@@ -45,9 +45,7 @@ using keyring_file::g_config_pod;
 */
 DLL_EXPORT int keyring_file_component_exported_symbol() { return 0; }
 
-namespace keyring_file {
-
-namespace config {
+namespace keyring_file::config {
 
 char *g_component_path = nullptr;
 char *g_instance_path = nullptr;
@@ -69,10 +67,7 @@ const std::string config_options[] = {"read_local_config", "path", "read_only"};
 bool find_and_read_config_file(std::unique_ptr<Config_pod> &config_pod,
                                std::string &err) {
   config_pod = std::make_unique<Config_pod>();
-  if (!config_pod) {
-    err = "Failed to allocate memory for configuration details";
-    return true;
-  }
+
   /* Get shared library location */
   std::string path(g_component_path);
 
@@ -86,7 +81,7 @@ bool find_and_read_config_file(std::unique_ptr<Config_pod> &config_pod,
     full_path.append(config_file_name);
     return false;
   };
-  if (set_config_path(path) == true) {
+  if (set_config_path(path)) {
     err = "Failed to set path to configuration file";
     return true;
   }
@@ -97,9 +92,9 @@ bool find_and_read_config_file(std::unique_ptr<Config_pod> &config_pod,
 
   {
     bool read_local_config = false;
-    if (config_reader->get_element<bool>(config_options[0],
-                                         read_local_config) == false) {
-      if (read_local_config == true) {
+    if (!config_reader->get_element<bool>(config_options[0],
+                                          read_local_config)) {
+      if (read_local_config) {
         config_reader.reset();
         /*
           Read config file from current working directory
@@ -107,20 +102,19 @@ bool find_and_read_config_file(std::unique_ptr<Config_pod> &config_pod,
           current working directory appropriately.
         */
         std::string instance_path(g_instance_path);
-        if (set_config_path(instance_path) == true)
-          instance_path = config_file_name;
+        if (set_config_path(instance_path)) instance_path = config_file_name;
         config_reader = std::make_unique<Config_reader>(instance_path);
       }
     }
   }
   std::string missing_option;
-  if (config_reader->get_element<std::string>(
-          config_options[1], config_pod.get()->config_file_path_)) {
+  if (config_reader->get_element<std::string>(config_options[1],
+                                              config_pod->config_file_path_)) {
     missing_option = config_options[1];
     goto error;
   }
   if (config_reader->get_element<bool>(config_options[2],
-                                       config_pod.get()->read_only_)) {
+                                       config_pod->read_only_)) {
     missing_option = config_options[2];
     goto error;
   }
@@ -136,7 +130,7 @@ bool create_config(
         &metadata) {
   metadata =
       std::make_unique<std::vector<std::pair<std::string, std::string>>>();
-  if (metadata.get() == nullptr) return true;
+  if (metadata == nullptr) return true;
   keyring_file::config::Config_pod config_pod;
   bool global_config_available = false;
   if (g_config_pod != nullptr) {
@@ -144,32 +138,31 @@ bool create_config(
     global_config_available = true;
   }
 
-  for (auto entry : keyring_file::config::s_component_metadata) {
-    metadata.get()->push_back(std::make_pair(entry[0], entry[1]));
+  for (const auto *entry : keyring_file::config::s_component_metadata) {
+    metadata->push_back(std::make_pair(entry[0], entry[1]));
   }
 
   /* Status */
-  metadata.get()->push_back(std::make_pair(
+  metadata->push_back(std::make_pair(
       "Component_status",
       keyring_file::g_component_callbacks->keyring_initialized() ? "Active"
                                                                  : "Disabled"));
 
   /* Backend file */
-  metadata.get()->push_back(std::make_pair(
-      "Data_file", ((global_config_available == true)
-                        ? ((config_pod.config_file_path_.length() == 0)
-                               ? "<NONE>"
-                               : config_pod.config_file_path_)
-                        : "<NOT APPLICABLE>")));
+  metadata->push_back(std::make_pair(
+      "Data_file",
+      (global_config_available ? ((config_pod.config_file_path_.length() == 0)
+                                      ? "<NONE>"
+                                      : config_pod.config_file_path_)
+                               : "<NOT APPLICABLE>")));
 
   /* Read only flag */
-  metadata.get()->push_back(std::make_pair(
-      "Read_only", ((global_config_available == true)
-                        ? ((config_pod.read_only_ == true) ? "Yes" : "No")
-                        : "<NOT APPLICABLE>")));
+  metadata->push_back(std::make_pair(
+      "Read_only",
+      (global_config_available ? (config_pod.read_only_ ? "Yes" : "No")
+                               : "<NOT APPLICABLE>")));
 
   return false;
 }
 
-}  // namespace config
-}  // namespace keyring_file
+}  // namespace keyring_file::config
