@@ -1,4 +1,4 @@
-/* Copyright (c) 2023, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2023, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -121,38 +121,36 @@ const char *assertion::get_rp_id() { return fido_assert_rp_id(m_assert); }
 /**
   Discover available devices
 
-  Caller should always free num_devices + 1.
-
   @param [in] num_devices Number of devices to open
 
   @returns handle to fido_dev_info_t array on success. null otherwise.
 */
 fido_dev_info_t *assertion::discover_fido2_devices(size_t num_devices) {
-  fido_dev_info_t *dev_infos = fido_dev_info_new(num_devices + 1);
+  fido_dev_info_t *dev_infos = fido_dev_info_new(num_devices);
   if (!dev_infos) {
     get_plugin_messages("Failed to allocate memory for fido_dev_info_t",
                         message_type::ERROR);
     return nullptr;
   }
-  auto cleanup_guard = create_scope_guard(
-      [&] { fido_dev_info_free(&dev_infos, num_devices + 1); });
+  auto cleanup_guard =
+      create_scope_guard([&] { fido_dev_info_free(&dev_infos, num_devices); });
   size_t olen = 0;
-  (void)fido_dev_info_manifest(dev_infos, num_devices + 1, &olen);
+  (void)fido_dev_info_manifest(dev_infos, num_devices, &olen);
   if (olen == 0) {
     get_plugin_messages("No FIDO device available on client host.",
                         message_type::ERROR);
     return nullptr;
   }
 
-  if (num_devices < olen) {
+  if (num_devices > olen) {
     std::stringstream error;
-    error
-        << "Expected maximum of '" << num_devices
-        << "' FIDO device(s). Please unplug some of the devices and try again.";
+    error << "Requested FIDO device '" << num_devices - 1
+          << "' not present. Please correct the device id supplied or make "
+             "sure the device is present.";
     get_plugin_messages(error.str(), message_type::ERROR);
     return nullptr;
   }
 
-  cleanup_guard.release();
+  cleanup_guard.commit();
   return dev_infos;
 }
