@@ -1,15 +1,16 @@
-/* Copyright (c) 2014, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -566,8 +567,9 @@ class Table_map_event : public Binary_log_event {
                                      columns, optimized to minimize
                                      space when many columns have the
                                      same charset. */
-    COLUMN_VISIBILITY             /* Flag to indicate column visibility
+    COLUMN_VISIBILITY,            /* Flag to indicate column visibility
                                      attribute. */
+    VECTOR_DIMENSIONALITY         /* Vector column dimensionality */
   };
 
   /**
@@ -604,6 +606,7 @@ class Table_map_event : public Binary_log_event {
     std::vector<str_vector> m_enum_str_value;
     std::vector<str_vector> m_set_str_value;
     std::vector<unsigned int> m_geometry_type;
+    std::vector<unsigned int> m_vector_dimensionality;
     /*
       The uint_pair means <column index, prefix length>.  Prefix length is 0 if
       whole column value is used.
@@ -908,10 +911,9 @@ class Rows_event : public Binary_log_event {
     Constructs an event directly. The members are assigned default values.
 
     @param type_arg          Type of ROW_EVENT. Expected types are:
-                             - WRITE_ROWS_EVENT, WRITE_ROWS_EVENT_V1
-                             - UPDATE_ROWS_EVENT, UPDATE_ROWS_EVENT_V1,
-                               PARTIAL_UPDATE_ROWS_EVENT
-                             - DELETE_ROWS_EVENT, DELETE_ROWS_EVENT_V1
+                             - WRITE_ROWS_EVENT
+                             - UPDATE_ROWS_EVENT, PARTIAL_UPDATE_ROWS_EVENT
+                             - DELETE_ROWS_EVENT
   */
   explicit Rows_event(Log_event_type type_arg)
       : Binary_log_event(type_arg),
@@ -1165,6 +1167,11 @@ class Delete_rows_event : public virtual Rows_event {
     <td>char array</td>
     <td>Records the original query executed in RBR </td>
   </tr>
+  <tr>
+    <td>m_rows_query_length</td>
+    <td>integer</td>
+    <td>Stores the original query length executed in RBR </td>
+  </tr>
   </table>
 */
 class Rows_query_event : public virtual Ignorable_event {
@@ -1190,12 +1197,15 @@ class Rows_query_event : public virtual Ignorable_event {
     ROWS_QUERY_LOG_EVENT in the header object in Binary_log_event.
   */
   Rows_query_event()
-      : Ignorable_event(ROWS_QUERY_LOG_EVENT), m_rows_query(nullptr) {}
+      : Ignorable_event(ROWS_QUERY_LOG_EVENT),
+        m_rows_query(nullptr),
+        m_rows_query_length(0) {}
 
   ~Rows_query_event() override;
 
  protected:
   char *m_rows_query;
+  size_t m_rows_query_length;
 };
 }  // namespace mysql::binlog::event
 
