@@ -1,4 +1,4 @@
-/* Copyright (c) 2014, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -612,8 +612,6 @@ end:
         "Fatal error during execution on the Applier module of Group "
         "Replication.";
     leave_group_on_failure::mask leave_actions;
-    if (applier_thd->killed)
-      LogPluginErr(ERROR_LEVEL, ER_GRP_RPL_APPLIER_THD_KILLED_BY_SQL_KILL);
     /*
       Only follow exit_state_action if we were already inside a group. We may
       happen to come across an applier error during the startup of GR (i.e.
@@ -716,7 +714,6 @@ int Applier_module::initialize_applier_thread() {
 
     struct timespec abstime;
     set_timespec(&abstime, 1);
-
     mysql_cond_timedwait(&run_cond, &run_lock, &abstime);
   }
 
@@ -869,7 +866,10 @@ int Applier_module::wait_for_applier_complete_suspension(
   */
   while (!suspended && !(*abort_flag) && !is_applier_thread_aborted() &&
          !applier_error) {
-    mysql_cond_wait(&suspension_waiting_condition, &suspend_lock);
+    struct timespec abstime;
+    set_timespec(&abstime, 1);
+    mysql_cond_timedwait(&suspension_waiting_condition, &suspend_lock,
+                         &abstime);
   }
 
   mysql_mutex_unlock(&suspend_lock);
