@@ -140,7 +140,7 @@ int Primary_election_handler::execute_primary_election(
         enum_primary_election_primary_change_status::
             PRIMARY_DID_NOT_CHANGE_NO_CANDIDATE,
         mode, PRIMARY_ELECTION_NO_CANDIDATES_ERROR);
-    if (enable_server_read_mode()) {
+    if (enable_server_read_mode("(GR) primary election failed")) {
       LogPluginErr(WARNING_LEVEL,
                    ER_GRP_RPL_ENABLE_READ_ONLY_FAILED); /* purecov: inspected */
     }
@@ -307,7 +307,7 @@ int Primary_election_handler::legacy_primary_election(
     member_actions_handler->trigger_actions(
         Member_actions::AFTER_PRIMARY_ELECTION);
   } else {
-    if (enable_server_read_mode()) {
+    if (enable_server_read_mode("(GR) new primary elected")) {
       LogPluginErr(WARNING_LEVEL,
                    ER_GRP_RPL_ENABLE_READ_ONLY_FAILED); /* purecov: inspected */
     }
@@ -444,45 +444,20 @@ Group_member_info_list_iterator sort_and_get_lowest_version_member_position(
    */
   it = all_members_info->begin();
   Group_member_info *first_member = *it;
-  uint32 lowest_major_version =
-      first_member->get_member_version().get_major_version();
 
   /* to avoid read compatibility issue leader should be picked only from lowest
      version members so save position where member version differs.
-     From 8.0.17 patch version will be considered during version comparison.
 
      set lowest_version_end when major version changes
 
-     eg: for a list: 5.7.18, 5.7.18, 5.7.19, 5.7.20, 5.7.21, 8.0.2
-         the members to be considered for election will be:
-            5.7.18, 5.7.18, 5.7.19, 5.7.20, 5.7.21
-         and server_uuid based algorithm will be used to elect primary
-
-     eg: for a list: 5.7.20, 5.7.21, 8.0.2, 8.0.2
-         the members to be considered for election will be:
-            5.7.20, 5.7.21
-         and member weight based algorithm will be used to elect primary
-
      eg: for a list: 8.0.17, 8.0.18, 8.0.19
          the members to be considered for election will be:
             8.0.17
-
-     eg: for a list: 8.0.13, 8.0.17, 8.0.18
-         the members to be considered for election will be:
-            8.0.13, 8.0.17, 8.0.18
-         and member weight based algorithm will be used to elect primary
   */
 
   for (it = all_members_info->begin() + 1; it != all_members_info->end();
        it++) {
-    if (first_member->get_member_version() >=
-            PRIMARY_ELECTION_PATCH_CONSIDERATION &&
-        (first_member->get_member_version() != (*it)->get_member_version())) {
-      lowest_version_end = it;
-      break;
-    }
-    if (lowest_major_version !=
-        (*it)->get_member_version().get_major_version()) {
+    if (first_member->get_member_version() != (*it)->get_member_version()) {
       lowest_version_end = it;
       break;
     }
