@@ -1,15 +1,16 @@
-/* Copyright (c) 2015, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -438,7 +439,9 @@ long Session_plugin_thread::wait_for_method_execution() {
   while (!m_method_execution_completed) {
     DBUG_PRINT("sleep",
                ("Waiting for the plugin session thread to execute a method"));
-    mysql_cond_wait(&m_method_cond, &m_method_lock);
+    struct timespec abstime;
+    set_timespec(&abstime, 1);
+    mysql_cond_timedwait(&m_method_cond, &m_method_lock, &abstime);
   }
   mysql_mutex_unlock(&m_method_lock);
   return m_method_execution_return_value;
@@ -473,7 +476,9 @@ int Session_plugin_thread::launch_session_thread(void *plugin_pointer_var,
   while (m_session_thread_state.is_alive_not_running() &&
          !m_session_thread_error) {
     DBUG_PRINT("sleep", ("Waiting for the plugin session thread to start"));
-    mysql_cond_wait(&m_run_cond, &m_run_lock);
+    struct timespec abstime;
+    set_timespec(&abstime, 1);
+    mysql_cond_timedwait(&m_run_cond, &m_run_lock, &abstime);
   }
 
   mysql_mutex_unlock(&m_run_lock);
@@ -579,7 +584,9 @@ int Session_plugin_thread::session_thread_handler() {
   while (!m_session_thread_terminate) {
     DBUG_PRINT("sleep", ("Waiting for the plugin session thread"
                          " to be signaled termination"));
-    mysql_cond_wait(&m_run_cond, &m_run_lock);
+    struct timespec abstime;
+    set_timespec(&abstime, 1);
+    mysql_cond_timedwait(&m_run_cond, &m_run_lock, &abstime);
   }
   mysql_mutex_unlock(&m_run_lock);
 
