@@ -1,16 +1,17 @@
 /*
- * Copyright (c) 2015, 2023, Oracle and/or its affiliates.
+ * Copyright (c) 2015, 2024, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
  * as published by the Free Software Foundation.
  *
- * This program is also distributed with certain software (including
+ * This program is designed to work with certain software (including
  * but not limited to OpenSSL) that is licensed under separate terms,
  * as designated in a particular file or component or in included license
  * documentation.  The authors of MySQL hereby grant you an additional
  * permission to link the program and your derivative works with the
- * separately licensed software that they have included with MySQL.
+ * separately licensed software that they have either included with
+ * the program or referenced in the documentation.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -151,7 +152,8 @@ int make_vio_timeout(const int64_t value) {
   return -1;
 }
 
-XError ssl_verify_server_cert(Vio *vio, const std::string &server_hostname) {
+XError ssl_verify_server_cert(Vio *vio, const std::string &server_hostname
+                              [[maybe_unused]]) {
   SSL *ssl = reinterpret_cast<SSL *>(vio->ssl_arg);
 
   if (nullptr == ssl) {
@@ -368,7 +370,8 @@ Connection_impl::Connection_impl(std::shared_ptr<Context> context)
 
 Connection_impl::~Connection_impl() { close(); }
 
-XError Connection_impl::connect_to_localhost(const std::string &unix_socket) {
+XError Connection_impl::connect_to_localhost(const std::string &unix_socket
+                                             [[maybe_unused]]) {
   m_connection_type = Connection_type::Unix_socket;
   m_hostname = "localhost";
 #if defined(HAVE_SYS_UN_H)
