@@ -88,6 +88,7 @@ const char *const native_mysql_functions[] = {
     "FROM_BASE64",
     "FROM_DAYS",
     "FROM_UNIXTIME",
+    "FROM_VECTOR",
     "GET_LOCK",
     "GREATEST",
     "GTID_SUBSET",
@@ -179,6 +180,7 @@ const char *const native_mysql_functions[] = {
     "STATEMENT_DIGEST",
     "STATEMENT_DIGEST_TEXT",
     "STRCMP",
+    "STRING_TO_VECTOR",
     "STR_TO_DATE",
     "ST_AREA",
     "ST_ASBINARY",
@@ -279,6 +281,7 @@ const char *const native_mysql_functions[] = {
     "TO_BASE64",
     "TO_DAYS",
     "TO_SECONDS",
+    "TO_VECTOR",
     "UCASE",
     "UNCOMPRESS",
     "UNCOMPRESSED_LENGTH",
@@ -290,9 +293,10 @@ const char *const native_mysql_functions[] = {
     "UUID_SHORT",
     "UUID_TO_BIN",
     "VALIDATE_PASSWORD_STRENGTH",
+    "VECTOR_DIM",
+    "VECTOR_TO_STRING",
     "VERSION",
     "WAIT_FOR_EXECUTED_GTID_SET",
-    "WAIT_UNTIL_SQL_THREAD_AFTER_GTIDS",
     "WEEKDAY",
     "WEEKOFYEAR",
     "YEARWEEK",
