@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2015, 2025, Oracle and/or its affiliates.
+ * Copyright (c) 2015, 2024, Oracle and/or its affiliates.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License, version 2.0,
@@ -35,8 +35,9 @@
 #include "plugin/x/tests/driver/driver_command_line_options.h"
 #include "plugin/x/tests/driver/processor/stream_processor.h"
 
-static void ignore_traces_from_libraries(enum loglevel ll, uint32_t ecode,
-                                         va_list args) {}
+static void ignore_traces_from_libraries(enum loglevel /*ll*/,
+                                         uint32_t /*ecode*/, va_list /*args*/) {
+}
 
 bool parse_mysql_connstring(const std::string &connstring,
                             std::string *protocol, std::string *user,
