@@ -1,16 +1,17 @@
 /*
-  Copyright (c) 2022, 2023, Oracle and/or its affiliates.
+  Copyright (c) 2022, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,7 +28,7 @@
 
 #include <array>
 
-static constexpr std::array<const char *, 19> metadata_cache_supported_options{{
+static constexpr std::array metadata_cache_supported_options{
     "user",
     "ttl",
     "auth_cache_ttl",
@@ -39,7 +40,6 @@ static constexpr std::array<const char *, 19> metadata_cache_supported_options{{
     "thread_stack_size",
     "use_gr_notifications",
     "cluster_type",
-    "bootstrap_server_addresses",
     "ssl_mode",
     "ssl_cipher",
     "tls_version",
@@ -47,6 +47,6 @@ static constexpr std::array<const char *, 19> metadata_cache_supported_options{{
     "ssl_capath",
     "ssl_crl",
     "ssl_crlpath",
-}};
+};
 
 #endif /* METADATA_CACHE_SUPPORTED_OPTIONS_INCLUDED */
