@@ -28,7 +28,6 @@
 #include <stdexcept>
 #include <string>
 #include <system_error>
-#include <type_traits>
 
 #ifdef _WIN32
 #include <WinSock2.h>
@@ -38,6 +37,7 @@
 #include <mysqld_error.h>
 #include <openssl/ssl.h>
 
+#include "duk_logging.h"
 #include "duk_module_shim.h"
 #include "duk_node_fs.h"
 #include "duktape.h"
@@ -544,11 +544,10 @@ struct DuktapeStatementReader::Pimpl {
   }
 
   template <class INT_TYPE>
-  INT_TYPE get_object_integer_value(duk_idx_t idx, const std::string &field,
-                                    const INT_TYPE default_val = 0,
-                                    bool is_required = false)
-    requires(std::is_unsigned_v<INT_TYPE>)
-  {
+  typename std::enable_if<std::is_unsigned<INT_TYPE>::value, INT_TYPE>::type
+  get_object_integer_value(duk_idx_t idx, const std::string &field,
+                           const INT_TYPE default_val = 0,
+                           bool is_required = false) {
     INT_TYPE value;
 
     duk_get_prop_string(ctx, idx, field.c_str());
@@ -1007,7 +1006,7 @@ static classic_protocol::message::server::Greeting default_server_greeting() {
 
   uint16_t status_flags = 0;
   uint8_t character_set = 0;
-  std::string auth_method = MySQLNativePassword::name;
+  std::string auth_method = CachingSha2Password::name;
   std::string nonce = "01234567890123456789";
 
   return {0x0a,
