@@ -1,16 +1,17 @@
 /*
-  Copyright (c) 2018, 2023, Oracle and/or its affiliates.
+  Copyright (c) 2018, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,6 +31,7 @@
 #include <optional>
 #include <string>
 #include <utility>
+#include <variant>
 #include <vector>
 
 #include <openssl/bio.h>
@@ -47,7 +49,9 @@
 #include "mysql/harness/net_ts/internet.h"
 #include "mysql/harness/net_ts/socket.h"
 #include "mysql/harness/stdx/expected.h"
+#include "mysql/harness/stdx/monitor.h"
 #include "mysqlrouter/classic_protocol_message.h"
+#include "mysqlrouter/classic_protocol_session_track.h"
 
 namespace server_mock {
 
@@ -56,12 +60,25 @@ namespace server_mock {
  **/
 using RowValueType = std::vector<std::optional<std::string>>;
 
+using session_tracker_field =
+    std::variant<classic_protocol::session_track::TransactionCharacteristics,
+                 classic_protocol::session_track::TransactionState,
+                 classic_protocol::session_track::SystemVariable,
+                 classic_protocol::session_track::Schema,
+                 classic_protocol::session_track::State,
+                 classic_protocol::session_track::Gtid>;
+
+std::string encode_session_trackers(
+    const std::vector<session_tracker_field> &trackers);
+
 /** @brief Keeps result data for single SQL statement that returns
  *         resultset.
  **/
 struct ResultsetResponse {
   std::vector<classic_protocol::message::server::ColumnMeta> columns;
   std::vector<RowValueType> rows;
+
+  classic_protocol::message::server::Eof end_of_rows;
 };
 
 using OkResponse = classic_protocol::message::server::Ok;
@@ -86,16 +103,10 @@ class ProtocolBase {
                TlsServerContext &tls_ctx);
 
   ProtocolBase(const ProtocolBase &) = delete;
-  ProtocolBase(ProtocolBase &&) = default;
+  ProtocolBase(ProtocolBase &&) = delete;
 
   ProtocolBase &operator=(const ProtocolBase &) = delete;
-  ProtocolBase &operator=(ProtocolBase &&rhs) {
-    client_socket_ = std::move(rhs.client_socket_);
-    ssl_ = std::move(rhs.ssl_);
-    tls_ctx_ = std::move(rhs.tls_ctx_);
-
-    return *this;
-  }
+  ProtocolBase &operator=(ProtocolBase &&rhs) = delete;
 
   virtual ~ProtocolBase() = default;
 
@@ -103,15 +114,7 @@ class ProtocolBase {
   virtual void encode_error(const ErrorResponse &resp) = 0;
 
   // throws std::system_error
-  virtual void encode_ok(const uint64_t affected_rows = 0,
-                         const uint64_t last_insert_id = 0,
-                         const uint16_t server_status = 0,
-                         const uint16_t warning_count = 0) = 0;
-
-  void encode_ok(const OkResponse &resp) {
-    encode_ok(resp.affected_rows(), resp.last_insert_id(),
-              resp.status_flags().to_ulong(), resp.warning_count());
-  }
+  virtual void encode_ok(const OkResponse &resp) = 0;
 
   // throws std::system_error
   virtual void encode_resultset(const ResultsetResponse &response) = 0;
@@ -234,12 +237,24 @@ class ProtocolBase {
 
   template <class CompletionToken>
   void async_receive(CompletionToken &&token) {
-    if (is_tls()) {
-      return async_receive_tls(std::forward<CompletionToken>(token));
-    } else {
-      return net::async_read(client_socket_, net::dynamic_buffer(recv_buffer_),
-                             std::forward<CompletionToken>(token));
-    }
+    is_terminated_([&](const bool killed) {
+      if (killed) {
+        net::async_completion<CompletionToken, void(std::error_code, size_t)>
+            init{token};
+
+        net::defer(client_socket_.get_executor(),
+                   [compl_handler = std::move(init.completion_handler)]() {
+                     compl_handler(
+                         make_error_code(std::errc::operation_canceled), 0);
+                   });
+      } else if (is_tls()) {
+        return async_receive_tls(std::forward<CompletionToken>(token));
+      } else {
+        return net::async_read(client_socket_,
+                               net::dynamic_buffer(recv_buffer_),
+                               std::forward<CompletionToken>(token));
+      }
+    });
   }
 
   template <class CompletionToken>
@@ -289,6 +304,22 @@ class ProtocolBase {
 
   std::string username() const { return username_; }
 
+  void server_auth_method_name(const std::string &auth_method_name) {
+    server_auth_method_name_ = auth_method_name;
+  }
+
+  std::string server_auth_method_name() const {
+    return server_auth_method_name_;
+  }
+
+  void server_auth_method_data(const std::string &auth_method_data) {
+    server_auth_method_data_ = auth_method_data;
+  }
+
+  std::string server_auth_method_data() const {
+    return server_auth_method_data_;
+  }
+
   void auth_method_name(const std::string &auth_method_name) {
     auth_method_name_ = auth_method_name;
   }
@@ -318,16 +349,32 @@ class ProtocolBase {
 
   void cancel();
 
+  /**
+   * terminate the current connection.
+   *
+   * sets is_terminated(true) and cancels the current operation.
+   *
+   * may be called from another thread.
+   */
+  void terminate();
+
   net::io_context &io_context() {
     return client_socket_.get_executor().context();
   }
 
+ private:
+  Monitor<bool> is_terminated_{false};
+
  protected:
   socket_type client_socket_;
   endpoint_type client_ep_;
   net::steady_timer exec_timer_{io_context()};
 
   std::string username_{};
+
+  std::string server_auth_method_name_{};
+  std::string server_auth_method_data_{};
+
   std::string auth_method_name_{};
   std::string auth_method_data_{};
 
