@@ -31,13 +31,16 @@
 #include <string>
 
 #include "mysql/harness/config_parser.h"
+#include "mysql/harness/dynamic_config.h"
 #include "mysql/harness/loader.h"
 #include "mysql/harness/logging/logging.h"
 #include "mysql/harness/plugin.h"
 #include "mysql/harness/plugin_config.h"
+#include "mysql/harness/section_config_exposer.h"
 #include "mysql/harness/utility/string.h"  // ::join()
 
-#include "mysqlrouter/http_server_component.h"
+#include "mysqlrouter/component/http_server_component.h"
+#include "mysqlrouter/http_constants.h"
 #include "mysqlrouter/rest_api_component.h"
 #include "mysqlrouter/supported_rest_options.h"
 
@@ -73,11 +76,11 @@ class RestMetadataCachePluginConfig : public mysql_harness::BasePluginConfig {
     GET_OPTION_CHECKED(require_realm, section, kRequireRealm, StringOption{});
   }
 
-  std::string get_default(const std::string & /* option */) const override {
+  std::string get_default(std::string_view /* option */) const override {
     return {};
   }
 
-  bool is_required(const std::string &option) const override {
+  bool is_required(std::string_view option) const override {
     if (option == kRequireRealm) return true;
     return false;
   }
@@ -148,57 +151,86 @@ static void init(mysql_harness::PluginFuncEnv *env) {
 using JsonPointer = RestApiComponent::JsonPointer;
 using JsonValue = RestApiComponent::JsonValue;
 
-#define STR(s) \
-  { s, strlen(s), rapidjson::kPointerInvalidIndex }
-static const std::array<JsonPointer::Token, 2> metadata_name_param_tokens{
-    {STR("parameters"), STR("metadataNameParam")}};
+static JsonPointer::Token make_json_pointer_token(std::string_view token) {
+  return {token.data(), token.size(), rapidjson::kPointerInvalidIndex};
+}
 
-static const std::array<JsonPointer::Token, 2> cluster_name_param_tokens{
-    {STR("parameters"), STR("clusterNameParam")}};
+static const std::array metadata_name_param_tokens{
+    make_json_pointer_token("parameters"),
+    make_json_pointer_token("metadataNameParam"),
+};
 
-static const std::array<JsonPointer::Token, 2> metadata_status_def_tokens{
-    {STR("definitions"), STR("MetadataStatus")}};
+static const std::array cluster_name_param_tokens{
+    make_json_pointer_token("parameters"),
+    make_json_pointer_token("clusterNameParam"),
+};
 
-static const std::array<JsonPointer::Token, 2> metadata_list_def_tokens{
-    {STR("definitions"), STR("MetadataList")}};
+static const std::array metadata_status_def_tokens{
+    make_json_pointer_token("definitions"),
+    make_json_pointer_token("MetadataStatus"),
+};
 
-static const std::array<JsonPointer::Token, 2> metadata_summary_def_tokens{
-    {STR("definitions"), STR("MetadataSummary")}};
+static const std::array metadata_list_def_tokens{
+    make_json_pointer_token("definitions"),
+    make_json_pointer_token("MetadataList"),
+};
 
-static const std::array<JsonPointer::Token, 2> metadata_config_def_tokens{
-    {STR("definitions"), STR("MetadataConfig")}};
+static const std::array metadata_summary_def_tokens{
+    make_json_pointer_token("definitions"),
+    make_json_pointer_token("MetadataSummary"),
+};
 
-static const std::array<JsonPointer::Token, 2> cluster_node_summary_def_tokens{
-    {STR("definitions"), STR("ClusterNodeSummary")}};
+static const std::array metadata_config_def_tokens{
+    make_json_pointer_token("definitions"),
+    make_json_pointer_token("MetadataConfig"),
+};
 
-static const std::array<JsonPointer::Token, 2> cluster_node_list_def_tokens{
-    {STR("definitions"), STR("ClusterNodeList")}};
+static const std::array cluster_node_summary_def_tokens{
+    make_json_pointer_token("definitions"),
+    make_json_pointer_token("ClusterNodeSummary"),
+};
 
-static const std::array<JsonPointer::Token, 2> cluster_summary_def_tokens{
-    {STR("definitions"), STR("ClusterSummary")}};
+static const std::array cluster_node_list_def_tokens{
+    make_json_pointer_token("definitions"),
+    make_json_pointer_token("ClusterNodeList"),
+};
 
-static const std::array<JsonPointer::Token, 2> cluster_list_def_tokens{
-    {STR("definitions"), STR("ClusterList")}};
+static const std::array cluster_summary_def_tokens{
+    make_json_pointer_token("definitions"),
+    make_json_pointer_token("ClusterSummary"),
+};
 
-static const std::array<JsonPointer::Token, 2> metadata_status_path_tokens{
-    {STR("paths"), STR("/metadata/{metadataName}/status")}};
+static const std::array cluster_list_def_tokens{
+    make_json_pointer_token("definitions"),
+    make_json_pointer_token("ClusterList"),
+};
 
-static const std::array<JsonPointer::Token, 2> metadata_config_path_tokens{
-    {STR("paths"), STR("/metadata/{metadataName}/config")}};
+static const std::array metadata_status_path_tokens{
+    make_json_pointer_token("paths"),
+    make_json_pointer_token("/metadata/{metadataName}/status"),
+};
 
-// static const std::array<JsonPointer::Token, 2> cluster_list_path_tokens{
-//    {STR("paths"), STR("/clusters")}};
+static const std::array metadata_config_path_tokens{
+    make_json_pointer_token("paths"),
+    make_json_pointer_token("/metadata/{metadataName}/config"),
+};
 
-// static const std::array<JsonPointer::Token, 2> cluster_node_list_path_tokens{
-//    {STR("paths"), STR("/clusters/{clusterName}/nodes")}};
+// static const std::array cluster_list_path_tokens{
+//    {make_json_pointer_token("paths"), make_json_pointer_token("/clusters")}};
 
-static const std::array<JsonPointer::Token, 2> metadata_list_path_tokens{
-    {STR("paths"), STR("/metadata")}};
+// static const std::array cluster_node_list_path_tokens{
+//    {make_json_pointer_token("paths"),
+//    make_json_pointer_token("/clusters/{clusterName}/nodes")}};
 
-static const std::array<JsonPointer::Token, 2> tags_append_tokens{
-    {STR("tags"), STR("-")}};
+static const std::array metadata_list_path_tokens{
+    make_json_pointer_token("paths"),
+    make_json_pointer_token("/metadata"),
+};
 
-#undef STR
+static const std::array tags_append_tokens{
+    make_json_pointer_token("tags"),
+    make_json_pointer_token("-"),
+};
 
 std::string json_pointer_stringfy(const JsonPointer &ptr) {
   rapidjson::StringBuffer sb;
@@ -825,20 +857,18 @@ static void start(mysql_harness::PluginFuncEnv *env) {
 
   const bool spec_adder_executed = rest_api_srv.try_process_spec(spec_adder);
 
-  std::array<RestApiComponentPath, 3> paths{{
-      {rest_api_srv, RestMetadataCacheStatus::path_regex,
-       std::make_unique<RestMetadataCacheStatus>(require_realm_metadata_cache)},
-      {rest_api_srv, RestMetadataCacheConfig::path_regex,
-       std::make_unique<RestMetadataCacheConfig>(require_realm_metadata_cache)},
-      {rest_api_srv, RestMetadataCacheList::path_regex,
-       std::make_unique<RestMetadataCacheList>(require_realm_metadata_cache)},
+  std::array paths{
+      RestApiComponentPath{rest_api_srv, RestMetadataCacheStatus::path_regex,
+                           std::make_unique<RestMetadataCacheStatus>(
+                               require_realm_metadata_cache)},
+      RestApiComponentPath{rest_api_srv, RestMetadataCacheConfig::path_regex,
+                           std::make_unique<RestMetadataCacheConfig>(
+                               require_realm_metadata_cache)},
+      RestApiComponentPath{rest_api_srv, RestMetadataCacheList::path_regex,
+                           std::make_unique<RestMetadataCacheList>(
+                               require_realm_metadata_cache)},
 
-      // The socpe of WL#12441 was limited and does not include those:
-      //  {rest_api_srv, RestClustersList::path_regex,
-      //                        std::make_unique<RestClustersList>(require_realm_metadata_cache)},
-      //  {rest_api_srv, RestClustersNodes::path_regex,
-      //                        std::make_unique<RestClustersNodes>(require_realm_metadata_cache)},
-  }};
+  };
 
   mysql_harness::on_service_ready(env);
 
@@ -856,11 +886,52 @@ static void start(mysql_harness::PluginFuncEnv *env) {
 #define DLLEXPORT
 #endif
 
-static const std::array<const char *, 2> required = {{
+static constexpr std::array required{
     "logger",
     // "metadata_cache",
     "rest_api",
-}};
+};
+
+namespace {
+
+class RestMetadataCacheConfigExposer
+    : public mysql_harness::SectionConfigExposer {
+ public:
+  using DC = mysql_harness::DynamicConfig;
+  RestMetadataCacheConfigExposer(
+      const bool initial, const RestMetadataCachePluginConfig &plugin_config,
+      const mysql_harness::ConfigSection &default_section)
+      : mysql_harness::SectionConfigExposer(
+            initial, default_section,
+            DC::SectionId{"rest_configs", kSectionName}),
+        plugin_config_(plugin_config) {}
+
+  void expose() override {
+    expose_option("require_realm", plugin_config_.require_realm,
+                  std::string(kHttpDefaultAuthRealmName));
+  }
+
+ private:
+  const RestMetadataCachePluginConfig &plugin_config_;
+};
+
+}  // namespace
+
+static void expose_configuration(mysql_harness::PluginFuncEnv *env,
+                                 const char * /*key*/, bool initial) {
+  const mysql_harness::AppInfo *info = get_app_info(env);
+
+  if (!info->config) return;
+
+  for (const mysql_harness::ConfigSection *section : info->config->sections()) {
+    if (section->name == kSectionName) {
+      RestMetadataCachePluginConfig config{section};
+      RestMetadataCacheConfigExposer(initial, config,
+                                     info->config->get_default_section())
+          .expose();
+    }
+  }
+}
 
 extern "C" {
 mysql_harness::Plugin DLLEXPORT harness_plugin_rest_metadata_cache = {
@@ -881,5 +952,6 @@ mysql_harness::Plugin DLLEXPORT harness_plugin_rest_metadata_cache = {
     true,     // declares_readiness
     rest_plugin_supported_options.size(),
     rest_plugin_supported_options.data(),
+    expose_configuration,
 };
 }
