@@ -1,16 +1,17 @@
 /*
-  Copyright (c) 2016, 2023, Oracle and/or its affiliates.
+  Copyright (c) 2016, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -52,21 +53,23 @@ IMPORT_LOG_FUNCTIONS()
 // TODO: possibly this should be made into a configurable option
 static const auto kPrimaryFailoverTimeout = 10s;
 
-static const std::set<std::string> supported_params{
+// we keep the allow_primary_reads on this list even though we no longer support
+// it, so that we give more specific error message for it
+static constexpr std::array supported_params{
     "role", "allow_primary_reads", "disconnect_on_promoted_to_primary",
     "disconnect_on_metadata_unavailable"};
 
 namespace {
 
-const constexpr std::array<
-    std::pair<std::string_view, DestMetadataCacheGroup::ServerRole>, 3>
-    known_roles{{
-        {"PRIMARY", DestMetadataCacheGroup::ServerRole::Primary},
-        {"SECONDARY", DestMetadataCacheGroup::ServerRole::Secondary},
-        {"PRIMARY_AND_SECONDARY",
-         DestMetadataCacheGroup::ServerRole::PrimaryAndSecondary},
-    }};
-}
+const constexpr auto known_roles = std::to_array<
+    std::pair<std::string_view, DestMetadataCacheGroup::ServerRole>>({
+    {"PRIMARY", DestMetadataCacheGroup::ServerRole::Primary},
+    {"SECONDARY", DestMetadataCacheGroup::ServerRole::Secondary},
+    {"PRIMARY_AND_SECONDARY",
+     DestMetadataCacheGroup::ServerRole::PrimaryAndSecondary},
+});
+
+}  // namespace
 
 DestMetadataCacheGroup::ServerRole get_server_role_from_uri(
     const mysqlrouter::URIQuery &uri) {
@@ -103,20 +106,6 @@ DestMetadataCacheGroup::ServerRole get_server_role_from_uri(
   return role_it->second;
 }
 
-namespace {
-std::string get_server_role_name(
-    const DestMetadataCacheGroup::ServerRole role) {
-  auto role_it =
-      std::find_if(known_roles.begin(), known_roles.end(),
-                   [role](const auto &p) { return p.second == role; });
-
-  if (role_it == known_roles.end()) {
-    return "unknown";
-  }
-
-  return std::string{role_it->first};
-}
-
 routing::RoutingStrategy get_default_routing_strategy(
     const DestMetadataCacheGroup::ServerRole role) {
   switch (role) {
@@ -129,27 +118,6 @@ routing::RoutingStrategy get_default_routing_strategy(
   return routing::RoutingStrategy::kUndefined;
 }
 
-/** @brief check that mode (if present) is correct for the role */
-bool mode_is_valid(const routing::Mode mode,
-                   const DestMetadataCacheGroup::ServerRole role) {
-  // no mode given, that's ok, nothing to check
-  if (mode == routing::Mode::kUndefined) {
-    return true;
-  }
-
-  switch (role) {
-    case DestMetadataCacheGroup::ServerRole::Primary:
-      return mode == routing::Mode::kReadWrite;
-    case DestMetadataCacheGroup::ServerRole::Secondary:
-    case DestMetadataCacheGroup::ServerRole::PrimaryAndSecondary:
-      return mode == routing::Mode::kReadOnly;
-    default:;  //
-               /* fall-through, no access mode is valid for that role */
-  }
-
-  return false;
-}
-
 // throws:
 // - runtime_error if invalid value for the option was discovered
 // - check_option_allowed() throws std::runtime_error (it is expected to throw
@@ -185,7 +153,7 @@ bool get_disconnect_on_promoted_to_primary(
   auto check_option_allowed = [&]() {
     if (role != DestMetadataCacheGroup::ServerRole::Secondary) {
       throw std::runtime_error("Option '" + kOptionName +
-                               "' is valid only for mode=SECONDARY");
+                               "' is valid only for role=SECONDARY");
     }
   };
 
@@ -203,8 +171,6 @@ bool get_disconnect_on_metadata_unavailable(const mysqlrouter::URIQuery &uri) {
                            check_option_allowed);
 }
 
-}  // namespace
-
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
 // doxygen confuses 'const mysqlrouter::URIQuery &query' with
 // 'std::map<std::string, std::string>'
@@ -212,12 +178,11 @@ DestMetadataCacheGroup::DestMetadataCacheGroup(
     net::io_context &io_ctx, const std::string &metadata_cache,
     const routing::RoutingStrategy routing_strategy,
     const mysqlrouter::URIQuery &query, const Protocol::Type protocol,
-    const routing::Mode mode, metadata_cache::MetadataCacheAPIBase *cache_api)
+    metadata_cache::MetadataCacheAPIBase *cache_api)
     : RouteDestination(io_ctx, protocol),
       cache_name_(metadata_cache),
       uri_query_(query),
       routing_strategy_(routing_strategy),
-      mode_(mode),
       server_role_(get_server_role_from_uri(query)),
       cache_api_(cache_api),
       disconnect_on_promoted_to_primary_(
@@ -319,66 +284,24 @@ DestMetadataCacheGroup::get_available_primaries(
 void DestMetadataCacheGroup::init() {
   // check if URI does not contain parameters that we don't understand
   for (const auto &uri_param : uri_query_) {
-    if (supported_params.count(uri_param.first) == 0) {
+    if (std::find(supported_params.begin(), supported_params.end(),
+                  uri_param.first) == supported_params.end()) {
       throw std::runtime_error(
           "Unsupported 'metadata-cache' parameter in URI: '" + uri_param.first +
           "'");
     }
   }
 
-  // if the routing strategy is set we don't allow mode to be set
-  if (routing_strategy_ != routing::RoutingStrategy::kUndefined &&
-      mode_ != routing::Mode::kUndefined) {
-    throw std::runtime_error(
-        "option 'mode' is not allowed together with 'routing_strategy' option");
-  }
-
-  bool routing_strategy_default{false};
   // if the routing_strategy is not set we go with the default based on the role
   if (routing_strategy_ == routing::RoutingStrategy::kUndefined) {
     routing_strategy_ = get_default_routing_strategy(server_role_);
-    routing_strategy_default = true;
   }
 
-  // check that mode (if present) is correct for the role
-  // we don't actually use it but support it for backward compatibility
-  // and parity with STANDALONE routing destinations
-  if (!mode_is_valid(mode_, server_role_)) {
-    throw std::runtime_error(
-        "mode '" + routing::get_mode_name(mode_) +
-        "' is not valid for 'role=" + get_server_role_name(server_role_) + "'");
-  }
-
-  // this is for backward compatibility
-  // old(allow_primary_reads + role=SECONDARY) = new
-  // (role=PRIMARY_AND_SECONDARY)
   auto query_part = uri_query_.find("allow_primary_reads");
   if (query_part != uri_query_.end()) {
-    if (server_role_ != ServerRole::Secondary) {
-      throw std::runtime_error(
-          "allow_primary_reads is supported only for SECONDARY routing");
-    }
-    if (!routing_strategy_default) {
-      throw std::runtime_error(
-          "allow_primary_reads is only supported for backward compatibility: "
-          "without routing_strategy but with mode defined, use "
-          "role=PRIMARY_AND_SECONDARY instead");
-    }
-    auto value = query_part->second;
-    std::transform(value.begin(), value.end(), value.begin(), ::tolower);
-    if (value == "yes") {
-      server_role_ = ServerRole::PrimaryAndSecondary;
-    } else if (value == "no") {
-      // it's a default but we allow it for consistency
-    } else {
-      throw std::runtime_error(
-          "Invalid value for allow_primary_reads option: '" +
-          query_part->second + "'");
-    }
-
-    log_warning(
-        "allow_primary_reads is deprecated, use role=PRIMARY_AND_SECONDARY "
-        "instead");
+    throw std::runtime_error(
+        "allow_primary_reads is no longer supported, use "
+        "role=PRIMARY_AND_SECONDARY instead");
   }
 
   // validate routing strategy:
