@@ -1,16 +1,17 @@
 /*
-  Copyright (c) 2023, Oracle and/or its affiliates.
+  Copyright (c) 2023, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -46,15 +47,19 @@
 #include "router_config.h"
 #include "router_test_helpers.h"
 #include "socket_operations.h"
+#include "stdx_expected_no_error.h"
 #include "tcp_port_pool.h"
 
 using mysqlrouter::ClusterType;
-using mysqlrouter::MetadataSchemaVersion;
-using mysqlrouter::MySQLSession;
-using ::testing::PrintToString;
 using namespace std::chrono_literals;
 using namespace std::string_literals;
 
+namespace mysqlrouter {
+std::ostream &operator<<(std::ostream &os, const MysqlError &e) {
+  return os << e.sql_state() << " code: " << e.value() << ": " << e.message();
+}
+}  // namespace mysqlrouter
+
 class GRStateTest : public RouterComponentMetadataTest {
  protected:
   std::string get_router_options_as_json_str(
@@ -83,7 +88,7 @@ class GRStateTest : public RouterComponentMetadataTest {
 
   std::string get_rw_split_routing_section(uint16_t accepting_port) {
     return get_metadata_cache_routing_section(
-        accepting_port, "PRIMARY_AND_SECONDARY", "round-robin", "", "rwsplit",
+        accepting_port, "PRIMARY_AND_SECONDARY", "round-robin", "rwsplit",
         "classic", {{"connection_sharing", "1"}, {"access_mode", "auto"}});
   }
 };
@@ -143,10 +148,10 @@ TEST_P(MetadataServerInvalidGRState, InvalidGRState) {
       get_metadata_cache_section(GetParam().cluster_type, "0.1");
   const auto router_rw_port = port_pool_.get_next_available();
   const std::string routing_rw_section = get_metadata_cache_routing_section(
-      router_rw_port, "PRIMARY", "first-available", "", "rw");
+      router_rw_port, "PRIMARY", "first-available", "rw");
   const auto router_ro_port = port_pool_.get_next_available();
   const std::string routing_ro_section = get_metadata_cache_routing_section(
-      router_ro_port, "SECONDARY", "round-robin", "", "ro");
+      router_ro_port, "SECONDARY", "round-robin", "ro");
   auto &router = launch_router(metadata_cache_section,
                                routing_rw_section + routing_ro_section,
                                md_servers_classic_ports, EXIT_SUCCESS,
@@ -163,26 +168,30 @@ TEST_P(MetadataServerInvalidGRState, InvalidGRState) {
   }
 
   // now promote first SECONDARY to become new PRIMARY
-  // make the old PRIMARY offline (static metadata does not change)
   for (const auto [i, http_port] :
        stdx::views::enumerate(md_servers_http_ports)) {
-    if (i == 0) {
-      // old PRIMARY sees itself as OFFLINE, does not see other nodes
-      const auto gr_nodes = std::vector<GRNode>{
-          {md_servers_classic_ports[0], "uuid-1", "OFFLINE", "PRIMARY"}};
-      set_mock_metadata(
-          http_port, "uuid", gr_nodes, 0,
-          classic_ports_to_cluster_nodes(md_servers_classic_ports));
-    } else {
-      // remaining nodes see the previous SECONDARY-1 as new primary
-      // they do not see old PRIMARY (it was expelled from the group)
-      const auto gr_nodes = std::vector<GRNode>{
-          {{md_servers_classic_ports[1], "uuid-2", "ONLINE", "PRIMARY"},
-           {md_servers_classic_ports[2], "uuid-3", "ONLINE", "SECONDARY"}}};
-      set_mock_metadata(
-          http_port, "uuid", gr_nodes, i - 1,
-          classic_ports_to_cluster_nodes(md_servers_classic_ports));
-    }
+    if (i == 0) continue;  // skip the PRIMARY for now.
+
+    // remaining nodes see the previous SECONDARY-1 as new primary
+    // they do not see old PRIMARY (it was expelled from the group)
+    const auto gr_nodes = std::vector<GRNode>{
+        {{md_servers_classic_ports[1], "uuid-2", "ONLINE", "PRIMARY"},
+         {md_servers_classic_ports[2], "uuid-3", "ONLINE", "SECONDARY"}}};
+    set_mock_metadata(http_port, "uuid", gr_nodes, i - 1,
+                      classic_ports_to_cluster_nodes(md_servers_classic_ports));
+  }
+
+  // ... make the old PRIMARY offline (static metadata does not change) _after_
+  // the SECONDARIES changed to get a consistent view when the metadata-cache
+  // queries the PRIMARY (and _then_ the SECONDARY)
+  {
+    auto http_port = md_servers_http_ports[0];
+
+    // old PRIMARY sees itself as OFFLINE, does not see other nodes
+    const auto gr_nodes = std::vector<GRNode>{
+        {md_servers_classic_ports[0], "uuid-1", "OFFLINE", "PRIMARY"}};
+    set_mock_metadata(http_port, "uuid", gr_nodes, 0,
+                      classic_ports_to_cluster_nodes(md_servers_classic_ports));
   }
 
   // check that the second metadata server (new PRIMARY) is queried for metadata
@@ -198,16 +207,26 @@ TEST_P(MetadataServerInvalidGRState, InvalidGRState) {
                      1);
 
   // new connections are now handled by new primary and the secon secondary
-  make_new_connection_ok(router_rw_port, md_servers_classic_ports[1]);
-  make_new_connection_ok(router_ro_port, md_servers_classic_ports[2]);
+  {
+    auto conn_res = make_new_connection(router_rw_port);
+    ASSERT_NO_ERROR(conn_res);
+    auto port_res = select_port(conn_res->get());
+    ASSERT_NO_ERROR(port_res);
+    EXPECT_EQ(*port_res, md_servers_classic_ports[1]);
+  }
+  {
+    auto conn_res = make_new_connection(router_ro_port);
+    ASSERT_NO_ERROR(conn_res);
+    auto port_res = select_port(conn_res->get());
+    ASSERT_NO_ERROR(port_res);
+    EXPECT_EQ(*port_res, md_servers_classic_ports[2]);
+  }
 }
 
 INSTANTIATE_TEST_SUITE_P(
     InvalidGRState, MetadataServerInvalidGRState,
     ::testing::Values(GRStateTestParams("metadata_dynamic_nodes_v2_gr.js",
-                                        "GR_V2", ClusterType::GR_V2),
-                      GRStateTestParams("metadata_dynamic_nodes.js", "GR_V1",
-                                        ClusterType::GR_V1)),
+                                        "GR_V2", ClusterType::GR_V2)),
     get_test_description);
 
 class MetadataServerNoQuorum
@@ -244,10 +263,10 @@ TEST_P(MetadataServerNoQuorum, NoQuorum) {
       get_metadata_cache_section(GetParam().cluster_type, "0.1");
   const auto router_rw_port = port_pool_.get_next_available();
   const std::string routing_rw_section = get_metadata_cache_routing_section(
-      router_rw_port, "PRIMARY", "first-available", "", "rw");
+      router_rw_port, "PRIMARY", "first-available", "rw");
   const auto router_ro_port = port_pool_.get_next_available();
   const std::string routing_ro_section = get_metadata_cache_routing_section(
-      router_ro_port, "SECONDARY", "round-robin", "", "ro");
+      router_ro_port, "SECONDARY", "round-robin", "ro");
   auto &router = launch_router(metadata_cache_section,
                                routing_rw_section + routing_ro_section,
                                md_servers_classic_ports, EXIT_SUCCESS,
@@ -266,28 +285,34 @@ TEST_P(MetadataServerNoQuorum, NoQuorum) {
   // now promote first SECONDARY to become new PRIMARY
   // make the old PRIMARY see other as OFFLINE and claim it is ONLINE
   // (static metadata does not change)
+  //
   for (const auto [i, http_port] :
        stdx::views::enumerate(md_servers_http_ports)) {
-    if (i == 0) {
-      // old PRIMARY still sees itself as ONLINE, but it lost quorum, do not
-      // see other GR members
-      const auto gr_nodes = std::vector<GRNode>{
-          {md_servers_classic_ports[0], "uuid-1", "ONLINE", "PRIMARY"},
-          {md_servers_classic_ports[1], "uuid-2", "UNREACHABLE", "SECONDARY"},
-          {md_servers_classic_ports[2], "uuid-3", "UNREACHABLE", "SECONDARY"}};
-      set_mock_metadata(
-          http_port, "uuid", gr_nodes, 0,
-          classic_ports_to_cluster_nodes(md_servers_classic_ports));
-    } else {
-      // remaining nodes see the previous SECONDARY-1 as new primary
-      // they do not see old PRIMARY (it was expelled from the group)
-      const auto gr_nodes = std::vector<GRNode>{
-          {{md_servers_classic_ports[1], "uuid-2", "ONLINE", "PRIMARY"},
-           {md_servers_classic_ports[2], "uuid-3", "ONLINE", "SECONDARY"}}};
-      set_mock_metadata(
-          http_port, "uuid", gr_nodes, i - 1,
-          classic_ports_to_cluster_nodes(md_servers_classic_ports));
-    }
+    if (i == 0) continue;  // skip the PRIMARY.
+
+    // remaining nodes see the previous SECONDARY-1 as new primary
+    // they do not see old PRIMARY (it was expelled from the group)
+    const auto gr_nodes = std::vector<GRNode>{
+        {{md_servers_classic_ports[1], "uuid-2", "ONLINE", "PRIMARY"},
+         {md_servers_classic_ports[2], "uuid-3", "ONLINE", "SECONDARY"}}};
+    set_mock_metadata(http_port, "uuid", gr_nodes, i - 1,
+                      classic_ports_to_cluster_nodes(md_servers_classic_ports));
+  }
+
+  // update the PRIMARY _after_ the SECONDARIES to ensure that the
+  // metadata-cache sees the same state all nodes once it switches
+  // from PRIMARY to SECONDARY
+  {
+    auto http_port = md_servers_http_ports[0];
+
+    // old PRIMARY still sees itself as ONLINE, but it lost quorum, do not
+    // see other GR members
+    const auto gr_nodes = std::vector<GRNode>{
+        {md_servers_classic_ports[0], "uuid-1", "ONLINE", "PRIMARY"},
+        {md_servers_classic_ports[1], "uuid-2", "UNREACHABLE", "SECONDARY"},
+        {md_servers_classic_ports[2], "uuid-3", "UNREACHABLE", "SECONDARY"}};
+    set_mock_metadata(http_port, "uuid", gr_nodes, 0,
+                      classic_ports_to_cluster_nodes(md_servers_classic_ports));
   }
 
   // check that the second metadata server (new PRIMARY) is queried for metadata
@@ -302,17 +327,28 @@ TEST_P(MetadataServerNoQuorum, NoQuorum) {
                          " is not a member of quorum group - skipping.",
                      1);
 
-  // new connections are now handled by new primary and the secon secondary
-  make_new_connection_ok(router_rw_port, md_servers_classic_ports[1]);
-  make_new_connection_ok(router_ro_port, md_servers_classic_ports[2]);
+  // new connections are now handled by new primary and the second secondary
+  {
+    auto conn_res = make_new_connection(router_rw_port);
+    ASSERT_NO_ERROR(conn_res);
+    auto port_res = select_port(conn_res->get());
+    ASSERT_NO_ERROR(port_res);
+    EXPECT_EQ(*port_res, md_servers_classic_ports[1]);
+  }
+
+  {
+    auto conn_res = make_new_connection(router_ro_port);
+    ASSERT_NO_ERROR(conn_res);
+    auto port_res = select_port(conn_res->get());
+    ASSERT_NO_ERROR(port_res);
+    EXPECT_EQ(*port_res, md_servers_classic_ports[2]);
+  }
 }
 
 INSTANTIATE_TEST_SUITE_P(
     NoQuorum, MetadataServerNoQuorum,
     ::testing::Values(GRStateTestParams("metadata_dynamic_nodes_v2_gr.js",
-                                        "GR_V2", ClusterType::GR_V2),
-                      GRStateTestParams("metadata_dynamic_nodes.js", "GR_V1",
-                                        ClusterType::GR_V1)),
+                                        "GR_V2", ClusterType::GR_V2)),
     get_test_description);
 
 class MetadataServerGRErrorStates
@@ -342,7 +378,7 @@ TEST_P(MetadataServerGRErrorStates, GRErrorStates) {
       get_metadata_cache_section(ClusterType::GR_V2, "0.1");
   const auto router_rw_port = port_pool_.get_next_available();
   const std::string routing_rw_section = get_metadata_cache_routing_section(
-      router_rw_port, "PRIMARY", "first-available", "", "rw");
+      router_rw_port, "PRIMARY", "first-available", "rw");
   auto &router = launch_router(metadata_cache_section, routing_rw_section,
                                {md_servers_classic_port}, EXIT_SUCCESS,
                                /*wait_for_notify_ready=*/-1s);
@@ -435,9 +471,9 @@ TEST_P(QuorumTest, Verify) {
   const std::string metadata_cache_section =
       get_metadata_cache_section(ClusterType::GR_V2, "0.2");
   const std::string routing_rw = get_metadata_cache_routing_section(
-      router_rw_port, "PRIMARY", "first-available", "", "rw");
+      router_rw_port, "PRIMARY", "first-available", "rw");
   const std::string routing_ro = get_metadata_cache_routing_section(
-      router_ro_port, "SECONDARY", "round-robin-with-fallback", "", "ro");
+      router_ro_port, "SECONDARY", "round-robin-with-fallback", "ro");
 
   const auto sync_point = (expect_rw_ok || expect_ro_ok)
                               ? ProcessManager::Spawner::SyncPoint::READY
@@ -457,13 +493,21 @@ TEST_P(QuorumTest, Verify) {
 
   for (int i = 0; i < 2; i++) {
     if (expect_rw_ok) {
-      make_new_connection_ok(router_rw_port, param.expected_rw_endpoints);
+      auto conn_res = make_new_connection(router_rw_port);
+      ASSERT_NO_ERROR(conn_res);
+      auto port_res = select_port(conn_res->get());
+      ASSERT_NO_ERROR(port_res);
+      EXPECT_THAT(param.expected_rw_endpoints, ::testing::Contains(*port_res));
     } else {
       verify_new_connection_fails(router_rw_port);
     }
 
     if (expect_ro_ok) {
-      make_new_connection_ok(router_ro_port, param.expected_ro_endpoints);
+      auto conn_res = make_new_connection(router_ro_port);
+      ASSERT_NO_ERROR(conn_res);
+      auto port_res = select_port(conn_res->get());
+      ASSERT_NO_ERROR(port_res);
+      EXPECT_THAT(param.expected_ro_endpoints, ::testing::Contains(*port_res));
     } else {
       verify_new_connection_fails(router_ro_port);
     }
@@ -722,8 +766,20 @@ TEST_F(QuorumConnectionLostStandaloneClusterTest, CheckInvalidConDropped) {
 
   // check that the new rw and rw-split connections don't go to the node that is
   // gone
-  make_new_connection_ok(router_rw_port, classic_ports[1]);
-  make_new_connection_ok(router_rw_split_port, classic_ports[2]);
+  {
+    auto conn_res = make_new_connection(router_rw_port);
+    ASSERT_NO_ERROR(conn_res);
+    auto port_res = select_port(conn_res->get());
+    ASSERT_NO_ERROR(port_res);
+    EXPECT_EQ(*port_res, classic_ports[1]);
+  }
+  {
+    auto conn_res = make_new_connection(router_rw_split_port);
+    ASSERT_NO_ERROR(conn_res);
+    auto port_res = select_port(conn_res->get());
+    ASSERT_NO_ERROR(port_res);
+    EXPECT_EQ(*port_res, classic_ports[2]);
+  }
 }
 
 struct AccessToPartitionWithNoQuorumTestParam {
@@ -781,9 +837,9 @@ TEST_P(AccessToPartitionWithNoQuorum, Spec) {
   const std::string metadata_cache_section =
       get_metadata_cache_section(ClusterType::GR_V2, "0.2");
   const std::string routing_rw = get_metadata_cache_routing_section(
-      router_rw_port, "PRIMARY", "first-available", "", "rw");
+      router_rw_port, "PRIMARY", "first-available", "rw");
   const std::string routing_ro = get_metadata_cache_routing_section(
-      router_ro_port, "SECONDARY", "round-robin-with-fallback", "", "ro");
+      router_ro_port, "SECONDARY", "round-robin-with-fallback", "ro");
   const std::string routing_rw_split =
       get_rw_split_routing_section(router_rw_split_port);
 
@@ -810,20 +866,31 @@ TEST_P(AccessToPartitionWithNoQuorum, Spec) {
   }
 
   if (GetParam().expect_rw_connection_ok) {
-    make_new_connection_ok(router_rw_port, classic_ports[2]);
+    auto conn_res = make_new_connection(router_rw_port);
+    ASSERT_NO_ERROR(conn_res);
+    auto port_res = select_port(conn_res->get());
+    ASSERT_NO_ERROR(port_res);
+    EXPECT_EQ(*port_res, classic_ports[2]);
   } else {
     verify_new_connection_fails(router_rw_port);
   }
 
   if (GetParam().expect_ro_connection_ok) {
-    make_new_connection_ok(router_ro_port, classic_ports[2]);
-
+    auto conn_res = make_new_connection(router_ro_port);
+    ASSERT_NO_ERROR(conn_res);
+    auto port_res = select_port(conn_res->get());
+    ASSERT_NO_ERROR(port_res);
+    EXPECT_EQ(*port_res, classic_ports[2]);
   } else {
     verify_new_connection_fails(router_ro_port);
   }
 
   if (GetParam().expect_rw_split_connection_ok) {
-    make_new_connection_ok(router_rw_split_port, classic_ports[2]);
+    auto conn_res = make_new_connection(router_rw_split_port);
+    ASSERT_NO_ERROR(conn_res);
+    auto port_res = select_port(conn_res->get());
+    ASSERT_NO_ERROR(port_res);
+    EXPECT_EQ(*port_res, classic_ports[2]);
   } else {
     verify_new_connection_fails(router_rw_split_port);
   }
@@ -944,9 +1011,9 @@ TEST_P(AccessToBothPartitions, Spec) {
   const std::string metadata_cache_section =
       get_metadata_cache_section(ClusterType::GR_V2, "0.2");
   const std::string routing_rw = get_metadata_cache_routing_section(
-      router_rw_port, "PRIMARY", "first-available", "", "rw");
+      router_rw_port, "PRIMARY", "first-available", "rw");
   const std::string routing_ro = get_metadata_cache_routing_section(
-      router_ro_port, "SECONDARY", "round-robin-with-fallback", "", "ro");
+      router_ro_port, "SECONDARY", "round-robin-with-fallback", "ro");
   const std::string routing_rw_split =
       get_rw_split_routing_section(router_rw_split_port);
 
@@ -961,9 +1028,27 @@ TEST_P(AccessToBothPartitions, Spec) {
   // Regardless of the unreachable_quorum_allowed_traffic option setting, the
   // Router should always use the partition with the quorum for the traffic as
   // it has an access to it
-  make_new_connection_ok(router_rw_port, classic_ports[1]);
-  make_new_connection_ok(router_ro_port, classic_ports[2]);
-  make_new_connection_ok(router_rw_split_port, classic_ports[1]);
+  {
+    auto conn_res = make_new_connection(router_rw_port);
+    ASSERT_NO_ERROR(conn_res);
+    auto port_res = select_port(conn_res->get());
+    ASSERT_NO_ERROR(port_res);
+    EXPECT_EQ(*port_res, classic_ports[1]);
+  }
+  {
+    auto conn_res = make_new_connection(router_ro_port);
+    ASSERT_NO_ERROR(conn_res);
+    auto port_res = select_port(conn_res->get());
+    ASSERT_NO_ERROR(port_res);
+    EXPECT_EQ(*port_res, classic_ports[2]);
+  }
+  {
+    auto conn_res = make_new_connection(router_rw_split_port);
+    ASSERT_NO_ERROR(conn_res);
+    auto port_res = select_port(conn_res->get());
+    ASSERT_NO_ERROR(port_res);
+    EXPECT_EQ(*port_res, classic_ports[1]);
+  }
 }
 
 INSTANTIATE_TEST_SUITE_P(
@@ -1096,29 +1181,31 @@ TEST_P(ClusterSetAccessToPartitionWithNoQuorum, Spec) {
   RecordProperty("Description", GetParam().test_description);
 
   const std::vector<size_t> gr_nodes_per_cluster{3, 3};
-  create_clusterset(/*view_id*/ 0, target_cluster_id, /*primary_cluster_id*/ 0,
-                    "metadata_clusterset.js", router_options, ".*", false,
-                    false, gr_nodes_per_cluster);
+
+  ClusterSetOptions cs_options;
+  cs_options.target_cluster_id = target_cluster_id;
+  cs_options.tracefile = "metadata_clusterset.js";
+  cs_options.router_options = router_options;
+  cs_options.gr_nodes_number = gr_nodes_per_cluster;
+  create_clusterset(cs_options);
 
   for (size_t i = 1; i <= 2; ++i) {
-    clusterset_data_.clusters[target_cluster_id].gr_nodes[i].member_status =
+    cs_options.topology.clusters[target_cluster_id].gr_nodes[i].member_status =
         "UNREACHABLE";
-    clusterset_data_.clusters[target_cluster_id].nodes[i].process->kill();
+    cs_options.topology.clusters[target_cluster_id].nodes[i].process->kill();
   }
 
   for (size_t i = 1; i <= 2; ++i) {
-    clusterset_data_.clusters[target_cluster_id]
+    cs_options.topology.clusters[target_cluster_id]
         .nodes[i]
         .process->wait_for_exit();
   }
 
   const auto http_port =
-      clusterset_data_.clusters[target_cluster_id].nodes[0].http_port;
-  set_mock_clusterset_metadata(/*view_id*/ 0,
+      cs_options.topology.clusters[target_cluster_id].nodes[0].http_port;
+  set_mock_clusterset_metadata(http_port,
                                /* this_cluster_id*/ target_cluster_id,
-                               /*this_node_id*/ 0,
-                               /*target_cluster_id*/ target_cluster_id,
-                               http_port, clusterset_data_, router_options);
+                               /*this_node_id*/ 0, cs_options);
 
   const auto router_ro_port = port_pool_.get_next_available();
   const auto router_rw_port = port_pool_.get_next_available();
@@ -1126,14 +1213,14 @@ TEST_P(ClusterSetAccessToPartitionWithNoQuorum, Spec) {
   const std::string metadata_cache_section =
       get_metadata_cache_section(ClusterType::GR_V2, "0.2");
   const std::string routing_rw = get_metadata_cache_routing_section(
-      router_rw_port, "PRIMARY", "first-available", "", "rw");
+      router_rw_port, "PRIMARY", "first-available", "rw");
   const std::string routing_ro = get_metadata_cache_routing_section(
-      router_ro_port, "SECONDARY", "round-robin-with-fallback", "", "ro");
+      router_ro_port, "SECONDARY", "round-robin-with-fallback", "ro");
   const std::string routing_rw_split =
       get_rw_split_routing_section(router_rw_split_port);
 
   const auto metadata_server_ports =
-      clusterset_data_.get_md_servers_classic_ports();
+      cs_options.topology.get_md_servers_classic_ports();
 
   const auto sync_point = (GetParam().expect_rw_connection_ok ||
                            GetParam().expect_ro_connection_ok ||
@@ -1155,27 +1242,37 @@ TEST_P(ClusterSetAccessToPartitionWithNoQuorum, Spec) {
   }
 
   if (GetParam().expect_rw_connection_ok) {
-    make_new_connection_ok(
-        router_rw_port,
-        clusterset_data_.clusters[target_cluster_id].nodes[0].classic_port);
+    auto conn_res = make_new_connection(router_rw_port);
+    ASSERT_NO_ERROR(conn_res);
+    auto port_res = select_port(conn_res->get());
+    ASSERT_NO_ERROR(port_res);
+    EXPECT_EQ(
+        *port_res,
+        cs_options.topology.clusters[target_cluster_id].nodes[0].classic_port);
   } else {
     verify_new_connection_fails(router_rw_port);
   }
 
   if (GetParam().expect_ro_connection_ok) {
-    make_new_connection_ok(
-        router_ro_port,
-        clusterset_data_.clusters[target_cluster_id].nodes[0].classic_port);
-
+    auto conn_res = make_new_connection(router_ro_port);
+    ASSERT_NO_ERROR(conn_res);
+    auto port_res = select_port(conn_res->get());
+    ASSERT_NO_ERROR(port_res);
+    EXPECT_EQ(
+        *port_res,
+        cs_options.topology.clusters[target_cluster_id].nodes[0].classic_port);
   } else {
     verify_new_connection_fails(router_ro_port);
   }
 
   if (GetParam().expect_rw_split_connection_ok) {
-    make_new_connection_ok(
-        router_rw_split_port,
-        clusterset_data_.clusters[target_cluster_id].nodes[0].classic_port);
-
+    auto conn_res = make_new_connection(router_rw_split_port);
+    ASSERT_NO_ERROR(conn_res);
+    auto port_res = select_port(conn_res->get());
+    ASSERT_NO_ERROR(port_res);
+    EXPECT_EQ(
+        *port_res,
+        cs_options.topology.clusters[target_cluster_id].nodes[0].classic_port);
   } else {
     verify_new_connection_fails(router_rw_split_port);
   }
