@@ -23,10 +23,12 @@
 
 #include "sql/create_field.h"
 
+#include "m_string.h"
+#include "mysql/strings/dtoa.h"
+#include "sql-common/my_decimal.h"
 #include "sql/derror.h"
 #include "sql/field.h"
 #include "sql/item.h"
-#include "sql/my_decimal.h"
 #include "sql/sql_class.h"
 #include "sql_string.h"
 #include "template_utils.h"
@@ -38,6 +40,8 @@ static constexpr const size_t MAX_BIT_FIELD_LENGTH{64};
 /** YYYYMMDDHHMMSS */
 static constexpr const size_t MAX_DATETIME_COMPRESSED_WIDTH{14};
 
+struct CHARSET_INFO;
+
 /**
     Constructs a column definition from an object representing an actual
     column. This is a reverse-engineering procedure that creates a column
@@ -143,7 +147,7 @@ Create_field::Create_field(Field *old_field, Field *orig_field)
 
     if (orig_field->has_insert_default_constant_expression()) {
       /* Get the value from default_values */
-      ptrdiff_t diff = orig_field->table->default_values_offset();
+      const ptrdiff_t diff = orig_field->table->default_values_offset();
       orig_field->move_field_offset(diff);  // Points now at default_values
       if (!orig_field->is_real_null()) {
         StringBuffer<MAX_FIELD_WIDTH> tmp(charset);
@@ -387,6 +391,18 @@ bool Create_field::init(
       break;
     case MYSQL_TYPE_STRING:
       break;
+    case MYSQL_TYPE_VECTOR: {
+      auto max_dimension_bytes =
+          Field_vector::dimension_bytes(Field_vector::max_dimensions);
+      if (m_max_display_width_in_codepoints > max_dimension_bytes) {
+        my_error(ER_EXCEEDS_VECTOR_MAX_DIMENSIONS, MYF(0),
+                 m_max_display_width_in_codepoints,
+                 m_max_display_width_in_codepoints / Field_vector::precision,
+                 max_dimension_bytes, Field_vector::max_dimensions, fld_name);
+        break;
+      }
+      [[fallthrough]];
+    }
     case MYSQL_TYPE_BLOB:
     case MYSQL_TYPE_TINY_BLOB:
     case MYSQL_TYPE_LONG_BLOB:
@@ -432,9 +448,8 @@ bool Create_field::init(
       break;
     case MYSQL_TYPE_FLOAT:
       /* change FLOAT(precision) to FLOAT or DOUBLE */
-      allowed_type_modifier = AUTO_INCREMENT_FLAG;
       if (display_width_in_codepoints && !fld_decimals) {
-        size_t tmp_length = m_max_display_width_in_codepoints;
+        const size_t tmp_length = m_max_display_width_in_codepoints;
         if (tmp_length > PRECISION_FOR_DOUBLE) {
           my_error(ER_WRONG_FIELD_SPEC, MYF(0), fld_name);
           return true;
@@ -457,7 +472,6 @@ bool Create_field::init(
       }
       break;
     case MYSQL_TYPE_DOUBLE:
-      allowed_type_modifier = AUTO_INCREMENT_FLAG;
       if (!display_width_in_codepoints && !fld_decimals) {
         m_max_display_width_in_codepoints = DBL_DIG + 7;
         decimals = DECIMAL_NOT_SPECIFIED;
@@ -713,8 +727,9 @@ size_t Create_field::max_display_width_in_bytes() const {
     // MAX_LONG_BLOB_WIDTH if the character set is multi-byte. So we must
     // ensure that we never return a value greater than
     // MAX_LONG_BLOB_WIDTH.
-    std::int64_t display_width = max_display_width_in_codepoints() *
-                                 static_cast<std::int64_t>(charset->mbmaxlen);
+    const std::int64_t display_width =
+        max_display_width_in_codepoints() *
+        static_cast<std::int64_t>(charset->mbmaxlen);
     return static_cast<size_t>(std::min(
         display_width, static_cast<std::int64_t>(Field::MAX_LONG_BLOB_WIDTH)));
   }
@@ -776,6 +791,10 @@ size_t Create_field::key_length() const {
       }
       return pack_length() + (max_display_width_in_bytes() & 7 ? 1 : 0);
     }
+    /* LCOV_EXCL_START */
+    case MYSQL_TYPE_VECTOR:
+      assert(false);  // Key on VECTOR type column is not supported.
+    /* LCOV_EXCL_STOP */
     default: {
       return pack_length(is_array);
     }
