@@ -1,18 +1,19 @@
 #ifndef ITEM_JSON_FUNC_INCLUDED
 #define ITEM_JSON_FUNC_INCLUDED
 
-/* Copyright (c) 2015, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -26,19 +27,23 @@
 #include <assert.h>
 #include <sys/types.h>
 
-#include <cstddef>
 #include <cstdint>
 #include <memory>
 #include <utility>  // std::forward
 
+#include "field_types.h"
+#include "my_alloc.h"
 #include "my_inttypes.h"
+#include "my_table_map.h"
 #include "my_time.h"
 #include "mysql/strings/m_ctype.h"
 #include "mysql/udf_registration_types.h"
 #include "mysql_com.h"
 #include "mysql_time.h"
-#include "prealloced_array.h"      // Prealloced_array
-#include "sql-common/json_path.h"  // Json_path
+#include "prealloced_array.h"  // Prealloced_array
+#include "sql-common/json_error_handler.h"
+#include "sql-common/json_path.h"    // Json_path
+#include "sql-common/json_schema.h"  //Json_schema_validator_holder
 #include "sql/enum_query_type.h"
 #include "sql/field.h"
 #include "sql/item.h"
@@ -50,16 +55,20 @@
 #include "sql/psi_memory_key.h"  // key_memory_JSON
 #include "sql_string.h"
 
-class Json_schema_validator;
 class Json_array;
+class Json_diff_vector;
 class Json_dom;
+class Json_object;
 class Json_scalar_holder;
+class Json_schema_validator;
 class Json_wrapper;
 class PT_item_list;
 class THD;
 class my_decimal;
 enum Cast_target : unsigned char;
 enum class Json_on_response_type : uint16;
+enum class enum_json_diff_status;
+
 struct Cast_type;
 struct TABLE;
 
@@ -181,6 +190,7 @@ class Item_json_func : public Item_func {
   }
 
   bool resolve_type(THD *) override {
+    if (reject_vector_args()) return true;
     set_nullable(true);
     return false;
   }
@@ -313,12 +323,14 @@ class Item_func_json_valid final : public Item_int_func {
   Item_func_json_valid(const POS &pos, Item *a) : Item_int_func(pos, a) {}
 
   const char *func_name() const override { return "json_valid"; }
+  enum Functype functype() const override { return JSON_VALID_FUNC; }
 
   bool is_bool_func() const override { return true; }
 
   longlong val_int() override;
 
   bool resolve_type(THD *thd) override {
+    if (reject_vector_args()) return true;
     if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_JSON)) return true;
     set_nullable(true);
     return false;
@@ -344,10 +356,7 @@ class Item_func_json_schema_valid final : public Item_bool_func {
   void cleanup() override;
 
  private:
-  // Wrap the object in a unique_ptr so that the relevant rapidjson destructors
-  // are called.
-  unique_ptr_destroy_only<const Json_schema_validator>
-      m_cached_schema_validator;
+  Json_schema_validator m_cached_schema_validator;
 };
 
 /**
@@ -367,6 +376,7 @@ class Item_func_json_schema_validation_report final : public Item_json_func {
   bool val_json(Json_wrapper *wr) override;
 
   bool resolve_type(THD *thd) override {
+    if (reject_vector_args()) return true;
     if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_JSON)) return true;
     set_nullable(true);
     return false;
@@ -377,10 +387,7 @@ class Item_func_json_schema_validation_report final : public Item_json_func {
   void cleanup() override;
 
  private:
-  // Wrap the object in a unique_ptr so that the relevant rapidjson destructors
-  // are called.
-  unique_ptr_destroy_only<const Json_schema_validator>
-      m_cached_schema_validator;
+  Json_schema_validator m_cached_schema_validator;
 };
 
 /**
@@ -404,6 +411,7 @@ class Item_func_json_contains final : public Item_int_func {
   longlong val_int() override;
 
   bool resolve_type(THD *thd) override {
+    if (reject_vector_args()) return true;
     if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_JSON)) return true;
     if (param_type_is_default(thd, 1, 3)) return true;
     set_nullable(true);
@@ -435,12 +443,14 @@ class Item_func_json_contains_path final : public Item_int_func {
         m_path_cache(thd, arg_count) {}
 
   const char *func_name() const override { return "json_contains_path"; }
+  enum Functype functype() const override { return JSON_CONTAINS_PATH_FUNC; }
 
   bool is_bool_func() const override { return true; }
 
   longlong val_int() override;
 
   bool resolve_type(THD *thd) override {
+    if (reject_vector_args()) return true;
     if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_JSON)) return true;
     if (param_type_is_default(thd, 1, -1)) return true;
     set_nullable(true);
@@ -465,6 +475,7 @@ class Item_func_json_type : public Item_str_func {
   Item_func_json_type(const POS &pos, Item *a) : Item_str_func(pos, a) {}
 
   const char *func_name() const override { return "json_type"; }
+  enum Functype functype() const override { return JSON_TYPE_FUNC; }
 
   bool resolve_type(THD *) override;
 
@@ -482,6 +493,7 @@ class Item_typecast_json final : public Item_json_func {
       : Item_json_func(thd, pos, a) {}
 
   bool resolve_type(THD *thd) override {
+    if (reject_vector_args()) return true;
     if (Item_json_func::resolve_type(thd)) return true;
     return args[0]->propagate_type(thd, MYSQL_TYPE_JSON, false, true);
   }
@@ -503,6 +515,7 @@ class Item_func_json_length final : public Item_int_func {
   Item_func_json_length(const POS &pos, Item *doc) : Item_int_func(pos, doc) {}
 
   bool resolve_type(THD *thd) override {
+    if (reject_vector_args()) return true;
     if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_JSON)) return true;
     if (param_type_is_default(thd, 1, 2)) return true;
     set_nullable(true);
@@ -510,6 +523,7 @@ class Item_func_json_length final : public Item_int_func {
   }
 
   const char *func_name() const override { return "json_length"; }
+  enum Functype functype() const override { return JSON_LENGTH_FUNC; }
 
   longlong val_int() override;
 };
@@ -524,8 +538,10 @@ class Item_func_json_depth final : public Item_int_func {
   Item_func_json_depth(const POS &pos, Item *a) : Item_int_func(pos, a) {}
 
   const char *func_name() const override { return "json_depth"; }
+  enum Functype functype() const override { return JSON_DEPTH_FUNC; }
 
   bool resolve_type(THD *thd) override {
+    if (reject_vector_args()) return true;
     if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_JSON)) return true;
     set_nullable(true);
     return false;
@@ -550,6 +566,7 @@ class Item_func_json_keys : public Item_json_func {
   const char *func_name() const override { return "json_keys"; }
 
   bool resolve_type(THD *thd) override {
+    if (reject_vector_args()) return true;
     if (Item_json_func::resolve_type(thd)) return true;
     if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_JSON)) return true;
     if (param_type_is_default(thd, 1, 2)) return true;
@@ -573,8 +590,10 @@ class Item_func_json_extract final : public Item_json_func {
       : Item_json_func(thd, pos, a, b) {}
 
   const char *func_name() const override { return "json_extract"; }
+  enum Functype functype() const override { return JSON_EXTRACT_FUNC; }
 
   bool resolve_type(THD *thd) override {
+    if (reject_vector_args()) return true;
     if (Item_json_func::resolve_type(thd)) return true;
     if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_JSON)) return true;
     if (param_type_is_default(thd, 1, -1)) return true;
@@ -583,7 +602,7 @@ class Item_func_json_extract final : public Item_json_func {
 
   bool val_json(Json_wrapper *wr) override;
 
-  bool eq(const Item *item, bool binary_cmp) const override;
+  bool eq(const Item *item) const override;
 };
 
 /// Base class for all the functions that take a JSON document as the first
@@ -710,6 +729,7 @@ class Item_func_json_array final : public Item_json_func {
   }
 
   const char *func_name() const override { return "json_array"; }
+  enum Functype functype() const override { return JSON_ARRAY_FUNC; }
 
   bool resolve_type(THD *thd) override {
     if (Item_json_func::resolve_type(thd)) return true;
@@ -736,6 +756,7 @@ class Item_func_json_row_object final : public Item_json_func {
   }
 
   const char *func_name() const override { return "json_object"; }
+  enum Functype functype() const override { return JSON_OBJECT_FUNC; }
 
   bool resolve_type(THD *thd) override {
     if (Item_json_func::resolve_type(thd)) return true;
@@ -770,6 +791,8 @@ class Item_func_json_search : public Item_json_func {
 
   const char *func_name() const override { return "json_search"; }
 
+  enum Functype functype() const override { return JSON_SEARCH_FUNC; }
+
   bool val_json(Json_wrapper *wr) override;
 
   /**
@@ -872,7 +895,10 @@ class Item_func_json_quote : public Item_str_func {
 
   const char *func_name() const override { return "json_quote"; }
 
+  enum Functype functype() const override { return JSON_QUOTE_FUNC; }
+
   bool resolve_type(THD *thd) override {
+    if (reject_vector_args()) return true;
     if (param_type_is_default(thd, 0, -1)) return true;
     set_nullable(true);
 
@@ -906,6 +932,7 @@ class Item_func_json_unquote : public Item_str_func {
   enum Functype functype() const override { return JSON_UNQUOTE_FUNC; }
 
   bool resolve_type(THD *thd) override {
+    if (reject_vector_args()) return true;
     if (param_type_is_default(thd, 0, -1)) return true;
     set_nullable(true);
     set_data_type_string(args[0]->max_char_length(), &my_charset_utf8mb4_bin);
@@ -924,7 +951,10 @@ class Item_func_json_pretty final : public Item_str_func {
 
   const char *func_name() const override { return "json_pretty"; }
 
+  enum Functype functype() const override { return JSON_PRETTY_FUNC; }
+
   bool resolve_type(THD *thd) override {
+    if (reject_vector_args()) return true;
     if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_JSON)) return true;
     set_data_type_string(MAX_BLOB_WIDTH, &my_charset_utf8mb4_bin);
     return false;
@@ -941,8 +971,10 @@ class Item_func_json_storage_size final : public Item_int_func {
   Item_func_json_storage_size(const POS &pos, Item *a)
       : Item_int_func(pos, a) {}
   const char *func_name() const override { return "json_storage_size"; }
+  enum Functype functype() const override { return JSON_STORAGE_SIZE_FUNC; }
 
   bool resolve_type(THD *thd) override {
+    if (reject_vector_args()) return true;
     if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_JSON)) return true;
     if (Item_int_func::resolve_type(thd)) return true;
     set_nullable(true);
@@ -960,8 +992,10 @@ class Item_func_json_storage_free final : public Item_int_func {
   Item_func_json_storage_free(const POS &pos, Item *a)
       : Item_int_func(pos, a) {}
   const char *func_name() const override { return "json_storage_free"; }
+  enum Functype functype() const override { return JSON_STORAGE_FREE_FUNC; }
 
   bool resolve_type(THD *thd) override {
+    if (reject_vector_args()) return true;
     if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_JSON)) return true;
     return false;
   }
@@ -1064,6 +1098,7 @@ class Item_func_member_of : public Item_bool_func {
   const char *func_name() const override { return "member of"; }
   enum Functype functype() const override { return MEMBER_OF_FUNC; }
   bool resolve_type(THD *thd) override {
+    if (reject_vector_args()) return true;
     if (param_type_is_default(thd, 0, 2, MYSQL_TYPE_JSON)) return true;
     args[0]->mark_json_as_scalar();
     return false;
@@ -1097,12 +1132,13 @@ class Item_func_json_value final : public Item_func {
                        Item *on_error_default);
   ~Item_func_json_value() override;
   const char *func_name() const override { return "json_value"; }
+  enum Functype functype() const override { return JSON_VALUE_FUNC; }
   enum Item_result result_type() const override;
   bool resolve_type(THD *) override;
   bool fix_fields(THD *thd, Item **ref) override;
   void print(const THD *thd, String *str,
              enum_query_type query_type) const override;
-  bool eq(const Item *item, bool binary_cmp) const override;
+  bool eq_specific(const Item *item) const override;
   bool val_json(Json_wrapper *wr) override;
   String *val_str(String *buffer) override;
   double val_real() override;
@@ -1110,6 +1146,8 @@ class Item_func_json_value final : public Item_func {
   my_decimal *val_decimal(my_decimal *value) override;
   bool get_date(MYSQL_TIME *ltime, my_time_flags_t flags) override;
   bool get_time(MYSQL_TIME *ltime) override;
+  Json_on_response_type on_empty_response_type() const;
+  Json_on_response_type on_error_response_type() const;
 
  private:
   /// Represents a default value given in JSON_VALUE's DEFAULT xxx ON EMPTY or
@@ -1245,10 +1283,18 @@ using Json_dom_ptr = std::unique_ptr<Json_dom>;
 
 bool parse_json(const String &res, Json_dom_ptr *dom, bool require_str_or_json,
                 const JsonParseErrorHandler &error_handler,
-                const JsonDocumentDepthHandler &depth_handler);
+                const JsonErrorHandler &depth_handler);
 
-typedef Prealloced_array<size_t, 16> Sorted_index_array;
-bool sort_and_remove_dups(const Json_wrapper &orig, Sorted_index_array *v);
+/**
+  Apply a sequence of JSON diffs to the value stored in a JSON column.
+
+    @param field  the column to update
+    @param diffs  the diffs to apply
+    @return an enum_json_diff_status value that tells if the diffs were
+    applied successfully
+ */
+enum_json_diff_status apply_json_diffs(Field_json *field,
+                                       const Json_diff_vector *diffs);
 
 bool save_json_to_field(THD *thd, Field *field, const Json_wrapper *w,
                         bool no_error);
