@@ -1,15 +1,16 @@
-/* Copyright (c) 2000, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -675,12 +676,25 @@ class Item_func_user : public Item_func_sysconst {
 
 class Item_func_current_user : public Item_func_user {
   typedef Item_func_user super;
+  /**
+     Used to pass a security context to the resolver functions.
+     Only used for definer views. In all other contexts, the security context
+     passed here is nullptr and is instead looked up dynamically at run time
+     from the current THD.
+  */
+  Name_resolution_context *m_name_resolution_ctx = nullptr;
 
-  Name_resolution_context *context;
+  /// Copied from m_name_resolution_ctx in fix_fields if the definer
+  /// Security_context is set in Name_resolution_context
+  LEX_CSTRING m_definer_priv_user = {};
+  LEX_CSTRING m_definer_priv_host = {};
 
  protected:
   type_conversion_status save_in_field_inner(Field *field, bool) override;
 
+  /// Overridden to copy definer priv_user and priv_host
+  bool resolve_type(THD *) override;
+
  public:
   explicit Item_func_current_user(const POS &pos) : super(pos) {}
 
@@ -718,37 +732,19 @@ class Item_func_elt final : public Item_str_func {
 class Item_func_make_set final : public Item_str_func {
   typedef Item_str_func super;
 
-  Item *item;
   String tmp_str;
 
  public:
-  Item_func_make_set(const POS &pos, Item *a, PT_item_list *opt_list)
-      : Item_str_func(pos, opt_list), item(a) {}
+  Item_func_make_set(const POS &pos, PT_item_list *opt_list)
+      : Item_str_func(pos, opt_list) {}
 
-  bool do_itemize(Parse_context *pc, Item **res) override;
   String *val_str(String *str) override;
   bool fix_fields(THD *thd, Item **ref) override;
-  void fix_after_pullout(Query_block *parent_query_block,
-                         Query_block *removed_query_block) override;
-  bool split_sum_func(THD *thd, Ref_item_array ref_item_array,
-                      mem_root_deque<Item *> *fields) override;
   bool resolve_type(THD *) override;
-  void update_used_tables() override;
   const char *func_name() const override { return "make_set"; }
 
-  bool walk(Item_processor processor, enum_walk walk, uchar *arg) override {
-    if ((walk & enum_walk::PREFIX) && (this->*processor)(arg)) return true;
-    if (item->walk(processor, walk, arg)) return true;
-    for (uint i = 0; i < arg_count; i++) {
-      if (args[i]->walk(processor, walk, arg)) return true;
-    }
-    return ((walk & enum_walk::POSTFIX) && (this->*processor)(arg));
-  }
-
-  Item *transform(Item_transformer transformer, uchar *arg) override;
   void print(const THD *thd, String *str,
              enum_query_type query_type) const override;
-  Item *get_item() { return item; }
 };
 
 class Item_func_format final : public Item_str_ascii_func {
@@ -774,14 +770,17 @@ class Item_func_char final : public Item_str_func {
   Item_func_char(const POS &pos, PT_item_list *list)
       : Item_str_func(pos, list) {
     collation.set(&my_charset_bin);
+    null_on_null = false;
   }
   Item_func_char(const POS &pos, PT_item_list *list, const CHARSET_INFO *cs)
       : Item_str_func(pos, list) {
     collation.set(cs);
+    null_on_null = false;
   }
   String *val_str(String *) override;
   bool resolve_type(THD *thd) override {
     if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_LONGLONG)) return true;
+    if (reject_vector_args()) return true;
     set_data_type_string(arg_count * 4U);
     return false;
   }
@@ -870,6 +869,7 @@ class Item_func_is_uuid final : public Item_bool_func {
   longlong val_int() override;
   const char *func_name() const override { return "is_uuid"; }
   bool resolve_type(THD *thd) override {
+    if (reject_vector_args()) return true;
     bool res = super::resolve_type(thd);
     set_nullable(true);
     return res;
@@ -926,6 +926,7 @@ class Item_func_like_range : public Item_str_func {
   bool resolve_type(THD *thd) override {
     if (param_type_is_default(thd, 0, 1)) return true;
     if (param_type_is_default(thd, 1, 2, MYSQL_TYPE_LONGLONG)) return true;
+    if (reject_vector_args()) return true;
     set_data_type_string(uint32{MAX_BLOB_WIDTH}, args[0]->collation);
     return false;
   }
@@ -1027,7 +1028,7 @@ class Item_typecast_char final : public Item_charset_conversion {
     m_cast_length = length_arg;
   }
   enum Functype functype() const override { return TYPECAST_FUNC; }
-  bool eq(const Item *item, bool binary_cmp) const override;
+  bool eq_specific(const Item *item) const override;
   const char *func_name() const override { return "cast_as_char"; }
   void print(const THD *thd, String *str,
              enum_query_type query_type) const override;
@@ -1049,6 +1050,7 @@ class Item_load_file final : public Item_str_func {
   }
   bool resolve_type(THD *thd) override {
     if (param_type_is_default(thd, 0, 1)) return true;
+    if (reject_vector_args()) return true;
     collation.set(&my_charset_bin, DERIVATION_COERCIBLE);
     set_data_type_blob(MYSQL_TYPE_LONG_BLOB, MAX_BLOB_WIDTH);
     set_nullable(true);
@@ -1102,6 +1104,7 @@ class Item_func_conv_charset final : public Item_charset_conversion {
   const char *func_name() const override { return "convert"; }
   void print(const THD *thd, String *str,
              enum_query_type query_type) const override;
+  bool eq_specific(const Item *item) const override;
 };
 
 class Item_func_set_collation final : public Item_str_func {
@@ -1117,7 +1120,7 @@ class Item_func_set_collation final : public Item_str_func {
   bool do_itemize(Parse_context *pc, Item **res) override;
   String *val_str(String *) override;
   bool resolve_type(THD *) override;
-  bool eq(const Item *item, bool binary_cmp) const override;
+  bool eq_specific(const Item *item) const override;
   const char *func_name() const override { return "collate"; }
   enum Functype functype() const override { return COLLATE_FUNC; }
   void print(const THD *thd, String *str,
@@ -1187,7 +1190,7 @@ class Item_func_weight_string final : public Item_str_func {
   bool do_itemize(Parse_context *pc, Item **res) override;
 
   const char *func_name() const override { return "weight_string"; }
-  bool eq(const Item *item, bool binary_cmp) const override;
+  bool eq_specific(const Item *item) const override;
   String *val_str(String *) override;
   bool resolve_type(THD *) override;
   void print(const THD *thd, String *str,
@@ -1219,6 +1222,7 @@ class Item_func_uncompressed_length final : public Item_int_func {
   const char *func_name() const override { return "uncompressed_length"; }
   bool resolve_type(THD *thd) override {
     if (param_type_is_default(thd, 0, 1)) return true;
+    if (reject_vector_args()) return true;
     max_length = 10;
     return false;
   }
@@ -1235,12 +1239,38 @@ class Item_func_compress final : public Item_str_func {
   String *val_str(String *str) override;
 };
 
+class Item_func_to_vector final : public Item_str_func {
+  String buffer;
+
+ public:
+  Item_func_to_vector(const POS &pos, Item *a) : Item_str_func(pos, a) {}
+  bool resolve_type(THD *thd) override;
+  const char *func_name() const override { return "to_vector"; }
+  String *val_str(String *str) override;
+};
+
+class Item_func_from_vector final : public Item_str_func {
+  static const uint32 per_value_chars = 16;
+  static const uint32 max_output_bytes =
+      (Field_vector::max_dimensions * Item_func_from_vector::per_value_chars);
+  String buffer;
+
+ public:
+  Item_func_from_vector(const POS &pos, Item *a) : Item_str_func(pos, a) {
+    collation.set(&my_charset_utf8mb4_0900_bin);
+  }
+  bool resolve_type(THD *thd) override;
+  const char *func_name() const override { return "from_vector"; }
+  String *val_str(String *str) override;
+};
+
 class Item_func_uncompress final : public Item_str_func {
   String buffer;
 
  public:
   Item_func_uncompress(const POS &pos, Item *a) : Item_str_func(pos, a) {}
   bool resolve_type(THD *thd) override {
+    if (reject_vector_args()) return true;
     if (Item_str_func::resolve_type(thd)) return true;
     set_nullable(true);
     set_data_type_string(uint32{MAX_BLOB_WIDTH});
