@@ -33,11 +33,11 @@
 #include <cstddef>
 
 #include "field_types.h"  // MYSQL_TYPE_DATETIME
-#include "m_ctype.h"
 
 #include "my_inttypes.h"
 #include "my_table_map.h"
 #include "my_time.h"
+#include "mysql/strings/m_ctype.h"
 #include "mysql/udf_registration_types.h"
 #include "mysql_time.h"
 #include "sql/enum_query_type.h"
@@ -69,6 +69,7 @@ class Item_func_period_add final : public Item_int_func {
       : Item_int_func(pos, a, b) {}
   longlong val_int() override;
   const char *func_name() const override { return "period_add"; }
+  enum Functype functype() const override { return PERIODADD_FUNC; }
   bool resolve_type(THD *thd) override;
 };
 
@@ -78,6 +79,7 @@ class Item_func_period_diff final : public Item_int_func {
       : Item_int_func(pos, a, b) {}
   longlong val_int() override;
   const char *func_name() const override { return "period_diff"; }
+  enum Functype functype() const override { return PERIODDIFF_FUNC; }
   bool resolve_type(THD *thd) override;
 };
 
@@ -150,7 +152,7 @@ class Item_func_month final : public Item_func {
     return (double)Item_func_month::val_int();
   }
   String *val_str(String *str) override {
-    longlong nr = val_int();
+    const longlong nr = val_int();
     if (null_value) return nullptr;
     str->set(nr, collation.collation);
     return str;
@@ -256,7 +258,7 @@ class Item_func_week final : public Item_int_func {
   Item_func_week(Item *a, Item *b) : Item_int_func(a, b) {}
   Item_func_week(const POS &pos, Item *a, Item *b) : super(pos, a, b) {}
 
-  bool itemize(Parse_context *pc, Item **res) override;
+  bool do_itemize(Parse_context *pc, Item **res) override;
 
   longlong val_int() override;
   const char *func_name() const override { return "week"; }
@@ -302,6 +304,8 @@ class Item_typecast_year final : public Item_int_func {
   const char *func_name() const override { return "cast_as_year"; }
   enum Functype functype() const override { return TYPECAST_FUNC; }
   bool resolve_type(THD *thd) override;
+  void print(const THD *thd, String *str,
+             enum_query_type query_type) const override;
 };
 
 /**
@@ -322,7 +326,7 @@ class Item_func_weekday : public Item_func {
     return static_cast<double>(val_int());
   }
   String *val_str(String *str) override {
-    assert(fixed == 1);
+    assert(fixed);
     str->set(val_int(), &my_charset_bin);
     return null_value ? nullptr : str;
   }
@@ -412,7 +416,7 @@ class Item_func_unix_timestamp final : public Item_timeval_func {
   const char *func_name() const override { return "unix_timestamp"; }
   enum Functype functype() const override { return UNIX_TIMESTAMP_FUNC; }
 
-  bool itemize(Parse_context *pc, Item **res) override;
+  bool do_itemize(Parse_context *pc, Item **res) override;
   enum_monotonicity_info get_monotonicity_info() const override;
   longlong val_int_endpoint(bool left_endp, bool *incl_endp) override;
   bool check_partition_func_processor(uchar *) override { return false; }
@@ -424,8 +428,15 @@ class Item_func_unix_timestamp final : public Item_timeval_func {
   bool check_valid_arguments_processor(uchar *) override {
     return !has_timestamp_args();
   }
+
+  table_map get_initial_pseudo_tables() const override {
+    if (arg_count == 0) return INNER_TABLE_BIT;
+    return 0;
+  }
+
   bool resolve_type(THD *thd) override {
     if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_DATETIME)) return true;
+    if (reject_vector_args()) return true;
     collation.set_numeric();
     const uint8 dec = arg_count == 0 ? 0 : args[0]->datetime_precision();
     if (dec > 0) {
@@ -472,6 +483,12 @@ class Item_temporal_func : public Item_func {
  protected:
   bool check_precision();
 
+ protected:
+  void add_json_info(Json_object *obj) override {
+    if (decimals > 0)
+      obj->add_alias("precision", create_dom_ptr<Json_uint>(decimals));
+  }
+
  public:
   Item_temporal_func() : Item_func() {}
   explicit Item_temporal_func(const POS &pos) : Item_func(pos) {}
@@ -526,6 +543,9 @@ class Item_temporal_hybrid_func : public Item_str_func {
   virtual bool val_datetime(MYSQL_TIME *ltime, my_time_flags_t fuzzy_date) = 0;
   type_conversion_status save_in_field_inner(Field *field,
                                              bool no_conversions) override;
+  void add_json_info(Json_object *obj) override {
+    Item_str_func::add_json_info(obj);
+  }
 
  public:
   Item_temporal_hybrid_func(Item *a, Item *b)
@@ -541,7 +561,7 @@ class Item_temporal_hybrid_func : public Item_str_func {
       or using collation.collation when VARCHAR
       (which is fixed from @collation_connection in resolve_type()).
     */
-    assert(fixed == 1);
+    assert(fixed);
     return data_type() == MYSQL_TYPE_STRING ? collation.collation
                                             : &my_charset_bin;
   }
@@ -602,7 +622,7 @@ class Item_date_func : public Item_temporal_func {
   enum Functype functype() const override { return DATE_FUNC; }
   bool resolve_type(THD *) override { return false; }
   my_decimal *val_decimal(my_decimal *decimal_value) override {
-    assert(fixed == 1);
+    assert(fixed);
     return val_decimal_from_date(decimal_value);
   }
   // All date functions must implement get_date()
@@ -660,7 +680,7 @@ class Item_datetime_func : public Item_temporal_func {
   longlong val_int() override { return val_int_from_datetime(); }
   longlong val_date_temporal() override;
   my_decimal *val_decimal(my_decimal *decimal_value) override {
-    assert(fixed == 1);
+    assert(fixed);
     return val_decimal_from_date(decimal_value);
   }
   bool get_time(MYSQL_TIME *ltime) override {
@@ -860,7 +880,7 @@ class Item_date_literal final : public Item_date_func {
   table_map used_tables() const override { return 0; }
   table_map not_null_tables() const override { return used_tables(); }
   void cleanup() override { assert(marker == MARKER_NONE); }
-  bool eq(const Item *item, bool binary_cmp) const override;
+  bool eq_specific(const Item *item) const override;
 };
 
 /**
@@ -901,7 +921,7 @@ class Item_time_literal final : public Item_time_func {
   table_map used_tables() const override { return 0; }
   table_map not_null_tables() const override { return used_tables(); }
   void cleanup() override { assert(marker == MARKER_NONE); }
-  bool eq(const Item *item, bool binary_cmp) const override;
+  bool eq_specific(const Item *item) const override;
 };
 
 /**
@@ -945,7 +965,7 @@ class Item_datetime_literal final : public Item_datetime_func {
   table_map used_tables() const override { return 0; }
   table_map not_null_tables() const override { return used_tables(); }
   void cleanup() override { assert(marker == MARKER_NONE); }
-  bool eq(const Item *item, bool binary_cmp) const override;
+  bool eq_specific(const Item *item) const override;
 };
 
 /**
@@ -1008,6 +1028,10 @@ class Item_func_curtime : public Item_time_func {
   // Abstract method that defines which time zone is used for conversion.
   virtual Time_zone *time_zone() = 0;
 
+  void add_json_info(Json_object *obj) override {
+    obj->add_alias("precision", create_dom_ptr<Json_uint>(decimals));
+  }
+
  public:
   /**
     Constructor for Item_func_curtime.
@@ -1018,7 +1042,7 @@ class Item_func_curtime : public Item_time_func {
     decimals = dec_arg;
   }
 
-  bool itemize(Parse_context *pc, Item **res) override;
+  bool do_itemize(Parse_context *pc, Item **res) override;
 
   /// This function must assign a new value for each execution
   table_map get_initial_pseudo_tables() const override {
@@ -1073,7 +1097,7 @@ class Item_func_curdate : public Item_date_func {
  public:
   explicit Item_func_curdate(const POS &pos) : Item_date_func(pos) {}
 
-  bool itemize(Parse_context *pc, Item **res) override;
+  bool do_itemize(Parse_context *pc, Item **res) override;
 
   /// This function must assign a new value for each execution
   table_map get_initial_pseudo_tables() const override {
@@ -1123,6 +1147,11 @@ class Item_func_now : public Item_datetime_func {
   type_conversion_status save_in_field_inner(Field *to,
                                              bool no_conversions) override;
 
+  void add_json_info(Json_object *obj) override {
+    if (decimals > 0)
+      obj->add_alias("precision", create_dom_ptr<Json_uint>(decimals));
+  }
+
  public:
   /**
     Constructor for Item_func_now.
@@ -1183,7 +1212,7 @@ class Item_func_now_utc final : public Item_func_now {
   Item_func_now_utc(const POS &pos, uint8 dec_arg)
       : Item_func_now(pos, dec_arg) {}
 
-  bool itemize(Parse_context *pc, Item **res) override;
+  bool do_itemize(Parse_context *pc, Item **res) override;
 
   const char *func_name() const override { return "utc_timestamp"; }
 };
@@ -1199,6 +1228,7 @@ class Item_func_sysdate_local final : public Item_datetime_func {
   }
   const char *func_name() const override { return "sysdate"; }
   bool resolve_type(THD *) override;
+  enum Functype functype() const override { return SYSDATE_FUNC; }
   bool get_date(MYSQL_TIME *res, my_time_flags_t fuzzy_date) override;
   /**
     This function is non-deterministic and hence depends on the 'RAND'
@@ -1223,6 +1253,7 @@ class Item_func_from_days final : public Item_date_func {
   }
   bool resolve_type(THD *thd) override {
     if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_LONGLONG)) return true;
+    if (reject_vector_args()) return true;
     return Item_date_func::resolve_type(thd);
   }
 };
@@ -1242,7 +1273,7 @@ class Item_func_date_format final : public Item_str_func {
   }
   bool resolve_type(THD *thd) override;
   uint format_length(const String *format);
-  bool eq(const Item *item, bool binary_cmp) const override;
+  bool eq_specific(const Item *item) const override;
 };
 
 class Item_func_from_unixtime final : public Item_datetime_func {
@@ -1291,12 +1322,14 @@ class Item_func_sec_to_time final : public Item_time_func {
       : Item_time_func(pos, item) {}
   bool resolve_type(THD *thd) override {
     if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_NEWDECIMAL)) return true;
+    if (reject_vector_args()) return true;
     set_data_type_time(
         std::min(args[0]->decimals, uint8{DATETIME_MAX_DECIMALS}));
     set_nullable(true);
     return false;
   }
   const char *func_name() const override { return "sec_to_time"; }
+  enum Functype functype() const override { return SEC_TO_TIME_FUNC; }
   bool get_time(MYSQL_TIME *ltime) override;
 };
 
@@ -1307,6 +1340,8 @@ extern const char *interval_names[];
   Also used for the synonym functions ADDDATE and SUBDATE.
 */
 class Item_date_add_interval final : public Item_temporal_hybrid_func {
+  typedef Item_temporal_hybrid_func super;
+
  public:
   Item_date_add_interval(const POS &pos, Item *a, Item *b, interval_type type,
                          bool subtract)
@@ -1324,12 +1359,15 @@ class Item_date_add_interval final : public Item_temporal_hybrid_func {
   const char *func_name() const override { return "date_add_interval"; }
   enum Functype functype() const override { return DATEADD_FUNC; }
   bool resolve_type(THD *) override;
-  bool eq(const Item *item, bool binary_cmp) const override;
+  bool eq_specific(const Item *item) const override;
   void print(const THD *thd, String *str,
              enum_query_type query_type) const override;
   interval_type get_interval_type() const { return m_interval_type; }
   bool is_subtract() const { return m_subtract; }
 
+ protected:
+  void add_json_info(Json_object *obj) override;
+
  private:
   bool val_datetime(MYSQL_TIME *ltime, my_time_flags_t fuzzy_date) override;
   bool get_date_internal(MYSQL_TIME *res, my_time_flags_t fuzzy_date);
@@ -1353,7 +1391,7 @@ class Item_extract final : public Item_int_func {
   enum Functype functype() const override { return EXTRACT_FUNC; }
   const char *func_name() const override { return "extract"; }
   bool resolve_type(THD *) override;
-  bool eq(const Item *item, bool binary_cmp) const override;
+  bool eq_specific(const Item *item) const override;
   void print(const THD *thd, String *str,
              enum_query_type query_type) const override;
   bool check_partition_func_processor(uchar *) override { return false; }
@@ -1408,6 +1446,7 @@ class Item_typecast_date final : public Item_date_func {
 
   bool resolve_type(THD *thd) override {
     if (args[0]->propagate_type(thd, MYSQL_TYPE_DATE, false, true)) return true;
+    if (reject_vector_args()) return true;
     return Item_date_func::resolve_type(thd);
   }
   void print(const THD *thd, String *str,
@@ -1447,6 +1486,7 @@ class Item_typecast_time final : public Item_time_func {
   bool get_time(MYSQL_TIME *ltime) override;
   const char *cast_type() const { return "time"; }
   bool resolve_type(THD *thd) override {
+    if (reject_vector_args()) return true;
     if (args[0]->propagate_type(thd, MYSQL_TYPE_DATETIME, false, true))
       return true;
     set_data_type_time(detect_precision_from_arg ? args[0]->time_precision()
@@ -1484,6 +1524,7 @@ class Item_typecast_datetime final : public Item_datetime_func {
   const char *cast_type() const { return "datetime"; }
   bool is_explicit_cast() const { return m_explicit_cast; }
   bool resolve_type(THD *thd) override {
+    if (reject_vector_args()) return true;
     if (args[0]->propagate_type(thd, MYSQL_TYPE_DATETIME, false, true))
       return true;
     set_data_type_datetime(
@@ -1504,6 +1545,7 @@ class Item_func_makedate final : public Item_date_func {
   bool get_date(MYSQL_TIME *ltime, my_time_flags_t fuzzy_date) override;
   enum Functype functype() const override { return MAKEDATE_FUNC; }
   bool resolve_type(THD *thd) override {
+    if (reject_vector_args()) return true;
     if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_LONGLONG)) return true;
     return Item_date_func::resolve_type(thd);
   }
@@ -1548,7 +1590,9 @@ class Item_func_timediff final : public Item_time_func {
   Item_func_timediff(const POS &pos, Item *a, Item *b)
       : Item_time_func(pos, a, b) {}
   const char *func_name() const override { return "timediff"; }
+  enum Functype functype() const override { return TIMEDIFF_FUNC; }
   bool resolve_type(THD *thd) override {
+    if (reject_vector_args()) return true;
     /*
       This function can operate on two TIME, or on two DATETIME (no mix).
       We infer the type from the other argument. If both arguments are '?', we
@@ -1572,6 +1616,7 @@ class Item_func_maketime final : public Item_time_func {
     set_nullable(true);
   }
   bool resolve_type(THD *thd) override {
+    if (reject_vector_args()) return true;
     if (param_type_is_default(thd, 0, 2, MYSQL_TYPE_LONGLONG)) return true;
     if (param_type_is_default(thd, 2, 3, MYSQL_TYPE_NEWDECIMAL)) return true;
     set_data_type_time(
@@ -1579,6 +1624,7 @@ class Item_func_maketime final : public Item_time_func {
     return false;
   }
   const char *func_name() const override { return "maketime"; }
+  enum Functype functype() const override { return MAKETIME_FUNC; }
   bool get_time(MYSQL_TIME *ltime) override;
 };
 
@@ -1607,6 +1653,7 @@ class Item_func_timestamp_diff final : public Item_int_func {
   interval_type intervaltype() const { return int_type; }
   longlong val_int() override;
   bool resolve_type(THD *thd) override {
+    if (reject_vector_args()) return true;
     if (param_type_is_default(thd, 0, -1, MYSQL_TYPE_DATETIME)) return true;
     set_nullable(true);
     return false;
@@ -1631,13 +1678,18 @@ class Item_func_get_format final : public Item_str_ascii_func {
       : Item_str_ascii_func(pos, a), type(type_arg) {}
   String *val_str_ascii(String *str) override;
   const char *func_name() const override { return "get_format"; }
-  bool resolve_type(THD *) override {
+  enum Functype functype() const override { return GET_FORMAT_FUNC; }
+  bool resolve_type(THD *thd) override {
+    if (reject_vector_args()) return true;
+    if (param_type_is_default(thd, 0, -1)) return true;
     set_nullable(true);
     set_data_type_string(17, default_charset());
     return false;
   }
   void print(const THD *thd, String *str,
              enum_query_type query_type) const override;
+
+  enum_mysql_timestamp_type timestamp_type() const { return type; }
 };
 
 class Item_func_str_to_date final : public Item_temporal_hybrid_func {
@@ -1663,6 +1715,7 @@ class Item_func_last_day final : public Item_date_func {
   enum Functype functype() const override { return LAST_DAY_FUNC; }
   bool get_date(MYSQL_TIME *res, my_time_flags_t fuzzy_date) override;
   bool resolve_type(THD *thd) override {
+    if (reject_vector_args()) return true;
     if (param_type_is_default(thd, 0, 1, MYSQL_TYPE_DATETIME)) return true;
     return Item_date_func::resolve_type(thd);
   }
