@@ -1,15 +1,16 @@
-/* Copyright (c) 2020, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2020, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -216,6 +217,8 @@ TABLE *GetBasicTable(const AccessPath *path) {
     // Basic access paths (those with no children, at least nominally).
     case AccessPath::TABLE_SCAN:
       return path->table_scan().table;
+    case AccessPath::SAMPLE_SCAN:
+      return path->sample_scan().table;
     case AccessPath::INDEX_SCAN:
       return path->index_scan().table;
     case AccessPath::INDEX_DISTANCE_SCAN:
@@ -1319,6 +1322,10 @@ unique_ptr_destroy_only<RowIterator> CreateIteratorFromAccessPath(
                                             std::move(job.children[0]));
         break;
       }
+      case AccessPath::SAMPLE_SCAN: { /* LCOV_EXCL_LINE */
+        // SampleScan can be executed only in the secondary engine.
+        assert(false); /* LCOV_EXCL_LINE */
+      }
     }
 
     if (iterator == nullptr) {
@@ -1332,6 +1339,12 @@ unique_ptr_destroy_only<RowIterator> CreateIteratorFromAccessPath(
 }
 
 void FindTablesToGetRowidFor(AccessPath *path) {
+  // A map of the tables for which other paths further down in the tree will
+  // take care of copying the correct row ID into table->file->ref. The hash
+  // join iterators and the BKA join iterators do that, so iterators higher up
+  // should not call handler::position(), as that would overwrite the copied row
+  // ID with the row ID of the last row that was read by the join. Sorting
+  // iterators, on the other hand, do not
   table_map handled_by_others = 0;
 
   auto add_tables_handled_by_others = [path, &handled_by_others](
@@ -1360,6 +1373,13 @@ void FindTablesToGetRowidFor(AccessPath *path) {
         // Doesn't really matter, we don't cross query blocks anyway.
         return true;
       }
+      case AccessPath::SORT:
+        // The sorting iterators do not populate handler::ref with the row ID
+        // while returning rows, so row IDs in any tables handled by paths below
+        // it have to be fetched again from the handler by the paths above the
+        // sort. Therefore, we don't add any of the tables in the subtree below
+        // SORT to handled_by_others.
+        return true;  // Skip the rest of the subtree.
       default:
         return false;
     }
@@ -1515,16 +1535,14 @@ void ExpandSingleFilterAccessPath(THD *thd, AccessPath *path, const JOIN *join,
       filter_cost +=
           EstimateFilterCost(thd, filter_rows, condition, join->query_block)
               .cost_if_not_materialized;
-      filter_rows *= EstimateSelectivity(thd, condition, *expr->companion_set,
-                                         /*trace=*/nullptr);
+      filter_rows *= EstimateSelectivity(thd, condition, *expr->companion_set);
     }
     for (Item *condition : expr->join_conditions) {
       items.push_back(condition);
       filter_cost +=
           EstimateFilterCost(thd, filter_rows, condition, join->query_block)
               .cost_if_not_materialized;
-      filter_rows *= EstimateSelectivity(thd, condition, *expr->companion_set,
-                                         /*trace=*/nullptr);
+      filter_rows *= EstimateSelectivity(thd, condition, *expr->companion_set);
     }
     assert(!items.is_empty());
 
@@ -1626,3 +1644,81 @@ table_map GetHashJoinTables(AccessPath *path) {
       });
   return tables;
 }
+
+void CollectStatusVariables(THD *thd, const JOIN *top_join,
+                            const AccessPath &top_path) {
+  MutableOverflowBitset seen_first_tables(thd->mem_root,
+                                          thd->lex->select_number);
+  WalkAccessPaths(
+      &top_path, top_join, WalkAccessPathPolicy::ENTIRE_TREE,
+      [thd, &seen_first_tables](const AccessPath *path, const JOIN *join) {
+        if (join == nullptr) {
+          // Skip paths that don't belong to a particular query block. In
+          // practice, this means the materialization path of a UNION used as a
+          // derived table.
+          return false;
+        }
+
+        const TABLE *const table = GetBasicTable(path);
+        if (table == nullptr) {
+          // Skip paths that don't represent a table access.
+          return false;
+        }
+        if (table->pos_in_table_list == nullptr) {
+          // Skip paths that read a table that is not in the FROM list.
+          // (Typically an internal temporary table created by the optimizer.)
+          // We count scans of tables in the FROM list only.
+          return false;
+        }
+
+        // Check if this is the first table we see in this query block. The
+        // first table of each query block is counted in a different status
+        // variable than the other tables.
+        const int query_block_bit = join->query_block->select_number - 1;
+        const bool first_table = !IsBitSet(query_block_bit, seen_first_tables);
+        if (first_table) {
+          seen_first_tables.SetBit(query_block_bit);
+        }
+
+        switch (path->type) {
+          case AccessPath::TABLE_SCAN:
+            thd->set_status_no_index_used();
+            [[fallthrough]];
+          case AccessPath::INDEX_SCAN:
+            if (first_table) {
+              thd->inc_status_select_scan();
+            } else {
+              thd->inc_status_select_full_join();
+            }
+            break;
+
+          case AccessPath::INDEX_RANGE_SCAN:
+          case AccessPath::INDEX_SKIP_SCAN:
+          case AccessPath::GROUP_INDEX_SKIP_SCAN:
+          case AccessPath::INDEX_MERGE:
+          case AccessPath::ROWID_INTERSECTION:
+          case AccessPath::ROWID_UNION:
+            if (first_table) {
+              thd->inc_status_select_range();
+            } else {
+              thd->inc_status_select_full_range_join();
+            }
+            break;
+
+          case AccessPath::DYNAMIC_INDEX_RANGE_SCAN:
+            thd->set_status_no_index_used();
+            thd->set_status_no_good_index_used();
+            thd->inc_status_select_range_check();
+            break;
+
+          default:;
+        }
+
+        // Stop traversing the sub-tree when we have seen a table access. There
+        // could be more paths below it (in particular for INDEX_MERGE,
+        // ROWID_INTERSECTION and ROWID_UNION), but they would all be for the
+        // same table, so we don't want to visit them and double-count the
+        // table. Returning true skips the sub-tree below this path.
+        return true;
+      });
+}
