@@ -1,15 +1,16 @@
-/* Copyright (c) 2020, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2020, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,6 +25,7 @@
 
 #include <sys/types.h>
 #include <algorithm>
+#include <bit>
 #include <initializer_list>
 #include <string>
 
@@ -36,6 +38,7 @@
 #include "sql/item_cmpfunc.h"
 #include "sql/item_func.h"
 #include "sql/join_optimizer/bit_utils.h"
+#include "sql/join_optimizer/optimizer_trace.h"
 #include "sql/join_optimizer/print_utils.h"
 #include "sql/join_optimizer/relational_expression.h"
 #include "sql/key.h"
@@ -53,7 +56,7 @@ namespace {
    Return the selectivity of 'field' derived from a histogram, or -1.0 if there
    was no histogram.
 */
-double HistogramSelectivity(const Field &field, string *trace) {
+double HistogramSelectivity(THD *thd, const Field &field) {
   const histograms::Histogram *const histogram =
       field.table->find_histogram(field.field_index());
 
@@ -68,16 +71,13 @@ double HistogramSelectivity(const Field &field, string *trace) {
         histogram->get_non_null_values_fraction() /
         std::max<double>(1.0, histogram->get_num_distinct_values());
 
-    if (trace != nullptr) {
-      std::ostringstream stream;
-      stream << " - estimating selectivity " << selectivity << " for field "
-             << field.table->alias << "." << field.field_name
-             << " from histogram showing "
-             << histogram->get_num_distinct_values()
-             << " distinct values and non-null fraction "
-             << histogram->get_non_null_values_fraction() << ".\n";
-
-      *trace += stream.str();
+    if (TraceStarted(thd)) {
+      Trace(thd) << " - estimating selectivity " << selectivity << " for field "
+                 << field.table->alias << "." << field.field_name
+                 << " from histogram showing "
+                 << histogram->get_num_distinct_values()
+                 << " distinct values and non-null fraction "
+                 << histogram->get_non_null_values_fraction() << ".\n";
     }
     return selectivity;
   } else {
@@ -91,7 +91,7 @@ double HistogramSelectivity(const Field &field, string *trace) {
   selectivity of 'field' (i.e. 1/'number of rows in table') and return
   that. If there is no such index, return 1.0.
 */
-double KeyCap(const Field &field, uint key_no, string *trace) {
+double KeyCap(THD *thd, const Field &field, uint key_no) {
   assert(key_no < field.table->s->keys);
   const KEY &key = field.table->key_info[key_no];
 
@@ -113,8 +113,8 @@ double KeyCap(const Field &field, uint key_no, string *trace) {
   const double field_cap =
       1.0 / std::max<double>(1.0, field.table->file->stats.records);
 
-  if (trace != nullptr) {
-    *trace += StringPrintf(
+  if (TraceStarted(thd)) {
+    Trace(thd) << StringPrintf(
         " - capping selectivity to %g since index is unique\n", field_cap);
   }
 
@@ -126,10 +126,10 @@ double KeyCap(const Field &field, uint key_no, string *trace) {
   upper bound on the selectivity of field (i.e. 1/'number of rows in table'). If
   there is no such index, return 1.0.
 */
-double FindSelectivityCap(const Field &field, string *trace) {
+double FindSelectivityCap(THD *thd, const Field &field) {
   for (uint i = field.key_start.get_first_set(); i != MY_BIT_NONE;
        i = field.key_start.get_next_set(i)) {
-    const double key_cap = KeyCap(field, i, trace);
+    const double key_cap = KeyCap(thd, field, i);
 
     if (key_cap < 1.0) {
       return key_cap;
@@ -205,8 +205,8 @@ using EqualFieldArray = Bounds_checked_array<const Field *const>;
   exploit the correlation between the fields in the prefix.
 */
 KeySelectivityResult EstimateSelectivityFromIndexStatistics(
-    const Field &equal_field, const CompanionSet &companion_set,
-    const TABLE &table, uint key_no, string *trace) {
+    THD *thd, const Field &equal_field, const CompanionSet &companion_set,
+    const TABLE &table, uint key_no) {
   const KEY &key = table.key_info[key_no];
   table_map joined_tables{~PSEUDO_TABLE_BITS};
 
@@ -236,7 +236,7 @@ KeySelectivityResult EstimateSelectivityFromIndexStatistics(
       present in companion_set. Then we still want to use the first
       key field.
     */
-    if (part_no > 0 && !AreMultipleBitsSet(joined_tables)) {
+    if (part_no > 0 && std::popcount(joined_tables) < 2) {
       break;
     }
 
@@ -260,14 +260,12 @@ KeySelectivityResult EstimateSelectivityFromIndexStatistics(
         }
       }();
 
-      if (trace != nullptr) {
-        std::ostringstream stream;
-        stream << " - found " << (part_no + 1)
-               << "-field prefix of candidate index " << key.name
-               << " with selectivity " << field_selectivity
-               << " for last field " << key_field.table->alias << "."
-               << key_field.field_name << "\n";
-        *trace += stream.str();
+      if (TraceStarted(thd)) {
+        Trace(thd) << " - found " << (part_no + 1)
+                   << "-field prefix of candidate index " << key.name
+                   << " with selectivity " << field_selectivity
+                   << " for last field " << key_field.table->alias << "."
+                   << key_field.field_name << "\n";
       }
 
       return {field_selectivity, part_no + 1};
@@ -301,18 +299,18 @@ KeySelectivityResult EstimateSelectivityFromIndexStatistics(
      clearly prefer the least risky choice here.
 
   Returns -1.0 if no index or no histogram was found. Lifted from
-  Item_equal::get_filtering_effect.
+  Item_multi_eq::get_filtering_effect.
 
+  @param[in] thd The current thread.
   @param[in] equal_fields  The equijoined fields for which we calculate
   selectivity.
   @param[in] companion_set The CompanionSet of the join.
-  @param[in,out] trace Optimizer trace.
   @returns The estimated selectivity of 'field' (or -1.0 if there was no
   suitable index or histogram).
 */
-double EstimateEqualPredicateSelectivity(const EqualFieldArray &equal_fields,
-                                         const CompanionSet &companion_set,
-                                         string *trace) {
+double EstimateEqualPredicateSelectivity(THD *thd,
+                                         const EqualFieldArray &equal_fields,
+                                         const CompanionSet &companion_set) {
   uint longest_prefix = 0;
   double selectivity = -1.0;
   double selectivity_cap = 1.0;
@@ -324,12 +322,12 @@ double EstimateEqualPredicateSelectivity(const EqualFieldArray &equal_fields,
       const KEY &key = equal_field->table->key_info[key_no];
       KeySelectivityResult key_data{-1.0, 0};
 
-      const double key_cap = KeyCap(*equal_field, key_no, trace);
+      const double key_cap = KeyCap(thd, *equal_field, key_no);
       if (key_cap < 1.0) {
         key_data = {key_cap, 1};
       } else if (key.has_records_per_key(0)) {
         key_data = EstimateSelectivityFromIndexStatistics(
-            *equal_field, companion_set, *equal_field->table, key_no, trace);
+            thd, *equal_field, companion_set, *equal_field->table, key_no);
       }
 
       selectivity_cap = std::min(selectivity_cap, key_cap);
@@ -348,7 +346,7 @@ double EstimateEqualPredicateSelectivity(const EqualFieldArray &equal_fields,
   } else {
     // Look for histograms if there was no suitable index.
     for (const Field *field : equal_fields) {
-      selectivity = std::max(selectivity, HistogramSelectivity(*field, trace));
+      selectivity = std::max(selectivity, HistogramSelectivity(thd, *field));
     }
   }
 
@@ -362,7 +360,7 @@ double EstimateEqualPredicateSelectivity(const EqualFieldArray &equal_fields,
   on a 0..1 scale (where 1.0 lets all records through).
  */
 double EstimateSelectivity(THD *thd, Item *condition,
-                           const CompanionSet &companion_set, string *trace) {
+                           const CompanionSet &companion_set) {
   // If the item is a true constant, we can say immediately whether it passes
   // or filters all rows. (Actually, calling get_filtering_effect() below
   // would crash if used_tables() is zero, which it is for const items.)
@@ -390,12 +388,12 @@ double EstimateSelectivity(THD *thd, Item *condition,
                                  down_cast<Item_field *>(right)->field};
 
         double selectivity = EstimateEqualPredicateSelectivity(
-            EqualFieldArray(fields, array_elements(fields)), companion_set,
-            trace);
+            thd, EqualFieldArray(fields, array_elements(fields)),
+            companion_set);
 
         if (selectivity >= 0.0) {
-          if (trace != nullptr) {
-            *trace += StringPrintf(
+          if (TraceStarted(thd)) {
+            Trace(thd) << StringPrintf(
                 " - used an index or a histogram for %s, selectivity = %g\n",
                 ItemToString(condition).c_str(), selectivity);
           }
@@ -408,12 +406,12 @@ double EstimateSelectivity(THD *thd, Item *condition,
         // index on this field.
         selectivity_cap = std::min(
             selectivity_cap,
-            FindSelectivityCap(*down_cast<Item_field *>(left)->field, trace));
+            FindSelectivityCap(thd, *down_cast<Item_field *>(left)->field));
       } else if (right->type() == Item::FIELD_ITEM) {
         // Same, for <anything> = field.
         selectivity_cap = std::min(
             selectivity_cap,
-            FindSelectivityCap(*down_cast<Item_field *>(right)->field, trace));
+            FindSelectivityCap(thd, *down_cast<Item_field *>(right)->field));
       }
     }
   }
@@ -448,8 +446,8 @@ double EstimateSelectivity(THD *thd, Item *condition,
   // If we get more sophisticated cardinality estimation, e.g. by histograms
   // or the likes, we need to revisit this assumption, and potentially adjust
   // our model here.
-  if (is_function_of_type(condition, Item_func::MULT_EQUAL_FUNC)) {
-    Item_equal *equal = down_cast<Item_equal *>(condition);
+  if (is_function_of_type(condition, Item_func::MULTI_EQ_FUNC)) {
+    Item_multi_eq *equal = down_cast<Item_multi_eq *>(condition);
 
     // These should have been expanded early, before we get here.
     assert(equal->const_arg() == nullptr);
@@ -459,11 +457,11 @@ double EstimateSelectivity(THD *thd, Item *condition,
     }
 
     double selectivity = EstimateEqualPredicateSelectivity(
-        EqualFieldArray(&fields[0], fields.size()), companion_set, trace);
+        thd, EqualFieldArray(&fields[0], fields.size()), companion_set);
 
     if (selectivity >= 0.0) {
-      if (trace != nullptr) {
-        *trace += StringPrintf(
+      if (TraceStarted(thd)) {
+        Trace(thd) << StringPrintf(
             " - used an index or a histogram for %s, selectivity = %g\n",
             ItemToString(condition).c_str(), selectivity);
       }
@@ -494,9 +492,9 @@ double EstimateSelectivity(THD *thd, Item *condition,
       /*rows_in_table=*/1000.0);
 
   selectivity = std::min(selectivity, selectivity_cap);
-  if (trace != nullptr) {
-    *trace += StringPrintf(" - fallback selectivity for %s = %g\n",
-                           ItemToString(condition).c_str(), selectivity);
+  if (TraceStarted(thd)) {
+    Trace(thd) << StringPrintf(" - fallback selectivity for %s = %g\n",
+                               ItemToString(condition).c_str(), selectivity);
   }
   return selectivity;
 }
