@@ -1,15 +1,16 @@
-/* Copyright (c) 2015, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -298,6 +299,21 @@ class Foreign_key_spec : public Key_spec {
    */
   bool validate(THD *thd, const char *table_name,
                 List<Create_field> &table_fields) const;
+
+  /**
+    Set referenced column list for a FK from the referenced table PK columns.
+
+    @param thd                     Thread handle.
+    @param is_self_referencing_fk  Represents self referencing foreign key.
+    @param key_list                key info list of parent/referenced table.
+
+    @retval false   when ref_columns list is populated successfully or when
+                    ref_columns list is empty since parent table or PK in
+                    parent table is missing.
+    @retval true    if any error
+  */
+  bool set_ref_columns_for_implicit_pk(THD *thd, bool is_self_referencing_fk,
+                                       Mem_root_array<Key_spec *> &key_list);
 };
 
 /**
