@@ -1,15 +1,16 @@
-/* Copyright (c) 2010, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -172,7 +173,6 @@ extern bool opt_help;
 extern bool opt_verbose;
 extern MYSQL_PLUGIN_IMPORT std::atomic<int32>
     connection_events_loop_aborted_flag;
-extern bool opt_no_dd_upgrade;
 extern long opt_upgrade_mode;
 extern bool opt_initialize;
 extern bool opt_safe_user_create;
@@ -221,8 +221,7 @@ extern const char *opt_secure_file_priv;
 extern bool opt_log_slow_admin_statements, opt_log_slow_replica_statements;
 extern bool sp_automatic_privileges, opt_noacl;
 extern bool trust_function_creators;
-extern bool check_proxy_users, mysql_native_password_proxy_users,
-    sha256_password_proxy_users;
+extern bool check_proxy_users, sha256_password_proxy_users;
 #ifdef _WIN32
 extern const char *shared_memory_base_name;
 #endif
@@ -262,7 +261,6 @@ extern const char *timestamp_type_names[];
 extern char *opt_general_logname, *opt_slow_logname, *opt_bin_logname,
     *opt_relay_logname;
 extern char *mysql_home_ptr, *pidfile_name_ptr;
-extern char *default_auth_plugin;
 extern uint default_password_lifetime;
 extern bool password_require_current;
 /*
@@ -344,7 +342,6 @@ enum enum_binlog_error_action {
 };
 extern const char *binlog_error_action_list[];
 extern char *opt_authentication_policy;
-extern std::vector<std::string> authentication_policy_list;
 
 extern ulong stored_program_cache_size;
 extern ulong back_log;
@@ -375,8 +372,6 @@ extern "C" MYSQL_PLUGIN_IMPORT int orig_argc;
 extern "C" MYSQL_PLUGIN_IMPORT char **orig_argv;
 extern bool server_shutting_down;
 extern my_thread_attr_t connection_attrib;
-extern bool old_mode;
-extern bool avoid_temporal_upgrade;
 extern LEX_STRING opt_init_connect, opt_init_replica;
 extern ulong connection_errors_internal;
 extern ulong connection_errors_peer_addr;
@@ -715,8 +710,8 @@ extern mysql_mutex_t LOCK_tls_ctx_options;
 extern mysql_mutex_t LOCK_admin_tls_ctx_options;
 extern mysql_mutex_t LOCK_rotate_binlog_master_key;
 extern mysql_mutex_t LOCK_partial_revokes;
-extern mysql_mutex_t LOCK_authentication_policy;
 extern mysql_mutex_t LOCK_global_conn_mem_limit;
+extern mysql_mutex_t LOCK_authentication_policy;
 
 extern mysql_cond_t COND_server_started;
 extern mysql_cond_t COND_compress_gtid_table;
@@ -801,10 +796,6 @@ void set_mysqld_partial_revokes(bool value);
 
 bool check_and_update_partial_revokes_sysvar(THD *thd);
 
-bool parse_authentication_policy(char *val,
-                                 std::vector<std::string> &policy_list);
-bool validate_authentication_policy(char *val);
-bool update_authentication_policy();
 #ifdef _WIN32
 
 bool is_windows_service();
@@ -827,6 +818,8 @@ extern SERVICE_TYPE_NO_CONST(registry) * srv_registry_no_lock;
    mysql_server component */
 extern SERVICE_TYPE(dynamic_loader_scheme_file) * scheme_file_srv;
 extern SERVICE_TYPE(dynamic_loader) * dynamic_loader_srv;
+extern SERVICE_TYPE(registry_registration) * registry_registration;
+extern SERVICE_TYPE(registry_registration) * registry_registration_no_lock;
 
 class Deployed_components;
 extern Deployed_components *g_deployed_components;
@@ -834,4 +827,28 @@ extern Deployed_components *g_deployed_components;
 extern bool opt_persist_sensitive_variables_in_plaintext;
 
 void persisted_variables_refresh_keyring_support();
+
+/**
+  Stores the value of argc during server start up that contains
+  the count of arguments specified by the user in the
+  configuration files and command line.
+  The server refers the cached argument count during
+  plugin and component installation.
+*/
+extern int argc_cached;
+/**
+  Stores the value of argv during server start up that contains
+  the vector of arguments specified by the user in the
+  configuration files and command line.
+  The server refers the cached argument vector during
+  plugin and component installation.
+*/
+extern char **argv_cached;
+
+/// Stores the last time the warning for non-composable engine is emitted
+extern std::atomic<time_t> last_mixed_non_transactional_engine_warning;
+/// The time period for which no warning for non-composable engines should
+/// be written to the error log after a similar warning was written
+
+const uint16_t mixed_non_transactional_engine_warning_period = 60 * 2;
 #endif /* MYSQLD_INCLUDED */
