@@ -1,15 +1,16 @@
-/* Copyright (c) 2011, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2011, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -1911,7 +1912,8 @@ bool explain_single_table_modification(THD *explain_thd, const THD *query_thd,
   const bool is_explain_into =
       explain_thd->lex->explain_format->is_explain_into();
 
-  if (explain_thd->lex->explain_format->is_iterator_based()) {
+  if (explain_thd->lex->explain_format->is_iterator_based(explain_thd,
+                                                          query_thd)) {
     // These kinds of queries don't have a JOIN with an iterator tree.
     return ExplainIterator(explain_thd, query_thd, nullptr);
   }
@@ -1961,7 +1963,6 @@ bool explain_single_table_modification(THD *explain_thd, const THD *query_thd,
       // Derived tables and const subqueries are already optimized
       if (!unit->is_optimized() &&
           unit->optimize(explain_thd, /*materialize_destination=*/nullptr,
-                         /*create_iterators=*/false,
                          /*finalize_access_paths=*/true))
         return true; /* purecov: inspected */
     }
@@ -2254,7 +2255,7 @@ bool explain_query(THD *explain_thd, const THD *query_thd,
 
   const bool is_explain_into = lex->explain_format->is_explain_into();
 
-  if (lex->explain_format->is_iterator_based()) {
+  if (lex->explain_format->is_iterator_based(explain_thd, query_thd)) {
     if (lex->is_explain_analyze) {
       if (secondary_engine) {
         my_error(ER_NOT_SUPPORTED_YET, MYF(0),
@@ -2286,11 +2287,14 @@ bool explain_query(THD *explain_thd, const THD *query_thd,
   }
 
   // Non-iterator-based formats are not supported with EXPLAIN ANALYZE.
-  if (lex->is_explain_analyze)
-    my_error(ER_NOT_SUPPORTED_YET, MYF(0),
-             (lex->explain_format->is_hierarchical()
-                  ? "EXPLAIN ANALYZE with JSON format"
-                  : "EXPLAIN ANALYZE with TRADITIONAL format"));
+  if (lex->is_explain_analyze) {
+    if (lex->explain_format->is_hierarchical()) {
+      my_error(ER_EXPLAIN_ANALYZE_JSON_FORMAT_VERSION_NOT_SUPPORTED, MYF(0));
+    } else {
+      my_error(ER_NOT_SUPPORTED_YET, MYF(0),
+               "EXPLAIN ANALYZE with TRADITIONAL format");
+    }
+  }
 
   // Non-iterator-based formats are not supported with the hypergraph
   // optimizer. But we still want to be able to use EXPLAIN with no format
