@@ -33,6 +33,8 @@
 #include <sys/types.h>
 
 #include <cstring>
+#include <optional>
+#include <string_view>
 
 #include "my_alloc.h"  // MEM_ROOT
 #include "my_compiler.h"
@@ -510,8 +512,21 @@ class Explain_format {
  protected:
   Query_result *output;  ///< output resulting data there
 
+ private:
+  std::optional<std::string_view> m_explain_into_variable_name;
+
  public:
+  /* Which schema this EXPLAIN statement should be run for. */
+  LEX_CSTRING m_schema_name_for_explain;
+
+ protected:
   Explain_format() : output(nullptr) {}
+  explicit Explain_format(
+      std::optional<std::string_view> explain_into_variable_name)
+      : output(nullptr),
+        m_explain_into_variable_name(explain_into_variable_name) {}
+
+ public:
   virtual ~Explain_format() = default;
 
   /**
@@ -535,7 +550,47 @@ class Explain_format {
     @retval true        Format is Iterator-based.
     @retval false       Format is not Iterator-based.
   */
-  virtual bool is_iterator_based() const { return false; }
+  virtual bool is_iterator_based(THD *explain_thd [[maybe_unused]],
+                                 const THD *query_thd [[maybe_unused]]) const {
+    return false;
+  }
+
+  /**
+   * Whether the output of an EXPLAIN statement should be stored in a user
+   * variable or sent to the client. If this function returns true,
+   * explain_into_variable_name() returns the name of the variable.
+   *
+   * @retval true       EXPLAIN output should be stored in a user variable.
+   * @retval false      EXPLAIN output should be sent to the client.
+   */
+  bool is_explain_into() const {
+    return m_explain_into_variable_name.has_value();
+  }
+
+  /**
+   * Whether the EXPLAIN statement should be run in another schema than the
+   * current active schema. If this returns true, m_schema_name_for_explain
+   * contains the name of the schema to use for EXPLAIN.
+   *
+   * @return true       The EXPLAIN statement should be run in another schema.
+   * @return false      The EXPLAIN statement should be run in the current
+   *                    active schema.
+   */
+  bool is_explain_for_schema() const {
+    return m_schema_name_for_explain.length != 0;
+  }
+
+  /**
+   * Returns the name of the user variable the output of this EXPLAIN
+   * statement is to be stored in. Should only be called if this is an
+   * EXPLAIN INTO statement.
+   *
+   * @return std::string_view The name of the variable to store the output in.
+   */
+  std::string_view explain_into_variable_name() const {
+    assert(is_explain_into());
+    return m_explain_into_variable_name.value();
+  }
 
   /**
     Send EXPLAIN header item(s) to output stream
