@@ -1,15 +1,16 @@
-/* Copyright (c) 2011, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2011, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -2114,3 +2115,10 @@ std::string Explain_format_JSON::ExplainJsonToString(Json_object *json) {
   }
   return {explain.ptr(), explain.length()};
 }
+
+bool Explain_format_JSON::is_iterator_based(THD *explain_thd,
+                                            const THD *query_thd) const {
+  return query_thd->optimizer_switch_flag(
+             OPTIMIZER_SWITCH_HYPERGRAPH_OPTIMIZER) ||
+         explain_thd->variables.explain_json_format_version == 2;
+}
