@@ -1,15 +1,16 @@
-/* Copyright (c) 2015, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -515,6 +516,553 @@ static bool set_join_hint_deps(JOIN *join,
   return false;
 }
 
+/**
+  Checks if any join order hints have been specified in query.
+
+  @return true if join order hints present, false otherwise.
+*/
+bool Opt_hints_qb::has_join_order_hints() const {
+  return join_order_hints.size() > 0;
+}
+
+/**
+  Deletes all the join order hints.
+*/
+void Opt_hints_qb::clear_join_order_hints() { join_order_hints.clear(); }
+
+/**
+ Check if a join table matches a hinted table. If the join table is an outer
+ join, semijoin or antijoin, check all its nested tables for a match with the
+ hinted table and return the matching table.
+
+ @param hint_table Table specified in hint
+ @param table      Table from join list
+
+ @return Join table which matches hinted table
+*/
+static Table_ref *find_hinted_table(const Hint_param_table *hint_table,
+                                    const Table_ref *table) {
+  if (!table->nested_join) {
+    if (!compare_table_name(hint_table, table))
+      return const_cast<Table_ref *>(table);
+    return nullptr;
+  }
+  for (auto nest_it = table->nested_join->m_tables.rbegin();
+       nest_it != table->nested_join->m_tables.rend();
+       ++nest_it) {  // The list goes backwards.
+    Table_ref *nest_tl = *nest_it;
+    if (nest_tl->nested_join) {
+      Table_ref *found_tab = find_hinted_table(hint_table, nest_tl);
+      if (found_tab != nullptr) return found_tab;
+    } else {
+      if (!compare_table_name(hint_table, nest_tl)) return nest_tl;
+    }
+  }
+  return nullptr;
+}
+
+/**
+ Return tablemap of tables present within the Table_ref. If the table is an
+ outer join, semijoin or antijoin, return the bitmap of all nested tables. If
+ not, return the bitmap of the table.
+
+ @param table Table for which bitmap(s) requested
+
+ @return bitmap of all tables within this Table_ref
+*/
+static table_map get_table_map(const Table_ref *table) {
+  return table->nested_join ? table->nested_join->used_tables : table->map();
+}
+/**
+ Add a table to the specified list. If an order_list is specified, tables must
+ be added maintaining the relative order in the order list.
+
+ @param table             Table to be added to list
+ @param exclude_table_map Bitmap of tables to be excluded from list
+ @param order_table_map   Bitmap of tables in order-list
+ @param order_list        List of required relative orders
+ @param list              List to add table to
+*/
+static void add_table_to_list(Table_ref *table, table_map exclude_table_map,
+                              table_map order_table_map,
+                              mem_root_deque<Table_ref *> *order_list,
+                              mem_root_deque<Table_ref *> *list) {
+  table_map curr_table_map = get_table_map(table);
+  if (Overlaps(curr_table_map, order_table_map)) {
+    // if any JOIN_ORDER_HINT table is found, replace it with the
+    // next table in the JOIN_ORDER_HINT order. This will ensure that
+    // all the JOIN_ORDER_HINT tables are present in relative order.
+    Table_ref *next_table_in_order = nullptr;
+    if (!order_list->empty()) next_table_in_order = order_list->back();
+
+    while (next_table_in_order &&
+           Overlaps(get_table_map(next_table_in_order), exclude_table_map)) {
+      order_list->pop_back();
+      if (order_list->empty()) {
+        next_table_in_order = nullptr;
+      } else {
+        next_table_in_order = order_list->back();
+      }
+    }
+    if (next_table_in_order != nullptr) {
+      order_list->pop_back();
+      list->push_front(next_table_in_order);
+    }
+  } else if (!Overlaps(curr_table_map, exclude_table_map)) {
+    list->push_front(table);
+  }
+  return;
+}
+
+/**
+  Sorts tables from the join list to create a new join list which contains the
+  tables in an order which complies with join order hints.
+
+  @param join          Pointer to JOIN object
+  @param join_list     Deque of tables in join
+  @param toplevel      False for subqueries, true otherwise
+
+  @return Deque of tables sorted in reverse hinted order.
+*/
+mem_root_deque<Table_ref *> *Opt_hints_qb::sort_tables_in_join_order(
+    JOIN *join, const mem_root_deque<Table_ref *> *join_list, bool toplevel) {
+  if (!has_join_order_hints()) return nullptr;
+  THD *thd = join->thd;
+  MEM_ROOT *mem_root_arg = thd->mem_root;
+  mem_root_deque<Table_ref *> *new_join_list =
+      new (mem_root_arg) mem_root_deque<Table_ref *>(mem_root_arg);
+
+  if (join_list->size() <= 1) {
+    new_join_list = const_cast<mem_root_deque<Table_ref *> *>(join_list);
+    return new_join_list;
+  }
+
+  mem_root_deque<Table_ref *> prefix_list(mem_root_arg),
+      suffix_list(mem_root_arg), order_list(mem_root_arg);
+  table_map prefix_table_map = 0, suffix_table_map = 0, order_table_map = 0;
+  // Iterate through the hint table list for each hint and place the tables
+  // into 3 lists: prefix_list, suffix_list and order_list
+  for (uint hint_idx = 0; hint_idx < join_order_hints.size(); hint_idx++) {
+    PT_qb_level_hint *hint = join_order_hints[hint_idx];
+    Hint_param_table_list *hint_table_list = hint->get_table_list();
+    if (hint_table_list->size() == 0) continue;
+
+    mem_root_deque<Table_ref *> curr_order_list(mem_root_arg);
+    table_map curr_order_table_map = 0;
+    bool conflicting_hint = false;
+
+    // Push all prefix/suffix tables backwards into join table list
+    for (uint hint_tab_idx = 0; hint_tab_idx < hint_table_list->size();
+         hint_tab_idx++) {
+      if (conflicting_hint) break;
+
+      const Hint_param_table &hint_table = (*hint_table_list)[hint_tab_idx];
+      Table_ref *match = nullptr, *curr_tab = nullptr;
+      for (auto it = join_list->rbegin(); it != join_list->rend(); ++it) {
+        curr_tab = *it;
+        match = find_hinted_table(&hint_table, curr_tab);
+        if (match != nullptr) break;
+      }
+      if (match == nullptr) {
+        if (toplevel) {
+          // unknown table
+          print_join_order_warn(thd, hint->type(), &hint_table);
+          conflicting_hint = true;
+        }
+        continue;
+      }
+      table_map match_map = match->map();
+      if (curr_tab->nested_join) {
+        table_map already_sorted = prefix_table_map | suffix_table_map |
+                                   order_table_map | curr_order_table_map;
+        if (!Overlaps(already_sorted, curr_tab->nested_join->used_tables) &&
+            (curr_tab->nested_join->m_tables.size() > 1)) {
+          // sort inner table list according to hints
+          mem_root_deque<Table_ref *> *nested_table_list =
+              sort_tables_in_join_order(join, &curr_tab->nested_join->m_tables);
+          if (nested_table_list != &curr_tab->nested_join->m_tables) {
+            curr_tab->nested_join->m_tables.clear();
+            curr_tab->nested_join->m_tables = *nested_table_list;
+          }
+        }
+        match = curr_tab;
+        match_map = curr_tab->nested_join->used_tables;
+
+        if (hint->type() == JOIN_PREFIX_HINT_ENUM && prefix_list.empty()) {
+          conflicting_hint = true;
+          push_warning_printf(thd, Sql_condition::SL_WARNING,
+                              ER_WARN_UNSUPPORTED_HINT,
+                              ER_THD(thd, ER_WARN_UNSUPPORTED_HINT),
+                              "hypergraph: first table in join order cannot be "
+                              "semijoin/antijoin nest");
+          continue;
+        }
+      }
+      table_map hinted_table_map = prefix_table_map | suffix_table_map;
+      if (Overlaps(hinted_table_map, match_map)) {
+        // table already hinted, ignore and add remaining hinted tables
+        // for best-effort matching of multiple hints
+        continue;
+      }
+      mem_root_deque<Table_ref *> *selected_list = nullptr;
+      table_map *selected_map = nullptr;
+      switch (hint->type()) {
+        case JOIN_PREFIX_HINT_ENUM:
+          if (Overlaps(order_table_map, match_map)) {
+            Table_ref *prefix_tab = nullptr, *order_tab = *order_list.rbegin();
+            auto order_it = order_list.rbegin();
+            // Find first table in order list which does not match a prefix
+            // table
+            for (auto prefix_it = prefix_list.rbegin();
+                 order_it != order_list.rend() &&
+                 prefix_it != prefix_list.rend();
+                 ++order_it, ++prefix_it) {
+              order_tab = *order_it;
+              prefix_tab = *prefix_it;
+              if (order_tab != prefix_tab) break;
+            }
+
+            if (*order_it != match) {
+              conflicting_hint = true;
+              continue;
+            }
+          }
+          selected_list = &prefix_list;
+          selected_map = &prefix_table_map;
+          break;
+        case JOIN_SUFFIX_HINT_ENUM:
+          if (Overlaps(order_table_map, match_map)) {
+            if (hint_tab_idx == (hint_table_list->size() - 1)) {
+              Table_ref *last_table_in_order = *order_list.begin();
+              if (last_table_in_order != match) {
+                conflicting_hint = true;
+                continue;
+              }
+            } else {
+              auto order_it = order_list.rbegin();
+              for (; order_it != order_list.rend(); ++order_it) {
+                Table_ref *order_tab = *order_it;
+                if (order_tab == match) break;
+              }
+              for (; order_it != order_list.rend(); ++order_it) {
+                Table_ref *order_tab = *order_it;
+                for (auto suffix_it = suffix_list.rbegin();
+                     suffix_it != suffix_list.rend(); ++suffix_it) {
+                  // if any table in order_list *after* match matches an
+                  // existing suffix table, then the suffix order is the
+                  // reverse of the existing hinted join order
+                  if (order_tab == *suffix_it) {
+                    conflicting_hint = true;
+                    break;
+                  }
+                }
+                if (conflicting_hint) break;
+              }
+              if (conflicting_hint) continue;
+            }
+          }
+          selected_list = &suffix_list;
+          selected_map = &suffix_table_map;
+          break;
+        case JOIN_ORDER_HINT_ENUM:
+          if (Overlaps(match_map, curr_order_table_map)) {
+            // table already hinted, continue
+            continue;
+          }
+          selected_list = &curr_order_list;
+          selected_map = &curr_order_table_map;
+          break;
+        default:
+          assert(false);
+          break;
+      }
+      selected_list->push_front(match);
+      (*selected_map) |= match_map;
+    }
+    if (conflicting_hint) {
+      join_order_hints_ignored |= 1ULL << hint_idx;
+      continue;
+    }
+
+    mem_root_deque<Table_ref *> merged_order_list(mem_root_arg);
+    auto curr_it = curr_order_list.rbegin();
+    // A query may have multiple JOIN_ORDER_HINTs. These individual orders
+    // need to be combined if possible to create an order which fulfils all
+    // the JOIN_ORDER_HINTs. For each JOIN_ORDER hint, a hint-specific table
+    // list called curr_order_list is created. After adding all the tables in
+    // the hint, the curr_order_list is merged with the combined order_list to
+    // create a new order_list which reflects the orders of all the JOIN_ORDER
+    // hints processed so far, including the current hint.
+    if (Overlaps(order_table_map, curr_order_table_map)) {
+      // multiple JOIN_ORDER hints may contain one or more
+      // common tables.
+      for (auto it = order_list.rbegin(); it != order_list.rend(); ++it) {
+        if (conflicting_hint) break;
+        Table_ref *tl = *it;
+        if (!Overlaps(get_table_map(tl), curr_order_table_map))
+          merged_order_list.push_front(tl);
+        else {
+          for (; curr_it != curr_order_list.rend(); ++curr_it) {
+            Table_ref *curr_tl = *curr_it;
+            if (Overlaps(get_table_map(curr_tl), order_table_map) &&
+                !Overlaps(get_table_map(curr_tl), get_table_map(tl))) {
+              // same tables as previous hint, but in different order
+              // ignore this hint
+              merged_order_list = order_list;
+              conflicting_hint = true;
+              break;
+            }
+            merged_order_list.push_front(curr_tl);
+            if (Overlaps(get_table_map(curr_tl), get_table_map(tl))) {
+              ++curr_it;
+              break;
+            }
+          }
+        }
+      }
+      if (conflicting_hint) {
+        join_order_hints_ignored |= 1ULL << hint_idx;
+        continue;
+      }
+    } else {
+      // No overlapping tables
+      merged_order_list = order_list;
+    }
+    // Append the current order list to the merged list.
+    for (; curr_it != curr_order_list.rend(); ++curr_it) {
+      Table_ref *curr_tl = *curr_it;
+      merged_order_list.push_front(curr_tl);
+    }
+    order_list = merged_order_list;
+    order_table_map |= curr_order_table_map;
+  }
+
+  // Now that the lists are in place, the new join order is constructed
+  // 1. Add all prefix tables.
+  // 2. Add all non-prefix and non-suffix tables while handling relative
+  //    table orders specified in order_list.
+  // 3. Add all suffix tables.
+  for (auto it = prefix_list.rbegin(); it != prefix_list.rend(); ++it) {
+    Table_ref *tl = *it;
+    new_join_list->push_front(tl);
+  }
+  table_map exclude_table_map = prefix_table_map | suffix_table_map;
+  for (auto it = join_list->rbegin(); it != join_list->rend(); ++it) {
+    Table_ref *tl = *it;
+    add_table_to_list(tl, exclude_table_map, order_table_map, &order_list,
+                      new_join_list);
+  }
+  for (auto it = suffix_list.rbegin(); it != suffix_list.rend(); ++it) {
+    Table_ref *tl = *it;
+    new_join_list->push_front(tl);
+  }
+  table_map prior_map = 0;
+  for (auto it = new_join_list->rbegin(); it != new_join_list->rend(); ++it) {
+    Table_ref *tl = *it;
+    if (tl->straight || tl->outer_join) {
+      if (!IsSubset(tl->dep_tables, prior_map)) {
+        // hinted order conflicts with OUTER JOIN directive, silently ignore
+        // hints by reverting to original join order.
+        new_join_list->clear();
+        new_join_list = const_cast<mem_root_deque<Table_ref *> *>(join_list);
+        return new_join_list;
+      }
+    } else if (tl->is_sj_or_aj_nest()) {
+      table_map outer_table_map = tl->nested_join->sj_depends_on;
+      if (!IsSubset(outer_table_map, prior_map)) {
+        new_join_list->clear();
+        new_join_list = const_cast<mem_root_deque<Table_ref *> *>(join_list);
+        push_warning_printf(thd, Sql_condition::SL_WARNING,
+                            ER_WARN_UNSUPPORTED_HINT,
+                            ER_THD(thd, ER_WARN_UNSUPPORTED_HINT),
+                            "hypergraph: semijoin/antijoin table precedes a"
+                            " table which it depends on");
+        return new_join_list;
+      }
+    }
+    prior_map |= get_table_map(tl);
+  }
+  return new_join_list;
+}
+
+/**
+  Checks if a combination of left and right RelationalExpressions satisfy
+  one or more join order hints. If the proposed join does not satisfy a hint,
+  the remaining hints are ignored.
+
+  @param left          Left side of proposed join
+  @param right         Right side of proposed join
+  @param join_list     Deque of tables in join
+
+  @return true if at least one hint matches, false otherwise
+*/
+bool Opt_hints_qb::check_join_order_hints(
+    RelationalExpression *left, RelationalExpression *right,
+    const mem_root_deque<Table_ref *> *join_list) {
+  if (!has_join_order_hints()) return true;
+  if (right->type != RelationalExpression::TABLE)
+    return false;  // right-deep or bushy plan
+
+  uint matched_hints = 0;
+  for (uint hint_idx = 0; hint_idx < join_order_hints.size(); hint_idx++) {
+    if (join_order_hints_ignored & (1ULL << hint_idx)) continue;
+    PT_qb_level_hint *hint = join_order_hints[hint_idx];
+    if (hinted_join_order(hint, left, right, join_list)) matched_hints++;
+  }
+  return matched_hints > 0;
+}
+
+/**
+  Checks if a combination of left and right RelationalExpressions satisfy
+  a hint.
+
+  @param hint          Hint info
+  @param left          Left side of proposed join
+  @param right         Right side of proposed join
+  @param join_list     Deque of tables in join
+
+  @return true if at least one hint matches, false otherwise
+*/
+bool Opt_hints_qb::hinted_join_order(
+    PT_qb_level_hint *hint, RelationalExpression *left,
+    RelationalExpression *right, const mem_root_deque<Table_ref *> *join_list) {
+  if (right->type != RelationalExpression::TABLE)
+    return false;  // right-deep or bushy plan
+
+  Hint_param_table_list *hint_table_list = hint->get_table_list();
+  if (hint_table_list->size() == 0) return false;
+
+  // Special case where both 'left' and 'right' are TABLEs
+  if (left->type == RelationalExpression::TABLE) {
+    uint hint_tbl_idx = 0;
+    const bool left_matched =
+        hint_table_list->size() > hint_tbl_idx &&
+        !compare_table_name(&(*hint_table_list)[hint_tbl_idx], left->table);
+    hint_tbl_idx += (left_matched == true);
+    const bool right_matched =
+        hint_table_list->size() > hint_tbl_idx &&
+        !compare_table_name(&(*hint_table_list)[hint_tbl_idx], right->table);
+    switch (hint->type()) {
+      case JOIN_PREFIX_HINT_ENUM:
+        return left_matched && (right_matched || hint_table_list->size() == 1);
+      case JOIN_SUFFIX_HINT_ENUM:
+        return right_matched;
+      case JOIN_ORDER_HINT_ENUM:
+        return left_matched && right_matched;
+      default:
+        assert(false);
+        break;
+    }
+    return false;
+  }
+  table_map hint_tab_map = 0;
+  uint last_hint_tab_in_left = 0;
+  for (uint hint_tbl_idx = 0; hint_tbl_idx < hint_table_list->size();
+       hint_tbl_idx++) {
+    const Hint_param_table &hint_table = (*hint_table_list)[hint_tbl_idx];
+    for (auto it = join_list->rbegin(); it != join_list->rend();
+         ++it) {  // The list goes backwards.
+      const Table_ref *tl = *it;
+      Table_ref *match = find_hinted_table(&hint_table, tl);
+      if (match != nullptr) {
+        table_map match_map = get_table_map(tl);
+        hint_tab_map |= match_map;
+        if (Overlaps(left->tables_in_subtree, match_map))
+          last_hint_tab_in_left = hint_tbl_idx;
+        break;
+      }
+    }
+  }
+  bool left_hinted =
+      hinted_join_order(hint, left->left, left->right, join_list);
+  switch (hint->type()) {
+    case JOIN_PREFIX_HINT_ENUM: {
+      if (!left_hinted) return false;
+      if (IsSubset(hint_tab_map, left->tables_in_subtree)) {
+        // Hint matches if prefix fully contained in left expr. Non-prefix
+        // tables can be present after prefix tables.
+        // E.g. left = {a, b, c, d}, prefix = {a, b, c}
+        return true;
+      } else if (Overlaps(left->tables_in_subtree, hint_tab_map) &&
+                 Overlaps(right->table->map(), hint_tab_map)) {
+        if (!left_hinted) return false;
+        // hint split across left and right expressions
+        uint next_prefix_tab = last_hint_tab_in_left + 1;
+        if (next_prefix_tab >= hint_table_list->size()) {
+          return false;
+        }
+        if (!compare_table_name(&((*hint_table_list)[next_prefix_tab]),
+                                right->table)) {
+          return true;
+        }
+      }
+      return false;
+      break;
+    }
+    case JOIN_SUFFIX_HINT_ENUM: {
+      if (!left_hinted) {
+        if (!compare_table_name(&((*hint_table_list)[0]), right->table)) {
+          // right expr contains first suffix table
+          return true;
+        } else if (!Overlaps(right->table->map(), hint_tab_map)) {
+          // Hint broken if right table not present
+          return false;
+        }
+      } else if (Overlaps(left->tables_in_subtree, hint_tab_map) &&
+                 Overlaps(right->table->map(), hint_tab_map)) {
+        // hint split across left and right expressions
+        uint next_suffix_tab = last_hint_tab_in_left + 1;
+        if (next_suffix_tab >= hint_table_list->size()) {
+          return false;
+        }
+        if (!compare_table_name(&((*hint_table_list)[next_suffix_tab]),
+                                right->table)) {
+          return true;
+        }
+      }
+      return false;
+      break;
+    }
+    case JOIN_ORDER_HINT_ENUM: {
+      if (!Overlaps(left->tables_in_subtree, hint_tab_map) &&
+          !Overlaps(right->table->map(), hint_tab_map)) {
+        // JOIN_ORDER tables not present in either left or right
+        return false;
+      } else if (left_hinted &&
+                 IsSubset(hint_tab_map, left->tables_in_subtree)) {
+        // All join order tables present in order in left expr, right expr is
+        // not part of hint
+        return false;
+      } else if (Overlaps(left->tables_in_subtree, hint_tab_map) &&
+                 Overlaps(right->table->map(), hint_tab_map)) {
+        // hint split across left and right expressions - get all tables which
+        // should precede right->table in hinted join order
+        table_map prior_map = 0;
+        for (uint hint_tbl_idx = 0; hint_tbl_idx <= last_hint_tab_in_left;
+             hint_tbl_idx++) {
+          const Hint_param_table &hint_table = (*hint_table_list)[hint_tbl_idx];
+          for (auto it = join_list->rbegin(); it != join_list->rend();
+               ++it) {  // The list goes backwards.
+            const Table_ref *tl = *it;
+            if (find_hinted_table(&hint_table, tl)) {
+              prior_map |= get_table_map(tl);
+              break;
+            }
+          }
+        }
+        // check that right expr does not contain a table which should
+        // precede any of the left expr tables in the hinted order
+        if (Overlaps(right->table->map(), prior_map)) return false;
+        return true;
+      }
+      break;
+    }
+    default:
+      break;
+  }
+  return false;
+}
+
 void Opt_hints_qb::apply_join_order_hints(JOIN *join) {
   for (uint hint_idx = 0; hint_idx < join_order_hints.size(); hint_idx++) {
     PT_qb_level_hint *hint = join_order_hints[hint_idx];
@@ -895,7 +1443,7 @@ bool hint_key_state(const THD *thd, const Table_ref *table, uint keyno,
   if (table_hints && keyno != MAX_KEY) {
     Opt_hints_key *key_hints = table_hints->keyinfo_array.size() > 0
                                    ? table_hints->keyinfo_array[keyno]
-                                   : NULL;
+                                   : nullptr;
     bool ret_val = false;
     if (get_hint_state(key_hints, table_hints, type_arg, &ret_val))
       return ret_val;
