@@ -73,6 +73,7 @@
 #include "sql/item_func.h"
 #include "sql/item_sum.h"  // Item_sum
 #include "sql/key.h"       // key_cmp_if_same
+#include "sql/range_optimizer/range_optimizer.h"
 #include "sql/sql_bitmap.h"
 #include "sql/sql_class.h"
 #include "sql/sql_const.h"
@@ -104,7 +105,7 @@ static ulonglong get_exact_record_count(Table_ref *tables) {
   ulonglong count = 1;
   for (Table_ref *tl = tables; tl; tl = tl->next_leaf) {
     ha_rows tmp = 0;
-    int error = tl->table->file->ha_records(&tmp);
+    const int error = tl->table->file->ha_records(&tmp);
     if (error != 0) return ULLONG_MAX;
     count *= tmp;
   }
@@ -349,7 +350,7 @@ bool optimize_aggregated_query(THD *thd, Query_block *select,
       using stats.records.
     */
     tables_filled &= !(tl->schema_table || tl->uses_materialization());
-    ulonglong table_flags = tl->table->file->ha_table_flags();
+    const ulonglong table_flags = tl->table->file->ha_table_flags();
     if ((table_flags & HA_STATS_RECORDS_IS_EXACT) && tables_filled &&
         !tl->table->force_index) {
       error = tl->fetch_number_of_rows();
@@ -471,7 +472,7 @@ bool optimize_aggregated_query(THD *thd, Query_block *select,
 
             ref.key_buff = key_buff;
             Item_field *item_field = down_cast<Item_field *>(expr);
-            Table_ref *tr = item_field->table_ref;
+            Table_ref *tr = item_field->m_table_ref;
             TABLE *table = tr->table;
             /*
               If table already has been read, we may use the values already
@@ -637,9 +638,9 @@ bool is_simple_predicate(Item_func *func_item, Item **args, bool *inv_order) {
   *inv_order = false;
   switch (func_item->argument_count()) {
     case 0:
-      /* MULT_EQUAL_FUNC */
+      /* MULTI_EQ_FUNC */
       {
-        Item_equal *item_equal = down_cast<Item_equal *>(func_item);
+        Item_multi_eq *item_equal = down_cast<Item_multi_eq *>(func_item);
         args[0] = item_equal->get_first();
         if (item_equal->members() > 1) return false;
         if (!(args[1] = item_equal->const_arg())) return false;
@@ -682,6 +683,50 @@ bool is_simple_predicate(Item_func *func_item, Item **args, bool *inv_order) {
   return true;
 }
 
+/**
+  Check if a predicate should make the endpoint for the MIN/MAX optimization
+  move towards -Inf for MAX or towards +Inf for MIN.
+
+  It is assumed that the current endpoint is stored in the field accessed by
+  "cond", and that "cond" represents a comparison of a field and constant values
+  using the <, >, <=, >= or BETWEEN operator.
+*/
+static bool move_endpoint(bool is_max, Item *cond) {
+  const Item_func::Functype type = down_cast<Item_func *>(cond)->functype();
+  assert(is_function_of_type(cond, Item_func::LT_FUNC) ||
+         is_function_of_type(cond, Item_func::LE_FUNC) ||
+         is_function_of_type(cond, Item_func::GT_FUNC) ||
+         is_function_of_type(cond, Item_func::GE_FUNC) ||
+         is_function_of_type(cond, Item_func::BETWEEN));
+
+  if (type != Item_func::BETWEEN) {
+    // For <, <=, >, >= operators, just check if "cond" evaluates to true on the
+    // current endpoint. If it does not, it represents a stricter condition than
+    // those seen before, so the endpoint should be moved.
+    return cond->val_int() == 0;
+  }
+
+  // For BETWEEN, we check similarly, but only using the upper bound constant
+  // for MAX and the lower bound constant for MIN.
+  Item_func_between *const between = down_cast<Item_func_between *>(cond);
+  Item_func_comparison *compare_endpoints = nullptr;
+  if (is_max) {
+    compare_endpoints =
+        new Item_func_le(between->get_arg(0), between->get_arg(2));
+  } else {
+    compare_endpoints =
+        new Item_func_ge(between->get_arg(0), between->get_arg(1));
+  }
+
+  if (compare_endpoints == nullptr) return false;
+
+  if (compare_endpoints->set_cmp_func()) return true;
+  compare_endpoints->update_used_tables();
+  compare_endpoints->quick_fix_field();
+
+  return compare_endpoints->val_int() == 0;
+}
+
 /**
   Check whether a condition matches a key to get {MAX|MIN}(field):.
 
@@ -773,7 +818,8 @@ static bool matching_cond(bool max_fl, Index_lookup *ref, KEY *keyinfo,
   bool is_null = false;          // IS NULL
   bool between = false;          // BETWEEN ... AND ...
 
-  switch (((Item_func *)cond)->functype()) {
+  const Item_func::Functype functype = down_cast<Item_func *>(cond)->functype();
+  switch (functype) {
     case Item_func::ISNULL_FUNC:
       is_null = true;
       [[fallthrough]];
@@ -801,20 +847,29 @@ static bool matching_cond(bool max_fl, Index_lookup *ref, KEY *keyinfo,
       if (((Item_func_between *)cond)->negated) return false;
 
       break;
-    case Item_func::MULT_EQUAL_FUNC:
+    case Item_func::MULTI_EQ_FUNC:
       eq_type = true;
       break;
     default:
       return false;  // Can't optimize function
   }
 
-  Item *args[3];
+  Item *args[] = {nullptr, nullptr, nullptr};
   bool inv;
 
   /* Test if this is a comparison of a field and constant */
   if (!is_simple_predicate(down_cast<Item_func *>(cond), args, &inv))
     return false;
 
+  // Test if the field and the constant values can be compared in an index.
+  Field *const field = down_cast<Item_field *>(args[0])->field;
+  for (Item *value : {args[1], args[2]}) {
+    if (value != nullptr &&
+        !comparable_in_index(cond, field, Field::itRAW, functype, value)) {
+      return false;
+    }
+  }
+
   if (!is_null_safe_eq && !is_null &&
       (args[1]->is_null() || (between && args[2]->is_null())))
     return false;
@@ -828,16 +883,17 @@ static bool matching_cond(bool max_fl, Index_lookup *ref, KEY *keyinfo,
 
   {
     if (part > field_part) return false;  // Field is beyond the tested parts
-    if (part->field->eq(((Item_field *)args[0])->field))
+    if (part->field->eq(field)) {
       break;  // Found a part of the key for the field
+    }
   }
 
-  bool is_field_part = part == field_part;
+  const bool is_field_part = part == field_part;
   if (!(is_field_part || eq_type)) return false;
 
-  key_part_map org_key_part_used = *key_part_used;
+  const key_part_map org_key_part_used = *key_part_used;
   if (eq_type || between || max_fl == less_fl) {
-    size_t length = (key_ptr - ref->key_buff) + part->store_length;
+    const size_t length = (key_ptr - ref->key_buff) + part->store_length;
     if (ref->key_length < length) {
       /* Ultimately ref->key_length will contain the length of the search key */
       ref->key_length = length;
@@ -862,7 +918,7 @@ static bool matching_cond(bool max_fl, Index_lookup *ref, KEY *keyinfo,
 
   if (org_key_part_used != *key_part_used ||
       (is_field_part && (between || eq_type || max_fl == less_fl) &&
-       !cond->val_int())) {
+       move_endpoint(max_fl, cond))) {
     /*
       It's the first predicate for this part or a predicate of the
       following form  that moves upper/lower bounds for max/min values:
@@ -896,7 +952,7 @@ static bool matching_cond(bool max_fl, Index_lookup *ref, KEY *keyinfo,
         check for TYPE_OK here. analyze_field_constant() does not yet handle
         string constants.
       */
-      type_conversion_status retval =
+      const type_conversion_status retval =
           value->save_in_field_no_warnings(part->field, true);
       if (!(retval == TYPE_OK ||
             (retval == TYPE_NOTE_TRUNCATED && part->field->is_text_key_type() &&
@@ -909,7 +965,7 @@ static bool matching_cond(bool max_fl, Index_lookup *ref, KEY *keyinfo,
     }
     if (is_field_part) {
       if (between || eq_type)
-        *range_fl &= ~(NO_MAX_RANGE | NO_MIN_RANGE);
+        *range_fl &= ~(NO_MAX_RANGE | NO_MIN_RANGE | NEAR_MAX | NEAR_MIN);
       else {
         *range_fl &= ~(max_fl ? NO_MAX_RANGE : NO_MIN_RANGE);
         if (noeq_type)
@@ -1011,7 +1067,7 @@ static bool find_key_for_maxmin(bool max_fl, Index_lookup *ref,
         key_part_map key_part_used = 0;
         *range_fl = NO_MIN_RANGE | NO_MAX_RANGE;
         if (matching_cond(max_fl, ref, keyinfo, part, cond,
-                          item_field->table_ref->map(), &key_part_used,
+                          item_field->m_table_ref->map(), &key_part_used,
                           range_fl, prefix_len) &&
             !(key_part_to_use & ~key_part_used)) {
           if (!max_fl && key_part_used == key_part_to_use && part->null_bit) {
@@ -1100,7 +1156,9 @@ static bool maxmin_in_range(bool max_fl, Item_field *item_field, Item *cond) {
   }
 
   // Check that condition actually references the specific table
-  if ((cond->used_tables() & item_field->table_ref->map()) == 0) return false;
+  if ((cond->used_tables() & item_field->m_table_ref->map()) == 0) {
+    return false;
+  }
   bool less_fl = false;
   switch (down_cast<Item_func *>(cond)->functype()) {
     case Item_func::BETWEEN:
@@ -1126,7 +1184,7 @@ static bool maxmin_in_range(bool max_fl, Item_field *item_field, Item *cond) {
     }
     case Item_func::EQ_FUNC:
     case Item_func::EQUAL_FUNC:
-    case Item_func::MULT_EQUAL_FUNC:
+    case Item_func::MULTI_EQ_FUNC:
     case Item_func::ISNULL_FUNC:
       break;
     default:          // Keep compiler happy
