@@ -1,15 +1,16 @@
-/* Copyright (c) 2011, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2011, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -45,6 +46,8 @@
 #include "sql/enum_query_type.h"
 #include "sql/handler.h"
 #include "sql/item.h"  // Item
+#include "sql/join_optimizer/optimizer_trace.h"
+#include "sql/sql_class.h"
 #include "sql/table.h"
 #include "sql_string.h"  // String
 #include "string_with_len.h"
@@ -127,6 +130,10 @@ class Opt_trace_stmt {
   /// Sets the quantity of allowed memory for this trace.
   void set_allowed_mem_size(size_t size);
 
+  void set_missing_unstructured_bytes(size_t num_bytes) {
+    m_missing_unstructured_bytes = num_bytes;
+  }
+
   /// @sa Opt_trace_context::set_query()
   void set_query(const char *query, size_t length, const CHARSET_INFO *charset);
 
@@ -212,6 +219,9 @@ class Opt_trace_stmt {
   */
   const char *make_unknown_key();
 
+  /// The current indentation level (when printing the JSON tree).
+  size_t indentation() const { return stack_of_current_structs.size() * 2; }
+
  private:
   bool ended;  ///< Whether @c end() has been called on this instance
 
@@ -240,6 +250,9 @@ class Opt_trace_stmt {
   Buffer trace_buffer;  ///< Where the trace is accumulated
   Buffer query_buffer;  ///< Where the original query is put
 
+  /// Trace bytes that were discarded by UnstructuredTrace because they
+  /// would exceed optimizer_trace_max_mem_size.
+  size_t m_missing_unstructured_bytes{0};
   /**
     Counter which serves to have unique autogenerated keys, needed if we
     autogenerate more than one key in a single object.
@@ -263,6 +276,60 @@ inline char closing_bracket(bool requires_key) {
 }
 }  // namespace
 
+/// Add an escape sequence to 'buffer' if 'c' is a character that should be
+/// escaped in optimizer trace.
+/// @param c The character to escape.
+/// @param buffer The buffer in which to put the escape sequence.
+/// @returns The next free position in in 'buffer'.
+static char *Escape(char c, char *buffer) {
+  char *pbuf{buffer};
+  char esc;
+  switch (c) {
+      // Don't use \u when possible for common chars, \ is easier to read:
+    case '\\':
+      esc = '\\';
+      break;
+    case '"':
+      esc = '\"';
+      break;
+    case '\n':
+      esc = 'n';
+      break;
+    case '\r':
+      esc = 'r';
+      break;
+    case '\t':
+      esc = 't';
+      break;
+    default:
+      esc = 0;
+      break;
+  }
+  if (esc != 0)  // Escaping with backslash.
+  {
+    *pbuf++ = '\\';
+    *pbuf++ = esc;
+  } else {
+    uint ascii_code = static_cast<uint>(c);
+    if (ascii_code < 32)  // Escaping with \u
+    {
+      *pbuf++ = '\\';
+      *pbuf++ = 'u';
+      *pbuf++ = '0';
+      *pbuf++ = '0';
+      if (ascii_code < 16) {
+        *pbuf++ = '0';
+      } else {
+        *pbuf++ = '1';
+        ascii_code -= 16;
+      }
+      *pbuf++ = dig_vec_lower[ascii_code];
+    } else
+      *pbuf++ = c;  // Normal character, no escaping needed.
+  }
+  return pbuf;
+}
+
 void Opt_trace_struct::do_construct(Opt_trace_context *ctx,
                                     bool requires_key_arg, const char *key,
                                     Opt_trace_context::feature_value feature) {
@@ -559,7 +626,7 @@ void Opt_trace_stmt::next_line() {
   if (ctx->get_one_line()) return;
   trace_buffer.append('\n');
 
-  size_t to_be_printed = 2 * stack_of_current_structs.size();
+  size_t to_be_printed = indentation();
   const size_t spaces_len = sizeof(my_spaces) - 1;
   while (to_be_printed > spaces_len) {
     trace_buffer.append(my_spaces, spaces_len);
@@ -609,8 +676,12 @@ void Opt_trace_stmt::fill_info(Opt_trace_info *info) const {
     info->query_ptr = query_buffer.ptr();
     info->query_length = query_buffer.length();
     info->query_charset = query_buffer.charset();
+
     info->missing_bytes =
-        trace_buffer.get_missing_bytes() + query_buffer.get_missing_bytes();
+        trace_buffer.get_missing_bytes() + query_buffer.get_missing_bytes() +
+        // Multiply with a fudge factor to compensate for indentation,
+        // adding quotes etc..
+        static_cast<size_t>(1.08 * m_missing_unstructured_bytes);
   }
 }
 
@@ -637,67 +708,7 @@ void Buffer::append_escaped(const char *str, size_t length) {
   char buf[128];  // Temporary output buffer.
   char *pbuf = buf;
   for (pstr = str, pstr_end = (str + length); pstr < pstr_end; pstr++) {
-    char esc;
-    const char c = *pstr;
-    /*
-      JSON syntax says that control characters must be escaped. Experience
-      confirms that this means ASCII 0->31 and " and \ . A few of
-      them are accepted with a short escaping syntax (using \ : like \n)
-      but for most of them, only \uXXXX works, where XXXX is a
-      hexadecimal value for the code point.
-      Rules also mention escaping / , but Python's and Perl's json modules
-      do not require it, and somewhere on Internet someone said JSON
-      allows escaping of / but does not require it.
-
-      Because UTF8 has the same characters in range 0-127 as ASCII does, and
-      other UTF8 characters don't contain 0-127 bytes, if we see a byte
-      equal to 0 it is really the UTF8 u0000 character (a.k.a. ASCII NUL)
-      and not a part of a longer character; if we see a newline, same,
-      etc. That wouldn't necessarily be true with another character set.
-    */
-    switch (c) {
-        // Don't use \u when possible for common chars, \ is easier to read:
-      case '\\':
-        esc = '\\';
-        break;
-      case '"':
-        esc = '\"';
-        break;
-      case '\n':
-        esc = 'n';
-        break;
-      case '\r':
-        esc = 'r';
-        break;
-      case '\t':
-        esc = 't';
-        break;
-      default:
-        esc = 0;
-        break;
-    }
-    if (esc != 0)  // Escaping with backslash.
-    {
-      *pbuf++ = '\\';
-      *pbuf++ = esc;
-    } else {
-      uint ascii_code = (uint)c;
-      if (ascii_code < 32)  // Escaping with \u
-      {
-        *pbuf++ = '\\';
-        *pbuf++ = 'u';
-        *pbuf++ = '0';
-        *pbuf++ = '0';
-        if (ascii_code < 16) {
-          *pbuf++ = '0';
-        } else {
-          *pbuf++ = '1';
-          ascii_code -= 16;
-        }
-        *pbuf++ = dig_vec_lower[ascii_code];
-      } else
-        *pbuf++ = c;  // Normal character, no escaping needed.
-    }
+    pbuf = Escape(*pstr, pbuf);
     /*
       To fit a next character, we need at most 6 bytes (happens when using
       \uXXXX syntax) before the buffer's end:
@@ -1115,6 +1126,32 @@ void Opt_trace_context::Opt_trace_context_impl::restore_I_S() {
   if (current_stmt_in_gen != nullptr) current_stmt_in_gen->restore_I_S();
 }
 
+/// Move unstructured trace text (as used by Hypergraph) into the JSON tree.
+void Opt_trace_context::ConsumeUnstructuredTrace(THD *thd) {
+  assert(thd->opt_trace.is_started());
+  const Opt_trace_object trace_wrapper{this};
+  Opt_trace_array join_optimizer{this, "join_optimizer"};
+  Mem_root_array<char> line{thd->mem_root};
+
+  unstructured_trace()->contents().Consume([&](char ch) {
+    if (ch == '\n') {
+      join_optimizer.add_utf8(line.data(), line.size());
+      line.clear();
+    } else {
+      line.push_back(ch);
+    }
+  });
+
+  // The last line should also be terminated by '\n'.
+  if (!line.empty()) {
+    line.push_back('\n');
+    join_optimizer.add_utf8(line.data(), line.size());
+  }
+
+  pimpl->current_stmt_in_gen->set_missing_unstructured_bytes(
+      unstructured_trace()->contents().excess_bytes());
+}
+
 void Opt_trace_context::missing_privilege() {
   /*
     By storing the 'missing_priv' mark in Opt_trace_stmt instead of in
