@@ -36,13 +36,13 @@
 #include <cstring>
 
 #include "lex_string.h"
-#include "m_ctype.h"
-#include "m_string.h"
 #include "my_dbug.h"
 #include "my_thread_local.h"
 #include "mysql/psi/mysql_file.h"
+#include "mysql/strings/m_ctype.h"
 #include "mysql/udf_registration_types.h"
 #include "mysql_com.h"
+#include "nulls.h"
 #include "sql/derror.h"  // ER_THD
 #include "sql/item.h"
 #include "sql/item_func.h"
@@ -57,6 +57,8 @@
 #include "sql/system_variables.h"
 #include "sql/visible_fields.h"
 #include "sql_string.h"
+#include "strmake.h"
+#include "strxnmov.h"
 #include "template_utils.h"  // pointer_cast
 
 using std::min;
@@ -205,7 +207,7 @@ void Query_result_to_file::cleanup() {
 static File create_file(THD *thd, char *path, sql_exchange *exchange,
                         IO_CACHE *cache) {
   File file;
-  uint option = MY_UNPACK_FILENAME | MY_RELATIVE_PATH;
+  const uint option = MY_UNPACK_FILENAME | MY_RELATIVE_PATH;
 
   if (!dirname_length(exchange->file_name)) {
     strxnmov(path, FN_REFLEN - 1, mysql_real_data_home,
@@ -367,6 +369,10 @@ bool Query_result_export::send_data(THD *thd,
     goto err;
   for (Item *item : VisibleFields(items)) {
     Item_result result_type = item->result_type();
+    if (item->actual_data_type() == MYSQL_TYPE_BIT) {
+      /* BIT is output as binary string */
+      result_type = STRING_RESULT;
+    }
     bool enclosed =
         (exchange->field.enclosed->length() &&
          (!exchange->field.opt_enclosed || result_type == STRING_RESULT));
