@@ -30,11 +30,11 @@
 #include <algorithm>
 #include <limits>
 
-#include "m_ctype.h"
 #include "mem_root_deque.h"
 #include "my_bitmap.h"
 #include "my_compiler.h"
 #include "my_dbug.h"
+#include "mysql/strings/m_ctype.h"
 #include "mysql/udf_registration_types.h"
 #include "sql/field.h"
 #include "sql/handler.h"
@@ -43,6 +43,7 @@
 #include "sql/item_func.h"
 #include "sql/item_sum.h"
 #include "sql/join_optimizer/access_path.h"
+#include "sql/join_optimizer/bit_utils.h"
 #include "sql/key.h"
 #include "sql/key_spec.h"
 #include "sql/opt_costmodel.h"
@@ -92,7 +93,7 @@ void trace_basic_info_group_index_skip_scan(THD *thd, const AccessPath *path,
       .add("max_aggregate", !param->max_functions.empty())
       .add("distinct_aggregate", param->have_agg_distinct)
       .add("rows", path->num_output_rows())
-      .add("cost", path->cost);
+      .add("cost", path->cost());
 
   const KEY_PART_INFO *key_part = param->index_info->key_part;
   Opt_trace_context *const trace = &thd->opt_trace;
@@ -107,11 +108,11 @@ void trace_basic_info_group_index_skip_scan(THD *thd, const AccessPath *path,
   Opt_trace_array trace_range(trace, "ranges");
 
   // can have group quick without ranges
-  if (param->index_tree_tracing_only) {
+  if (param->index_tree) {
     String range_info;
     range_info.set_charset(system_charset_info);
     append_range_all_keyparts(&trace_range, nullptr, &range_info,
-                              param->index_tree_tracing_only, key_part, false);
+                              param->index_tree, key_part, false);
   }
 }
 
@@ -133,12 +134,26 @@ static bool min_max_inspect_cond_for_fields(Item *cond,
                                             bool *min_max_arg_present,
                                             bool *non_min_max_arg_present);
 
-static void cost_group_min_max(TABLE *table, uint key, uint used_key_parts,
-                               uint group_key_parts, SEL_TREE *range_tree,
-                               ha_rows quick_prefix_records, bool have_min,
-                               bool have_max, uint infix_factor,
-                               Cost_estimate *cost_est, ha_rows *records,
-                               bool single_group);
+static void cost_group_skip_scan(TABLE *table, uint key, uint used_key_parts,
+                                 uint group_key_parts, SEL_TREE *range_tree,
+                                 ha_rows quick_prefix_records, bool have_min,
+                                 bool have_max, uint infix_factor,
+                                 Cost_estimate *cost_est, ha_rows *records,
+                                 bool single_group);
+
+void collect_group_skip_scans(
+    THD *thd, RANGE_OPT_PARAM *param, SEL_TREE *tree,
+    enum_order order_direction, bool skip_records_in_range,
+    Mem_root_array<GroupIndexSkipScanInfo *> *possible_group_skip_scans,
+    bool &have_min, bool &have_max);
+
+GroupIndexSkipScanInfo *select_best_group_skip_scan(
+    Mem_root_array<GroupIndexSkipScanInfo *> *possible_group_skip_scans);
+
+AccessPath *make_group_skip_scan_path(
+    THD *thd, RANGE_OPT_PARAM *param, SEL_TREE *tree,
+    GroupIndexSkipScanInfo *group_skip_scan_info, double cost_est,
+    bool have_min, bool have_max);
 
 /**
   Test if this access method is applicable to a GROUP query with MIN/MAX
@@ -274,47 +289,79 @@ static void cost_group_min_max(TABLE *table, uint key, uint used_key_parts,
  @param  thd       Thread handle
  @param  param     Parameter from test_quick_select
  @param  tree      Range tree generated by get_mm_tree
- @param  cost_est  Best cost so far (=table/index scan time)
  @param  order_direction The sort order the range access method must be able
                    to provide. Three-value logic: asc/desc/don't care
  @param  skip_records_in_range  Same value as JOIN_TAB::skip_records_in_range().
- @return table read plan
-   @retval NULL  Loose index scan not applicable or mem_root == NULL
-   @retval !NULL Loose index scan table read plan
-*/
+ @param  cost_est  Best cost so far (=table/index scan time)
 
-AccessPath *get_best_group_min_max(THD *thd, RANGE_OPT_PARAM *param,
-                                   SEL_TREE *tree, enum_order order_direction,
-                                   bool skip_records_in_range,
-                                   double cost_est) {
-  JOIN *join = param->query_block->join;
-  TABLE *table = param->table;
+ @return group_skip_scan_path access path for best group skip scan
+   @retval NULL  Group index skip scan not applicable or mem_root == NULL
+   @retval !NULL Group index skip scan table read plan
+*/
+AccessPath *get_best_group_skip_scan(THD *thd, RANGE_OPT_PARAM *param,
+                                     SEL_TREE *tree, enum_order order_direction,
+                                     bool skip_records_in_range,
+                                     double cost_est) {
   bool have_min = false; /* true if there is a MIN function. */
   bool have_max = false; /* true if there is a MAX function. */
+
+  // collect all candidate group skip_scans
+  Mem_root_array<GroupIndexSkipScanInfo *> possible_group_skip_scans(
+      param->return_mem_root);
+  collect_group_skip_scans(thd, param, tree, order_direction,
+                           skip_records_in_range, &possible_group_skip_scans,
+                           have_min, have_max);
+  if (possible_group_skip_scans.size() == 0) return nullptr;
+
+  // select group skip scan with lowest 'cost' value
+  GroupIndexSkipScanInfo *best_group_skip_scan =
+      select_best_group_skip_scan(&possible_group_skip_scans);
+  // build access path for best group skip scan
+  AccessPath *group_skip_scan_path = make_group_skip_scan_path(
+      thd, param, tree, best_group_skip_scan, cost_est, have_min, have_max);
+  return group_skip_scan_path;
+}
+
+/**
+  Analyze indexes to see if a group index skip scan is possible and save
+  GroupIndexSkipScanInfo data for every possible group index skip scan. Return
+  a list of GroupIndexSkipScanInfo* objects which can be used to build a
+  GROUP_INDEX_SKIP_SCAN AccessPath.
+
+  @param  thd       Thread info
+  @param  param     Range optimizer parameter
+  @param  tree      Range tree generated by get_mm_tree
+  @param  order_direction The sort order the range access method must be able
+                    to provide. Three-value logic: asc/desc/don't care
+  @param  skip_records_in_range Same value as JOIN_TAB::skip_records_in_range()
+  @param  possible_group_skip_scans List to populate with candidate group skip
+                    scans
+  @param  have_min  Set to true if MIN() function is present
+  @param  have_max  Set to true if MAX() function is present
+*/
+void collect_group_skip_scans(
+    THD *thd, RANGE_OPT_PARAM *param, SEL_TREE *tree,
+    enum_order order_direction, bool skip_records_in_range,
+    Mem_root_array<GroupIndexSkipScanInfo *> *possible_group_skip_scans,
+    bool &have_min, bool &have_max) {
+  JOIN *join = param->query_block->join;
+  TABLE *table = param->table;
+
+  if (Overlaps(table->file->ha_table_flags(), HA_NO_INDEX_ACCESS)) return;
+
   Item_field *min_max_arg_item =
-      nullptr;  // The argument of all MIN/MAX functions
-  KEY_PART_INFO *min_max_arg_part = nullptr; /* The corresponding keypart. */
-  uint group_prefix_len = 0; /* Length (in bytes) of the key prefix. */
-  KEY *index_info = nullptr; /* The index chosen for data access. */
-  uint index = 0;            /* The id of the chosen index. */
-  uint group_key_parts = 0;  // Number of index key parts in the group prefix.
-  uint used_key_parts = 0;   /* Number of index key parts used for access. */
-  uint key_infix_len = 0;    /* Length of key_infix. */
+      nullptr; /* The argument of all MIN/MAX functions */
   uint key_part_nr;
   ORDER *tmp_group;
   Item_field *item_field;
   bool is_agg_distinct;
-  /* Cost-related variables for the best index so far. */
   Cost_estimate best_read_cost;
-  ha_rows best_records = 0;
-  SEL_ROOT *best_index_tree = nullptr;
-  ha_rows best_quick_prefix_records = 0;
-  uint best_param_idx = 0;
+
   Opt_trace_context *const trace = &thd->opt_trace;
 
   DBUG_TRACE;
 
-  Opt_trace_object trace_group(trace, "group_index_range",
+  Opt_trace_object trace_group(trace, "group_index_skip_scan",
                                Opt_trace_context::RANGE_OPTIMIZER);
   const char *cause = nullptr;
   best_read_cost.set_max_cost();
@@ -322,7 +369,8 @@ AccessPath *get_best_group_min_max(THD *thd, RANGE_OPT_PARAM *param,
   /* Perform few 'cheap' tests whether this access method is applicable. */
   if (!join)
     cause = "no_join";
-  else if (join->primary_tables != 1) /* Query must reference one table. */
+  else if (param->query_block->leaf_table_count !=
+           1) /* Query must reference one table. */
     cause = "not_single_table";
   else if (join->query_block->olap == ROLLUP_TYPE) /* Check (B3) for ROLLUP */
     cause = "rollup";
@@ -332,7 +380,7 @@ AccessPath *get_best_group_min_max(THD *thd, RANGE_OPT_PARAM *param,
     cause = "cannot_do_reverse_ordering";
   if (cause != nullptr) {
     trace_group.add("chosen", false).add_alnum("cause", cause);
-    return nullptr;
+    return;
   }
 
   /* Check (SA1,SA4) and store the only MIN/MAX argument - the C attribute.*/
@@ -343,7 +391,7 @@ AccessPath *get_best_group_min_max(THD *thd, RANGE_OPT_PARAM *param,
       (!join->select_distinct) && !is_agg_distinct) {
     trace_group.add("chosen", false)
         .add_alnum("cause", "not_group_by_or_distinct");
-    return nullptr;
+    return;
   }
   /* Analyze the query in more detail. */
 
@@ -363,19 +411,19 @@ AccessPath *get_best_group_min_max(THD *thd, RANGE_OPT_PARAM *param,
       else {
         trace_group.add("chosen", false)
             .add_alnum("cause", "not_applicable_aggregate_function");
-        return nullptr;
+        return;
       }
 
       /* The argument of MIN/MAX. */
       Item *expr = min_max_item->get_arg(0)->real_item();
       if (expr->type() == Item::FIELD_ITEM) /* Is it an attribute? */
       {
-        if (!min_max_arg_item)
-          min_max_arg_item = (Item_field *)expr;
-        else if (!min_max_arg_item->eq(expr, true))
-          return nullptr;
+        if (min_max_arg_item == nullptr)
+          min_max_arg_item = down_cast<Item_field *>(expr);
+        else if (!min_max_arg_item->eq(expr))
+          return;
       } else
-        return nullptr;
+        return;
     }
   }
 
@@ -392,7 +440,7 @@ AccessPath *get_best_group_min_max(THD *thd, RANGE_OPT_PARAM *param,
     */
     trace_group.add("chosen", false)
         .add_alnum("cause", "disjuntive_predicate_present");
-    return nullptr;
+    return;
   } else if (!tree && join->where_cond && min_max_arg_item) {
     /**
       Skip loose index scan if min_max attribute is present along with
@@ -407,7 +455,7 @@ AccessPath *get_best_group_min_max(THD *thd, RANGE_OPT_PARAM *param,
                                         &non_min_max_arg_present)) {
       trace_group.add("chosen", false)
           .add_alnum("cause", "minmax_keypart_in_disjunctive_query");
-      return nullptr;
+      return;
     }
   }
 
@@ -415,14 +463,14 @@ AccessPath *get_best_group_min_max(THD *thd, RANGE_OPT_PARAM *param,
   if (is_agg_distinct && (have_max || have_min)) {
     trace_group.add("chosen", false)
         .add_alnum("cause", "have_both_agg_distinct_and_min_max");
-    return nullptr;
+    return;
   }
 
   /* Check (SA5). */
   if (join->select_distinct) {
     trace_group.add("distinct_query", true);
     for (Item *item : VisibleFields(*join->fields)) {
-      if (item->real_item()->type() != Item::FIELD_ITEM) return nullptr;
+      if (item->real_item()->type() != Item::FIELD_ITEM) return;
     }
   }
 
@@ -432,7 +480,7 @@ AccessPath *get_best_group_min_max(THD *thd, RANGE_OPT_PARAM *param,
     if ((*tmp_group->item)->real_item()->type() != Item::FIELD_ITEM) {
       trace_group.add("chosen", false)
           .add_alnum("cause", "group_field_is_expression");
-      return nullptr;
+      return;
     }
   }
 
@@ -441,11 +489,10 @@ AccessPath *get_best_group_min_max(THD *thd, RANGE_OPT_PARAM *param,
     (GA1,GA2) are all true. If there is more than one such index, select the
     first one. Here we set the variables: group_prefix_len and index_info.
   */
-
   const uint pk = param->table->s->primary_key;
   SEL_ROOT *cur_index_tree = nullptr;
   ha_rows cur_quick_prefix_records = 0;
-  Opt_trace_array trace_indexes(trace, "potential_group_range_indexes");
+  Opt_trace_array trace_indexes(trace, "potential_group_skip_scan_indexes");
   // We go through allowed indexes
   for (uint cur_param_idx = 0; cur_param_idx < param->keys; ++cur_param_idx) {
     const uint cur_index = param->real_keynr[cur_param_idx];
@@ -780,33 +827,34 @@ AccessPath *get_best_group_min_max(THD *thd, RANGE_OPT_PARAM *param,
                                   cur_index_tree, key_part, false);
       }
     }
-    cost_group_min_max(table, cur_index, cur_used_key_parts,
-                       cur_group_key_parts, tree, cur_quick_prefix_records,
-                       have_min, have_max, cur_infix_factor, &cur_read_cost,
-                       &cur_records, is_eq_range_pred);
-    /*
-      If cur_read_cost is lower than best_read_cost use cur_index.
-      Do not compare doubles directly because they may have different
-      representations (64 vs. 80 bits).
-    */
+    cost_group_skip_scan(table, cur_index, cur_used_key_parts,
+                         cur_group_key_parts, tree, cur_quick_prefix_records,
+                         have_min, have_max, cur_infix_factor, &cur_read_cost,
+                         &cur_records, is_eq_range_pred);
     trace_idx.add("rows", cur_records).add("cost", cur_read_cost);
     {
-      Cost_estimate min_diff_cost = cur_read_cost;
-      min_diff_cost.multiply(DBL_EPSILON);
-      if (cur_read_cost < (best_read_cost - min_diff_cost)) {
-        index_info = cur_index_info;
-        index = cur_index;
-        best_read_cost = cur_read_cost;
-        best_records = cur_records;
-        best_index_tree = cur_index_tree;
-        best_quick_prefix_records = cur_quick_prefix_records;
-        best_param_idx = cur_param_idx;
-        group_key_parts = cur_group_key_parts;
-        group_prefix_len = cur_group_prefix_len;
-        key_infix_len = cur_key_infix_len;
-        used_key_parts = cur_used_key_parts;
-        min_max_arg_part = cur_min_max_arg_part;
-      }
+      GroupIndexSkipScanInfo *group_skip_scan_info =
+          new (param->return_mem_root) GroupIndexSkipScanInfo;
+      GroupIndexSkipScanParameters *group_skip_scan_param =
+          new (param->return_mem_root) GroupIndexSkipScanParameters;
+
+      group_skip_scan_param->have_agg_distinct = is_agg_distinct;
+      group_skip_scan_param->min_max_arg_part = cur_min_max_arg_part;
+      group_skip_scan_param->group_prefix_len = cur_group_prefix_len;
+      group_skip_scan_param->group_key_parts = cur_group_key_parts;
+      group_skip_scan_param->index_info = cur_index_info;
+      group_skip_scan_param->key_infix_len = cur_key_infix_len;
+      group_skip_scan_param->index_tree = cur_index_tree;
+      group_skip_scan_param->used_key_part = param->key[cur_param_idx];
+      group_skip_scan_info->param = group_skip_scan_param;
+      group_skip_scan_info->quick_prefix_records = cur_quick_prefix_records;
+      group_skip_scan_info->cost = cur_read_cost;
+      group_skip_scan_info->records = cur_records;
+      group_skip_scan_info->param_idx = cur_param_idx;
+      group_skip_scan_info->num_used_key_parts = cur_used_key_parts;
+      group_skip_scan_info->min_max_arg_item = min_max_arg_item;
+
+      possible_group_skip_scans->push_back(group_skip_scan_info);
     }
 
   next_index:
@@ -816,22 +864,121 @@ AccessPath *get_best_group_min_max(THD *thd, RANGE_OPT_PARAM *param,
     }
   }
   trace_indexes.end();
+  trace_group.end();
+  return;
+}
 
-  if (!index_info) /* No usable index found. */
-    return nullptr;
+/**
+ Iterates over list of group skip scan candidates and returns pointer
+ to candidate with lowest cost.
+ @param possible_group_skip_scans List of candidate group index skip scans
+ @retval best_scan Pointer to element in possible_group_skip_scans which has
+ lowest cost.
+*/
+GroupIndexSkipScanInfo *select_best_group_skip_scan(
+    Mem_root_array<GroupIndexSkipScanInfo *> *possible_group_skip_scans) {
+  GroupIndexSkipScanInfo *best_scan = nullptr;
+  for (GroupIndexSkipScanInfo *gskip_scan : *possible_group_skip_scans) {
+    /*
+      If current scan cost is lower than best scan cost, use current scan.
+      Do not compare doubles directly because they may have different
+      representations (64 vs. 80 bits).
+    */
+    Cost_estimate min_diff_cost = gskip_scan->cost;
+    min_diff_cost.multiply(DBL_EPSILON);
+    if ((best_scan == nullptr) ||
+        (gskip_scan->cost < (best_scan->cost - min_diff_cost))) {
+      best_scan = gskip_scan;
+    }
+  }
+  return best_scan;
+}
+
+/**
+  Test if group index skip scan is applicable and if so, construct a new
+  AccessPath for every candidate group index skip scan.
+
+  @param  thd       Thread info
+  @param  param     Range optimizer parameter
+  @param  tree      Range tree generated by get_mm_tree
+  @param  order_direction The sort order the range access method must be able
+                    to provide. Three-value logic: asc/desc/don't care
+  @param  skip_records_in_range Same value as JOIN_TAB::skip_records_in_range()
+  @param  cost_est  Best cost so far (=table/index scan time)
+
+  @retval Mem_root_array of candidate GROUP_INDEX_SKIP_SCAN AccessPaths.
+*/
+Mem_root_array<AccessPath *> get_all_group_skip_scans(
+    THD *thd, RANGE_OPT_PARAM *param, SEL_TREE *tree,
+    enum_order order_direction, bool skip_records_in_range, double cost_est) {
+  bool have_min = false; /* true if there is a MIN function. */
+  bool have_max = false; /* true if there is a MAX function. */
+  Mem_root_array<AccessPath *> group_skip_scan_paths(param->temp_mem_root);
+  Mem_root_array<GroupIndexSkipScanInfo *> possible_group_skip_scans(
+      param->temp_mem_root);
+
+  collect_group_skip_scans(thd, param, tree, order_direction,
+                           skip_records_in_range, &possible_group_skip_scans,
+                           have_min, have_max);
+  double rows = kUnknownRowCount;
+  // Retrieve highest rowcount estimate and use for all group skip scans.
+  for (GroupIndexSkipScanInfo *gskip_scan : possible_group_skip_scans) {
+    rows = max<double>(rows, gskip_scan->records);
+  }
+  for (GroupIndexSkipScanInfo *gskip_scan : possible_group_skip_scans) {
+    AccessPath *cur_path = make_group_skip_scan_path(
+        thd, param, tree, gskip_scan, cost_est, have_min, have_max);
+    if (cur_path != nullptr) {
+      // Adjust num_output_rows for hypergraph to match aggregate path rowcounts
+      cur_path->set_num_output_rows(rows > 1 ? rows - 1 : rows);
+      cur_path->num_output_rows_before_filter = cur_path->num_output_rows();
+      group_skip_scan_paths.push_back(cur_path);
+    }
+  }
+  return group_skip_scan_paths;
+}
+
+/**
+ Build a GROUP_INDEX_SKIP_SCAN AccessPath based on scan info
+
+  @param  thd       Thread info
+  @param  param     Range optimizer parameter
+  @param  tree      Range tree generated by get_mm_tree
+  @param  group_skip_scan_info Info for a candidate group skip scan
+  @param  cost_est  Best cost so far (=table/index scan time)
+  @param  have_min  Set to true if MIN() function is present
+  @param  have_max  Set to true if MAX() function is present
+*/
+AccessPath *make_group_skip_scan_path(
+    THD *thd, RANGE_OPT_PARAM *param, SEL_TREE *tree,
+    GroupIndexSkipScanInfo *group_skip_scan_info, double cost_est,
+    bool have_min, bool have_max) {
+  if (group_skip_scan_info == nullptr) return nullptr;
+
+  Opt_trace_context *const trace = &thd->opt_trace;
+  DBUG_TRACE;
+  Opt_trace_object trace_group_skip(trace, "make_group_skip_scan_path",
+                                    Opt_trace_context::RANGE_OPTIMIZER);
+
+  JOIN *join = param->query_block->join;
+  TABLE *table = param->table;
 
   // TODO(sgunders): Figure out why (this was kept across a refactoring,
   // and there is an assert about it further below.).
-  if (tree != nullptr && best_quick_prefix_records == 0) {
+  if (tree != nullptr && group_skip_scan_info->quick_prefix_records == 0) {
     return nullptr;
   }
 
+  GroupIndexSkipScanParameters *group_skip_scan_param =
+      group_skip_scan_info->param;
   /* Check (SA3) for the where clause. */
-  if (join->where_cond && min_max_arg_item &&
+  if (join->where_cond && group_skip_scan_info->min_max_arg_item &&
       !check_group_min_max_predicates(
-          join->where_cond, min_max_arg_item,
-          (index_info->flags & HA_SPATIAL) ? Field::itMBR : Field::itRAW)) {
-    trace_group.add("usable", false)
+          join->where_cond, group_skip_scan_info->min_max_arg_item,
+          (group_skip_scan_param->index_info->flags & HA_SPATIAL)
+              ? Field::itMBR
+              : Field::itRAW)) {
+    trace_group_skip.add("usable", false)
         .add_alnum("cause", "unsupported_predicate_on_agg_attribute");
     return nullptr;
   }
@@ -839,17 +986,20 @@ AccessPath *get_best_group_min_max(THD *thd, RANGE_OPT_PARAM *param,
   // Populate key_infix_ranges from index_tree.
   MEM_ROOT *const return_mem_root = param->return_mem_root;
   SEL_TREE *const range_tree = tree;
-  SEL_ROOT *const index_tree = best_index_tree;
-  const uint keyno = param->real_keynr[best_param_idx];
+  SEL_ROOT *const index_tree = group_skip_scan_param->index_tree;
+  const uint keyno = param->real_keynr[group_skip_scan_info->param_idx];
   Quick_ranges_array key_infix_ranges(return_mem_root);
-  uint num_infix_keyparts = used_key_parts - group_key_parts;
+  uint num_infix_keyparts = group_skip_scan_info->num_used_key_parts -
+                            group_skip_scan_param->group_key_parts;
   for (uint i = 0; i < num_infix_keyparts; i++) {
     key_infix_ranges.push_back(new (return_mem_root)
                                    Quick_ranges(return_mem_root));
   }
-  if (key_infix_len > 0) {
+  if (group_skip_scan_param->key_infix_len > 0) {
     if (range_tree) {
-      KEY_PART_INFO *infix_keypart = index_info->key_part + group_key_parts;
+      KEY_PART_INFO *infix_keypart =
+          group_skip_scan_param->index_info->key_part +
+          group_skip_scan_param->group_key_parts;
 
       // Find the start of key infix ranges in the range tree
       SEL_ARG *infix_key = index_tree->root;
@@ -868,7 +1018,8 @@ AccessPath *get_best_group_min_max(THD *thd, RANGE_OPT_PARAM *param,
         */
         bool is_ascending = infix_key->is_ascending;
         KEY_PART_INFO *key_infix_part =
-            index_info->key_part + group_key_parts + i;
+            group_skip_scan_param->index_info->key_part +
+            group_skip_scan_param->group_key_parts + i;
         for (SEL_ARG *cur_range = is_ascending ? infix_key->first()
                                                : infix_key->last();
              cur_range != nullptr;
@@ -887,6 +1038,7 @@ AccessPath *get_best_group_min_max(THD *thd, RANGE_OPT_PARAM *param,
   }
 
   Quick_ranges min_max_ranges(return_mem_root);
+  KEY_PART_INFO *min_max_arg_part = group_skip_scan_param->min_max_arg_part;
   if (range_tree && min_max_arg_part) {
     /*
       Extract the SEL_ARG subtree that contains only ranges for the MIN/MAX
@@ -924,16 +1076,17 @@ AccessPath *get_best_group_min_max(THD *thd, RANGE_OPT_PARAM *param,
     is skipped, and then a shorter key will be used. However this is data
     dependent and can't be easily estimated here.
    */
-  uint real_prefix_len = group_prefix_len + key_infix_len;
+  uint real_prefix_len = group_skip_scan_param->group_prefix_len +
+                         group_skip_scan_param->key_infix_len;
   uint max_used_key_length = real_prefix_len;
-  unsigned real_key_parts = used_key_parts;
+  unsigned real_key_parts = group_skip_scan_info->num_used_key_parts;
   if (min_max_ranges.size() > 0) {
     // Check if the right-most range has a lower boundary,
     // or the left-most range has an upper boundary.
     if ((have_min && !(min_max_ranges.back()->flag & NO_MIN_RANGE)) ||
         (have_max && !(min_max_ranges[0]->flag & NO_MAX_RANGE))) {
       max_used_key_length += min_max_arg_part->store_length;
-      used_key_parts++;
+      group_skip_scan_info->num_used_key_parts++;
     }
   } else if (have_min && min_max_arg_part &&
              min_max_arg_part->field->is_nullable()) {
@@ -946,22 +1099,22 @@ AccessPath *get_best_group_min_max(THD *thd, RANGE_OPT_PARAM *param,
       usable key length(see next_min()).
     */
     max_used_key_length += min_max_arg_part->store_length;
-    used_key_parts++;
+    group_skip_scan_info->num_used_key_parts++;
   }
 
   Quick_ranges prefix_ranges(return_mem_root);
-  KEY_PART *const used_key_part = param->key[best_param_idx];
+  KEY_PART *const used_key_part = param->key[group_skip_scan_info->param_idx];
   if (range_tree) {
-    assert(best_quick_prefix_records > 0);
-    if (best_quick_prefix_records != HA_POS_ERROR) {
+    assert(group_skip_scan_info->quick_prefix_records > 0);
+    if (group_skip_scan_info->quick_prefix_records != HA_POS_ERROR) {
       /* Prepare for a IndexRangeScanIterator to be used for group prefix
        * retrieval.
        */
       unsigned used_key_parts_unused, num_exact_key_parts_unused;
-      if (get_ranges_from_tree(return_mem_root, table, used_key_part, keyno,
-                               index_tree, group_key_parts,
-                               &used_key_parts_unused,
-                               &num_exact_key_parts_unused, &prefix_ranges)) {
+      if (get_ranges_from_tree(
+              return_mem_root, table, used_key_part, keyno, index_tree,
+              group_skip_scan_param->group_key_parts, &used_key_parts_unused,
+              &num_exact_key_parts_unused, &prefix_ranges)) {
         return nullptr;
       }
       // Opens the ranges if there are more conditions in
@@ -983,7 +1136,7 @@ AccessPath *get_best_group_min_max(THD *thd, RANGE_OPT_PARAM *param,
         prefix_max_length =
             std::max(prefix_max_length, uint(range->max_length));
       }
-      if (group_prefix_len < prefix_max_length) {
+      if (group_skip_scan_param->group_prefix_len < prefix_max_length) {
         for (QUICK_RANGE *range : prefix_ranges) {
           range->flag &= ~(NEAR_MIN | NEAR_MAX);
         }
@@ -994,51 +1147,49 @@ AccessPath *get_best_group_min_max(THD *thd, RANGE_OPT_PARAM *param,
   /* The query passes all tests, so construct a new AccessPath. */
   AccessPath *path = new (param->return_mem_root) AccessPath;
   path->type = AccessPath::GROUP_INDEX_SKIP_SCAN;
-  path->cost = best_read_cost.total_cost();
-  path->set_num_output_rows(best_records);
+  path->set_cost(group_skip_scan_info->cost.total_cost());
+  path->set_num_output_rows(group_skip_scan_info->records);
+  path->has_group_skip_scan = true;
 
-  // Extract the list of MIN and MAX functions; join->sum_funcs will change
-  // after temporary table setup, so it needs to be done before the iterator
-  // is created.
-  GroupIndexSkipScanParameters *p =
-      new (param->return_mem_root) GroupIndexSkipScanParameters;
-  p->min_functions.init(return_mem_root);
-  p->max_functions.init(return_mem_root);
-  if (min_max_arg_part != nullptr) {
+  // Extract the list of MIN and MAX functions; join->sum_funcs will
+  // change after temporary table setup, so it needs to be done before the
+  // iterator is created.
+  group_skip_scan_param->min_functions.init(param->return_mem_root);
+  group_skip_scan_param->max_functions.init(param->return_mem_root);
+  if (group_skip_scan_param->min_max_arg_part != nullptr) {
     Item_sum *min_max_item;
     Item_sum **func_ptr = join->sum_funcs;
     while ((min_max_item = *(func_ptr++))) {
       if (min_max_item->sum_func() == Item_sum::MIN_FUNC)
-        p->min_functions.push_back(min_max_item);
+        group_skip_scan_param->min_functions.push_back(min_max_item);
       else if (min_max_item->sum_func() == Item_sum::MAX_FUNC)
-        p->max_functions.push_back(min_max_item);
+        group_skip_scan_param->max_functions.push_back(min_max_item);
     }
   }
-  p->have_agg_distinct = is_agg_distinct;
-  p->min_max_arg_part = min_max_arg_part;
-  p->group_prefix_len = group_prefix_len;
-  p->group_key_parts = group_key_parts;
-  p->index_info = index_info;
-  p->key_infix_len = key_infix_len;
-  p->index_tree_tracing_only = best_index_tree;
-  p->used_key_part = param->key[best_param_idx];
-  p->real_key_parts = real_key_parts;
-  p->max_used_key_length = max_used_key_length;
-  p->prefix_ranges = std::move(prefix_ranges);
-  p->key_infix_ranges = std::move(key_infix_ranges);
-  p->min_max_ranges = std::move(min_max_ranges);
-  if (cost_est < best_read_cost.total_cost() && is_agg_distinct) {
-    trace_group.add("index_scan", true);
-    path->cost = 0.0;
-    p->is_index_scan = true;
+
+  group_skip_scan_param->real_key_parts = real_key_parts;
+  group_skip_scan_param->max_used_key_length = max_used_key_length;
+  group_skip_scan_param->prefix_ranges = std::move(prefix_ranges);
+  group_skip_scan_param->key_infix_ranges = std::move(key_infix_ranges);
+  group_skip_scan_param->min_max_ranges = std::move(min_max_ranges);
+  if (cost_est < group_skip_scan_info->cost.total_cost() &&
+      group_skip_scan_param->have_agg_distinct) {
+    trace_group_skip.add("index_scan", true);
+    path->set_cost(0.0);
+    group_skip_scan_param->is_index_scan = true;
   } else {
-    p->is_index_scan = false;
+    group_skip_scan_param->is_index_scan = false;
   }
 
   path->group_index_skip_scan().table = table;
-  path->group_index_skip_scan().index = index;
-  path->group_index_skip_scan().num_used_key_parts = used_key_parts;
-  path->group_index_skip_scan().param = p;
+  path->group_index_skip_scan().index =
+      param->real_keynr[group_skip_scan_info->param_idx];
+  path->group_index_skip_scan().num_used_key_parts =
+      group_skip_scan_info->num_used_key_parts;
+  path->group_index_skip_scan().param = group_skip_scan_param;
+
+  trace_group_skip.end();
+
   return path;
 }
 
@@ -1096,7 +1247,7 @@ static bool check_group_min_max_predicates(Item *cond,
     Need to handle subselect in min_max_inspect_cond_for_fields() once this
     is fixed.
   */
-  if (cond_type == Item::SUBSELECT_ITEM) return false;
+  if (cond_type == Item::SUBQUERY_ITEM) return false;
 
   /*
     Condition of the form 'field' is equivalent to 'field <> 0' and thus
@@ -1117,7 +1268,7 @@ static bool check_group_min_max_predicates(Item *cond,
     Item_func. In this particular case, the where clause should
     be equivalent to "where max(1) <> 0". A where clause
     phrased that way does not satisfy the SA3 condition of
-    get_best_group_min_max(). The "where max(1) = true" clause
+    get_best_group_skip_scan(). The "where max(1) = true" clause
     causes this method to reject the access method
     (i.e., to return false).
 
@@ -1153,7 +1304,7 @@ static bool check_group_min_max_predicates(Item *cond,
     cur_arg = arguments[arg_idx]->real_item();
     DBUG_PRINT("info", ("cur_arg: %s", cur_arg->full_name()));
     if (cur_arg->type() == Item::FIELD_ITEM) {
-      if (min_max_arg_item->eq(cur_arg, true)) {
+      if (min_max_arg_item->eq(cur_arg)) {
         /*
           If pred references the MIN/MAX argument, check whether pred is a range
           condition that compares the MIN/MAX argument with a constant.
@@ -1230,7 +1381,7 @@ static inline void util_min_max_inspect_item(Item *item_field,
                                              bool *min_max_arg_present,
                                              bool *non_min_max_arg_present) {
   if (item_field->type() == Item::FIELD_ITEM) {
-    if (min_max_arg_item->eq(item_field, true))
+    if (min_max_arg_item->eq(item_field))
       *min_max_arg_present = true;
     else
       *non_min_max_arg_present = true;
@@ -1306,12 +1457,12 @@ static bool min_max_inspect_cond_for_fields(Item *cond,
         if (*min_max_arg_present && *non_min_max_arg_present) return true;
       }
 
-      if (pred->functype() == Item_func::MULT_EQUAL_FUNC) {
+      if (pred->functype() == Item_func::MULTI_EQ_FUNC) {
         /*
           Analyze participating fields in a multiequal condition.
         */
         for (Item_field &item_field :
-             down_cast<Item_equal *>(cond)->get_fields()) {
+             down_cast<Item_multi_eq *>(cond)->get_fields()) {
           util_min_max_inspect_item(&item_field, min_max_arg_item,
                                     min_max_arg_present,
                                     non_min_max_arg_present);
@@ -1351,7 +1502,7 @@ static bool min_max_inspect_cond_for_fields(Item *cond,
     index_info             [in]  Pointer to KEY object
 
   DESCRIPTION
-    Test conditions (NGA1, NGA2) from get_best_group_min_max(). Namely,
+    Test conditions (NGA1, NGA2) from get_best_group_skip_scan(). Namely,
     for each keypart field NGF_i not in GROUP-BY, check that there is at least
     one equality range predicate for each key part among conds with the form
     (NGF_i = const_ci) or (const_ci = NGF_i).
@@ -1455,10 +1606,11 @@ static inline uint get_field_keypart(KEY *index, const Field *field) {
 }
 
 /*
-  Compute the cost of a quick_group_min_max_query_block for a particular index.
+  Compute the cost of a quick_group_skip_scan_query_block for a particular
+  index.
 
   SYNOPSIS
-    cost_group_min_max()
+    cost_group_skip_scan()
     table                [in] The table being accessed
     key                  [in] The index used to access the table
     used_key_parts       [in] Number of key parts used to access the index
@@ -1529,12 +1681,12 @@ static inline uint get_field_keypart(KEY *index, const Field *field) {
     None
 */
 
-static void cost_group_min_max(TABLE *table, uint key, uint used_key_parts,
-                               uint group_key_parts, SEL_TREE *range_tree,
-                               ha_rows quick_prefix_records, bool have_min,
-                               bool have_max, uint infix_factor,
-                               Cost_estimate *cost_est, ha_rows *records,
-                               bool single_group) {
+static void cost_group_skip_scan(TABLE *table, uint key, uint used_key_parts,
+                                 uint group_key_parts, SEL_TREE *range_tree,
+                                 ha_rows quick_prefix_records, bool have_min,
+                                 bool have_max, uint infix_factor,
+                                 Cost_estimate *cost_est, ha_rows *records,
+                                 bool single_group) {
   ha_rows table_records;
   uint num_groups;
   uint num_blocks;
@@ -1687,7 +1839,7 @@ void dbug_dump_group_index_skip_scan(int indent, bool, const AccessPath *path) {
       path->group_index_skip_scan().param;
 
   fprintf(DBUG_FILE,
-          "%*squick_group_min_max_query_block: index %s (%d), length: %d\n",
+          "%*squick_group_skip_scan_query_block: index %s (%d), length: %d\n",
           indent, "", param->index_info->name,
           path->group_index_skip_scan().index, param->max_used_key_length);
   if (param->key_infix_len > 0) {
