@@ -1,15 +1,16 @@
-/* Copyright (c) 2021, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2021, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -36,6 +37,7 @@
 #include "sql/range_optimizer/index_skip_scan_plan.h"
 #include "sql/range_optimizer/range_optimizer.h"
 #include "sql/range_optimizer/rowid_ordered_retrieval_plan.h"
+#include "sql/sql_opt_exec_shared.h"
 
 inline bool is_loose_index_scan(const AccessPath *path) {
   return path->type == AccessPath::INDEX_SKIP_SCAN ||
@@ -197,6 +199,20 @@ inline unsigned get_used_key_parts(const AccessPath *path) {
       return path->index_skip_scan().num_used_key_parts;
     case AccessPath::GROUP_INDEX_SKIP_SCAN:
       return path->group_index_skip_scan().num_used_key_parts;
+    case AccessPath::REF:
+      return path->ref().ref->key_parts;
+    case AccessPath::REF_OR_NULL:
+      return path->ref_or_null().ref->key_parts;
+    case AccessPath::EQ_REF:
+      return path->eq_ref().ref->key_parts;
+    case AccessPath::PUSHED_JOIN_REF:
+      return path->pushed_join_ref().ref->key_parts;
+    case AccessPath::FULL_TEXT_SEARCH:
+      return path->full_text_search().ref->key_parts;
+    case AccessPath::MRR:
+      return path->mrr().ref->key_parts;
+    case AccessPath::INDEX_DISTANCE_SCAN:
+    case AccessPath::INDEX_SCAN:
     case AccessPath::INDEX_MERGE:
     case AccessPath::ROWID_INTERSECTION:
     case AccessPath::ROWID_UNION:
