@@ -1,15 +1,16 @@
-/* Copyright (c) 2000, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -42,6 +43,7 @@
 #include "sql/current_thd.h"
 #include "sql/derror.h"
 #include "sql/field.h"
+#include "sql/field_common_properties.h"
 #include "sql/handler.h"
 #include "sql/item.h"
 #include "sql/item_cmpfunc.h"
@@ -771,15 +773,15 @@ static SEL_TREE *get_full_func_mm_tree(THD *thd, RANGE_OPT_PARAM *param,
     Item_field *item_field = static_cast<Item_field *>(predicand);
     Field *field = item_field->field;
 
-    if (!((ref_tables | item_field->table_ref->map()) & param_comp))
+    if (!((ref_tables | item_field->m_table_ref->map()) & param_comp))
       ftree = get_func_mm_tree(thd, param, prev_tables, read_tables,
                                remove_jump_scans, predicand, op, value, inv);
-    Item_equal *item_equal = item_field->item_equal;
+    Item_multi_eq *item_equal = item_field->multi_equality();
     if (item_equal != nullptr) {
       for (Item_field &item : item_equal->get_fields()) {
         Field *f = item.field;
         if (!field->eq(f) &&
-            !((ref_tables | item.table_ref->map()) & param_comp)) {
+            !((ref_tables | item.m_table_ref->map()) & param_comp)) {
           tree = get_func_mm_tree(thd, param, prev_tables, read_tables,
                                   remove_jump_scans, &item, op, value, inv);
           ftree = !ftree ? tree : tree_and(param, ftree, tree);
@@ -967,15 +969,15 @@ SEL_TREE *get_mm_tree(THD *thd, RANGE_OPT_PARAM *param, table_map prev_tables,
       break;
     }  // end case Item_func::IN_FUNC
 
-    case Item_func::MULT_EQUAL_FUNC: {
-      Item_equal *item_equal = down_cast<Item_equal *>(cond);
+    case Item_func::MULTI_EQ_FUNC: {
+      Item_multi_eq *item_equal = down_cast<Item_multi_eq *>(cond);
       Item *value = item_equal->const_arg();
       if (value == nullptr) return nullptr;
       table_map ref_tables = value->used_tables();
       for (Item_field &field_item : item_equal->get_fields()) {
         Field *field = field_item.field;
         table_map param_comp = ~(prev_tables | read_tables | current_table);
-        if (!((ref_tables | field_item.table_ref->map()) & param_comp)) {
+        if (!((ref_tables | field_item.m_table_ref->map()) & param_comp)) {
           SEL_TREE *tree =
               get_mm_parts(thd, param, prev_tables, read_tables, item_equal,
                            field, Item_func::EQ_FUNC, value);
@@ -985,7 +987,7 @@ SEL_TREE *get_mm_tree(THD *thd, RANGE_OPT_PARAM *param, table_map prev_tables,
 
       dbug_print_tree("tree_returned", ftree, param);
       return ftree;
-    }  // end case Item_func::MULT_EQUAL_FUNC
+    }  // end case Item_func::MULTI_EQ_FUNC
 
     default: {
       Item *const arg_left = cond_func->arguments()[0];
@@ -1050,6 +1052,7 @@ static bool is_spatial_operator(Item_func::Functype op_type) {
   switch (op_type) {
     case Item_func::SP_EQUALS_FUNC:
     case Item_func::SP_DISJOINT_FUNC:
+    case Item_func::SP_DISTANCE_FUNC:
     case Item_func::SP_INTERSECTS_FUNC:
     case Item_func::SP_TOUCHES_FUNC:
     case Item_func::SP_CROSSES_FUNC:
@@ -1126,6 +1129,14 @@ static SEL_TREE *get_mm_parts(THD *thd, RANGE_OPT_PARAM *param,
       tree->set_key(key_part->key,
                     sel_add(tree->release_key(key_part->key), sel_root));
       tree->keys_map.set_bit(key_part->key);
+      // A range constructed for a multi-valued index is never exact.
+      // So it needs the filter to be placed on top of the range access.
+      if (param->using_real_indexes) {
+        int index = param->real_keynr[key_part->key];
+        if (param->table->key_info[index].flags & HA_MULTI_VALUED_KEY) {
+          tree->inexact = true;
+        }
+      }
     }
   }
 
@@ -1212,12 +1223,28 @@ static bool save_value_and_handle_conversion(
   thd->variables.sql_mode = orig_sql_mode;
 
   switch (err) {
-    case TYPE_NOTE_TRUNCATED:
-    case TYPE_WARN_TRUNCATED:
-      *inexact = true;
-      [[fallthrough]];
     case TYPE_OK:
       return false;
+    case TYPE_NOTE_TRUNCATED:
+      // Insignificant truncation (trailing zero/space). Use it as an inexact
+      // range predicate.
+      *inexact = true;
+      return false;
+    case TYPE_WARN_TRUNCATED:
+      // Truncation of possibly significant parts. We may still be able to use
+      // it as a range predicate, but we need a filter to make sure we don't
+      // return too many rows.
+      *inexact = true;
+      // Use the truncated value in the range predicate, unless it's a string
+      // with a non-binary collation with a non-trivial strnxfrm function. For
+      // example, if c is a VARCHAR(1) column with the utf8mb4_0900_ai_ci
+      // collation, c <= 'ss' should match the value 'ß', but if we truncate it
+      // to c <= 's' to fit in the column type, it will not match 'ß'. For such
+      // predicates, we assume the predicate is always true, and let a filter
+      // decide the outcome.
+      return is_string_type(field->type()) &&
+             !my_binary_compare(field->charset()) &&
+             use_strnxfrm(field->charset());
     case TYPE_WARN_INVALID_STRING:
       /*
         An invalid string does not produce any rows when used with
@@ -1229,8 +1256,7 @@ static bool save_value_and_handle_conversion(
       }
       /*
         For other operations on invalid strings, we assume that the range
-        predicate is always true and let evaluate_join_record() decide
-        the outcome.
+        predicate is always true and let a filter decide the outcome.
       */
       *inexact = true;
       return true;
@@ -1243,7 +1269,7 @@ static bool save_value_and_handle_conversion(
 
         instead of always false. Because of this, we assume that the
         range predicate is always true instead of always false and let
-        evaluate_join_record() decide the outcome.
+        a filter decide the outcome.
       */
       *inexact = true;
       return true;
