@@ -31,7 +31,8 @@
 #include "sql/rpl_replica.h"
 #include "sql/rpl_rli.h"
 #include "sql/rpl_rli_pdb.h"
-#include "sql/sql_backup_lock.h"  // is_instance_backup_locked et al.
+#include "sql/sql_backup_lock.h"
+#include "string_with_len.h"
 
 /**
    It manages a stage and the related mutex and makes the process of
@@ -185,9 +186,9 @@ Log_event *Rpl_applier_reader::read_next_event() {
         the checkpoint routine must be periodically invoked.
 
         mta_checkpoint_routine has to be called before enter_stage().
-        Otherwise, it will cause a deadlock with STOP SLAVE or other
+        Otherwise, it will cause a deadlock with STOP REPLICA or other
         thread has the same lock pattern.
-        STOP SLAVE Thread                   Coordinator Thread
+        STOP REPLICA Thread                   Coordinator Thread
         =================                   ==================
         lock LOCK_thd_data                  lock LOCK_binlog_end_pos
                                             enter_stage(LOCK_binlog_end_pos)
@@ -220,6 +221,12 @@ Log_event *Rpl_applier_reader::read_next_event() {
       /* Check it again to avoid missing update signals from receiver thread */
       if (read_active_log_end_pos()) break;
 
+      if (m_rli->is_until_satisfied_all_transactions_read_from_relay_log()) {
+        // Make it stop on the next execution
+        m_rli->abort_slave = true;
+        return nullptr;
+      }
+
       reset_seconds_behind_master();
       /* It should be protected by relay_log.LOCK_binlog_end_pos */
       if (m_rli->ign_master_log_name_end[0]) return generate_rotate_event();
@@ -301,7 +308,7 @@ bool Rpl_applier_reader::wait_for_new_event() {
   mysql_mutex_assert_owner(&m_rli->data_lock);
   /*
     We can, and should release data_lock while we are waiting for
-    update. If we do not, show slave status will block
+    update. If we do not, show replica status will block
   */
   mysql_mutex_unlock(&m_rli->data_lock);
 
@@ -489,7 +496,7 @@ void Rpl_applier_reader::debug_print_next_event_positions() {
   DBUG_PRINT(
       "info",
       ("assertion skip %u file pos %llu event relay log pos %llu file %s\n",
-       m_rli->slave_skip_counter, m_relaylog_file_reader.position(),
+       m_rli->slave_skip_counter.load(), m_relaylog_file_reader.position(),
        m_rli->get_event_relay_log_pos(), m_rli->get_event_relay_log_name()));
 
   /* This is an assertion which sometimes fails, let's try to track it */
@@ -525,7 +532,7 @@ void Rpl_applier_reader::debug_print_next_event_positions() {
 
 void Rpl_applier_reader::reset_seconds_behind_master() {
   /*
-    We say in Seconds_Behind_Master that we have "caught up". Note that for
+    We say in Seconds_Behind_Source that we have "caught up". Note that for
     example if network link is broken but I/O slave thread hasn't noticed it
     (replica_net_timeout not elapsed), then we'll say "caught up" whereas we're
     not really caught up. Fixing that would require internally cutting timeout
@@ -536,13 +543,13 @@ void Rpl_applier_reader::reset_seconds_behind_master() {
 
     Transient phases like this can be fixed with implementing Heartbeat event
     which provides the slave the status of the master at time the master does
-    not have any new update to send. Seconds_Behind_Master would be zero only
+    not have any new update to send. Seconds_Behind_Source would be zero only
     when master has no more updates in binlog for slave. The heartbeat can be
     sent in a (small) fraction of replica_net_timeout. Until it's done
     m_rli->last_master_timestamp is temporarily (for time of waiting for the
     following event) reset whenever EOF is reached.
 
-    Note, in MTS case Seconds_Behind_Master resetting follows
+    Note, in MTS case Seconds_Behind_Source resetting follows
     slightly different schema where reaching EOF is not enough.  The status
     parameter is updated per some number of processed group of events. The
     number can't be greater than @@global.replica_checkpoint_group and anyway
