@@ -1,4 +1,4 @@
-/* Copyright (c) 2015, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
@@ -70,7 +70,6 @@
 #include "sql/rpl_trx_boundary_parser.h"
 #include "sql/sql_class.h"
 #include "sql/sql_lex.h"
-#include "sql/transaction.h"  // trans_begin
 
 /**
   Auxiliary function to stop all the running channel threads according to the
@@ -509,7 +508,9 @@ int channel_start(const char *channel, Channel_connection_info *connection_info,
       wait for the thread to start
     */
     while (thread_start_id == mi->slave_run_id) {
-      mysql_cond_wait(&mi->start_cond, &mi->run_lock);
+      struct timespec abstime;
+      set_timespec(&abstime, 1);
+      mysql_cond_timedwait(&mi->start_cond, &mi->run_lock, &abstime);
     }
     mysql_mutex_unlock(&mi->run_lock);
 
@@ -1413,8 +1414,6 @@ bool start_failover_channels() {
 bool channel_change_source_connection_auto_failover(const char *channel,
                                                     bool status) {
   bool error = false;
-  bool thd_created = false;
-  THD *thd = current_thd;
   channel_map.assert_some_wrlock();
 
   Master_info *mi = channel_map.get_mi(channel);
@@ -1424,30 +1423,9 @@ bool channel_change_source_connection_auto_failover(const char *channel,
     return true;
   }
 
-  if (!thd) {
-    thd_created = true;
-    thd = create_surrogate_thread();
-    thd->set_skip_readonly_check();
-  }
-
   mi->channel_wrlock();
   lock_slave_threads(mi);
 
-  /*
-    When autocommit= 0, we force a new transaction to prevent
-    table access deadlocks when a master and slave info repositories
-    transaction is executed after changing SOURCE_CONNECTION_AUTO_FAILOVER.
-  */
-  if (thd->variables.option_bits & OPTION_NOT_AUTOCOMMIT) {
-    assert(!(thd->server_status & SERVER_STATUS_IN_TRANS));
-    if (trans_begin(thd)) {
-      unlock_slave_threads(mi);
-      mi->channel_unlock();
-      if (thd_created) delete_surrogate_thread(thd);
-      return true;
-    }
-  }
-
   if (status && !mi->is_source_connection_auto_failover()) {
     mi->set_source_connection_auto_failover();
     error |= (flush_master_info(mi, true, true, false) != 0);
@@ -1458,25 +1436,9 @@ bool channel_change_source_connection_auto_failover(const char *channel,
     error |= (flush_master_info(mi, true, true, false) != 0);
   }
 
-  /*
-    When autocommit= 0, we force a new transaction to prevent
-    table access deadlocks when a master and slave info repositories
-    transaction is executed after changing SOURCE_CONNECTION_AUTO_FAILOVER.
-  */
-  if (thd->variables.option_bits & OPTION_NOT_AUTOCOMMIT) {
-    if (trans_commit(thd)) {
-      unlock_slave_threads(mi);
-      mi->channel_unlock();
-      if (thd_created) delete_surrogate_thread(thd);
-      return true;
-    }
-  }
-
   unlock_slave_threads(mi);
   mi->channel_unlock();
 
-  if (thd_created) delete_surrogate_thread(thd);
-
   return error;
 }
 
