@@ -1,15 +1,16 @@
-/* Copyright (c) 2014, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -261,6 +262,8 @@ void Binlog_group_commit_ctx::mark_as_already_waited() {
 void Binlog_group_commit_ctx::reset() {
   this->m_session_ticket = binlog::BgcTicket(binlog::BgcTicket::kTicketUnset);
   this->m_has_waited = false;
+  m_max_size_exceeded = false;
+  m_force_rotate = false;
 }
 
 std::string Binlog_group_commit_ctx::to_string() const {
@@ -283,8 +286,18 @@ memory::Aligned_atomic<bool> &Binlog_group_commit_ctx::manual_ticket_setting() {
   return flag;
 }
 
+std::pair<bool, bool> Binlog_group_commit_ctx::aggregate_rotate_settings(
+    THD *queue) {
+  bool exceeded = false;
+  bool force_rotate = false;
+  for (THD *thd = queue; thd; thd = thd->next_to_commit) {
+    exceeded |= thd->rpl_thd_ctx.binlog_group_commit_ctx().m_max_size_exceeded;
+    force_rotate |= thd->rpl_thd_ctx.binlog_group_commit_ctx().m_force_rotate;
+  }
+  return {exceeded, force_rotate};
+}
+
 void Rpl_thd_context::init() {
-  m_dependency_tracker_ctx.set_last_session_sequence_number(0);
   m_tx_rpl_delegate_stage_status = TX_RPL_STAGE_INIT;
 }
 
