@@ -1,15 +1,16 @@
-/* Copyright (c) 2013, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2013, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,8 +28,8 @@
 #include <atomic>
 
 #include <mysql/components/my_service.h>
-#include <mysql/components/services/component_sys_var_service.h>
 #include <mysql/components/services/group_replication_status_service.h>
+#include <mysql/components/services/mysql_system_variable.h>
 #include "my_dbug.h"
 #include "my_inttypes.h"
 #include "my_sys.h"
@@ -60,13 +61,8 @@
 #include "sql/tztime.h"            // my_tz_UTC
 #include "string_with_len.h"
 
-REQUIRES_SERVICE_PLACEHOLDER(component_sys_variable_register);
-REQUIRES_SERVICE_PLACEHOLDER(component_sys_variable_unregister);
-
 class THD;
 
-extern ulong opt_mi_repository_id;
-extern ulong opt_rli_repository_id;
 std::atomic_flag start_stop_executing = ATOMIC_FLAG_INIT;
 
 /*
@@ -183,11 +179,11 @@ int group_replication_start(char **error_message, THD *thd) {
       goto err;
     }
     if (lex != nullptr &&
-        (lex->slave_connection.user || lex->slave_connection.plugin_auth)) {
+        (lex->replica_connection.user || lex->replica_connection.plugin_auth)) {
       if (Rpl_channel_credentials::get_instance().store_credentials(
-              "group_replication_recovery", lex->slave_connection.user,
-              lex->slave_connection.password,
-              lex->slave_connection.plugin_auth)) {
+              "group_replication_recovery", lex->replica_connection.user,
+              lex->replica_connection.password,
+              lex->replica_connection.plugin_auth)) {
         // Parallel START/STOP GR is blocked.
         // So by now START GR command should fail if running or stop should have
         // cleared credentials. Post UNINSTALL we should not reach here.
@@ -492,10 +488,6 @@ void get_server_startup_prerequirements(Trans_context_info &requirements) {
   requirements.binlog_checksum_options = binlog_checksum_options;
   requirements.gtid_mode = global_gtid_mode.get();
   requirements.log_replica_updates = opt_log_replica_updates;
-  requirements.transaction_write_set_extraction =
-      global_system_variables.transaction_write_set_extraction;
-  requirements.mi_repository_type = opt_mi_repository_id;
-  requirements.rli_repository_type = opt_rli_repository_id;
   requirements.parallel_applier_type = mts_parallel_option;
   requirements.parallel_applier_workers = opt_mts_replica_parallel_workers;
   requirements.parallel_applier_preserve_commit_order =
@@ -507,7 +499,7 @@ void get_server_startup_prerequirements(Trans_context_info &requirements) {
 
 bool get_server_encoded_gtid_executed(uchar **encoded_gtid_executed,
                                       size_t *length) {
-  Checkable_rwlock::Guard g(*global_sid_lock, Checkable_rwlock::WRITE_LOCK);
+  Checkable_rwlock::Guard g(*global_tsid_lock, Checkable_rwlock::WRITE_LOCK);
 
   assert(global_gtid_mode.get() != Gtid_mode::OFF);
 
@@ -523,9 +515,9 @@ bool get_server_encoded_gtid_executed(uchar **encoded_gtid_executed,
 
 #if !defined(NDEBUG)
 char *encoded_gtid_set_to_string(uchar *encoded_gtid_set, size_t length) {
-  /* No sid_lock because this is a completely local object. */
-  Sid_map sid_map(nullptr);
-  Gtid_set set(&sid_map);
+  /* No tsid_lock because this is a completely local object. */
+  Tsid_map tsid_map(nullptr);
+  Gtid_set set(&tsid_map);
 
   if (set.add_gtid_encoding(encoded_gtid_set, length) != RETURN_STATUS_OK)
     return nullptr;
@@ -545,7 +537,7 @@ void global_thd_manager_remove_thd(THD *thd) {
 }
 
 bool is_gtid_committed(const Gtid &gtid) {
-  Checkable_rwlock::Guard g(*global_sid_lock, Checkable_rwlock::READ_LOCK);
+  Checkable_rwlock::Guard g(*global_tsid_lock, Checkable_rwlock::READ_LOCK);
 
   assert(global_gtid_mode.get() != Gtid_mode::OFF);
 
@@ -559,12 +551,12 @@ bool is_gtid_committed(const Gtid &gtid) {
 bool wait_for_gtid_set_committed(const char *gtid_set_text, double timeout,
                                  bool update_thd_status) {
   THD *thd = current_thd;
-  Gtid_set wait_for_gtid_set(global_sid_map, nullptr);
+  Gtid_set wait_for_gtid_set(global_tsid_map, nullptr);
 
-  global_sid_lock->rdlock();
+  global_tsid_lock->rdlock();
 
   if (wait_for_gtid_set.add_gtid_text(gtid_set_text) != RETURN_STATUS_OK) {
-    global_sid_lock->unlock();
+    global_tsid_lock->unlock();
     return true;
   }
 
@@ -575,7 +567,7 @@ bool wait_for_gtid_set_committed(const char *gtid_set_text, double timeout,
   */
   if (thd->owned_gtid.sidno > 0 &&
       wait_for_gtid_set.contains_gtid(thd->owned_gtid)) {
-    global_sid_lock->unlock();
+    global_tsid_lock->unlock();
     return true;
   }
 
@@ -584,7 +576,7 @@ bool wait_for_gtid_set_committed(const char *gtid_set_text, double timeout,
                                               update_thd_status);
   gtid_state->end_gtid_wait();
 
-  global_sid_lock->unlock();
+  global_tsid_lock->unlock();
 
   return result;
 }
@@ -630,11 +622,11 @@ std::string get_group_replication_group_name() {
 }
 
 bool get_group_replication_view_change_uuid(std::string &uuid) {
-  my_h_service component_sys_variable_register_service_handler = nullptr;
-  SERVICE_TYPE(component_sys_variable_register)
-  *component_sys_variable_register_service = nullptr;
-  srv_registry->acquire("component_sys_variable_register",
-                        &component_sys_variable_register_service_handler);
+  my_h_service component_sys_variable_reader_service_handler = nullptr;
+  SERVICE_TYPE(mysql_system_variable_reader)
+  *component_sys_variable_reader_service = nullptr;
+  srv_registry->acquire("mysql_system_variable_reader",
+                        &component_sys_variable_reader_service_handler);
 
   char *var_value = nullptr;
   // uuid length + sizeof('\0')
@@ -643,22 +635,23 @@ bool get_group_replication_view_change_uuid(std::string &uuid) {
 
   bool error = false;
 
-  if (nullptr == component_sys_variable_register_service_handler) {
+  if (nullptr == component_sys_variable_reader_service_handler) {
     error = true; /* purecov: inspected */
     goto end;     /* purecov: inspected */
   }
 
-  component_sys_variable_register_service =
-      reinterpret_cast<SERVICE_TYPE(component_sys_variable_register) *>(
-          component_sys_variable_register_service_handler);
+  component_sys_variable_reader_service =
+      reinterpret_cast<SERVICE_TYPE(mysql_system_variable_reader) *>(
+          component_sys_variable_reader_service_handler);
 
   if ((var_value = new char[var_len]) == nullptr) {
     error = true; /* purecov: inspected */
     goto end;     /* purecov: inspected */
   }
 
-  if (!component_sys_variable_register_service->get_variable(
-          "mysql_server", "group_replication_view_change_uuid",
+  if (!component_sys_variable_reader_service->get(
+          nullptr, "GLOBAL", "mysql_server",
+          "group_replication_view_change_uuid",
           reinterpret_cast<void **>(&var_value), &var_len)) {
     uuid.assign(var_value, var_len);
   } else if (var_len != var_buffer_capacity) {
@@ -672,7 +665,7 @@ bool get_group_replication_view_change_uuid(std::string &uuid) {
   }
 
 end:
-  srv_registry->release(component_sys_variable_register_service_handler);
+  srv_registry->release(component_sys_variable_reader_service_handler);
   delete[] var_value;
   return error;
 }
