@@ -1,15 +1,16 @@
-/* Copyright (c) 2006, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2006, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -30,10 +31,10 @@
 #include "include/compression.h"
 #include "include/mutex_lock.h"
 #include "my_dbug.h"
-#include "my_loglevel.h"
 #include "my_sys.h"
+#include "mysql/components/services/bits/psi_stage_bits.h"
 #include "mysql/components/services/log_builtins.h"
-#include "mysql/components/services/psi_stage_bits.h"
+#include "mysql/my_loglevel.h"
 #include "mysql_version.h"
 #include "mysqld_error.h"
 #include "prealloced_array.h"
@@ -42,21 +43,23 @@
 #include "sql/log.h"
 #include "sql/mysqld.h"  // sync_masterinfo_period
 #include "sql/rpl_info_handler.h"
-#include "sql/rpl_msr.h"    // channel_map
-#include "sql/rpl_slave.h"  // master_retry_count
+#include "sql/rpl_msr.h"      // channel_map
+#include "sql/rpl_replica.h"  // source_retry_count
 #include "sql/sql_class.h"
+#include "string_with_len.h"
+#include "strmake.h"
 
 enum {
   LINES_IN_MASTER_INFO_WITH_SSL = 14,
 
-  /* 5.1.16 added value of master_ssl_verify_server_cert */
-  LINE_FOR_MASTER_SSL_VERIFY_SERVER_CERT = 15,
+  /* 5.1.16 added value of source_ssl_verify_server_cert */
+  LINE_FOR_SOURCE_SSL_VERIFY_SERVER_CERT = 15,
 
-  /* 5.5 added value of master_heartbeat_period */
-  LINE_FOR_MASTER_HEARTBEAT_PERIOD = 16,
+  /* 5.5 added value of source_heartbeat_period */
+  LINE_FOR_SOURCE_HEARTBEAT_PERIOD = 16,
 
-  /* MySQL Cluster 6.3 added master_bind */
-  LINE_FOR_MASTER_BIND = 17,
+  /* MySQL Cluster 6.3 added source_bind */
+  LINE_FOR_SOURCE_BIND = 17,
 
   /* 6.0 added value of master_ignore_server_id */
   LINE_FOR_REPLICATE_IGNORE_SERVER_IDS = 18,
@@ -64,8 +67,8 @@ enum {
   /* 6.0 added value of master_uuid */
   LINE_FOR_MASTER_UUID = 19,
 
-  /* line for master_retry_count */
-  LINE_FOR_MASTER_RETRY_COUNT = 20,
+  /* line for source_retry_count */
+  LINE_FOR_SOURCE_RETRY_COUNT = 20,
 
   /* line for ssl_crl */
   LINE_FOR_SSL_CRL = 21,
@@ -82,20 +85,20 @@ enum {
   /* line for tls_version */
   LINE_FOR_TLS_VERSION = 25,
 
-  /* line for master_public_key_path */
+  /* line for source_public_key_path */
   LINE_FOR_PUBLIC_KEY_PATH = 26,
 
-  /* line for get_master_public_key */
+  /* line for get_source_public_key */
   LINE_FOR_GET_PUBLIC_KEY = 27,
 
   /* line for network_namespace */
   LINE_FOR_NETWORK_NAMESPACE = 28,
 
-  /* line for master_compression_algorithm */
-  LINE_FOR_MASTER_COMPRESSION_ALGORITHM = 29,
+  /* line for source_compression_algorithm */
+  LINE_FOR_SOURCE_COMPRESSION_ALGORITHM = 29,
 
-  /* line for master_zstd_compression_level */
-  LINE_FOR_MASTER_ZSTD_COMPRESSION_LEVEL = 30,
+  /* line for source_zstd_compression_level */
+  LINE_FOR_SOURCE_ZSTD_COMPRESSION_LEVEL = 30,
 
   /* line for tls_ciphersuites */
   LINE_FOR_TLS_CIPHERSUITES = 31,
@@ -103,19 +106,22 @@ enum {
   /* line for source_connection_auto_failover */
   LINE_FOR_SOURCE_CONNECTION_AUTO_FAILOVER = 32,
 
+  /* line for gtid_only */
+  LINE_FOR_GTID_ONLY = 33,
+
   /* Number of lines currently used when saving master info file */
-  LINES_IN_MASTER_INFO = LINE_FOR_SOURCE_CONNECTION_AUTO_FAILOVER
+  LINES_IN_MASTER_INFO = LINE_FOR_GTID_ONLY
 
 };
 
 /*
-  Please every time you add a new field to the mater info, update
+  Please every time you add a new field to the master info, update
   what follows. For now, this is just used to get the number of
   fields.
 */
 const char *info_mi_fields[] = {"number_of_lines",
-                                "master_log_name",
-                                "master_log_pos",
+                                "source_log_name",
+                                "source_log_pos",
                                 "host",
                                 "user",
                                 "password",
@@ -141,10 +147,11 @@ const char *info_mi_fields[] = {"number_of_lines",
                                 "public_key_path",
                                 "get_public_key",
                                 "network_namespace",
-                                "master_compression_algorithm",
-                                "master_zstd_compression_level",
+                                "source_compression_algorithm",
+                                "source_zstd_compression_level",
                                 "tls_ciphersuites",
-                                "source_connection_auto_failover"};
+                                "source_connection_auto_failover",
+                                "gtid_only"};
 
 const uint info_mi_table_pk_field_indexes[] = {
     LINE_FOR_CHANNEL - 1,
@@ -187,13 +194,15 @@ Master_info::Master_info(
       received_heartbeats(0),
       last_heartbeat(0),
       master_id(0),
-      checksum_alg_before_fd(binary_log::BINLOG_CHECKSUM_ALG_UNDEF),
-      retry_count(master_retry_count),
+      checksum_alg_before_fd(mysql::binlog::event::BINLOG_CHECKSUM_ALG_UNDEF),
+      retry_count(source_retry_count),
       mi_description_event(nullptr),
       auto_position(false),
-      transaction_parser(
-          Transaction_boundary_parser::TRX_BOUNDARY_PARSER_RECEIVER),
-      reset(false) {
+      transaction_parser(mysql::binlog::event::Transaction_boundary_parser::
+                             TRX_BOUNDARY_PARSER_RECEIVER),
+      reset(false),
+      m_gtid_only_mode(false),
+      m_is_receiver_position_info_invalid(false) {
   host[0] = 0;
   user[0] = 0;
   bind_addr[0] = 0;
@@ -216,10 +225,7 @@ Master_info::Master_info(
   ignore_server_ids = new Server_ids;
   strcpy(compression_algorithm, COMPRESSION_ALGORITHM_UNCOMPRESSED);
   zstd_compression_level = default_zstd_compression_level;
-  server_extn.m_user_data = nullptr;
-  server_extn.m_before_header = nullptr;
-  server_extn.m_after_header = nullptr;
-  server_extn.compress_ctx.algorithm = MYSQL_UNCOMPRESSED;
+  net_server_ext_init(&server_extn);
   gtid_monitoring_info = new Gtid_monitoring_info(&data_lock);
 
   mysql_mutex_init(*key_info_rotate_lock, &this->rotate_lock,
@@ -305,7 +311,7 @@ bool Master_info::is_rotate_requested() {
    Reports if the s_id server has been configured to ignore events
    it generates with
 
-      CHANGE MASTER IGNORE_SERVER_IDS= ( list of server ids )
+      CHANGE REPLICATION SOURCE IGNORE_SERVER_IDS= ( list of server ids )
 
    Method is called from the io thread event receiver filtering.
 
@@ -359,7 +365,7 @@ void Master_info::end_info() {
 
 int Master_info::flush_info(bool force) {
   DBUG_TRACE;
-  DBUG_PRINT("enter", ("master_pos: %lu", (ulong)master_log_pos));
+  DBUG_PRINT("enter", ("source_pos: %lu", (ulong)master_log_pos));
 
   bool skip_flushing = !inited;
   /*
@@ -388,7 +394,7 @@ int Master_info::flush_info(bool force) {
   return 0;
 
 err:
-  LogErr(ERROR_LEVEL, ER_RPL_ERROR_WRITING_MASTER_CONFIGURATION);
+  LogErr(ERROR_LEVEL, ER_RPL_ERROR_WRITING_SOURCE_CONFIGURATION);
   return 1;
 }
 
@@ -425,7 +431,7 @@ int Master_info::mi_init_info() {
 err:
   handler->end_info();
   inited = false;
-  LogErr(ERROR_LEVEL, ER_RPL_ERROR_READING_MASTER_CONFIGURATION);
+  LogErr(ERROR_LEVEL, ER_RPL_ERROR_READING_SOURCE_CONFIGURATION);
   return 1;
 }
 
@@ -522,22 +528,22 @@ bool Master_info::read_info(Rpl_info_handler *from) {
     Starting from 5.1.16 ssl_verify_server_cert might be
     in the file
   */
-  if (lines >= LINE_FOR_MASTER_SSL_VERIFY_SERVER_CERT) {
+  if (lines >= LINE_FOR_SOURCE_SSL_VERIFY_SERVER_CERT) {
     if (!!from->get_info(&temp_ssl_verify_server_cert, 0)) return true;
   }
 
   /*
-    Starting from 5.5 master_heartbeat_period might be
+    Starting from 5.5 source_heartbeat_period might be
     in the file
   */
-  if (lines >= LINE_FOR_MASTER_HEARTBEAT_PERIOD) {
+  if (lines >= LINE_FOR_SOURCE_HEARTBEAT_PERIOD) {
     if (!!from->get_info(&heartbeat_period, (float)0.0)) return true;
   }
 
   /*
-    Starting from 5.5 master_bind might be in the file
+    Starting from 5.5 source_bind might be in the file
   */
-  if (lines >= LINE_FOR_MASTER_BIND) {
+  if (lines >= LINE_FOR_SOURCE_BIND) {
     if (!!from->get_info(bind_addr, sizeof(bind_addr), "")) return true;
   }
 
@@ -555,10 +561,10 @@ bool Master_info::read_info(Rpl_info_handler *from) {
       return true;
   }
 
-  /* Starting from 5.5 the master_retry_count may be in the repository. */
-  retry_count = master_retry_count;
-  if (lines >= LINE_FOR_MASTER_RETRY_COUNT) {
-    if (!!from->get_info(&retry_count, master_retry_count)) return true;
+  /* Starting from 5.5 the source_retry_count may be in the repository. */
+  retry_count = source_retry_count;
+  if (lines >= LINE_FOR_SOURCE_RETRY_COUNT) {
+    if (!!from->get_info(&retry_count, source_retry_count)) return true;
   }
 
   if (lines >= LINE_FOR_SSL_CRLPATH) {
@@ -603,7 +609,7 @@ bool Master_info::read_info(Rpl_info_handler *from) {
   auto_position = temp_auto_position;
   get_public_key = (bool)temp_get_public_key;
 
-  if (lines >= LINE_FOR_MASTER_COMPRESSION_ALGORITHM) {
+  if (lines >= LINE_FOR_SOURCE_COMPRESSION_ALGORITHM) {
     char algorithm_name[COMPRESSION_ALGORITHM_NAME_BUFFER_SIZE];
     if (!!from->get_info(algorithm_name, sizeof(algorithm_name), nullptr))
       return true;
@@ -619,7 +625,7 @@ bool Master_info::read_info(Rpl_info_handler *from) {
     }
   }
 
-  if (lines >= LINE_FOR_MASTER_ZSTD_COMPRESSION_LEVEL) {
+  if (lines >= LINE_FOR_SOURCE_ZSTD_COMPRESSION_LEVEL) {
     int level;
     if (!!from->get_info(&level, (int)0)) return true;
     if (is_zstd_compression_level_valid(level))
@@ -658,6 +664,14 @@ bool Master_info::read_info(Rpl_info_handler *from) {
     m_source_connection_auto_failover = temp_source_connection_auto_failover;
   }
 
+  auto temp_gtid_only{0};
+  if (lines >= LINE_FOR_GTID_ONLY) {
+    if (!!from->get_info(&temp_gtid_only, 0)) return true;
+  } else {
+    if (channel_map.is_group_replication_channel_name(channel))
+      temp_gtid_only = 1;
+  }
+  m_gtid_only_mode = temp_gtid_only;
   return false;
 }
 
@@ -697,7 +711,8 @@ bool Master_info::write_info(Rpl_info_handler *to) {
       to->set_info((int)zstd_compression_level) ||
       to->set_info(tls_ciphersuites.first ? nullptr
                                           : tls_ciphersuites.second.c_str()) ||
-      to->set_info((int)m_source_connection_auto_failover))
+      to->set_info((int)m_source_connection_auto_failover) ||
+      to->set_info((int)m_gtid_only_mode))
     return true;
 
   return false;
@@ -780,8 +795,22 @@ void Master_info::update_flushed_relay_log_info() {
   }
 }
 
-void Master_info::get_flushed_relay_log_info(LOG_INFO *linfo) {
+void Master_info::get_flushed_relay_log_info(Log_info *linfo) {
   strmake(linfo->log_file_name, flushed_relay_log_info.log_file_name,
           sizeof(linfo->log_file_name) - 1);
   linfo->pos = flushed_relay_log_info.pos;
 }
+
+void Master_info::set_receiver_position_info_invalid(bool invalid) {
+  m_is_receiver_position_info_invalid = invalid;
+}
+
+bool Master_info::is_receiver_position_info_invalid() const {
+  return m_is_receiver_position_info_invalid;
+}
+
+void Master_info::set_gtid_only_mode(bool gtid_only_mode) {
+  m_gtid_only_mode = gtid_only_mode;
+}
+
+bool Master_info::is_gtid_only_mode() const { return m_gtid_only_mode; }
