@@ -29,11 +29,10 @@
 #include <atomic>
 
 #include "compression.h"  // COMPRESSION_ALGORITHM_NAME_BUFFER_SIZE
-#include "libbinlogevents/include/binlog_event.h"  // enum_binlog_checksum_alg
-#include "m_string.h"
 #include "my_inttypes.h"
 #include "my_io.h"
 #include "my_psi_config.h"
+#include "mysql/binlog/event/binlog_event.h"  // enum_binlog_checksum_alg
 #include "mysql/components/services/bits/psi_mutex_bits.h"
 #include "mysql/psi/mysql_mutex.h"
 #include "mysql_com.h"
@@ -44,6 +43,7 @@
 #include "sql/rpl_rli.h"                  // rli->get_log_lock()
 #include "sql/rpl_trx_boundary_parser.h"  // Transaction_boundary_parser
 #include "sql/sql_const.h"
+#include "strmake.h"
 
 class Rpl_info_handler;
 class Server_ids;
@@ -70,11 +70,11 @@ struct MYSQL;
 
   log_name
   log_pos
-  master_host
-  master_user
-  master_pass
-  master_port
-  master_connect_retry
+  source_host
+  source_user
+  source_pass
+  source_port
+  source_connect_retry
 
   To write out the contents of master.info to disk a call to flush_info()
   is required. Currently, it is needed every time we read and queue data
@@ -88,6 +88,11 @@ class Master_info : public Rpl_info {
   friend class Rpl_info_factory;
 
  public:
+  /// In case we start replication from the first binary log file and
+  /// source log name is empty, we use first_source_log_name instead of
+  /// 'master_log_name' in the error log
+  static constexpr auto first_source_log_name = "<First log>";
+
   /**
     Host name or ip address stored in the master.info.
   */
@@ -105,7 +110,7 @@ class Master_info : public Rpl_info {
 
  private:
   /**
-    If true, USER/PASSWORD was specified when running START SLAVE.
+    If true, USER/PASSWORD was specified when running START REPLICA.
   */
   bool start_user_configured;
   /**
@@ -117,20 +122,20 @@ class Master_info : public Rpl_info {
   */
   char password[MAX_PASSWORD_LENGTH + 1];
   /**
-    User specified when running START SLAVE.
+    User specified when running START REPLICA.
   */
   char start_user[USERNAME_LENGTH + 1];
   /**
-    Password specified when running START SLAVE.
+    Password specified when running START REPLICA.
   */
   char start_password[MAX_PASSWORD_LENGTH + 1];
   /**
-    Stores the authentication plugin specified when running START SLAVE.
+    Stores the authentication plugin specified when running START REPLICA.
   */
   char start_plugin_auth[FN_REFLEN + 1];
   /**
     Stores the authentication plugin directory specified when running
-    START SLAVE.
+    START REPLICA.
   */
   char start_plugin_dir[FN_REFLEN + 1];
 
@@ -167,13 +172,13 @@ class Master_info : public Rpl_info {
  public:
   /**
     Returns if USER/PASSWORD was specified when running
-    START SLAVE.
+    START REPLICA.
 
     @return true or false.
   */
   bool is_start_user_configured() const { return start_user_configured; }
   /**
-    Returns if DEFAULT_AUTH was specified when running START SLAVE.
+    Returns if DEFAULT_AUTH was specified when running START REPLICA.
 
     @return true or false.
   */
@@ -181,7 +186,7 @@ class Master_info : public Rpl_info {
     return (start_plugin_auth[0] != 0);
   }
   /**
-    Returns if PLUGIN_DIR was specified when running START SLAVE.
+    Returns if PLUGIN_DIR was specified when running START REPLICA.
 
     @return true or false.
   */
@@ -190,7 +195,7 @@ class Master_info : public Rpl_info {
   }
   /**
     Defines that USER/PASSWORD was specified or not when running
-    START SLAVE.
+    START REPLICA.
 
     @param config is true or false.
   */
@@ -199,7 +204,7 @@ class Master_info : public Rpl_info {
   }
   /**
     Sets either user's name in the master.info repository when CHANGE
-    MASTER is executed or user's name used in START SLAVE if USER is
+    MASTER is executed or user's name used in START REPLICA if USER is
     specified.
 
     @param user_arg is user's name.
@@ -220,7 +225,7 @@ class Master_info : public Rpl_info {
     return (start_user_configured ? sizeof(start_user) : sizeof(user));
   }
   /**
-    If an user was specified when running START SLAVE, this function returns
+    If an user was specified when running START REPLICA, this function returns
     such user. Otherwise, it returns the user stored in master.info.
 
     @return user's name.
@@ -230,7 +235,7 @@ class Master_info : public Rpl_info {
   }
   /**
     Stores either user's password in the master.info repository when CHANGE
-    MASTER is executed or user's password used in START SLAVE if PASSWORD
+    MASTER is executed or user's password used in START REPLICA if PASSWORD
     is specified.
 
     @param password_arg is user's password.
@@ -239,7 +244,7 @@ class Master_info : public Rpl_info {
   void set_password(const char *password_arg);
   /**
     Returns either user's password in the master.info repository or
-    user's password used in START SLAVE.
+    user's password used in START REPLICA.
 
     @param[out] password_arg is user's password.
     @param[out] password_arg_size is user's password size.
@@ -248,29 +253,29 @@ class Master_info : public Rpl_info {
   */
   bool get_password(char *password_arg, size_t *password_arg_size);
   /**
-    Cleans in-memory password defined by START SLAVE.
+    Cleans in-memory password defined by START REPLICA.
   */
   void reset_start_info();
   /**
-    Returns the DEFAULT_AUTH defined by START SLAVE.
+    Returns the DEFAULT_AUTH defined by START REPLICA.
 
     @return DEFAULT_AUTH.
   */
   const char *get_start_plugin_auth() { return start_plugin_auth; }
   /**
-    Returns the PLUGIN_DIR defined by START SLAVE.
+    Returns the PLUGIN_DIR defined by START REPLICA.
 
     @return PLUGIN_DIR.
   */
   const char *get_start_plugin_dir() { return start_plugin_dir; }
   /**
-    Stores the DEFAULT_AUTH defined by START SLAVE.
+    Stores the DEFAULT_AUTH defined by START REPLICA.
   */
   void set_plugin_auth(const char *src) {
     if (src) strmake(start_plugin_auth, src, sizeof(start_plugin_auth) - 1);
   }
   /**
-    Stores the DEFAULT_AUTH defined by START SLAVE.
+    Stores the DEFAULT_AUTH defined by START REPLICA.
   */
   void set_plugin_dir(const char *src) {
     if (src) strmake(start_plugin_dir, src, sizeof(start_plugin_dir) - 1);
@@ -310,7 +315,8 @@ class Master_info : public Rpl_info {
 
   */
   long clock_diff_with_master;
-  float heartbeat_period;         // interface with CHANGE MASTER or master.info
+  float heartbeat_period;         // interface with CHANGE REPLICATION SOURCE or
+                                  // master.info
   ulonglong received_heartbeats;  // counter of received heartbeat events
 
   ulonglong last_heartbeat;
@@ -323,7 +329,7 @@ class Master_info : public Rpl_info {
     Initialized to novalue, then set to the queried from master
     @@global.binlog_checksum and deactivated once FD has been received.
   */
-  binary_log::enum_binlog_checksum_alg checksum_alg_before_fd;
+  mysql::binlog::event::enum_binlog_checksum_alg checksum_alg_before_fd;
   ulong retry_count;
   char master_uuid[UUID_LENGTH + 1];
   char bind_addr[HOSTNAME_LENGTH + 1];
@@ -662,10 +668,10 @@ class Master_info : public Rpl_info {
     This is the channel lock. It is a rwlock used to serialize all replication
     administrative commands that cannot be performed concurrently for a given
     replication channel:
-    - START SLAVE;
-    - STOP SLAVE;
-    - CHANGE MASTER;
-    - RESET SLAVE;
+    - START REPLICA;
+    - STOP REPLICA;
+    - CHANGE REPLICATION SOURCE;
+    - RESET REPLICA;
     - end_slave() (when mysqld stops)).
     Any of these commands must hold the wrlock from the start till the end.
   */
@@ -731,7 +737,7 @@ class Master_info : public Rpl_info {
   */
   void wait_until_no_reference(THD *thd);
 
-  /* Set true when the Master_info object was cleared by a RESET SLAVE */
+  /* Set true when the Master_info object was cleared by a RESET REPLICA */
   bool reset;
 
   /**
@@ -758,7 +764,7 @@ class Master_info : public Rpl_info {
     @param [out] linfo Where the relay log coordinates shall be stored.
   */
 
-  void get_flushed_relay_log_info(LOG_INFO *linfo);
+  void get_flushed_relay_log_info(Log_info *linfo);
 
   /**
     Marks the receiver position information (master_log_name, master_log_pos)
@@ -796,7 +802,7 @@ class Master_info : public Rpl_info {
     Holds the relay log coordinates (file name and position) of the last master
     coordinates flushed into Master_info repository.
   */
-  LOG_INFO flushed_relay_log_info;
+  Log_info flushed_relay_log_info;
 
   /**
     Is the replica working in GTID only mode, meaning it does not
