@@ -1,15 +1,16 @@
-/* Copyright (c) 2021, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2021, 2024, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
 as published by the Free Software Foundation.
 
-This program is also distributed with certain software (including
+This program is designed to work with certain software (including
 but not limited to OpenSSL) that is licensed under separate terms,
 as designated in a particular file or component or in included license
 documentation.  The authors of MySQL hereby grant you an additional
 permission to link the program and your derivative works with the
-separately licensed software that they have included with MySQL.
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -46,7 +47,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 /**
   Return the system variable type given a type name.
 */
-static enum_var_type sysvar_type(const char *type_name) {
+enum_var_type sysvar_type(const char *type_name) {
   if (type_name) {
     if (!strcmp(type_name, "GLOBAL"))
       return OPT_GLOBAL;
@@ -121,6 +122,11 @@ static bool common_system_variable_update_set(Set_variables_helper &hlp,
                                               my_h_string variable_base,
                                               my_h_string variable_name,
                                               Item *variable_value) {
+  /* Cannot set system variable when server is shutting down. */
+  rwlock_scoped_lock rdlock(&LOCK_server_shutting_down, false, __FILE__,
+                            __LINE__);
+  if (server_shutting_down) return true;
+
   String *base = reinterpret_cast<String *>(variable_base);
   String *name = reinterpret_cast<String *>(variable_name);
 
