@@ -1,15 +1,16 @@
-/* Copyright (c) 2016, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2024, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
 as published by the Free Software Foundation.
 
-This program is also distributed with certain software (including
+This program is designed to work with certain software (including
 but not limited to OpenSSL) that is licensed under separate terms,
 as designated in a particular file or component or in included license
 documentation.  The authors of MySQL hereby grant you an additional
 permission to link the program and your derivative works with the
-separately licensed software that they have included with MySQL.
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -41,7 +42,9 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 #include "mysql/components/services/mysql_query_attributes.h"
 #include "mysql/components/services/mysql_runtime_error_service.h"
 #include "mysql/components/services/mysql_rwlock_service.h"
+#include "mysql/components/services/mysql_signal_handler.h"
 #include "mysql/components/services/mysql_simple_error_log.h"
+#include "mysql/components/services/mysql_statement_service.h"
 #include "mysql/components/services/mysql_status_variable_reader.h"
 #include "mysql/components/services/mysql_system_variable.h"
 #include "mysql/components/services/table_access_service.h"
@@ -66,6 +69,7 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 #include "log_builtins_filter_imp.h"
 #include "log_builtins_imp.h"
 #include "log_sink_perfschema_imp.h"
+#include "my_compiler.h"
 #include "my_inttypes.h"
 #include "mysql_audit_print_service_double_data_source_imp.h"
 #include "mysql_audit_print_service_longlong_data_source_imp.h"
@@ -76,16 +80,20 @@ Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 #include "mysql_command_services_imp.h"
 #include "mysql_connection_attributes_iterator_imp.h"
 #include "mysql_current_thread_reader_imp.h"
+#include "mysql_global_variable_attributes_service_imp.h"
 #include "mysql_ongoing_transaction_query_imp.h"
 #include "mysql_page_track_imp.h"
 #include "mysql_runtime_error_imp.h"
 #include "mysql_server_event_tracking_bridge_imp.h"
 #include "mysql_server_keyring_lockable_imp.h"
 #include "mysql_server_runnable_imp.h"
+#include "mysql_signal_handler_imp.h"
 #include "mysql_simple_error_log_imp.h"
+#include "mysql_statement_service_imp.h"
 #include "mysql_status_variable_reader_imp.h"
 #include "mysql_stored_program_imp.h"
 #include "mysql_string_service_imp.h"
+#include "mysql_system_variable_reader_imp.h"
 #include "mysql_system_variable_update_imp.h"
 #include "mysql_thd_attributes_imp.h"
 #include "mysql_thd_store_imp.h"
@@ -293,9 +301,19 @@ mysql_udf_metadata_imp::argument_get, mysql_udf_metadata_imp::result_get,
     mysql_udf_metadata_imp::argument_set,
     mysql_udf_metadata_imp::result_set END_SERVICE_IMPLEMENTATION();
 
+/*
+ Here the new mysql_service_mysql_system_variable_reader->get() service cannot
+ be used instead of get_variable because the below code is for the
+ implementation of the old get_variable() service. Hence adding below
+ suppression deprecated warning code to skip the deprecation warning while
+ compiling the code.
+*/
+MY_COMPILER_DIAGNOSTIC_PUSH()
+MY_COMPILER_CLANG_DIAGNOSTIC_IGNORE("-Wdeprecated-declarations")
 BEGIN_SERVICE_IMPLEMENTATION(mysql_server, component_sys_variable_register)
 mysql_component_sys_variable_imp::register_variable,
     mysql_component_sys_variable_imp::get_variable END_SERVICE_IMPLEMENTATION();
+MY_COMPILER_DIAGNOSTIC_POP()
 
 BEGIN_SERVICE_IMPLEMENTATION(mysql_server, mysql_connection_attributes_iterator)
 mysql_connection_attributes_iterator_imp::init,
@@ -334,7 +352,8 @@ Bulk_data_convert::mysql_format, Bulk_data_convert::mysql_format_from_raw,
     Bulk_data_convert::get_row_metadata END_SERVICE_IMPLEMENTATION();
 
 BEGIN_SERVICE_IMPLEMENTATION(mysql_server, bulk_data_load)
-Bulk_data_load::begin, Bulk_data_load::load, Bulk_data_load::end,
+Bulk_data_load::begin, Bulk_data_load::load, Bulk_data_load::open_blob,
+    Bulk_data_load::write_blob, Bulk_data_load::close_blob, Bulk_data_load::end,
     Bulk_data_load::is_table_supported, Bulk_data_load::get_se_memory_size,
     END_SERVICE_IMPLEMENTATION();
 
@@ -474,6 +493,9 @@ mysql_system_variable_update_imp::set_signed,
 BEGIN_SERVICE_IMPLEMENTATION(mysql_server, mysql_system_variable_update_default)
 mysql_system_variable_update_imp::set_default END_SERVICE_IMPLEMENTATION();
 
+BEGIN_SERVICE_IMPLEMENTATION(mysql_server, mysql_system_variable_reader)
+mysql_system_variable_reader_imp::get END_SERVICE_IMPLEMENTATION();
+
 BEGIN_SERVICE_IMPLEMENTATION(mysql_server, mysql_thd_attributes)
 mysql_thd_attributes_imp::get,
     mysql_thd_attributes_imp::set END_SERVICE_IMPLEMENTATION();
@@ -656,6 +678,12 @@ BEGIN_SERVICE_IMPLEMENTATION(mysql_server, mysql_debug_sync_service)
 mysql_debug_sync_service_imp::debug_sync END_SERVICE_IMPLEMENTATION();
 #endif
 
+BEGIN_SERVICE_IMPLEMENTATION(mysql_server,
+                             mysql_stored_program_external_program_handle)
+mysql_stored_program_external_program_handle_imp::get,
+    mysql_stored_program_external_program_handle_imp::set,
+    END_SERVICE_IMPLEMENTATION();
+
 BEGIN_SERVICE_IMPLEMENTATION(mysql_server, mysql_stored_program_metadata_query)
 mysql_stored_program_metadata_query_imp::get END_SERVICE_IMPLEMENTATION();
 
@@ -713,6 +741,11 @@ mysql_stored_program_runtime_argument_string_imp::get,
     mysql_stored_program_runtime_argument_string_imp::set
     END_SERVICE_IMPLEMENTATION();
 
+BEGIN_SERVICE_IMPLEMENTATION(
+    mysql_server, mysql_stored_program_runtime_argument_string_charset)
+mysql_stored_program_runtime_argument_string_charset_imp::set
+END_SERVICE_IMPLEMENTATION();
+
 BEGIN_SERVICE_IMPLEMENTATION(mysql_server,
                              mysql_stored_program_runtime_argument_int)
 mysql_stored_program_runtime_argument_int_imp::get,
@@ -761,6 +794,11 @@ BEGIN_SERVICE_IMPLEMENTATION(mysql_server,
                              mysql_stored_program_return_value_string)
 mysql_stored_program_return_value_string_imp::set END_SERVICE_IMPLEMENTATION();
 
+BEGIN_SERVICE_IMPLEMENTATION(mysql_server,
+                             mysql_stored_program_return_value_string_charset)
+mysql_stored_program_return_value_string_charset_imp::set
+END_SERVICE_IMPLEMENTATION();
+
 BEGIN_SERVICE_IMPLEMENTATION(mysql_server,
                              mysql_stored_program_return_value_int)
 mysql_stored_program_return_value_int_imp::set, END_SERVICE_IMPLEMENTATION();
@@ -777,6 +815,67 @@ mysql_stored_program_return_value_float_imp::set, END_SERVICE_IMPLEMENTATION();
 BEGIN_SERVICE_IMPLEMENTATION(mysql_server, mysql_simple_error_log)
 mysql_simple_error_log_imp::emit END_SERVICE_IMPLEMENTATION();
 
+BEGIN_SERVICE_IMPLEMENTATION(mysql_server, mysql_stmt_factory)
+mysql_stmt_factory_imp::init, mysql_stmt_factory_imp::close,
+    END_SERVICE_IMPLEMENTATION();
+
+BEGIN_SERVICE_IMPLEMENTATION(mysql_server, mysql_stmt_execute)
+mysql_stmt_execute_imp::execute, mysql_stmt_execute_imp::prepare,
+    mysql_stmt_execute_imp::reset, END_SERVICE_IMPLEMENTATION();
+
+BEGIN_SERVICE_IMPLEMENTATION(mysql_server, mysql_stmt_execute_direct)
+mysql_stmt_execute_direct_imp::execute, END_SERVICE_IMPLEMENTATION();
+
+BEGIN_SERVICE_IMPLEMENTATION(mysql_server, mysql_stmt_bind)
+mysql_stmt_bind_imp::bind_param, END_SERVICE_IMPLEMENTATION();
+
+BEGIN_SERVICE_IMPLEMENTATION(mysql_server, mysql_stmt_result)
+
+mysql_stmt_result_imp::next_result, mysql_stmt_result_imp::fetch,
+    END_SERVICE_IMPLEMENTATION();
+
+BEGIN_SERVICE_IMPLEMENTATION(mysql_server, mysql_stmt_diagnostics)
+mysql_stmt_diagnostics_imp::affected_rows,
+    mysql_stmt_diagnostics_imp::insert_id, mysql_stmt_diagnostics_imp::error_id,
+    mysql_stmt_diagnostics_imp::error, mysql_stmt_diagnostics_imp::sqlstate,
+    mysql_stmt_diagnostics_imp::num_warnings,
+    mysql_stmt_diagnostics_imp::get_warning,
+    mysql_stmt_diagnostics_imp::warning_level,
+    mysql_stmt_diagnostics_imp::warning_code,
+    mysql_stmt_diagnostics_imp::warning_message END_SERVICE_IMPLEMENTATION();
+
+BEGIN_SERVICE_IMPLEMENTATION(mysql_server, mysql_stmt_attributes)
+mysql_stmt_attributes_imp::get, mysql_stmt_attributes_imp::set,
+    END_SERVICE_IMPLEMENTATION();
+
+BEGIN_SERVICE_IMPLEMENTATION(mysql_server, mysql_stmt_metadata)
+mysql_stmt_metadata_imp::param_count, mysql_stmt_metadata_imp::param_metadata,
+    END_SERVICE_IMPLEMENTATION();
+
+BEGIN_SERVICE_IMPLEMENTATION(mysql_server, mysql_stmt_resultset_metadata)
+mysql_stmt_resultset_metadata_imp::fetch_field,
+    mysql_stmt_resultset_metadata_imp::field_count,
+    mysql_stmt_resultset_metadata_imp::field_info, END_SERVICE_IMPLEMENTATION();
+
+BEGIN_SERVICE_IMPLEMENTATION(mysql_server, mysql_stmt_get_string)
+mysql_stmt_get_string_imp::get, END_SERVICE_IMPLEMENTATION();
+
+BEGIN_SERVICE_IMPLEMENTATION(mysql_server, mysql_stmt_get_time)
+mysql_stmt_get_time_imp::get, END_SERVICE_IMPLEMENTATION();
+
+BEGIN_SERVICE_IMPLEMENTATION(mysql_server, mysql_stmt_get_double)
+mysql_stmt_get_double_imp::get, END_SERVICE_IMPLEMENTATION();
+
+BEGIN_SERVICE_IMPLEMENTATION(mysql_server, mysql_stmt_get_unsigned_integer)
+mysql_stmt_get_unsigned_integer_imp::get, END_SERVICE_IMPLEMENTATION();
+
+BEGIN_SERVICE_IMPLEMENTATION(mysql_server, mysql_stmt_get_integer)
+mysql_stmt_get_integer_imp::get, END_SERVICE_IMPLEMENTATION();
+
+BEGIN_SERVICE_IMPLEMENTATION(mysql_server, my_signal_handler)
+my_signal_handler_imp::add, my_signal_handler_imp::remove,
+    END_SERVICE_IMPLEMENTATION();
+
 BEGIN_COMPONENT_PROVIDES(mysql_server)
 PROVIDES_SERVICE(mysql_server_path_filter, dynamic_loader_scheme_file),
     PROVIDES_SERVICE(mysql_server, persistent_dynamic_loader),
@@ -908,6 +1007,12 @@ PROVIDES_SERVICE(mysql_server_path_filter, dynamic_loader_scheme_file),
     PROVIDES_SERVICE(performance_schema, psi_metric_v1),
     PROVIDES_SERVICE(performance_schema, pfs_plugin_column_text_v1),
 
+    PROVIDES_SERVICE(mysql_server, mysql_global_variable_attributes),
+    PROVIDES_SERVICE(mysql_server, mysql_global_variable_attributes_iterator),
+
+    PROVIDES_SERVICE(mysql_server, pfs_notification_v3),
+    PROVIDES_SERVICE(mysql_server, pfs_resource_group_v3),
+
     PROVIDES_SERVICE(mysql_server, mysql_query_attributes_iterator),
     PROVIDES_SERVICE(mysql_server, mysql_query_attribute_string),
     PROVIDES_SERVICE(mysql_server, mysql_query_attribute_isnull),
@@ -923,6 +1028,7 @@ PROVIDES_SERVICE(mysql_server_path_filter, dynamic_loader_scheme_file),
     PROVIDES_SERVICE(mysql_server, mysql_system_variable_update_string),
     PROVIDES_SERVICE(mysql_server, mysql_system_variable_update_integer),
     PROVIDES_SERVICE(mysql_server, mysql_system_variable_update_default),
+    PROVIDES_SERVICE(mysql_server, mysql_system_variable_reader),
 
     PROVIDES_SERVICE(mysql_server, table_access_factory_v1),
     PROVIDES_SERVICE(mysql_server, table_access_v1),
@@ -962,6 +1068,8 @@ PROVIDES_SERVICE(mysql_server_path_filter, dynamic_loader_scheme_file),
     PROVIDES_SERVICE(mysql_server, mysql_command_field_metadata),
     PROVIDES_SERVICE(mysql_server, dynamic_loader_services_loaded_notification),
     PROVIDES_SERVICE(mysql_server, dynamic_loader_services_unload_notification),
+    PROVIDES_SERVICE(mysql_server,
+                     mysql_stored_program_external_program_handle),
     PROVIDES_SERVICE(mysql_server, mysql_stored_program_metadata_query),
     PROVIDES_SERVICE(mysql_server,
                      mysql_stored_program_argument_metadata_query),
@@ -977,6 +1085,8 @@ PROVIDES_SERVICE(mysql_server_path_filter, dynamic_loader_scheme_file),
     PROVIDES_SERVICE(mysql_server, mysql_stored_program_runtime_argument_null),
     PROVIDES_SERVICE(mysql_server,
                      mysql_stored_program_runtime_argument_string),
+    PROVIDES_SERVICE(mysql_server,
+                     mysql_stored_program_runtime_argument_string_charset),
     PROVIDES_SERVICE(mysql_server, mysql_stored_program_runtime_argument_int),
     PROVIDES_SERVICE(mysql_server,
                      mysql_stored_program_runtime_argument_unsigned_int),
@@ -988,17 +1098,36 @@ PROVIDES_SERVICE(mysql_server_path_filter, dynamic_loader_scheme_file),
     PROVIDES_SERVICE(mysql_server, mysql_stored_program_return_value_timestamp),
     PROVIDES_SERVICE(mysql_server, mysql_stored_program_return_value_null),
     PROVIDES_SERVICE(mysql_server, mysql_stored_program_return_value_string),
+    PROVIDES_SERVICE(mysql_server,
+                     mysql_stored_program_return_value_string_charset),
     PROVIDES_SERVICE(mysql_server, mysql_stored_program_return_value_int),
     PROVIDES_SERVICE(mysql_server,
                      mysql_stored_program_return_value_unsigned_int),
     PROVIDES_SERVICE(mysql_server, mysql_stored_program_return_value_float),
     PROVIDES_SERVICE(mysql_server, thread_cleanup_register),
     PROVIDES_SERVICE(mysql_server, mysql_simple_error_log),
+    PROVIDES_SERVICE(mysql_server, mysql_stmt_factory),
+    PROVIDES_SERVICE(mysql_server, mysql_stmt_execute),
+    PROVIDES_SERVICE(mysql_server, mysql_stmt_execute_direct),
+    PROVIDES_SERVICE(mysql_server, mysql_stmt_attributes),
+    PROVIDES_SERVICE(mysql_server, mysql_stmt_bind),
+    PROVIDES_SERVICE(mysql_server, mysql_stmt_get_string),
+    PROVIDES_SERVICE(mysql_server, mysql_stmt_get_time),
+    PROVIDES_SERVICE(mysql_server, mysql_stmt_get_double),
+    PROVIDES_SERVICE(mysql_server, mysql_stmt_get_unsigned_integer),
+    PROVIDES_SERVICE(mysql_server, mysql_stmt_get_integer),
+    PROVIDES_SERVICE(mysql_server, mysql_stmt_diagnostics),
+    PROVIDES_SERVICE(mysql_server, mysql_stmt_metadata),
+    PROVIDES_SERVICE(mysql_server, mysql_stmt_resultset_metadata),
+    PROVIDES_SERVICE(mysql_server, mysql_stmt_result),
 #if !defined(NDEBUG)
     PROVIDES_SERVICE(mysql_server, mysql_debug_keyword_service),
     PROVIDES_SERVICE(mysql_server, mysql_debug_sync_service),
 #endif
     PROVIDES_SERVICE(mysql_server, dynamic_privilege_deprecation),
+    PROVIDES_SERVICE(mysql_server, my_signal_handler),
+
+    PROVIDES_SERVICE(mysql_server, table_access_binlog),
     END_COMPONENT_PROVIDES();
 
 static BEGIN_COMPONENT_REQUIRES(mysql_server) END_COMPONENT_REQUIRES();
