@@ -1,15 +1,16 @@
-/* Copyright (c) 2020, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2020, 2024, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License, version 2.0,
 as published by the Free Software Foundation.
 
-This program is also distributed with certain software (including
+This program is designed to work with certain software (including
 but not limited to OpenSSL) that is licensed under separate terms,
 as designated in a particular file or component or in included license
 documentation.  The authors of MySQL hereby grant you an additional
 permission to link the program and your derivative works with the
-separately licensed software that they have included with MySQL.
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -518,6 +519,9 @@ class Table_access_impl {
   int rollback();
   TABLE *get_table(size_t index);
 
+  bool binlog_get();
+  void binlog_set(bool binlog);
+
  private:
   /** Array of size m_max_count. */
   Table_ref *m_table_array;
@@ -739,6 +743,17 @@ size_t Table_access_impl::add_table(const char *schema_name,
 }
 
 int Table_access_impl::begin() {
+  /*
+    Read lock must be acquired during entire open_and_lock_tables function
+    call, because shutdown process can make its internals unavailable in the
+    middle of the call. If tables are acquired before the shutdown process, the
+    shutdown process will not deallocate internals until tables are closed.
+   */
+  rwlock_scoped_lock rdlock(&LOCK_server_shutting_down, false, __FILE__,
+                            __LINE__);
+
+  if (server_shutting_down) return TA_ERROR_OPEN;
+
   if (m_write && m_parent_thd != nullptr) {
     if (m_parent_thd->global_read_lock.is_acquired()) {
       /*
@@ -779,6 +794,19 @@ TABLE *Table_access_impl::get_table(size_t index) {
   return nullptr;
 }
 
+bool Table_access_impl::binlog_get() {
+  assert(!m_in_tx);
+  return 0 != (m_child_thd->variables.option_bits & OPTION_BIN_LOG);
+}
+
+void Table_access_impl::binlog_set(bool binlog) {
+  assert(!m_in_tx);
+  if (binlog)
+    m_child_thd->variables.option_bits |= OPTION_BIN_LOG;
+  else
+    m_child_thd->variables.option_bits &= ~OPTION_BIN_LOG;
+}
+
 int Table_access_impl::commit() {
   assert(m_in_tx);
   m_in_tx = false;
@@ -1333,6 +1361,17 @@ int impl_get_field_any_value(Table_access /* api_ta */, TA_table api_table,
   return 0;
 }
 
+static bool impl_ta_binlog_get(Table_access api_ta) {
+  Table_access_impl *ta = Table_access_impl::from_api(api_ta);
+  assert(ta != nullptr);
+  return ta->binlog_get();
+}
+static void impl_ta_binlog_set(Table_access api_ta, bool binlog) {
+  Table_access_impl *ta = Table_access_impl::from_api(api_ta);
+  assert(ta != nullptr);
+  ta->binlog_set(binlog);
+}
+
 SERVICE_TYPE(table_access_factory_v1)
 SERVICE_IMPLEMENTATION(mysql_server, table_access_factory_v1) = {
     impl_create_table_access, impl_destroy_table_access};
@@ -1370,3 +1409,7 @@ SERVICE_IMPLEMENTATION(mysql_server, field_varchar_access_v1) = {
 SERVICE_TYPE(field_any_access_v1)
 SERVICE_IMPLEMENTATION(mysql_server, field_any_access_v1) = {
     impl_set_field_any_value, impl_get_field_any_value};
+
+SERVICE_TYPE(table_access_binlog)
+SERVICE_IMPLEMENTATION(mysql_server, table_access_binlog) = {
+    impl_ta_binlog_get, impl_ta_binlog_set};
