@@ -1,15 +1,16 @@
-/* Copyright (c) 2016, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2016, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -111,8 +112,9 @@ static bool populate_table(THD *thd, LEX *lex) {
 
   if (lock_tables(thd, lex->query_tables, lex->table_count, 0)) return true;
 
-  if (unit->optimize(thd, nullptr, true, /*finalize_access_paths=*/true))
-    return true;
+  if (unit->optimize(thd, nullptr, /*finalize_access_paths=*/true)) return true;
+
+  DBUG_EXECUTE_IF("ast", { unit->DebugPrintQueryPlan(thd, "ast"); });
 
   // Calculate the current statement cost.
   accumulate_statement_cost(lex);
@@ -120,12 +122,12 @@ static bool populate_table(THD *thd, LEX *lex) {
   // Perform secondary engine optimizations, if needed.
   if (optimize_secondary_engine(thd)) return true;
 
+  // Create iterators for the chosen query plan before execution.
+  if (unit->create_iterators(thd)) return true;
+
   if (unit->execute(thd)) return true;
 
-  if (thd->m_current_query_cost >
-      thd->variables.secondary_engine_cost_threshold) {
-    notify_plugins_after_select(thd, lex->m_sql_cmd);
-  }
+  notify_plugins_after_select(thd, lex->m_sql_cmd);
 
   return false;
 }
@@ -331,7 +333,6 @@ bool Sql_cmd_create_table::execute(THD *thd) {
 
     if (!query_expression->is_prepared()) {
       // Use the desired optimizer during following preparation
-
       lex->set_using_hypergraph_optimizer(need_hypergraph_optimizer);
     }
     if (need_hypergraph_optimizer != lex->using_hypergraph_optimizer() &&
@@ -386,10 +387,6 @@ bool Sql_cmd_create_table::execute(THD *thd) {
         return true;
       }
 
-      // Use the hypergraph optimizer for the SELECT statement, if enabled.
-      lex->set_using_hypergraph_optimizer(
-          thd->optimizer_switch_flag(OPTIMIZER_SWITCH_HYPERGRAPH_OPTIMIZER));
-
       if (query_expression->prepare(thd, result, nullptr, SELECT_NO_UNLOCK,
                                     0)) {
         lex->link_first_table_back(create_table, link_to_local);
