@@ -1,18 +1,19 @@
 #ifndef SQL_EXECUTOR_INCLUDED
 #define SQL_EXECUTOR_INCLUDED
 
-/* Copyright (c) 2000, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2000, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -228,6 +229,8 @@ int join_read_const_table(JOIN_TAB *tab, POSITION *pos);
 
 int do_sj_dups_weedout(THD *thd, SJ_TMP_TABLE *sjtbl);
 int update_item_cache_if_changed(List<Cached_item> &list);
+size_t compute_ria_idx(const mem_root_deque<Item *> &fields, size_t i,
+                       size_t added_non_hidden_fields, size_t border);
 
 // Create list for using with temporary table
 bool change_to_use_tmp_fields(mem_root_deque<Item *> *fields, THD *thd,
@@ -534,12 +537,13 @@ void SplitConditions(Item *condition, QEP_TAB *current_table,
 
   If a ZERO_ROWS access path is materialized, we simply replace the MATERIALIZE
   path with the ZERO_ROWS path, since there is nothing to materialize.
+  @param thd The current thread.
   @param path the MATERIALIZE path.
   @param query_block The query block in which 'path' belongs.
   @returns The new root of the set of AccessPaths formed by 'path' and its
   descendants.
  */
-AccessPath *MoveCompositeIteratorsFromTablePath(AccessPath *path,
+AccessPath *MoveCompositeIteratorsFromTablePath(THD *thd, AccessPath *path,
                                                 const Query_block &query_block);
 
 AccessPath *GetAccessPathForDerivedTable(
