@@ -54,16 +54,16 @@
 #include "sql/temp_table_param.h"
 
 enum class Subquery_strategy : int;
-class COND_EQUAL;
+class Item_multi_eq;
 class Item_subselect;
 class Item_sum;
 class Opt_trace_context;
 class THD;
 class Window;
 struct AccessPath;
+struct COND_EQUAL;
 struct MYSQL_LOCK;
 
-class Item_equal;
 template <class T>
 class mem_root_deque;
 
@@ -328,7 +328,9 @@ class JOIN {
   /**
     The cost of best complete join plan found so far during optimization,
     after optimization phase - cost of picked join order (not taking into
-    account the changes made by test_if_skip_sort_order()).
+    account the changes made by test_if_skip_sort_order(). Exception: Single
+    table query cost is updated after access change in
+    test_if_skip_sort_order()).
   */
   double best_read{0.0};
   /**
@@ -804,6 +806,11 @@ class JOIN {
    */
   void refresh_base_slice();
 
+  /**
+    Similar to refresh_base_slice(), but refreshes only the specified slice.
+   */
+  void assign_fields_to_slice(int sliceno);
+
   /**
     Whether this query block needs finalization (see
     FinalizePlanForQueryBlock()) before it can be actually used.
@@ -1047,15 +1054,15 @@ class JOIN {
     WHERE 1=2). This will normally yield no output rows, but if we have implicit
     aggregation, it might yield a single one.
    */
-  void create_access_paths_for_zero_rows();
+  AccessPath *create_access_paths_for_zero_rows() const;
 
  private:
   void create_access_paths_for_index_subquery();
 
   /** @{ Helpers for create_access_paths. */
   AccessPath *create_root_access_path_for_join();
-  AccessPath *attach_access_paths_for_having_and_limit(AccessPath *path);
-  AccessPath *attach_access_path_for_update_or_delete(AccessPath *path);
+  AccessPath *attach_access_paths_for_having_and_limit(AccessPath *path) const;
+  AccessPath *attach_access_path_for_update_or_delete(AccessPath *path) const;
   /** @} */
 
   /**
@@ -1098,6 +1105,8 @@ class Switch_ref_item_slice {
   ~Switch_ref_item_slice() { join->set_ref_item_slice(saved); }
 };
 
+uint get_tmp_table_rec_length(const mem_root_deque<Item *> &items,
+                              bool include_hidden, bool can_skip_aggs);
 bool uses_index_fields_only(Item *item, TABLE *tbl, uint keyno,
                             bool other_tbls_ok);
 bool remove_eq_conds(THD *thd, Item *cond, Item **retcond,
@@ -1108,10 +1117,6 @@ bool optimize_cond(THD *thd, Item **conds, COND_EQUAL **cond_equal,
 Item *substitute_for_best_equal_field(THD *thd, Item *cond,
                                       COND_EQUAL *cond_equal,
                                       JOIN_TAB **table_join_idx);
-bool build_equal_items(THD *thd, Item *cond, Item **retcond,
-                       COND_EQUAL *inherited, bool do_inherit,
-                       mem_root_deque<Table_ref *> *join_list,
-                       COND_EQUAL **cond_equal_ref);
 bool is_indexed_agg_distinct(JOIN *join,
                              mem_root_deque<Item_field *> *out_args);
 Key_use_array *create_keyuse_for_table(
@@ -1224,11 +1229,12 @@ bool evaluate_during_optimization(const Item *item, const Query_block *select);
                              cond_equal)
 
   @return
-    - Item_equal for the found multiple equality predicate if a success;
+    - Item_multi_eq for the found multiple equality predicate if a success;
     - nullptr otherwise.
 */
-Item_equal *find_item_equal(COND_EQUAL *cond_equal,
-                            const Item_field *item_field, bool *inherited_fl);
+Item_multi_eq *find_item_equal(COND_EQUAL *cond_equal,
+                               const Item_field *item_field,
+                               bool *inherited_fl);
 
 /**
   Find an artificial cap for ref access. This is mostly a crutch to mitigate
