@@ -32,13 +32,15 @@
 #include <sys/types.h>
 
 #include <climits>
+#include <string_view>
 
 #include "my_base.h"
 
 #include "my_inttypes.h"
 #include "my_sqlcommand.h"
 #include "my_table_map.h"
-#include "sql/field.h"         // Copy_field
+#include "sql/field.h"  // Copy_field
+#include "sql/handler.h"
 #include "sql/item_cmpfunc.h"  // Item_cond_and
 #include "sql/opt_costmodel.h"
 #include "sql/sql_bitmap.h"
@@ -83,7 +85,8 @@ class Sql_cmd_select : public Sql_cmd_dml {
 
   bool accept(THD *thd, Select_lex_visitor *visitor) override;
 
-  const MYSQL_LEX_CSTRING *eligible_secondary_storage_engine() const override;
+  const MYSQL_LEX_CSTRING *eligible_secondary_storage_engine(
+      THD *thd) const override;
 
  protected:
   bool may_use_cursor() const override { return true; }
@@ -873,7 +876,8 @@ bool check_field_is_const(Item *cond, const Item *order_item,
                           const Field *order_field = nullptr,
                           Item **const_item = nullptr);
 bool test_if_subpart(ORDER *a, ORDER *b);
-void calc_group_buffer(JOIN *join, ORDER *group);
+void calc_group_buffer(JOIN *join, ORDER *group,
+                       Temp_table_param *tmp_table_param = nullptr);
 bool make_join_readinfo(JOIN *join, uint no_jbuf_after);
 bool create_ref_for_key(JOIN *join, JOIN_TAB *j, Key_use *org_keyuse,
                         table_map used_tables);
@@ -1063,9 +1067,9 @@ const handlerton *get_secondary_engine_handlerton(const LEX *lex);
   Checks if any of the tables referenced belong to an external engine.
   If an external table is found, return true, false otherwise.
 
-  @param query_tables the referenced tables.
+  @param lex the statement
 */
-bool has_external_table(Table_ref *query_tables);
+bool has_external_table(const LEX *lex);
 
 /**
   Sets the reason of failure for the statement to the external engine.
@@ -1075,4 +1079,24 @@ bool has_external_table(Table_ref *query_tables);
 */
 void set_external_engine_fail_reason(const LEX *lex, const char *reason);
 
+/**
+  Notify plugins about an executed SELECT statement.
+
+  @param thd the current session
+  @param cmd command to be notified about
+*/
+void notify_plugins_after_select(THD *thd, const Sql_cmd *cmd);
+
+std::string_view get_secondary_engine_fail_reason(const LEX *lex);
+
+void set_fail_reason_and_raise_error(const LEX *lex, std::string_view reason);
+
+const MYSQL_LEX_CSTRING *get_eligible_secondary_engine_from(const LEX *lex);
+
+std::string_view find_secondary_engine_fail_reason(const LEX *lex);
+
+void find_and_set_offload_fail_reason(const LEX *lex);
+
+bool reads_not_secondary_columns(const LEX *lex);
+
 #endif /* SQL_SELECT_INCLUDED */
