@@ -145,7 +145,8 @@ int get_quote_char_for_identifier(const THD *thd, const char *name,
                                   size_t length);
 
 void show_sql_type(enum_field_types type, bool is_array, uint metadata,
-                   String *str, const CHARSET_INFO *field_cs = nullptr);
+                   String *str, const CHARSET_INFO *field_cs = nullptr,
+                   unsigned int vector_dimensionality = 0);
 
 bool do_fill_information_schema_table(THD *thd, Table_ref *table_list,
                                       Item *condition);
@@ -437,11 +438,12 @@ class Sql_cmd_show_keys : public Sql_cmd_show_table_base {
   Sql_cmd_show_keys() : Sql_cmd_show_table_base(SQLCOM_SHOW_KEYS) {}
 };
 
-/// Represents SHOW MASTER STATUS statement.
+/// Represents SHOW BINARY LOG STATUS statement.
 
-class Sql_cmd_show_master_status : public Sql_cmd_show_noplan {
+class Sql_cmd_show_binary_log_status : public Sql_cmd_show_noplan {
  public:
-  Sql_cmd_show_master_status() : Sql_cmd_show_noplan(SQLCOM_SHOW_MASTER_STAT) {}
+  Sql_cmd_show_binary_log_status()
+      : Sql_cmd_show_noplan(SQLCOM_SHOW_BINLOG_STATUS) {}
   bool check_privileges(THD *thd) override;
   bool execute_inner(THD *thd) override;
 };
@@ -488,6 +490,13 @@ class Sql_cmd_show_processlist : public Sql_cmd_show {
   bool m_use_pfs{false};
 };
 
+/// Represents SHOW PARSE_TREE statement.
+
+class Sql_cmd_show_parse_tree : public Sql_cmd_show {
+ public:
+  Sql_cmd_show_parse_tree() : Sql_cmd_show(SQLCOM_SHOW_PARSE_TREE) {}
+};
+
 /// Represents SHOW PROFILE statement.
 
 class Sql_cmd_show_profile : public Sql_cmd_show {
@@ -522,7 +531,7 @@ class Sql_cmd_show_relaylog_events : public Sql_cmd_show_noplan {
 
 class Sql_cmd_show_replicas : public Sql_cmd_show_noplan {
  public:
-  Sql_cmd_show_replicas() : Sql_cmd_show_noplan(SQLCOM_SHOW_SLAVE_HOSTS) {}
+  Sql_cmd_show_replicas() : Sql_cmd_show_noplan(SQLCOM_SHOW_REPLICAS) {}
   bool check_privileges(THD *thd) override;
   bool execute_inner(THD *thd) override;
 };
@@ -531,7 +540,8 @@ class Sql_cmd_show_replicas : public Sql_cmd_show_noplan {
 
 class Sql_cmd_show_replica_status : public Sql_cmd_show_noplan {
  public:
-  Sql_cmd_show_replica_status() : Sql_cmd_show_noplan(SQLCOM_SHOW_SLAVE_STAT) {}
+  Sql_cmd_show_replica_status()
+      : Sql_cmd_show_noplan(SQLCOM_SHOW_REPLICA_STATUS) {}
   bool check_privileges(THD *thd) override;
   bool execute_inner(THD *thd) override;
 };
