@@ -27,7 +27,6 @@
 #include <atomic>
 
 #include "lex_string.h"
-#include "m_ctype.h"
 #include "my_compiler.h"
 #include "my_dbug.h"
 #include "my_io.h"
@@ -40,6 +39,7 @@
 #include "mysql/plugin.h"
 #include "mysql/psi/mysql_mutex.h"
 #include "mysql/service_thd_engine_lock.h"
+#include "mysql/strings/m_ctype.h"
 #include "mysql_com.h"
 #include "sql/auth/auth_acls.h"
 #include "sql/auth/sql_security_ctx.h"
@@ -65,6 +65,7 @@
 #include "sql/transaction_info.h"
 #include "sql/xa.h"
 #include "sql_string.h"
+#include "string_with_len.h"
 #include "violite.h"
 
 struct MYSQL_LEX_STRING;
@@ -276,6 +277,17 @@ my_socket thd_get_fd(THD *thd) {
   return thd->get_protocol_classic()->get_socket();
 }
 
+/**
+  Get MYSQL_SOCKET struct for this connection
+
+  @param thd            THD object
+
+  @retval               MYSQL_SOCKET struct of the connection
+*/
+MYSQL_SOCKET thd_get_mysql_socket(THD *thd) {
+  return thd->get_protocol_classic()->get_vio()->mysql_socket;
+}
+
 /**
   Set thread specific environment required for thd cleanup in thread pool.
 
@@ -403,7 +415,15 @@ int thd_sql_command(const MYSQL_THD thd) { return (int)thd->lex->sql_command; }
 
 int thd_tx_isolation(const MYSQL_THD thd) { return (int)thd->tx_isolation; }
 
-int thd_tx_is_read_only(const MYSQL_THD thd) { return (int)thd->tx_read_only; }
+int thd_tx_is_read_only(const MYSQL_THD thd) {
+  // If the transaction is marked to be skipped read-only  then we ignore
+  // the value of tx_read_only variable and treat the transaction as
+  // a normal read-write trx.
+  if (thd->tx_read_only && thd->is_cmd_skip_transaction_read_only())
+    return 0;
+  else
+    return (int)thd->tx_read_only;
+}
 
 int thd_tx_priority(const MYSQL_THD thd) {
   return (thd->thd_tx_priority != 0 ? thd->thd_tx_priority : thd->tx_priority);
