@@ -1,15 +1,16 @@
-/* Copyright (c) 2001, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2001, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -470,6 +471,10 @@ bool Query_expression::prepare_query_term(THD *thd, Query_term *qt,
       auto *qb = qts->query_block();
       assert(qts->m_children.size() >= 2);
 
+      if (qts->term_type() == QT_EXCEPT &&
+          qts->m_first_distinct == std::numeric_limits<int64_t>::max())
+        m_contains_except_all = true;
+
       qb->make_active_options(
           (added_options & (OPTION_FOUND_ROWS | OPTION_BUFFER_RESULT)) |
               OPTION_NO_CONST_TABLES | SELECT_NO_UNLOCK,
@@ -684,9 +689,9 @@ bool Query_expression::can_materialize_directly_into_result() const {
 }
 
 /**
-  Prepares all query blocks of a query expression, including
-  fake_query_block. If a recursive query expression, this also creates the
-  materialized temporary table.
+  Prepares all query blocks of a query expression.
+  If a recursive query expression, this also creates the materialized temporary
+  table.
 
   @param thd           Thread handler
   @param sel_result    Result object where the unit's output should go.
@@ -891,7 +896,7 @@ bool Query_expression::prepare(THD *thd, Query_result *sel_result,
 /// have been created.
 static bool finalize_full_text_functions(THD *thd,
                                          Query_expression *query_expression) {
-  assert(thd->lex->using_hypergraph_optimizer);
+  assert(thd->lex->using_hypergraph_optimizer());
   for (Query_expression *qe = query_expression; qe != nullptr;
        qe = qe->next_query_expression()) {
     for (Query_block *qb = qe->first_query_block(); qb != nullptr;
@@ -989,14 +994,8 @@ static bool use_iterator(TABLE *materialize_destination,
 }
 
 bool Query_expression::optimize(THD *thd, TABLE *materialize_destination,
-                                bool create_iterators,
                                 bool finalize_access_paths) {
   DBUG_TRACE;
-
-  if (!finalize_access_paths) {
-    assert(!create_iterators);
-  }
-
   assert(is_prepared() && !is_optimized());
 
   Change_current_query_block save_query_block(thd);
@@ -1114,46 +1113,6 @@ bool Query_expression::optimize(THD *thd, TABLE *materialize_destination,
     }
   }
 
-  if (create_iterators && IteratorsAreNeeded(thd, m_root_access_path)) {
-    JOIN *join = query_term()->query_block()->join;
-
-    DBUG_EXECUTE_IF(
-        "ast", Query_term *qn = m_query_term;
-        DBUG_PRINT("ast", ("\n%s", thd->query().str)); if (qn) {
-          std::ostringstream buf;
-          qn->debugPrint(0, buf);
-          DBUG_PRINT("ast", ("\n%s", buf.str().c_str()));
-        });
-
-    DBUG_EXECUTE_IF(
-        "ast", bool is_root_of_join = (join != nullptr); DBUG_PRINT(
-            "ast", ("Query plan:\n%s\n",
-                    PrintQueryPlan(0, m_root_access_path, join, is_root_of_join)
-                        .c_str())););
-
-    m_root_iterator = CreateIteratorFromAccessPath(
-        thd, m_root_access_path, join, /*eligible_for_batch_mode=*/true);
-    if (m_root_iterator == nullptr) {
-      return true;
-    }
-
-    if (thd->lex->using_hypergraph_optimizer) {
-      if (finalize_full_text_functions(thd, this)) {
-        return true;
-      }
-    }
-
-    if (false) {
-      // This can be useful during debugging.
-      // TODO(sgunders): Consider adding the SET DEBUG force-subplan line here,
-      // like we have on EXPLAIN FORMAT=tree if subplan_tokens is active.
-      bool is_root_of_join = (join != nullptr);
-      fprintf(
-          stderr, "Query plan:\n%s\n",
-          PrintQueryPlan(0, m_root_access_path, join, is_root_of_join).c_str());
-    }
-  }
-
   // When done with the outermost query expression, and if max_join_size is in
   // effect, estimate the total number of row accesses in the query, and error
   // out if it exceeds max_join_size.
@@ -1183,15 +1142,28 @@ bool Query_expression::finalize(THD *thd) {
 }
 
 bool Query_expression::force_create_iterators(THD *thd) {
+  assert(IteratorsAreNeeded(thd, m_root_access_path));
   if (m_root_iterator == nullptr) {
-    JOIN *join = is_set_operation() ? nullptr : first_query_block()->join;
-    m_root_iterator = CreateIteratorFromAccessPath(
-        thd, m_root_access_path, join, /*eligible_for_batch_mode=*/true);
+    return create_iterators(thd);
   }
+  return false;
+}
+
+bool Query_expression::create_iterators(THD *thd) {
+  assert(m_root_iterator == nullptr);
+
+  if (!IteratorsAreNeeded(thd, m_root_access_path)) {
+    return false;
+  }
+
+  JOIN *const top_join = query_term()->query_block()->join;
+  m_root_iterator =
+      CreateIteratorFromAccessPath(thd, m_root_access_path, top_join,
+                                   /*eligible_for_batch_mode=*/true);
 
   if (m_root_iterator == nullptr) return true;
 
-  if (thd->lex->using_hypergraph_optimizer) {
+  if (thd->lex->using_hypergraph_optimizer()) {
     if (finalize_full_text_functions(thd, this)) {
       return true;
     }
@@ -1200,6 +1172,25 @@ bool Query_expression::force_create_iterators(THD *thd) {
   return false;
 }
 
+#ifndef NDEBUG
+void Query_expression::DebugPrintQueryPlan(THD *thd,
+                                           const char *keyword) const {
+  DBUG_PRINT(keyword, ("\n%s", thd->query().str));
+  if (m_query_term != nullptr) {
+    std::ostringstream buf;
+    m_query_term->debugPrint(0, buf);
+    DBUG_PRINT(keyword, ("\n%s", buf.str().c_str()));
+  }
+
+  JOIN *const join = query_term()->query_block()->join;
+  const bool is_root_of_join = join != nullptr;
+  DBUG_PRINT(
+      keyword,
+      ("Query plan:\n%s\n",
+       PrintQueryPlan(0, m_root_access_path, join, is_root_of_join).c_str()));
+}
+#endif
+
 /**
   Helper method: create a materialized access path, estimate its cost and
   move it to the best place, cf. doc for MoveCompositeIteratorsFromTablePath
@@ -1224,7 +1215,7 @@ static AccessPath *add_materialized_access_path(
                                   /*rematerialize=*/true, limit,
                                   /*reject_multiple_rows=*/false);
   EstimateMaterializeCost(thd, path);
-  return MoveCompositeIteratorsFromTablePath(path, *qt->query_block());
+  return MoveCompositeIteratorsFromTablePath(thd, path, *qt->query_block());
 }
 
 /**
@@ -1302,7 +1293,7 @@ AccessPath *make_set_op_access_path(
                                               &join->tmp_table_param, dest,
                                               /*ref_slice=*/-1);
           param.join = join;
-          CopyBasicProperties(*join->root_access_path(), param.path);
+          CopyBasicProperties(*child_path, param.path);
           union_all_subpaths->push_back(param);
         }
       } else if (parent != nullptr) {
@@ -1793,6 +1784,9 @@ bool Query_expression::ExecuteIteratorQuery(THD *thd) {
         return true;
       }
       thd->get_stmt_da()->inc_current_row_for_condition();
+
+      DBUG_EXECUTE_IF("simulate_partial_result_set_scenario",
+                      my_error(ER_UNKNOWN_ERROR, MYF(0)););
     }
 
     // NOTE: join_cleanup must be done before we send EOF, so that we get the
@@ -1820,6 +1814,12 @@ bool Query_expression::execute(THD *thd) {
 
   assert(!unfinished_materialization());
 
+  // Collect information about the chosen query plan.
+  if (!is_executed()) {
+    CollectStatusVariables(thd, query_term()->query_block()->join,
+                           *root_access_path());
+  }
+
   /*
     Even if we return "true" the statement might continue
     (e.g. ER_SUBQUERY_1_ROW in stmt with IGNORE), so we want to restore
@@ -2076,7 +2076,7 @@ void Query_block::cleanup(bool full) {
     if (full) {
       assert(join->query_block == this);
       join->destroy();
-      ::destroy(join);
+      ::destroy_at(join);
       join = nullptr;
     } else
       join->cleanup();
