@@ -30,8 +30,8 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
-#ifndef YY_MY_SQL_PARSER_VAR_LIB_PB2_SB_1_16116654_1720812213_18_DIST_GPL_SQL_SQL_YACC_H_INCLUDED
-# define YY_MY_SQL_PARSER_VAR_LIB_PB2_SB_1_16116654_1720812213_18_DIST_GPL_SQL_SQL_YACC_H_INCLUDED
+#ifndef YY_MY_SQL_PARSER_VAR_LIB_PB2_SB_1_16116943_1720812561_91_DIST_GPL_SQL_SQL_YACC_H_INCLUDED
+# define YY_MY_SQL_PARSER_VAR_LIB_PB2_SB_1_16116943_1720812561_91_DIST_GPL_SQL_SQL_YACC_H_INCLUDED
 /* Debug traces.  */
 #ifndef MY_SQL_PARSER_DEBUG
 # if defined YYDEBUG
@@ -859,11 +859,12 @@ extern int my_sql_parser_debug;
     MANUAL_SYM = 1212,
     BERNOULLI_SYM = 1213,
     TABLESAMPLE_SYM = 1214,
-    KEYWORD_USED_AS_IDENT = 1216,
-    KEYWORD_USED_AS_KEYWORD = 1217,
-    CONDITIONLESS_JOIN = 1218,
-    PREFER_PARENTHESES = 1219,
-    EMPTY_FROM_CLAUSE = 1220
+    VECTOR_SYM = 1215,
+    KEYWORD_USED_AS_IDENT = 1217,
+    KEYWORD_USED_AS_KEYWORD = 1218,
+    CONDITIONLESS_JOIN = 1219,
+    PREFER_PARENTHESES = 1220,
+    EMPTY_FROM_CLAUSE = 1221
   };
 #endif
 /* Tokens.  */
@@ -1673,11 +1674,12 @@ extern int my_sql_parser_debug;
 #define MANUAL_SYM 1212
 #define BERNOULLI_SYM 1213
 #define TABLESAMPLE_SYM 1214
-#define KEYWORD_USED_AS_IDENT 1216
-#define KEYWORD_USED_AS_KEYWORD 1217
-#define CONDITIONLESS_JOIN 1218
-#define PREFER_PARENTHESES 1219
-#define EMPTY_FROM_CLAUSE 1220
+#define VECTOR_SYM 1215
+#define KEYWORD_USED_AS_IDENT 1217
+#define KEYWORD_USED_AS_KEYWORD 1218
+#define CONDITIONLESS_JOIN 1219
+#define PREFER_PARENTHESES 1220
+#define EMPTY_FROM_CLAUSE 1221
 
 /* Value type.  */
 
@@ -1699,4 +1701,4 @@ struct MY_SQL_PARSER_LTYPE
 
 int my_sql_parser_parse (class THD *YYTHD, class Parse_tree_root **parse_tree);
 
-#endif /* !YY_MY_SQL_PARSER_VAR_LIB_PB2_SB_1_16116654_1720812213_18_DIST_GPL_SQL_SQL_YACC_H_INCLUDED  */
+#endif /* !YY_MY_SQL_PARSER_VAR_LIB_PB2_SB_1_16116943_1720812561_91_DIST_GPL_SQL_SQL_YACC_H_INCLUDED  */
