@@ -291,12 +291,6 @@ static const char *fts_end_delete_sql =
     "DELETE FROM $being_deleted;\n"
     "DELETE FROM $being_deleted_cache;\n";
 
-/** Set fts_zip_t::word to empty string. */
-static void fts_zip_clear_word(fts_zip_t &zip) {
-  zip.word.f_len = 0;
-  memset(zip.word.f_str, 0, FTS_MAX_WORD_LEN);
-}
-
 /** Initialize fts_zip_t. */
 static void fts_zip_initialize(
     fts_zip_t *zip) /*!< out: zip instance to initialize */
@@ -308,7 +302,8 @@ static void fts_zip_initialize(
 
   zip->last_big_block = 0;
 
-  fts_zip_clear_word(*zip);
+  zip->word.f_len = 0;
+  memset(zip->word.f_str, 0, FTS_MAX_WORD_LEN);
 
   ib_vector_reset(zip->blocks);
 
@@ -673,36 +668,19 @@ static bool fts_zip_read_word(fts_zip_t *zip,     /*!< in: Zip state + data */
  @return false on EOF */
 static bool fts_fetch_index_words(
     void *row,      /*!< in: sel_node_t* */
-    void *user_arg) /*!< in: pointer to fts_optimize_t */
+    void *user_arg) /*!< in: pointer to ib_vector_t */
 {
-  const auto *const sel_node = static_cast<sel_node_t *>(row);
-  const auto optim = static_cast<fts_optimize_t *>(user_arg);
-  fts_zip_t *const zip = optim->zip;
-  que_node_t *const exp = sel_node->select_list;
-  const CHARSET_INFO *const charset = optim->fts_index_table.charset;
-  const dfield_t *const dfield = que_node_get_val(exp);
+  sel_node_t *sel_node = static_cast<sel_node_t *>(row);
+  fts_zip_t *zip = static_cast<fts_zip_t *>(user_arg);
+  que_node_t *exp = sel_node->select_list;
+  dfield_t *dfield = que_node_get_val(exp);
   ushort len = static_cast<ushort>(dfield_get_len(dfield));
-  void *const data = dfield_get_data(dfield);
+  void *data = dfield_get_data(dfield);
 
-  /* Ensure each class of collation-equal tokens is compressed only once. */
-  {
-    /* Compare current word with previous using collation. */
-    auto comp = my_strnncoll(charset, static_cast<uint8_t *>(data), len,
-                             zip->word.f_str, zip->word.f_len);
-    /* Duplicate detection is guaranteed by callback invocations being
-    grouped into sequences (corresponding to aux tables), where all
-    collation-equal words are in the same sequence; between these
-    sequences zip->word is reset to an empty string (every non-empty
-    word is assumed to be later in sort order than the empty string).
-    Within each sequence words are sorted (using collation) in ascending
-    order, guaranteeing all collation-equal words are passed to the
-    callback in consecutive calls, which allows us to eliminate
-    duplicates by comparing each word only with the one preceding it. */
-    ut_ad(comp >= 0);
-    /* Skip the duplicate words. */
-    if (comp == 0) {
-      return true;
-    }
+  /* Skip the duplicate words. */
+  if (zip->word.f_len == static_cast<ulint>(len) &&
+      !memcmp(zip->word.f_str, data, len)) {
+    return true;
   }
 
   ut_a(len <= FTS_MAX_WORD_LEN);
@@ -842,30 +820,9 @@ static void fts_zip_deflate_end(
 
     optim->fts_index_table.suffix = fts_get_suffix(selected_aux_idx);
 
-    /* The list of words to be processed may not have any duplicates -
-    meaning here words that are equal in collation order. Elimination
-    of duplicates is performed inside the fts_fetch_index_words callback
-    using optim->zip->word as the previous word processed. It relies on
-    all collation-equal words being passed to the callback in consecutive
-    calls. This property holds because:
-    1. All words equal by collation order are indexed in the same aux
-       table,
-    2. Words from a given aux table are processed by collation
-       (ascending) order.
-    (1) is ensured by the way words are assigned to aux table number -
-    see fts_select_index function. (2) is guaranteed by 'ORDER BY word'
-    clause in the query below.
-
-    optim->zip->word is cleared out when moving from one aux table to
-    another; it is not needed, as there can be no duplicates between
-    tables, but doing so allows the callback to assert ordering within
-    the table. */
-
-    fts_zip_clear_word(*optim->zip);
-
     info = pars_info_create();
 
-    pars_info_bind_function(info, "my_func", fts_fetch_index_words, optim);
+    pars_info_bind_function(info, "my_func", fts_fetch_index_words, optim->zip);
 
     pars_info_bind_varchar_literal(info, "word", word->f_str, word->f_len);
 
@@ -1033,7 +990,7 @@ dberr_t fts_table_fetch_doc_ids(
   if (error == DB_SUCCESS) {
     fts_sql_commit(trx);
 
-    ib_vector_sort(doc_ids->doc_ids, fts_doc_id_field_cmp<fts_update_t>);
+    ib_vector_sort(doc_ids->doc_ids, fts_update_doc_id_cmp);
   } else {
     fts_sql_rollback(trx);
   }
