@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1994, 2025, Oracle and/or its affiliates.
+Copyright (c) 1994, 2024, Oracle and/or its affiliates.
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
@@ -157,7 +157,7 @@ If len>0, tests the first len bytes of the content for equality.
 @return true if both fields are NULL or if they are equal */
 [[nodiscard]] inline bool dfield_datas_are_binary_equal(const dfield_t *field1,
                                                         const dfield_t *field2,
-                                                        ulint len = 0);
+                                                        ulint len);
 /** Tests if dfield data length and content is equal to the given.
 @param[in] field Field
 @param[in] len   Data length or UNIV_SQL_NULL
@@ -405,14 +405,7 @@ struct multi_value_data {
   If current data array is for INSERT and DELETE, this can(should) be
   nullptr since all values in current array should be handled in these
   two cases. */
-  Bitset<> *bitset;
-
-#ifdef UNIV_DEBUG
-  /** The size of bitmap buffer in bytes. The bitset might point into
-  some prefix of it, and be later made shorter or longer, but not longer
-  than this length of underlying buffer. */
-  size_t bitmap_bytes{};
-#endif
+  Bitset *bitset;
 
   /** Allocate specified number of elements for all arrays and initialize
   the structure accordingly
@@ -517,12 +510,7 @@ struct multi_value_data {
            sizeof(*conv_buf) * multi_value->num_v);
     if (multi_value->bitset != nullptr) {
       ut_ad(bitset != nullptr);
-      ut_ad_le(multi_value->bitset->size_bytes(), bitmap_bytes);
-      /* adjust the size to match that of multi_value, but keep pointing to the
-      old buffer */
-      *bitset = Bitset<>(bitset->data(), multi_value->bitset->size_bytes());
-      /* copy the actual data into the buffer from multi_value's buffer */
-      bitset->copy_from(multi_value->bitset->data());
+      *bitset = *multi_value->bitset;
     }
     num_v = multi_value->num_v;
   }
@@ -623,6 +611,8 @@ struct dfield_t {
   unsigned len; /*!< data length; UNIV_SQL_NULL if SQL null */
   dtype_t type; /*!< type of data */
 
+  bool is_ext() const { return ext; }
+
   bool is_virtual() const { return (type.is_virtual()); }
 
   void reset() {
