@@ -45,10 +45,11 @@ this program; if not, write to the Free Software Foundation, Inc.,
 
 #include "ut0byte.h"
 #include "ut0new.h"
+#include "ut0todo_counter.h"
 
 #include <list>
-#include <set>
 #include <unordered_map>
+#include <unordered_set>
 
 class MetadataRecover;
 class PersistentTableMetadata;
@@ -399,7 +400,7 @@ struct recv_sys_t {
     Pages m_pages;
   };
 
-  using Missing_Ids = std::set<space_id_t>;
+  using Missing_Ids = std::unordered_set<space_id_t>;
 
   using Spaces = std::unordered_map<space_id_t, Space, std::hash<space_id_t>,
                                     std::equal_to<space_id_t>>;
@@ -521,8 +522,8 @@ struct recv_sys_t {
 
 #ifndef UNIV_HOTBACKUP
 
-  /*!< mutex protecting the fields apply_log_recs, n_addrs, and the
-  state field in each recv_addr struct */
+  /** mutex protecting the fields apply_log_recs, decrements of
+  n_pages_to_recover, and the state field in each recv_addr struct */
   ib_mutex_t mutex;
 
   /** mutex coordinating flushing between recv_writer_thread and
@@ -548,9 +549,6 @@ struct recv_sys_t {
   application */
   bool apply_log_recs;
 
-  /** This is true when a log rec application batch is running */
-  bool apply_batch_on;
-
   /** Buffer for parsing log records */
   byte *buf;
 
@@ -616,8 +614,11 @@ struct recv_sys_t {
   /** Hash table of pages, indexed by SpaceID. */
   Spaces *spaces;
 
-  /** Number of not processed hashed file addresses in the hash table */
-  ulint n_addrs;
+  /** Number of unique unprocessed page ids in the spaces nested hash table.
+  Increments are done only from the main recovery thread before apply starts.
+  Decrements are done from multiple threads during batch apply phase, and are
+  protected by the recv_sys_t::mutex. */
+  ut::Todo_counter n_pages_to_recover;
 
   /** Doublewrite buffer pages, destroyed after recovery completes */
   dblwr::recv::DBLWR *dblwr;
@@ -676,10 +677,6 @@ roll-forward */
 
 extern size_t recv_n_frames_for_pages_per_pool_instance;
 
-/** A list of tablespaces for which (un)encryption process was not
-completed before crash. */
-extern std::list<space_id_t> recv_encr_ts_list;
-
 #include "log0recv.ic"
 
 #endif
