@@ -39,6 +39,13 @@ this program; if not, write to the Free Software Foundation, Inc.,
 
 #include "my_dbug.h"
 
+#define WLOG(x)                                                        \
+  {                                                                    \
+    std::cout << "[BULK] thread=" << std::this_thread::get_id() << ":" \
+              << __func__ << ":" << strrchr(__FILE__, '/') + 1 << ":"  \
+              << __LINE__ << ": " << x << std::endl;                   \
+  }
+// ----------------------------------------------------------------------------
 namespace lob {
 
 /** A BLOB field reference has all the bits set to zero, except the "being
@@ -161,7 +168,7 @@ int zReader::setup_zstream() {
 
   m_stream.next_out = m_rctx.m_buf + local_prefix;
   m_stream.avail_out = static_cast<uInt>(m_rctx.m_len - local_prefix);
-  m_stream.next_in = Z_NULL;
+  m_stream.next_in = nullptr;
   m_stream.avail_in = 0;
 
   /* Zlib inflate needs 32 kilobytes for the default
@@ -662,7 +669,6 @@ byte *btr_rec_copy_externally_stored_field_func(
   const byte *field_ref = data + local_len - BTR_EXTERN_FIELD_REF_SIZE;
 
   lob::ref_t ref(const_cast<byte *>(field_ref));
-
   ut_a(local_len >= BTR_EXTERN_FIELD_REF_SIZE);
 
   /* Verify if the LOB reference is sane. */
@@ -922,7 +928,6 @@ byte *btr_copy_externally_stored_field_func(
 
   if (extern_len == 0) {
     /* The lob has already been purged. */
-    ut_ad(ref_t::page_no(field_ref) == FIL_NULL);
     *len = 0;
     return (buf);
   }
