@@ -1,6 +1,6 @@
 /*****************************************************************************
 
-Copyright (c) 1995, 2023, Oracle and/or its affiliates.
+Copyright (c) 1995, 2024, Oracle and/or its affiliates.
 Copyright (c) 2009, Google Inc.
 
 Portions of this file contain modifications contributed and copyrighted by
@@ -13,12 +13,13 @@ This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License, version 2.0, as published by the
 Free Software Foundation.
 
-This program is also distributed with certain software (including but not
-limited to OpenSSL) that is licensed under separate terms, as designated in a
-particular file or component or in included license documentation. The authors
-of MySQL hereby grant you an additional permission to link the program and
-your derivative works with the separately licensed software that they have
-included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
@@ -227,12 +228,12 @@ Redo log consists of following data layers:
    formats data in proper way: include headers/footers of log blocks,
    calculates checksums, maintains boundaries of record groups.
 
--# %Log recent written buffer (e.g. 4MB) - tracks recent writes to the
+-# %Log recent written buffer (e.g. 1MB) - tracks recent writes to the
    log buffer. Allows to have concurrent writes to the log buffer and tracks
    up to which lsn all such writes have been already finished.
 
--# %Log recent closed buffer (e.g. 4MB) - tracks for which recent writes,
-   corresponding dirty pages have been already added to the flush lists.
+-# %The Buf_flush_list_added_lsns buffer (e.g. 2MB) - tracks for which recent
+   writes, corresponding dirty pages have been already added to the flush lists.
    Allows to relax order in which dirty pages have to be added to the flush
    lists and tracks up to which lsn, all dirty pages have been added.
    This is required to not make checkpoint at lsn which is larger than
@@ -284,7 +285,7 @@ Redo log consists of following data layers:
 
 -# User threads need to wait if there is no space in the log files.
 
-   @diafile storage/innobase/log/arch_deleting.dia "Reclaiming space in the redo log"
+   @diafile arch_deleting.dia "Reclaiming space in the redo log"
 
 -# Well thought out and tested set of _MONITOR_ counters is maintained and
    documented.
@@ -296,10 +297,6 @@ Redo log consists of following data layers:
 -# All the new buffers could be resized dynamically during runtime. In practice,
    only size of the log buffer is accessible without the _EXPERIMENTAL_ mode.
 
-   @note
-   This is a functional change - the log buffer could be resized dynamically
-   by users (also decreased).
-
 @section sect_redo_log_lsn_values Glossary of lsn values
 
 Different fragments of head of the redo log are tracked by different values:
@@ -308,7 +305,8 @@ Different fragments of head of the redo log are tracked by different values:
   - @ref subsect_redo_log_sn.
 
 Different fragments of the redo log's tail are tracked by different values:
-  - @ref subsect_redo_log_buf_dirty_pages_added_up_to_lsn,
+
+  - @ref subsect_buf_flush_list_smallest_not_added,
   - @ref subsect_redo_log_available_for_checkpoint_lsn,
   - @ref subsect_redo_log_last_checkpoint_lsn.
 
@@ -320,14 +318,15 @@ be overwritten in cyclic manner for lsn values larger by _log.buf_size_.
 
 Value is updated by: [log writer thread](@ref sect_redo_log_writer).
 
-@subsection subsect_redo_log_buf_ready_for_write_lsn log.buf_ready_for_write_lsn
+@subsection subsect_redo_log_buf_ready_for_write_lsn log_buffer_ready_for_write_lsn()
 
+It's internally implemented as log.recent_written.tail().
 Up to this lsn, all concurrent writes to log buffer have been finished.
 We don't need older part of the log recent-written buffer.
 
 It obviously holds:
 
-        log.buf_ready_for_write_lsn >= log.write_lsn
+        log_buffer_ready_for_write_lsn() >= log.write_lsn
 
 Value is updated by: [log writer thread](@ref sect_redo_log_writer).
 
@@ -348,12 +347,11 @@ data bytes).
 
 It obviously holds:
 
-        log.sn >= log_translate_lsn_to_sn(log.buf_ready_for_write_lsn)
+        log.sn >= log_translate_lsn_to_sn(log_buffer_ready_for_write_lsn())
 
 Value is updated by: user threads during reservation of space.
 
-@subsection subsect_redo_log_buf_dirty_pages_added_up_to_lsn
-log.buf_dirty_pages_added_up_to_lsn
+@subsection subsect_buf_flush_list_smallest_not_added buf_flush_list_smallest_not_added()
 
 Up to this lsn user threads have added all dirty pages to flush lists.
 
@@ -362,9 +360,9 @@ That's because there could be a page with _oldest_modification_ smaller than
 the minimum _oldest_modification_ available in flush lists. Note that such page
 is just about to be added to flush list by a user thread, but there is no mutex
 protecting access to the minimum _oldest_modification_, which would be acquired
-by the user thread before writing to redo log. Hence for any lsn greater than
-_buf_dirty_pages_added_up_to_lsn_ we cannot trust that flush lists are complete
-and minimum calculated value (or its approximation) is valid.
+by the user thread before writing to redo log. Hence for any lsn greater or
+equal to _buf_flush_list_smallest_not_added()_ we cannot trust that flush lists
+are complete and minimum calculated value (or its approximation) is valid.
 
 @note
 Note that we do not delete redo log records physically, but we still can delete
@@ -373,10 +371,9 @@ them logically by doing checkpoint at given lsn.
 It holds (unless the log writer thread misses an update of the
 @ref subsect_redo_log_buf_ready_for_write_lsn):
 
-        log.buf_dirty_pages_added_up_to_lsn <= log.buf_ready_for_write_lsn.
+    buf_flush_list_added->smallest_not_added_lsn() <= log_buffer_ready_for_write_lsn().
 
-@subsection subsect_redo_log_available_for_checkpoint_lsn
-log.available_for_checkpoint_lsn
+@subsection subsect_redo_log_available_for_checkpoint_lsn log.available_for_checkpoint_lsn
 
 Up to this lsn all dirty pages have been flushed to disk. However, this value
 is not guaranteed to be the maximum such value. As insertion order to flush
@@ -384,11 +381,11 @@ lists is relaxed, the buf_pool_get_oldest_modification_approx() returns
 modification time of some page that was inserted the earliest, it doesn't
 have to be the oldest modification though. However, the maximum difference
 between the first page in flush list, and one with the oldest modification
-lsn is limited by the number of entries in the log recent closed buffer.
+lsn is limited by the buf_flush_list_added->order_lag().
 
 That's why from result of buf_pool_get_oldest_modification_approx() size of
-the log recent closed buffer is subtracted. The result is used to update the
-lsn available for a next checkpoint.
+the buf_flush_list_added->order_lag() is subtracted. The result is used to
+update the lsn available for a next checkpoint.
 
 This has impact on the redo format, because the checkpoint_lsn can now point
 to the middle of some group of log records (even to the middle of a single
@@ -416,7 +413,7 @@ It holds:
 
         log.last_checkpoint_lsn
         <= log.available_for_checkpoint_lsn
-        <= log.buf_dirty_pages_added_up_to_lsn.
+        <= buf_flush_list_added->smallest_not_added_lsn().
 
 
 Read more about redo log details:
@@ -541,14 +538,6 @@ static void log_allocate_recent_written(log_t &log);
 @param[out]     log     redo log */
 static void log_deallocate_recent_written(log_t &log);
 
-/** Allocates the log recent closed buffer.
-@param[out]     log     redo log */
-static void log_allocate_recent_closed(log_t &log);
-
-/** Deallocates the log recent closed buffer.
-@param[out]     log     redo log */
-static void log_deallocate_recent_closed(log_t &log);
-
 /** Resets the log encryption buffer (used to write encryption headers).
 @param[out]     log     redo log */
 static void log_reset_encryption_buffer(log_t &log);
@@ -660,7 +649,6 @@ static void log_sys_create() {
   log_allocate_buffer(log);
   log_allocate_write_ahead_buffer(log);
   log_allocate_recent_written(log);
-  log_allocate_recent_closed(log);
   log_allocate_flush_events(log);
   log_allocate_write_events(log);
 
@@ -714,6 +702,21 @@ dberr_t log_start(log_t &log, lsn_t checkpoint_lsn, lsn_t start_lsn,
   ut_a(checkpoint_lsn >= LOG_START_LSN);
   ut_a(start_lsn >= checkpoint_lsn);
   ut_a(arch_log_sys == nullptr || !arch_log_sys->is_active());
+  /* If flags in the header on disc indicate redo logging was disabled, then let
+  the module know logging should be turned off. We need to do it before anyone
+  attempts to write to the redo log, and conveniently the call to log_start()
+  indicates the moment the caller wants to start writing to the redo log. The
+  mtr_t::s_logging.disable(..) will set LOG_HEADER_FLAG_CRASH_UNSAFE flag in
+  redo header on disc, which we only know how to do in the currently supported
+  format. We should never write to files which are in the older format. This is
+  why we call this function only after upgrade was already performed. */
+  ut_a(log.m_format == Log_format::CURRENT);
+  if (log_file_header_check_flag(log.m_log_flags, LOG_HEADER_FLAG_NO_LOGGING)) {
+    auto result = mtr_t::s_logging.disable(nullptr);
+    /* Currently never fails. */
+    ut_a(result == 0);
+    srv_redo_log = false;
+  }
 
   log.write_to_file_requests_total.store(0);
   log.write_to_file_requests_interval.store(std::chrono::seconds::zero());
@@ -738,15 +741,18 @@ dberr_t log_start(log_t &log, lsn_t checkpoint_lsn, lsn_t start_lsn,
   log.recent_written.advance_tail();
   ut_a(log_buffer_ready_for_write_lsn(log) == start_lsn);
 
-  log.recent_closed.add_link(0, start_lsn);
-  log.recent_closed.advance_tail();
-  ut_a(log_buffer_dirty_pages_added_up_to_lsn(log) == start_lsn);
-
   log.write_lsn = start_lsn;
   log.flushed_to_disk_lsn = start_lsn;
 
   log.write_ahead_end_offset = 0;
 
+  /* It's a bit ugly that log_start is responsible for calling
+  buf_flush_list_added->assume_added_up_to(log.flushed_to_disk_lsn),
+  but the alternative seems to be to hunt down 4 (or more?) places where we
+  start the log and add this call immediately after them, which seems even more
+  difficult to maintain and verify. */
+  buf_flush_list_added->assume_added_up_to(log.flushed_to_disk_lsn);
+
   lsn_t block_lsn;
   byte *block;
 
@@ -758,28 +764,22 @@ dberr_t log_start(log_t &log, lsn_t checkpoint_lsn, lsn_t start_lsn,
 
   Log_data_block_header block_header;
 
-  if (log.m_format == Log_format::CURRENT) {
-    {
-      auto file = log.m_files.find(block_lsn);
-      ut_a(file != log.m_files.end());
-      auto file_handle = file->open(Log_file_access_mode::READ_ONLY);
-      ut_a(file_handle.is_open());
+  {
+    auto file = log.m_files.find(block_lsn);
+    ut_a(file != log.m_files.end());
+    auto file_handle = file->open(Log_file_access_mode::READ_ONLY);
+    ut_a(file_handle.is_open());
 
-      const auto err = log_data_blocks_read(
-          file_handle, file->offset(block_lsn), OS_FILE_LOG_BLOCK_SIZE, block);
-      if (err != DB_SUCCESS) {
-        return err;
-      }
-      log_data_block_header_deserialize(block, block_header);
+    const auto err = log_data_blocks_read(file_handle, file->offset(block_lsn),
+                                          OS_FILE_LOG_BLOCK_SIZE, block);
+    if (err != DB_SUCCESS) {
+      return err;
     }
-
-    /* FOLLOWING IS NOT NEEDED IF WE DON'T ALLOW DISABLING crc32 checksum */
-    log_fix_first_rec_group(block_lsn, block_header, start_lsn);
-  } else {
-    ut_a(start_lsn % OS_FILE_LOG_BLOCK_SIZE == LOG_BLOCK_HDR_SIZE);
-    std::memset(block, 0x00, OS_FILE_LOG_BLOCK_SIZE);
-    block_header.m_first_rec_group = LOG_BLOCK_HDR_SIZE;
   }
+  log_data_block_header_deserialize(block, block_header);
+
+  /* FOLLOWING IS NOT NEEDED IF WE DON'T ALLOW DISABLING crc32 checksum */
+  log_fix_first_rec_group(block_lsn, block_header, start_lsn);
 
   block_header.set_lsn(block_lsn);
   /* The last mtr in the block might have been incomplete, thus we trim the
@@ -810,7 +810,6 @@ static void log_sys_free() {
 
   log_deallocate_write_events(log);
   log_deallocate_flush_events(log);
-  log_deallocate_recent_closed(log);
   log_deallocate_recent_written(log);
   log_deallocate_write_ahead_buffer(log);
   log_deallocate_buffer(log);
@@ -1135,7 +1134,7 @@ void log_print(const log_t &log, FILE *file) {
   log_files_mutex_enter(log);
 
   last_checkpoint_lsn = log.last_checkpoint_lsn.load();
-  dirty_pages_added_up_to_lsn = log_buffer_dirty_pages_added_up_to_lsn(log);
+  dirty_pages_added_up_to_lsn = buf_flush_list_added->smallest_not_added_lsn();
   ready_for_write_lsn = log_buffer_ready_for_write_lsn(log);
   write_lsn = log.write_lsn.load();
   flush_lsn = log.flushed_to_disk_lsn.load();
@@ -1417,16 +1416,6 @@ static void log_deallocate_recent_written(log_t &log) {
   log.recent_written.validate_no_links();
   log.recent_written = {};
 }
-
-static void log_allocate_recent_closed(log_t &log) {
-  log.recent_closed = Link_buf<lsn_t>{srv_log_recent_closed_size};
-}
-
-static void log_deallocate_recent_closed(log_t &log) {
-  log.recent_closed.validate_no_links();
-  log.recent_closed = {};
-}
-
 static void log_reset_encryption_buffer(log_t &log) {
   std::memset(log.m_encryption_buf, 0x00, OS_FILE_LOG_BLOCK_SIZE);
 }
@@ -1897,19 +1886,7 @@ dberr_t log_sys_init(bool expect_no_files, lsn_t flushed_lsn,
 
   /* Check creator of log files and mark fields of recv_sys: is_cloned_db,
   is_meb_db if needed. */
-  err = log_sys_handle_creator(log);
-  if (err != DB_SUCCESS) {
-    return err;
-  }
-
-  if (log_file_header_check_flag(log_flags, LOG_HEADER_FLAG_NO_LOGGING)) {
-    auto result = mtr_t::s_logging.disable(nullptr);
-    /* Currently never fails. */
-    ut_a(result == 0);
-    srv_redo_log = false;
-  }
-
-  return DB_SUCCESS;
+  return log_sys_handle_creator(log);
 }
 
 void log_sys_close() {
