@@ -94,6 +94,9 @@ description and its property/type and corresponding monitor_id.
 Please note: If you add a monitor here, please add its corresponding
 monitor_id to "enum monitor_id_value" structure in srv0mon.h file. */
 
+// NOTE: please keep the counter descriptions below in sync with the
+// description of the matching metrics in handler/ha_innodb.cc
+
 static monitor_info_t innodb_counter_info[] = {
     /* A dummy item to mark the module start, this is
     to accommodate the default value (0) set for the
@@ -317,7 +320,7 @@ static monitor_info_t innodb_counter_info[] = {
      MONITOR_DEFAULT_START, MONITOR_OVLD_PAGES_READ},
 
     {"buffer_data_reads", "buffer",
-     "Amount of data read in bytes (innodb_data_reads)",
+     "Amount of data read in bytes (innodb_data_read)",
      static_cast<monitor_type_t>(MONITOR_EXISTING | MONITOR_DEFAULT_ON),
      MONITOR_DEFAULT_START, MONITOR_OVLD_BYTE_READ},
 
@@ -916,7 +919,8 @@ static monitor_info_t innodb_counter_info[] = {
      static_cast<monitor_type_t>(MONITOR_EXISTING | MONITOR_DEFAULT_ON),
      MONITOR_DEFAULT_START, MONITOR_OVLD_LOG_WAITS},
 
-    {"log_write_requests", "log", "Number of log write requests",
+    {"log_write_requests", "log",
+     "Number of log write requests (innodb_log_write_requests)",
      static_cast<monitor_type_t>(MONITOR_EXISTING | MONITOR_DEFAULT_ON),
      MONITOR_DEFAULT_START, MONITOR_OVLD_LOG_WRITE_REQUEST},
 
@@ -1018,9 +1022,10 @@ static monitor_info_t innodb_counter_info[] = {
      MONITOR_LOG_ON_RECENT_WRITTEN_WAIT_LOOPS},
 
     {"log_on_recent_closed_wait_loops", "log",
-     "Loops of waits in user threads on space in log.recent_closed",
-     /* Non-zero values of the counter => writes to log buffer are delayed.
-     Then innodb_log_recent_closed_slots should be increased. */
+     "Loops of waits in user threads on space in "
+     "innodb_buf_flush_list_added_size",
+     /* Non-zero values of the counter => appends to flush lists are delayed.
+     Then innodb_buf_flush_list_added_size should be increased. */
      MONITOR_NONE, MONITOR_DEFAULT_START,
      MONITOR_LOG_ON_RECENT_CLOSED_WAIT_LOOPS},
 
@@ -1720,7 +1725,8 @@ void srv_mon_process_existing_counter(
       value = stat.n_pages_read;
       break;
 
-    /* innodb_data_reads, the total number of data reads */
+    /* innodb_data_read, the amount of data read since the server was started
+    (in bytes) */
     case MONITOR_OVLD_BYTE_READ:
       value = srv_stats.data_read;
       break;
@@ -1959,7 +1965,7 @@ void srv_mon_process_existing_counter(
     } break;
 
     case MONITOR_OVLD_LSN_BUF_DIRTY_PAGES_ADDED:
-      value = (mon_type_t)log_buffer_dirty_pages_added_up_to_lsn(*log_sys);
+      value = (mon_type_t)buf_flush_list_added->smallest_not_added_lsn();
       break;
 
     case MONITOR_OVLD_BUF_OLDEST_LSN_APPROX:
