@@ -1,5 +1,5 @@
 /*
-   Copyright (c) 2009, 2025, Oracle and/or its affiliates.
+   Copyright (c) 2009, 2024, Oracle and/or its affiliates.
 
 
    This program is free software; you can redistribute it and/or modify
@@ -24,6 +24,10 @@
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
  */
 
+#include <fstream>
+#include <iostream>
+#include <string>
+
 #include <NdbEnv.h>
 #include <NDBT.hpp>
 #include <NDBT_Find.hpp>
@@ -33,6 +37,9 @@
 #include <NdbProcess.hpp>
 #include <portlib/NdbDir.hpp>
 #include "ConfigFactory.hpp"
+#include "portlib/ssl_applink.h"
+#include "util/TlsKeyManager.hpp"
+#include "util/ndb_openssl3_compat.h"
 #include "util/require.h"
 
 static const char *exe_valgrind = 0;
@@ -104,7 +111,6 @@ class Mgmd {
 
   ~Mgmd() {
     if (m_proc) {
-      // stop the process
       stop();
     }
   }
@@ -136,18 +142,22 @@ class Mgmd {
     return (bool)m_proc;
   }
 
-  bool start_from_config_ini(const char *working_dir,
-                             const char *first_extra_arg = NULL, ...) {
-    NdbProcess::Args args;
+  void common_args(NdbProcess::Args &args, const char *working_dir) {
     args.add("--no-defaults");
     args.add("--configdir=", working_dir);
-    args.add("-f config.ini");
+    args.add("--config-file=", "config.ini");
     if (with_nodeid) {
       args.add("--ndb-nodeid=", m_nodeid);
     }
     args.add("--nodaemon");
     args.add("--log-name=", name());
     if (m_verbose) args.add("--verbose");
+  }
+
+  bool start_from_config_ini(const char *working_dir,
+                             const char *first_extra_arg = NULL, ...) {
+    NdbProcess::Args args;
+    common_args(args, working_dir);
 
     if (first_extra_arg) {
       // Append any extra args
@@ -197,7 +207,7 @@ class Mgmd {
       return false;  // Can't kill with -9 -> fatal error
     }
     int ret;
-    if (!m_proc->wait(ret, 300)) {
+    if (!m_proc->wait(ret, 30000)) {
       fprintf(stderr, "Failed to wait for process %s\n", name());
       return false;  // Can't wait after kill with -9 -> fatal error
     }
@@ -215,10 +225,10 @@ class Mgmd {
     return true;
   }
 
-  bool wait(int &ret, int timeout = 300) {
+  bool wait(int &ret, int timeout = 30000) {
     g_info << "Waiting for " << name() << endl;
 
-    if (m_proc == 0 || !m_proc->wait(ret, timeout)) {
+    if (!m_proc || !m_proc->wait(ret, timeout)) {
       fprintf(stderr, "Failed to wait for process %s\n", name());
       return false;
     }
@@ -248,6 +258,10 @@ class Mgmd {
                                  retry_delay_in_seconds);
   }
 
+  int client_start_tls(struct ssl_ctx_st *ctx) {
+    return m_mgmd_client.start_tls(ctx);
+  }
+
   bool wait_confirmed_config(int timeout = 30) {
     if (!m_mgmd_client.is_connected()) {
       g_err << "wait_confirmed_config: not connected!" << endl;
@@ -274,6 +288,10 @@ class Mgmd {
 
   NdbMgmHandle handle() { return m_mgmd_client.handle(); }
 
+  NdbSocket convert_to_transporter() {
+    return m_mgmd_client.convert_to_transporter();
+  }
+
  private:
   bool get_section_string(const Properties &config, const char *section_name,
                           const char *key, const char **value) const {
@@ -308,19 +326,24 @@ class MgmdProcessList : public Vector<Mgmd *> {
 
 class Ndbd : public Mgmd {
  public:
-  Ndbd(int nodeid) : Mgmd(nodeid) {
+  Ndbd(int nodeid) : Mgmd(nodeid), m_args() {
+    m_args.add("--ndb-nodeid=", m_nodeid);
+    m_args.add("--foreground");
+    m_args.add("--loose-core-file=0");
     m_name.assfmt("ndbd_%d", nodeid);
     NDBT_find_ndbd(m_exe);
   }
 
+  NdbProcess::Args &args() { return m_args; }
+
+  void set_connect_string(BaseString connect_string) {
+    m_args.add("-c");
+    m_args.add(connect_string.c_str());
+  }
+
   bool start(const char *working_dir, BaseString connect_string) {
-    NdbProcess::Args args;
-    args.add("-c");
-    args.add(connect_string.c_str());
-    args.add("--ndb-nodeid=", m_nodeid);
-    args.add("--foreground");
-    args.add("--loose-core-file=0");
-    return Mgmd::start(working_dir, args);
+    set_connect_string(connect_string);
+    return Mgmd::start(working_dir, m_args);
   }
 
   bool wait_started(NdbMgmHandle &mgm_handle, int timeout = 30,
@@ -347,11 +370,100 @@ class Ndbd : public Mgmd {
            << m_nodeid << endl;
     return false;
   }
+
+ private:
+  NdbProcess::Args m_args;
 };
 
-#include <fstream>
-#include <iostream>
-#include <string>
+static bool create_CA(NDBT_Workingdir &wd, const BaseString &exe) {
+  int ret;
+  NdbProcess::Args args;
+
+  args.add("--passphrase=", "Trondheim");
+  args.add("--create-CA");
+  args.add("--CA-search-path=", wd.path());
+  auto proc = NdbProcess::create("Create CA", exe, wd.path(), args);
+  bool r = proc->wait(ret, 10000);
+
+  return (r && (ret == 0));
+}
+
+static bool sign_tls_keys(NDBT_Workingdir &wd) {
+  int ret;
+  BaseString cfg_path = path(wd.path(), "config.ini", nullptr);
+
+  /* Find executable */
+  BaseString exe;
+  NDBT_find_sign_keys(exe);
+
+  /* Create CA */
+  if (!create_CA(wd, exe)) return false;
+
+  /* Create keys and certificates for all nodes in config */
+  NdbProcess::Args args;
+  args.add("--config-file=", cfg_path.c_str());
+  args.add("--passphrase=", "Trondheim");
+  args.add("--ndb-tls-search-path=", wd.path());
+  args.add("--create-key");
+  auto proc = NdbProcess::create("Create Keys", exe, wd.path(), args);
+  bool r = proc->wait(ret, 10000);
+  return (r && (ret == 0));
+}
+
+static bool create_expired_cert(NDBT_Workingdir &wd) {
+  int ret;
+
+  BaseString cfg_path = path(wd.path(), "config.ini", nullptr);
+
+  /* Find executable */
+  BaseString exe;
+  NDBT_find_sign_keys(exe);
+
+  /* Create CA */
+  if (!create_CA(wd, exe)) return false;
+
+  /* Create an expired certificate for a data node */
+  NdbProcess::Args args;
+  args.add("--create-key");
+  args.add("--ndb-tls-search-path=", wd.path());
+  args.add("--passphrase=", "Trondheim");
+  args.add("-l");                     // no-config mode
+  args.add("--node-type=", "db");     // type db
+  args.add("--duration=", "-50000");  // negative seconds; already expired
+
+  auto proc = NdbProcess::create("Create Keys", exe, wd.path(), args);
+  bool r = proc->wait(ret, 10000);
+  return (r && (ret == 0));
+}
+
+/* Print some information about a cert, and check that its validity is at
+   least 120 days. Return true if ok.
+*/
+static bool check_cert(const NDBT_Workingdir &wd, Node::Type type) {
+  static constexpr int MinDuration = 120 * CertLifetime::SecondsPerDay;
+
+  int duration = 0;
+  PkiFile::PathName certFile;
+  TlsSearchPath searchPath(wd.path());
+  if (ActiveCertificate::find(&searchPath, 0, type, certFile)) {
+    fprintf(stderr, "Reading cert file: %s \n", certFile.c_str());
+    X509 *cert = Certificate::open_one(certFile);
+    if (cert) {
+      char name[65];
+      Certificate::get_common_name(cert, name, sizeof(name));
+      const NodeCertificate *nc = NodeCertificate::for_peer(cert);
+      if (nc) {
+        duration = nc->duration();
+        printf(" ... Cert CN:       %s\n", name);
+        printf(" ... Cert Duration: %d\n", duration);
+        printf(" ... Cert Serial:   %s\n", nc->serial_number().c_str());
+        delete nc;
+      }
+      Certificate::free(cert);
+    }
+  }
+  return (duration >= MinDuration);
+}
 
 bool Print_find_in_file(const char *path, Vector<BaseString> search_string) {
   std::ifstream indata;
@@ -508,7 +620,7 @@ int runTestBug45495(NDBT_Context *ctx, NDBT_Step *step) {
   for (unsigned i = 0; i < mgmds.size(); i++) {
     CHECK(mgmds[i]->stop());
     // Start from config2.ini
-    CHECK(mgmds[i]->start_from_config_ini(wd.path(), "-f config2.ini",
+    CHECK(mgmds[i]->start_from_config_ini(wd.path(), "-f", "config2.ini",
                                           "--initial", NULL));
 
     // Wait for mgmd to exit and check return status
@@ -535,7 +647,7 @@ int runTestBug45495(NDBT_Context *ctx, NDBT_Step *step) {
   for (unsigned i = 0; i < mgmds.size(); i++) {
     CHECK(mgmds[i]->stop());
     // Start from config2.ini
-    CHECK(mgmds[i]->start_from_config_ini(wd.path(), "-f config2.ini",
+    CHECK(mgmds[i]->start_from_config_ini(wd.path(), "-f", "config2.ini",
                                           "--reload", NULL));
     CHECK(mgmds[i]->connect(config));
     CHECK(mgmds[i]->wait_confirmed_config());
@@ -553,7 +665,7 @@ int runTestBug45495(NDBT_Context *ctx, NDBT_Step *step) {
   g_err << "** Reload mgmd initial(from generation=2)" << endl;
   for (unsigned i = 0; i < mgmds.size(); i++) {
     CHECK(mgmds[i]->stop());
-    CHECK(mgmds[i]->start_from_config_ini(wd.path(), "-f config2.ini",
+    CHECK(mgmds[i]->start_from_config_ini(wd.path(), "-f", "config2.ini",
                                           "--reload", "--initial", NULL));
 
     CHECK(mgmds[i]->connect(config));
@@ -777,8 +889,8 @@ int runTestNowaitNodes(NDBT_Context *ctx, NDBT_Step *step) {
     Mgmd *mgmd2 = mgmds[1];
     CHECK(mgmd2->stop());
     // Start from config2.ini
-    CHECK(mgmd2->start_from_config_ini(wd.path(), "-f config2.ini", "--reload",
-                                       NULL));
+    CHECK(mgmd2->start_from_config_ini(wd.path(), "-f", "config2.ini",
+                                       "--reload", NULL));
     CHECK(mgmd2->connect(config));
     CHECK(mgmd1->wait_confirmed_config());
     CHECK(mgmd2->wait_confirmed_config());
@@ -828,16 +940,18 @@ int runTestNowaitNodes2(NDBT_Context *ctx, NDBT_Step *step) {
   g_err << "** Start mgmd2 from config2.ini" << endl;
   Mgmd *mgmd2 = new Mgmd(2);
   mgmds.push_back(mgmd2);
-  CHECK(mgmd2->start_from_config_ini(wd.path(), "-f config2.ini", "--initial",
-                                     "--nowait-nodes=1-255", NULL));
+  CHECK(mgmd2->start_from_config_ini(wd.path(), "-f", "config2.ini",
+                                     "--initial", "--nowait-nodes=1-255",
+                                     NULL));
   CHECK(mgmd2->wait(ret));
   CHECK(ret == 1);
 
   CHECK(mgmd1->stop());
 
   g_err << "** Start mgmd2 again from config2.ini" << endl;
-  CHECK(mgmd2->start_from_config_ini(wd.path(), "-f config2.ini", "--initial",
-                                     "--nowait-nodes=1-255", NULL));
+  CHECK(mgmd2->start_from_config_ini(wd.path(), "-f", "config2.ini",
+                                     "--initial", "--nowait-nodes=1-255",
+                                     NULL));
 
   CHECK(mgmd2->connect(config));
   CHECK(mgmd2->wait_confirmed_config());
@@ -900,7 +1014,7 @@ int runBug56844(NDBT_Context *ctx, NDBT_Step *step) {
     for (unsigned i = 0; i < mgmds.size(); i++) {
       // Start from config2.ini
       CHECK(mgmds[i]->start_from_config_ini(
-          wd.path(), (l & 1) == 1 ? "-f config.ini" : "-f config2.ini",
+          wd.path(), "-f", (l & 1) == 1 ? "config.ini" : "config2.ini",
           "--reload", NULL));
     }
     for (unsigned i = 0; i < mgmds.size(); i++) {
@@ -1298,8 +1412,8 @@ int runTestUnresolvedHosts2(NDBT_Context *ctx, NDBT_Step *step) {
   int ndbd_exit_code;
   Ndbd ndbd2(2);
   CHECK(ndbd2.start(wd.path(), mgmd.connectstring(config)));
-  CHECK(ndbd2.wait(ndbd_exit_code, 200) == 0);  // first 20-second wait
-  CHECK(ndbd2.wait(ndbd_exit_code, 400) == 1);  // second 20-second wait
+  CHECK(ndbd2.wait(ndbd_exit_code, 20000) == 0);
+  CHECK(ndbd2.wait(ndbd_exit_code, 40000) == 1);
   CHECK(ndbd1.stop());
   CHECK(mgmd.stop());
 
@@ -1339,7 +1453,7 @@ int runTestMgmdwithoutnodeid(NDBT_Context *ctx, NDBT_Step *step) {
   // TEST 1: start mgmd without nodeid and unknown address
   with_nodeid = false;
   CHECK(mgmd.start_from_config_ini(wd.path()));
-  CHECK(mgmd.wait(exit_value, 300));
+  CHECK(mgmd.wait(exit_value));
   CHECK(exit_value == 1);
   with_nodeid = true;
   search_list.push_back(
@@ -1358,9 +1472,9 @@ int runTestMgmdwithoutnodeid(NDBT_Context *ctx, NDBT_Step *step) {
   CHECK(ConfigFactory::write_config_ini(
       config, path(wd.path(), "config2.ini", NULL).c_str()));
   with_nodeid = false;
-  CHECK(mgmd.start_from_config_ini(wd.path(), "-f config2.ini", "--initial",
+  CHECK(mgmd.start_from_config_ini(wd.path(), "-f", "config2.ini", "--initial",
                                    NULL));
-  CHECK(mgmd.wait(exit_value, 300));
+  CHECK(mgmd.wait(exit_value));
   CHECK(exit_value == 1);
   with_nodeid = true;
   search_list.push_back(
@@ -1382,9 +1496,9 @@ int runTestMgmdwithoutnodeid(NDBT_Context *ctx, NDBT_Step *step) {
   CHECK(ConfigFactory::write_config_ini(
       config3, path(wd.path(), "config3.ini", NULL).c_str()));
   with_nodeid = false;
-  CHECK(mgmd.start_from_config_ini(wd.path(), "-f config3.ini", "--initial",
+  CHECK(mgmd.start_from_config_ini(wd.path(), "-f", "config3.ini", "--initial",
                                    NULL));
-  CHECK(mgmd.wait(exit_value, 300));
+  CHECK(mgmd.wait(exit_value));
   CHECK(exit_value == 1);
   with_nodeid = true;
 
@@ -1501,33 +1615,377 @@ int runTestMultiMGMDDisconnection(NDBT_Context *ctx, NDBT_Step *step) {
   return NDBT_OK;
 }
 
-int runTestMyCnf(NDBT_Context *ctx, NDBT_Step *step) {
-  /* Construct path name to my.cnf */
-  NDBT_Workingdir wd("test_mgmd");
-  BaseString my_cnf_file = path(wd.path(), "my.cnf", NULL);
+int runTestSshKeySigning(NDBT_Context *ctx, NDBT_Step *step) {
+  /* Skip this test in PB2 environments, where "ssh localhost"
+     does not necessarily work.
+  */
+  if (getenv("PB2WORKDIR")) {
+    printf("Skipping test SshKeySigning\n");
+    return NDBT_OK;
+  }
 
-  /* Create args to start mgmd using a my.cnf file */
+  NDBT_Workingdir wd("test_mgmd");  // temporary working directory
+  Properties config = ConfigFactory::create();
+  ConfigFactory::put(config, "ndb_mgmd", 1, "RequireCertificate", "true");
+  BaseString cfg_path = path(wd.path(), "config.ini", nullptr);
+  CHECK(ConfigFactory::write_config_ini(config, cfg_path.c_str()));
+
+  /* Find executable */
+  BaseString exe;
+  NDBT_find_sign_keys(exe);
+
+  /* Create CA */
+  if (!create_CA(wd, exe)) return false;
+
+  /* Create keys and certificates for all nodes, via ssh to localhost */
+  /* There will be a parent ndb_sign_keys process plus 3 ssh invocations */
   NdbProcess::Args args;
-  args.add("--defaults-file=", my_cnf_file.c_str());
-  args.add("--configdir=", wd.path());
-  args.add("--ndb-nodeid=", 1);
-  args.add("--mycnf");
-
-  /* Write a my.cnf file containing an illegal config parameter */
-  FILE *fp = fopen(my_cnf_file.c_str(), "w");
-  CHECK(fp != nullptr);
-  fprintf(fp, "[cluster_config]\n");
-  fprintf(fp, "ndbcluster=on\n");
-  fclose(fp);
-
-  /* Start mgmd */
-  Mgmd mgmd;
-  CHECK(mgmd.start(wd.path(), args));
-
-  /* Mgmd exits with status 1 due to syntax error in my.cnf */
   int ret;
-  mgmd.wait(ret);
-  CHECK(ret == 1);
+  {
+    args.add("--config-file=", cfg_path.c_str());
+    args.add("--passphrase=", "Trondheim");
+    args.add("--ndb-tls-search-path=", wd.path());
+    args.add("--create-key");
+    args.add("--remote-exec-path=", exe.c_str());
+    args.add("--remote-CA-host=", "localhost");
+    auto proc = NdbProcess::create("Create Keys", exe, wd.path(), args);
+    bool r = proc->wait(ret, 5000);
+    if (!r) proc->stop();
+    CHECK(r);
+    CHECK(ret == 0);
+  }
+  CHECK(check_cert(wd, Node::Type::DB));
+
+  /* Sign again, this time using openssl. ndb_sign_keys is called with
+     the --remote-openssl option, and with --CA-cert and --CA-key holding
+     the full paths to the CA PEM files on the remote server.
+  */
+  {
+    BaseString ca_cert(wd.path());
+    ca_cert.append(DIR_SEPARATOR).append(ClusterCertAuthority::CertFile);
+    BaseString ca_key(wd.path());
+    ca_key.append(DIR_SEPARATOR).append(ClusterCertAuthority::KeyFile);
+
+    args.clear();
+    args.add("--config-file=", cfg_path.c_str());
+    args.add("--passphrase=", "Trondheim");
+    args.add("--ndb-tls-search-path=", wd.path());
+    args.add("--remote-openssl");
+    args.add("--remote-CA-host=", "localhost");
+    args.add("--CA-cert=", ca_cert.c_str());
+    args.add("--CA-key=", ca_key.c_str());
+    auto proc = NdbProcess::create("OpenSSL", exe, wd.path(), args);
+    bool r = proc->wait(ret, 5000);
+    if (!r) proc->stop();
+    CHECK(r);
+    CHECK(ret == 0);
+  }
+  CHECK(check_cert(wd, Node::Type::DB));
+
+  /* Prove that the certificates created above are usable, by starting the mgmd.
+   */
+  args.clear();
+  Mgmd mgmd(1);
+  mgmd.common_args(args, wd.path());
+  args.add("--ndb-tls-search-path=", wd.path());
+  CHECK(mgmd.start(wd.path(), args));
+  CHECK(mgmd.connect(config, 1, 5));
+  CHECK(mgmd.wait_confirmed_config());
+  CHECK(mgmd.stop());
+
+  return NDBT_OK;
+}
+
+int runTestKeySigningTool(NDBT_Context *, NDBT_Step *) {
+  NDBT_Workingdir wd("test_mgmd");  // temporary working directory
+  Properties config = ConfigFactory::create();
+  BaseString cfg_path = path(wd.path(), "config.ini", nullptr);
+  CHECK(ConfigFactory::write_config_ini(config, cfg_path.c_str()));
+
+  /* Find executable */
+  BaseString exe;
+  NDBT_find_sign_keys(exe);
+
+  /* Create CA */
+  if (!create_CA(wd, exe)) return false;
+
+  /* Create key and certificate for node 2 */
+  NdbProcess::Args args;
+  int ret = -1;
+  args.add("--config-file=", cfg_path.c_str());
+  args.add("--passphrase=", "Trondheim");
+  args.add("--ndb-tls-search-path=", wd.path());
+  args.add("--create-key");
+  args.add("-n", 2);
+  args.add("--CA-tool=", exe.c_str());
+  auto proc = NdbProcess::create("Create Keys", exe, wd.path(), args);
+  bool r = proc->wait(ret, 10000);
+  if (!r) proc->stop();
+  CHECK(r);
+  CHECK(ret == 0);
+  return NDBT_OK;
+}
+
+int runTestMgmdWithoutCert(NDBT_Context *ctx, NDBT_Step *step) {
+  NDBT_Workingdir wd("test_mgmd");  // temporary working directory
+  BaseString cfg_path = path(wd.path(), "config.ini", nullptr);
+
+  Properties config = ConfigFactory::create();
+  ConfigFactory::put(config, "ndb_mgmd", 1, "RequireCertificate", "true");
+  CHECK(ConfigFactory::write_config_ini(config, cfg_path.c_str()));
+
+  Mgmd mgmd(1);
+  int exitCode;
+  CHECK(mgmd.start_from_config_ini(wd.path()));  // Start management node
+  CHECK(mgmd.wait(exitCode));
+  CHECK(exitCode == 1);
+  return NDBT_OK;
+}
+
+int runTestApiWithoutCert(NDBT_Context *ctx, NDBT_Step *step) {
+  NDBT_Workingdir wd("test_tls");  // temporary working directory
+
+  BaseString cfg_path = path(wd.path(), "config.ini", nullptr);
+  Properties config = ConfigFactory::create();
+  CHECK(ConfigFactory::put(config, "ndbd", 2, "RequireTls", "true"));
+  CHECK(ConfigFactory::write_config_ini(config, cfg_path.c_str()));
+
+  CHECK(sign_tls_keys(wd));
+
+  Mgmd mgmd(1);
+  Ndbd ndbd(2);
+
+  NdbProcess::Args mgmdArgs;
+  mgmd.common_args(mgmdArgs, wd.path());
+
+  CHECK(mgmd.start(wd.path(), mgmdArgs));  // Start management node
+  CHECK(mgmd.connect(config));             // Connect to management node
+  CHECK(mgmd.wait_confirmed_config());     // Wait for configuration
+
+  ndbd.args().add("--ndb-tls-search-path=", wd.path());
+  ndbd.start(wd.path(), mgmd.connectstring(config));  // Start data node
+  NdbMgmHandle handle = mgmd.handle();
+  CHECK(ndbd.wait_started(handle));
+
+  /* API has no TLS context and should fail to connect */
+  Ndb_cluster_connection con(mgmd.connectstring(config).c_str());
+  con.set_name("api_without_cert");
+  int r = con.connect(0, 0, 1);
+  CHECK(r == -1);
+  printf("ERROR %d: %s\n", con.get_latest_error(), con.get_latest_error_msg());
+
+  ndbd.stop();
+  mgmd.stop();
+  return NDBT_OK;
+}
+
+int runTestNdbdWithoutCert(NDBT_Context *ctx, NDBT_Step *step) {
+  NDBT_Workingdir wd("test_mgmd");  // temporary working directory
+  BaseString cfg_path = path(wd.path(), "config.ini", nullptr);
+
+  Properties config = ConfigFactory::create();
+  Properties db;
+  db.put("RequireCertificate", "true");
+  config.put("DB Default", &db);
+
+  CHECK(ConfigFactory::write_config_ini(config, cfg_path.c_str()));
+
+  Mgmd mgmd(1);
+  Ndbd ndbd(2);
+
+  CHECK(mgmd.start_from_config_ini(wd.path()));  // Start management node
+  CHECK(mgmd.connect(config));                   // Connect to management node
+  CHECK(mgmd.wait_confirmed_config());           // Wait for configuration
+
+  int exit_code;  // Start ndbd; it will fail
+  CHECK(ndbd.start(wd.path(), mgmd.connectstring(config)));
+  CHECK(ndbd.wait(exit_code, 5000));  // should fail quickly
+  require(exit_code == 255);
+
+  CHECK(mgmd.stop());
+  return NDBT_OK;
+}
+
+int runTestNdbdWithExpiredCert(NDBT_Context *ctx, NDBT_Step *step) {
+  NDBT_Workingdir wd("test_tls");  // temporary working directory
+
+  BaseString cfg_path = path(wd.path(), "config.ini", nullptr);
+
+  Properties config = ConfigFactory::create();
+  Properties db;
+  db.put("RequireCertificate", "true");
+  config.put("DB Default", &db);
+  CHECK(ConfigFactory::write_config_ini(config, cfg_path.c_str()));
+
+  CHECK(create_expired_cert(wd));
+
+  Mgmd mgmd(1);
+  Ndbd ndbd(2);
+
+  CHECK(mgmd.start_from_config_ini(wd.path()));  // Start management node
+  CHECK(mgmd.connect(config));                   // Connect to management node
+  CHECK(mgmd.wait_confirmed_config());           // Wait for configuration
+
+  ndbd.args().add("--ndb-tls-search-path=", wd.path());
+  ndbd.start(wd.path(), mgmd.connectstring(config));  // Start data node
+
+  int exit_code;
+  CHECK(ndbd.wait(exit_code, 5000));  // should fail quickly
+  CHECK(exit_code == 255);
+
+  CHECK(mgmd.stop());
+  return NDBT_OK;
+}
+
+int runTestNdbdWithCert(NDBT_Context *ctx, NDBT_Step *step) {
+  NDBT_Workingdir wd("test_tls");  // temporary working directory
+
+  BaseString cfg_path = path(wd.path(), "config.ini", nullptr);
+  Properties config = ConfigFactory::create();
+  Properties db;
+  db.put("RequireCertificate", "true");
+  config.put("DB Default", &db);
+  ConfigFactory::put(config, "ndb_mgmd", 1, "RequireTls", "true");
+  CHECK(ConfigFactory::write_config_ini(config, cfg_path.c_str()));
+
+  CHECK(sign_tls_keys(wd));
+
+  Mgmd mgmd(1);
+  Ndbd ndbd(2);
+
+  NdbProcess::Args mgmdArgs;
+  mgmd.common_args(mgmdArgs, wd.path());
+  mgmdArgs.add("--ndb-tls-search-path=", wd.path());
+
+  TlsKeyManager tls_km;
+  tls_km.init_mgm_client(wd.path(), Node::Type::DB);
+
+  CHECK(mgmd.start(wd.path(), mgmdArgs));  // Start management node
+  CHECK(mgmd.connect(config));             // Connect to management node
+  CHECK(mgmd.client_start_tls(tls_km.ctx()) == 0);  // Start TLS
+  CHECK(mgmd.wait_confirmed_config());              // Wait for configuration
+
+  ndbd.args().add("--ndb-tls-search-path=", wd.path());
+  ndbd.args().add("--ndb-mgm-tls=strict");
+  ndbd.start(wd.path(), mgmd.connectstring(config));  // Start data node
+  NdbMgmHandle handle = mgmd.handle();
+  CHECK(ndbd.wait_started(handle));
+
+  CHECK(mgmd.stop());
+  CHECK(ndbd.stop());
+  return NDBT_OK;
+}
+
+int runTestStartTls(NDBT_Context *ctx, NDBT_Step *step) {
+  NDBT_Workingdir wd("test_tls");  // temporary working directory
+  TlsKeyManager tls_km;
+  int major, minor, build, r;
+  char ver[128];
+  static constexpr int len = sizeof(ver);
+
+  BaseString cfg_path = path(wd.path(), "config.ini", nullptr);
+  Properties config = ConfigFactory::create();
+  CHECK(ConfigFactory::write_config_ini(config, cfg_path.c_str()));
+
+  sign_tls_keys(wd);
+
+  Mgmd mgmd(1);
+
+  NdbProcess::Args mgmdArgs;
+  mgmd.common_args(mgmdArgs, wd.path());
+  mgmdArgs.add("--ndb-tls-search-path=", wd.path());
+
+  CHECK(mgmd.start(wd.path(), mgmdArgs));  // Start management node
+  CHECK(mgmd.connect(config));             // Connect to management node
+  CHECK(mgmd.wait_confirmed_config());     // Wait for configuration
+
+  tls_km.init_mgm_client(wd.path());
+  CHECK(tls_km.ctx());
+
+  r = ndb_mgm_get_version(mgmd.handle(), &major, &minor, &build, len, ver);
+  CHECK(r == 1);
+  printf("Version: %d.%d.%d %s\n", major, minor, build, ver);
+
+  r = ndb_mgm_start_tls(mgmd.handle());
+  CHECK(r == -1);  // -1 is "SSL CTX required"
+  CHECK(ndb_mgm_get_latest_error(mgmd.handle()) == NDB_MGM_TLS_ERROR);
+
+  r = ndb_mgm_set_ssl_ctx(mgmd.handle(), tls_km.ctx());
+  CHECK(r == 0);  // first time setting ctx succeeds
+  r = ndb_mgm_set_ssl_ctx(mgmd.handle(), nullptr);
+  CHECK(r == -1);  // second time setting ctx fails
+
+  r = ndb_mgm_start_tls(mgmd.handle());
+  printf("ndb_mgm_start_tls(): %d\n", r);
+  CHECK(r == 0);
+
+  r = ndb_mgm_start_tls(mgmd.handle());
+  CHECK(r == -2);  // -2 is "Socket already has TLS"
+
+  /* We have switched to TLS. Now run a command. */
+  r = ndb_mgm_get_version(mgmd.handle(), &major, &minor, &build, len, ver);
+  CHECK(r == 1);
+
+  /* And run another command. */
+  struct ndb_mgm_cluster_state *state = ndb_mgm_get_status(mgmd.handle());
+  CHECK(state != nullptr);
+
+  /* Now convert the socket to a transporter */
+  NdbSocket s = mgmd.convert_to_transporter();
+  CHECK(s.is_valid());
+  CHECK(s.close() == 0);
+
+  return NDBT_OK;
+}
+
+int runTestRequireTls(NDBT_Context *ctx, NDBT_Step *step) {
+  /* Create a configuration file in the working directory */
+  NDBT_Workingdir wd("test_tls");
+  BaseString cfg_path = path(wd.path(), "config.ini", nullptr);
+  Properties config = ConfigFactory::create();
+  ConfigFactory::put(config, "ndb_mgmd", 1, "RequireTls", "true");
+  CHECK(ConfigFactory::write_config_ini(config, cfg_path.c_str()));
+
+  /* Create keys in test_tls, and initialize our own TLS context */
+  TlsKeyManager tls_km;
+  bool k = sign_tls_keys(wd);
+  CHECK(k);
+  tls_km.init_mgm_client(wd.path());
+  CHECK(tls_km.ctx());
+
+  /* Start a management server that will require TLS */
+  Mgmd mgmd(1);
+  NdbProcess::Args mgmdArgs;
+  mgmd.common_args(mgmdArgs, wd.path());
+  mgmdArgs.add("--ndb-tls-search-path=", wd.path());
+  CHECK(mgmd.start(wd.path(), mgmdArgs));  // Start management node
+  sleep(1);                                // Wait for confirmed config
+
+  /* Our management client */
+  NdbMgmHandle handle = ndb_mgm_create_handle();
+  ndb_mgm_set_connectstring(handle, mgmd.connectstring(config).c_str());
+  ndb_mgm_set_ssl_ctx(handle, tls_km.ctx());
+
+  int r = ndb_mgm_connect(handle, 3, 5, 1);  // Connect to management node
+  CHECK(r == 0);
+
+  ndb_mgm_severity sev = {NDB_MGM_EVENT_SEVERITY_ON, 1};
+  r = ndb_mgm_get_clusterlog_severity_filter(handle, &sev, 1);
+  CHECK(r < 1);  // COMMAND IS NOT YET ALLOWED
+  int err = ndb_mgm_get_latest_error(handle);
+  CHECK(err == NDB_MGM_AUTH_REQUIRES_TLS);
+
+  struct ndb_mgm_cluster_state *st = ndb_mgm_get_status(handle);
+  CHECK(st == nullptr);  // COMMAND IS NOT YET ALLOWED
+  err = ndb_mgm_get_latest_error(handle);
+  CHECK(err == NDB_MGM_AUTH_REQUIRES_TLS);
+
+  r = ndb_mgm_start_tls(handle);
+  printf("ndb_mgm_start_tls(): %d\n", r);  // START TLS
+  CHECK(r == 0);
+
+  r = ndb_mgm_get_clusterlog_severity_filter(handle, &sev, 1);
+  CHECK(r == 1);  // NOW COMMAND IS ALLOWED
 
   return NDBT_OK;
 }
@@ -1604,10 +2062,47 @@ TESTCASE("MultiMGMDDisconnection",
          "Test multi mgmd robustness against other mgmd disconnections") {
   INITIALIZER(runTestMultiMGMDDisconnection);
 }
-TESTCASE("MyCnf", "Test reading config from my.cnf") {
-  INITIALIZER(runTestMyCnf);
+
+#if OPENSSL_VERSION_NUMBER >= NDB_TLS_MINIMUM_OPENSSL
+
+TESTCASE("MgmdWithoutCertificate",
+         "Test MGM server startup with TLS required but no certificate"){
+    INITIALIZER(runTestMgmdWithoutCert)}
+
+TESTCASE("NdbdWithoutCertificate",
+         "Test data node startup with TLS required but no certificate"){
+    INITIALIZER(runTestNdbdWithoutCert)}
+
+TESTCASE("ApiWithoutCertificate",
+         "Test API node without certificate where TRP TLS is required"){
+    INITIALIZER(runTestApiWithoutCert)}
+
+TESTCASE("NdbdWithExpiredCertificate",
+         "Test data node startup with expired certificate"){
+    INITIALIZER(runTestNdbdWithExpiredCert)}
+
+TESTCASE("NdbdWithCertificate", "Test data node startup with certificate"){
+    INITIALIZER(runTestNdbdWithCert)}
+
+TESTCASE("StartTls", "Test START TLS in MGM protocol") {
+  INITIALIZER(runTestStartTls);
 }
 
+TESTCASE("RequireTls", "Test MGM server that requires TLS") {
+  INITIALIZER(runTestRequireTls);
+}
+
+TESTCASE("KeySigningTool", "Test key signing using a co-process tool") {
+  INITIALIZER(runTestKeySigningTool);
+}
+
+TESTCASE("SshKeySigning",
+         "Test remote key signing over ssh using ndb_sign_keys") {
+  INITIALIZER(runTestSshKeySigning);
+}
+
+#endif
+
 NDBT_TESTSUITE_END(testMgmd)
 
 int main(int argc, const char **argv) {
