@@ -40,6 +40,7 @@
 #include <util/File.hpp>
 
 #include <NdbSleep.h>
+#include <ndb_opts.h>
 #include <ndb_version.h>
 #include <vector>
 #include "my_alloc.h"  // MEM_ROOT
@@ -171,6 +172,8 @@ static struct my_option g_options[] = {
      NO_ARG, 0, 0, 0, 0, 0, 0},
     {"version", 'V', "Output version information and exit.", 0, 0, 0, GET_BOOL,
      NO_ARG, 0, 0, 0, 0, 0, 0},
+    NdbStdOpt::tls_search_path,
+    NdbStdOpt::mgm_tls,
     {"site", 256, "Site", &g_site, &g_site, 0, GET_STR, REQUIRED_ARG, 0, 0, 0,
      0, 0, 0},
     {"clusters", 256, "Cluster", &g_clusters, &g_clusters, 0, GET_STR,
@@ -1016,7 +1019,11 @@ void test_case_results(TestResult *test_result, const atrt_testcase &testcase) {
   res_dir.assfmt("result.%d", testcase.test_no);
   remove_dir(res_dir.c_str(), true);
 
-  if (testcase.m_report || test_result->result != ERR_OK) {
+  if (!testcase.m_report && test_result->result == ERR_TEST_SKIPPED &&
+      rename("result/ndb_api.1", res_dir.c_str()) == 0) {
+    g_logger.debug("Test skipped, storing only test output");
+    remove_dir("result", true);
+  } else if (testcase.m_report || test_result->result != ERR_OK) {
     if (rename("result", res_dir.c_str()) != 0) {
       g_logger.critical("Failed to rename %s as %s", "result", res_dir.c_str());
       remove_dir("result", true);
