@@ -1,15 +1,16 @@
-/* Copyright (c) 2008, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -35,11 +36,14 @@
 #include "my_alloc.h"
 #include "my_compiler.h"
 #include "my_dbug.h"
+#include "my_inttypes.h"
 #include "my_sys.h"
 #include "mysql/components/services/pfs_plugin_table_service.h"
 #include "mysql/plugin.h"
 #include "mysql/status_var.h"
+#include "mysql/strings/int2str.h"
 #include "mysqld_error.h"
+#include "nulls.h"
 #include "sql/hostname_cache.h"
 #include "sql/mysqld.h"
 #include "sql/sql_class.h"
@@ -107,7 +111,7 @@ handlerton *pfs_hton = nullptr;
 #define PFS_ENABLED() \
   (pfs_initialized && (pfs_enabled || m_table_share->m_perpetual))
 
-#define IS_NATIVE_TABLE(X) ((X)->m_st_table.open_table == NULL) ? true : false
+#define IS_NATIVE_TABLE(X) ((X)->m_st_table.open_table == NULL)
 
 static void lock_pfs_external_table_shares() {
   if (!opt_initialize) {
@@ -127,7 +131,7 @@ static handler *pfs_create_handler(handlerton *hton, TABLE_SHARE *table, bool,
 }
 
 static size_t size_of_global_error_stat_buffer() {
-  size_t size = sizeof(PFS_error_single_stat) * max_global_server_errors;
+  const size_t size = sizeof(PFS_error_single_stat) * max_global_server_errors;
   return size;
 }
 
@@ -150,12 +154,6 @@ static size_t size_of_session_error_stat_buffer() {
 */
 static bool pfs_show_status(handlerton *, THD *thd, stat_print_fn *print,
                             enum ha_stat_type stat) {
-  char buf[1024];
-  uint buflen;
-  const char *name;
-  int i;
-  size_t size;
-
   DBUG_TRACE;
 
   /*
@@ -171,7 +169,10 @@ static bool pfs_show_status(handlerton *, THD *thd, stat_print_fn *print,
 
   size_t total_memory = 0;
 
-  for (i = 0; /* empty */; i++) {
+  for (int i = 0; /* empty */; i++) {
+    const char *name;
+    size_t size;
+
     switch (i) {
       case 0:
         name = "events_waits_current.size";
@@ -816,7 +817,6 @@ static bool pfs_show_status(handlerton *, THD *thd, stat_print_fn *print,
         size = global_prepared_stmt_container.get_memory();
         total_memory += size;
         break;
-
       case 143:
         name = "(account_hash).count";
         size = account_hash.count;
@@ -901,7 +901,6 @@ static bool pfs_show_status(handlerton *, THD *thd, stat_print_fn *print,
         name = "host_cache.size";
         size = sizeof(Host_entry);
         break;
-
       case 162:
         name = "(pfs_memory_class).row_size";
         size = sizeof(PFS_memory_class);
@@ -915,7 +914,6 @@ static bool pfs_show_status(handlerton *, THD *thd, stat_print_fn *print,
         size = memory_class_max * sizeof(PFS_memory_class);
         total_memory += size;
         break;
-
       case 165:
         name = "memory_summary_by_thread_by_event_name.row_size";
         size = sizeof(PFS_memory_safe_stat);
@@ -1295,11 +1293,38 @@ static bool pfs_show_status(handlerton *, THD *thd, stat_print_fn *print,
         name = "(max_session_server_errors).count";
         size = max_session_server_errors;
         break;
+      case 247:
+        name = "(pfs_meter_class).size";
+        size = sizeof(PFS_meter_class);
+        break;
+      case 248:
+        name = "(pfs_meter_class).count";
+        size = meter_class_max;
+        break;
+      case 249:
+        name = "(pfs_meter_class).memory";
+        size = meter_class_max * sizeof(PFS_meter_class);
+        total_memory += size;
+        break;
+      case 250:
+        name = "(pfs_metric_class).size";
+        size = sizeof(PFS_metric_class);
+        break;
+      case 251:
+        name = "(pfs_metric_class).count";
+        size = metric_class_max;
+        break;
+      case 252:
+        name = "(pfs_metric_class).memory";
+        size = metric_class_max * sizeof(PFS_metric_class);
+        total_memory += size;
+        break;
+
       /*
         This case must be last,
         for aggregation in total_memory.
       */
-      case 247:
+      case 253:
         name = "performance_schema.memory";
         size = total_memory;
         break;
@@ -1308,7 +1333,8 @@ static bool pfs_show_status(handlerton *, THD *thd, stat_print_fn *print,
         break;
     }
 
-    buflen = (uint)(longlong10_to_str(size, buf, 10) - buf);
+    char buf[1024];
+    const uint buflen = (uint)(longlong10_to_str(size, buf, 10) - buf);
     if (print(thd, PERFORMANCE_SCHEMA_str.str, PERFORMANCE_SCHEMA_str.length,
               name, strlen(name), buf, buflen)) {
       return true;
@@ -1375,10 +1401,11 @@ static PFS_engine_table_share *find_table_share(const char *db,
   @retval false                   Success - no errors.
 */
 
-static bool pfs_dict_init(
-    dict_init_mode_t dict_init_mode, uint version MY_ATTRIBUTE((unused)),
-    List<const Plugin_table> *tables,
-    List<const Plugin_tablespace> *tablespaces MY_ATTRIBUTE((unused))) {
+static bool pfs_dict_init(dict_init_mode_t dict_init_mode,
+                          uint version [[maybe_unused]],
+                          List<const Plugin_table> *tables,
+                          List<const Plugin_tablespace> *tablespaces
+                          [[maybe_unused]]) {
   if (dict_init_mode != DICT_INIT_CREATE_FILES) {
     return false;
   }
@@ -1387,6 +1414,48 @@ static bool pfs_dict_init(
   return false;
 }
 
+static bool pfs_sdi_set_ignored(handlerton *, const dd::Tablespace &,
+                                const dd::Table *, const sdi_key_t *,
+                                const void *, uint64) {
+  return false;
+}
+
+static bool pfs_sdi_delete_ignored(const dd::Tablespace &, const dd::Table *,
+                                   const sdi_key_t *) {
+  return false;
+}
+
+static bool pfs_sdi_get_ignored(const dd::Tablespace &, const sdi_key_t *,
+                                void *, uint64 *) {
+  return false;
+}
+
+static bool pfs_sdi_create_ignored(dd::Tablespace *) { return false; }
+
+static bool pfs_sdi_drop_ignored(dd::Tablespace *) { return false; }
+
+static bool pfs_sdi_get_keys_ignored(const dd::Tablespace &, sdi_vector_t &) {
+  return false;
+}
+
+void pfs_sdi_disable() {
+  pfs_hton->sdi_set = pfs_sdi_set_ignored;
+  pfs_hton->sdi_delete = pfs_sdi_delete_ignored;
+  pfs_hton->sdi_get = pfs_sdi_get_ignored;
+  pfs_hton->sdi_create = pfs_sdi_create_ignored;
+  pfs_hton->sdi_drop = pfs_sdi_drop_ignored;
+  pfs_hton->sdi_get_keys = pfs_sdi_get_keys_ignored;
+}
+
+void pfs_sdi_enable() {
+  pfs_hton->sdi_set = nullptr;
+  pfs_hton->sdi_delete = nullptr;
+  pfs_hton->sdi_get = nullptr;
+  pfs_hton->sdi_create = nullptr;
+  pfs_hton->sdi_drop = nullptr;
+  pfs_hton->sdi_get_keys = nullptr;
+}
+
 static int pfs_init_func(void *p) {
   DBUG_TRACE;
 
@@ -1509,6 +1578,10 @@ static SHOW_VAR pfs_status_vars[] = {
      SHOW_SCOPE_GLOBAL},
     {"Performance_schema_metadata_lock_lost",
      (char *)&global_mdl_container.m_lost, SHOW_LONG, SHOW_SCOPE_GLOBAL},
+    {"Performance_schema_meter_lost", (char *)&meter_class_lost, SHOW_LONG,
+     SHOW_SCOPE_GLOBAL},
+    {"Performance_schema_metric_lost", (char *)&metric_class_lost, SHOW_LONG,
+     SHOW_SCOPE_GLOBAL},
     {NullS, NullS, SHOW_LONG, SHOW_SCOPE_GLOBAL}};
 
 struct st_mysql_storage_engine pfs_storage_engine = {
@@ -1540,7 +1613,7 @@ mysql_declare_plugin_end;
 ha_perfschema::ha_perfschema(handlerton *hton, TABLE_SHARE *share)
     : handler(hton, share), m_table_share(nullptr), m_table(nullptr) {}
 
-ha_perfschema::~ha_perfschema() {}
+ha_perfschema::~ha_perfschema() = default;
 
 int ha_perfschema::open(const char *, int, uint, const dd::Table *) {
   DBUG_TRACE;
@@ -1566,7 +1639,7 @@ int ha_perfschema::open(const char *, int, uint, const dd::Table *) {
   return 0;
 }
 
-int ha_perfschema::close(void) {
+int ha_perfschema::close() {
   DBUG_TRACE;
 
   /* Only for table added by plugin/components */
@@ -1598,7 +1671,7 @@ int ha_perfschema::write_row(uchar *buf) {
   return result;
 }
 
-void ha_perfschema::use_hidden_primary_key(void) {
+void ha_perfschema::use_hidden_primary_key() {
   /*
     This is also called in case of row based replication,
     see TABLE::mark_columns_needed_for_update().
@@ -1620,7 +1693,8 @@ int ha_perfschema::update_row(const uchar *old_data, uchar *new_data) {
 
   assert(m_table);
   ha_statistic_increment(&System_status_var::ha_update_count);
-  int result = m_table->update_row(table, old_data, new_data, table->field);
+  const int result =
+      m_table->update_row(table, old_data, new_data, table->field);
   return result;
 }
 
@@ -1632,7 +1706,7 @@ int ha_perfschema::delete_row(const uchar *buf) {
 
   assert(m_table);
   ha_statistic_increment(&System_status_var::ha_delete_count);
-  int result = m_table->delete_row(table, buf, table->field);
+  const int result = m_table->delete_row(table, buf, table->field);
   return result;
 }
 
@@ -1658,7 +1732,7 @@ int ha_perfschema::rnd_init(bool scan) {
   return result;
 }
 
-int ha_perfschema::rnd_end(void) {
+int ha_perfschema::rnd_end() {
   DBUG_TRACE;
   assert(m_table);
   delete m_table;
@@ -1719,7 +1793,7 @@ int ha_perfschema::info(uint flag) {
   return 0;
 }
 
-int ha_perfschema::delete_all_rows(void) {
+int ha_perfschema::delete_all_rows() {
   int result;
 
   DBUG_TRACE;
@@ -1830,7 +1904,7 @@ ulong ha_perfschema::index_flags(uint, uint, bool) const {
     /* ha_perfschema::index_flags is const, can not save in m_table_share. */
   }
 
-  ulong flags = HA_KEY_SCAN_NOT_ROR;
+  constexpr ulong flags = HA_KEY_SCAN_NOT_ROR;
 
   if (!tmp) {
     unlock_pfs_external_table_shares();
