@@ -1,15 +1,16 @@
-/* Copyright (c) 2010, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -176,7 +177,7 @@ PFS_account *find_or_create_account(PFS_thread *thread,
   PFS_account **entry;
   PFS_account *pfs;
   uint retry_count = 0;
-  const uint retry_max = 3;
+  constexpr uint retry_max = 3;
   pfs_dirty_state dirty_state;
 
 search:
@@ -211,9 +212,9 @@ search:
     }
 
     int res;
-    pfs->m_lock.dirty_to_allocated(&dirty_state);
     res = lf_hash_insert(&account_hash, pins, &pfs);
     if (likely(res == 0)) {
+      pfs->m_lock.dirty_to_allocated(&dirty_state);
       return pfs;
     }
 
@@ -226,7 +227,7 @@ search:
       pfs->m_host = nullptr;
     }
 
-    global_account_container.deallocate(pfs);
+    global_account_container.dirty_to_free(&dirty_state, pfs);
 
     if (res > 0) {
       if (++retry_count > retry_max) {
@@ -601,7 +602,7 @@ void PFS_account::release() { dec_refcount(); }
 
 void PFS_account::rebase_memory_stats() {
   PFS_memory_shared_stat *stat = m_instr_class_memory_stats;
-  PFS_memory_shared_stat *stat_last = stat + memory_class_max;
+  const PFS_memory_shared_stat *stat_last = stat + memory_class_max;
   for (; stat < stat_last; stat++) {
     stat->reset();
   }
