@@ -1,15 +1,16 @@
-/* Copyright (c) 2014, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2014, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -93,12 +94,17 @@ class PFS_buffer_default_array {
     size_t monotonic_max;
     value_type *pfs;
 
-    if (m_full) {
+    if (m_full.load()) {
       return nullptr;
     }
 
+    // Immutable
+    size_t max = m_max.load();
+    // Immutable
+    T *ptr = m_ptr.load();
+
     monotonic = m_monotonic.m_size_t++;
-    monotonic_max = monotonic + m_max;
+    monotonic_max = monotonic + max;
 
     if (unlikely(monotonic >= monotonic_max)) {
       /*
@@ -108,12 +114,12 @@ class PFS_buffer_default_array {
       */
       m_monotonic.m_size_t.store(0);
       monotonic = 0;
-      monotonic_max = m_max;
+      monotonic_max = max;
     }
 
     while (monotonic < monotonic_max) {
-      index = monotonic % m_max;
-      pfs = m_ptr + index;
+      index = monotonic % max;
+      pfs = ptr + index;
 
       if (pfs->m_lock.free_to_dirty(dirty_state)) {
         return pfs;
@@ -121,25 +127,68 @@ class PFS_buffer_default_array {
       monotonic = m_monotonic.m_size_t++;
     }
 
-    m_full = true;
+    m_full.store(true);
     return nullptr;
   }
 
   void deallocate(value_type *pfs) {
     pfs->m_lock.allocated_to_free();
-    m_full = false;
+
+    if (m_full.load()) {
+      m_full.store(false);
+    }
   }
 
   T *get_first() { return m_ptr; }
 
   T *get_last() { return m_ptr + m_max; }
 
-  bool m_full;
+  /**
+   Page full flag.
+
+   Concurrency profile:
+   - mostly read during normal operations.
+   - do not perform useless write,
+     check for previous value first
+   - occasional write on state changes
+  */
+  std::atomic<bool> m_full{false};
+
+  /**
+   Monotonic counter.
+   This counter is used to access items in the page.
+
+   Concurrency profile:
+   - frequent read and write, must be on its own cacheline
+  */
   PFS_cacheline_atomic_size_t m_monotonic;
-  T *m_ptr;
-  size_t m_max;
-  /** Container. */
-  PFS_opaque_container *m_container;
+
+  /**
+   Array of values.
+
+   Concurrency profile:
+   - written once on page creation
+   - read only during normal operations.
+  */
+  std::atomic<T *> m_ptr{nullptr};
+
+  /**
+   Max number of items in the page.
+
+   Concurrency profile:
+   - written once on page creation
+   - read only during normal operations.
+  */
+  std::atomic<size_t> m_max{0};
+
+  /**
+   Container.
+
+   Concurrency profile:
+   - written once on page creation
+   - read only during normal operations.
+  */
+  std::atomic<PFS_opaque_container *> m_container{nullptr};
 };
 
 template <class T>
@@ -204,7 +253,7 @@ class PFS_buffer_container {
   int init(size_t max_size) {
     if (max_size > 0) {
       m_array.m_max = max_size;
-      int rc = m_allocator->alloc_array(&m_array);
+      const int rc = m_allocator->alloc_array(&m_array);
       if (rc != 0) {
         m_allocator->free_array(&m_array);
         return 1;
@@ -227,7 +276,7 @@ class PFS_buffer_container {
     value_type *pfs;
 
     pfs = m_array.allocate(dirty_state, m_max);
-    if (pfs == NULL) {
+    if (pfs == nullptr) {
       m_lost++;
     }
 
@@ -251,7 +300,7 @@ class PFS_buffer_container {
       if (pfs->m_lock.is_populated()) {
         fct(pfs);
       }
-      pfs++;
+      ++pfs;
     }
   }
 
@@ -261,7 +310,7 @@ class PFS_buffer_container {
 
     while (pfs < pfs_last) {
       fct(pfs);
-      pfs++;
+      ++pfs;
     }
   }
 
@@ -273,7 +322,7 @@ class PFS_buffer_container {
       if (pfs->m_lock.is_populated()) {
         proc(pfs);
       }
-      pfs++;
+      ++pfs;
     }
   }
 
@@ -283,7 +332,7 @@ class PFS_buffer_container {
 
     while (pfs < pfs_last) {
       proc(pfs);
-      pfs++;
+      ++pfs;
     }
   }
 
@@ -295,13 +344,13 @@ class PFS_buffer_container {
       return pfs;
     }
 
-    return NULL;
+    return nullptr;
   }
 
   value_type *get(uint index, bool *has_more) {
     if (index >= m_max) {
       *has_more = false;
-      return NULL;
+      return nullptr;
     }
 
     *has_more = true;
@@ -309,18 +358,17 @@ class PFS_buffer_container {
   }
 
   value_type *sanitize(value_type *unsafe) {
-    intptr offset;
     value_type *pfs = m_array.get_first();
     value_type *pfs_last = m_array.get_last();
 
     if ((pfs <= unsafe) && (unsafe < pfs_last)) {
-      offset = ((intptr)unsafe - (intptr)pfs) % sizeof(value_type);
+      const intptr offset = ((intptr)unsafe - (intptr)pfs) % sizeof(value_type);
       if (offset == 0) {
         return unsafe;
       }
     }
 
-    return NULL;
+    return nullptr;
   }
 
   ulong m_lost;
@@ -335,16 +383,16 @@ class PFS_buffer_container {
 
     while (pfs < pfs_last) {
       if (pfs->m_lock.is_populated()) {
-        uint found = pfs - pfs_first;
+        const uint found = pfs - pfs_first;
         *found_index = found;
         index = found + 1;
         return pfs;
       }
-      pfs++;
+      ++pfs;
     }
 
     index = m_max;
-    return NULL;
+    return nullptr;
   }
 
   size_t m_max;
@@ -388,6 +436,11 @@ class PFS_buffer_scalable_container {
   explicit PFS_buffer_scalable_container(allocator_type *allocator) {
     m_allocator = allocator;
     m_initialized = false;
+    m_full = true;
+    m_max = PFS_PAGE_COUNT * PFS_PAGE_SIZE;
+    m_max_page_count = PFS_PAGE_COUNT;
+    m_last_page_size = PFS_PAGE_SIZE;
+    m_lost = 0;
   }
 
   int init(long max_size) {
@@ -444,12 +497,14 @@ class PFS_buffer_scalable_container {
       return;
     }
 
+    allocator_type *allocator = m_allocator.load();
+
     native_mutex_lock(&m_critical_section);
 
     for (i = 0; i < PFS_PAGE_COUNT; i++) {
       page = m_pages[i];
       if (page != nullptr) {
-        m_allocator->free_array(page);
+        allocator->free_array(page);
         delete page;
         m_pages[i] = nullptr;
       }
@@ -462,7 +517,7 @@ class PFS_buffer_scalable_container {
   }
 
   size_t get_row_count() {
-    size_t page_count = m_max_page_index.m_size_t.load();
+    const size_t page_count = m_max_page_index.m_size_t.load();
     size_t result = page_count * PFS_PAGE_SIZE;
 
     if ((page_count > 0) && (m_last_page_size != PFS_PAGE_SIZE)) {
@@ -477,14 +532,12 @@ class PFS_buffer_scalable_container {
   size_t get_memory() { return get_row_count() * get_row_size(); }
 
   value_type *allocate(pfs_dirty_state *dirty_state) {
-    if (m_full) {
+    if (m_full.load()) {
       m_lost++;
       return nullptr;
     }
 
     size_t index;
-    size_t monotonic;
-    size_t monotonic_max;
     size_t current_page_count;
     value_type *pfs;
     array_type *array;
@@ -495,8 +548,8 @@ class PFS_buffer_scalable_container {
     current_page_count = m_max_page_index.m_size_t.load();
 
     if (current_page_count != 0) {
-      monotonic = m_monotonic.m_size_t.load();
-      monotonic_max = monotonic + current_page_count;
+      size_t monotonic = m_monotonic.m_size_t.load();
+      size_t monotonic_max = monotonic + current_page_count;
 
       if (unlikely(monotonic >= monotonic_max)) {
         /*
@@ -599,9 +652,12 @@ class PFS_buffer_scalable_container {
           builtin_memory_scalable_buffer.count_alloc(sizeof(array_type));
 
           array->m_max = get_page_logical_size(current_page_count);
-          int rc = m_allocator->alloc_array(array);
+
+          allocator_type *allocator = m_allocator.load();
+
+          const int rc = allocator->alloc_array(array);
           if (rc != 0) {
-            m_allocator->free_array(array);
+            allocator->free_array(array);
             delete array;
             builtin_memory_scalable_buffer.count_free(sizeof(array_type));
             m_lost++;
@@ -638,10 +694,29 @@ class PFS_buffer_scalable_container {
     }
 
     m_lost++;
-    m_full = true;
+    m_full.store(true);
     return nullptr;
   }
 
+  void dirty_to_free(pfs_dirty_state *dirty_state, value_type *safe_pfs) {
+    /* Find the containing page */
+    PFS_opaque_container_page *opaque_page = safe_pfs->m_page;
+    auto *page = reinterpret_cast<array_type *>(opaque_page);
+
+    /* Mark the object free */
+    safe_pfs->m_lock.dirty_to_free(dirty_state);
+
+    /* Flag the containing page as not full. */
+    if (page->m_full.load()) {
+      page->m_full.store(false);
+    }
+
+    /* Flag the overall container as not full. */
+    if (m_full.load()) {
+      m_full.store(false);
+    }
+  }
+
   void deallocate(value_type *safe_pfs) {
     /* Find the containing page */
     PFS_opaque_container_page *opaque_page = safe_pfs->m_page;
@@ -651,10 +726,14 @@ class PFS_buffer_scalable_container {
     safe_pfs->m_lock.allocated_to_free();
 
     /* Flag the containing page as not full. */
-    page->m_full = false;
+    if (page->m_full.load()) {
+      page->m_full.store(false);
+    }
 
     /* Flag the overall container as not full. */
-    m_full = false;
+    if (m_full.load()) {
+      m_full.store(false);
+    }
   }
 
   static void static_deallocate(value_type *safe_pfs) {
@@ -666,7 +745,9 @@ class PFS_buffer_scalable_container {
     safe_pfs->m_lock.allocated_to_free();
 
     /* Flag the containing page as not full. */
-    page->m_full = false;
+    if (page->m_full.load()) {
+      page->m_full.store(false);
+    }
 
     /* Find the containing buffer */
     PFS_opaque_container *opaque_container = page->m_container;
@@ -674,7 +755,9 @@ class PFS_buffer_scalable_container {
     container = reinterpret_cast<container_type *>(opaque_container);
 
     /* Flag the overall container as not full. */
-    container->m_full = false;
+    if (container->m_full.load()) {
+      container->m_full.store(false);
+    }
   }
 
   iterator_type iterate() {
@@ -704,7 +787,7 @@ class PFS_buffer_scalable_container {
           if (pfs->m_lock.is_populated()) {
             fct(pfs);
           }
-          pfs++;
+          ++pfs;
         }
       }
     }
@@ -724,7 +807,7 @@ class PFS_buffer_scalable_container {
 
         while (pfs < pfs_last) {
           fct(pfs);
-          pfs++;
+          ++pfs;
         }
       }
     }
@@ -746,7 +829,7 @@ class PFS_buffer_scalable_container {
           if (pfs->m_lock.is_populated()) {
             proc(pfs);
           }
-          pfs++;
+          ++pfs;
         }
       }
     }
@@ -760,13 +843,13 @@ class PFS_buffer_scalable_container {
 
     for (i = 0; i < PFS_PAGE_COUNT; i++) {
       page = m_pages[i];
-      if (page != NULL) {
+      if (page != nullptr) {
         pfs = page->get_first();
         pfs_last = page->get_last();
 
         while (pfs < pfs_last) {
           proc(pfs);
-          pfs++;
+          ++pfs;
         }
       }
     }
@@ -885,16 +968,16 @@ class PFS_buffer_scalable_container {
 
       while (pfs < pfs_last) {
         if (pfs->m_lock.is_populated()) {
-          uint found =
+          const uint found =
               index_1 * PFS_PAGE_SIZE + static_cast<uint>(pfs - pfs_first);
           *found_index = found;
           index = found + 1;
           return pfs;
         }
-        pfs++;
+        ++pfs;
       }
 
-      index_1++;
+      ++index_1;
       index_2 = 0;
     }
 
@@ -902,15 +985,89 @@ class PFS_buffer_scalable_container {
     return nullptr;
   }
 
-  bool m_initialized;
-  bool m_full;
-  size_t m_max;
+  /**
+   Initialized full flag.
+
+   Concurrency profile:
+   - write in init / cleanup
+   - readonly during normal operations
+  */
+  std::atomic<bool> m_initialized{false};
+
+  /**
+   Buffer full flag.
+
+   Concurrency profile:
+   - mostly read during normal operations.
+   - do not perform useless write,
+     check for previous value first
+   - occasional write on state changes
+  */
+  std::atomic<bool> m_full{false};
+
+  /**
+   Max number of items in the buffer.
+
+   Concurrency profile:
+   - written once on page creation
+   - read only during normal operations.
+  */
+  std::atomic<size_t> m_max{0};
+
+  /**
+   Monotonic page counter.
+   This counter is used to access pages in the array.
+
+   Concurrency profile:
+   - frequent read and write, must be on its own cacheline
+  */
   PFS_cacheline_atomic_size_t m_monotonic;
+
+  /**
+   Current page index.
+
+   Concurrency profile:
+   - occasional write on buffer extend
+   - mostly read otherwise
+  */
   PFS_cacheline_atomic_size_t m_max_page_index;
-  size_t m_max_page_count;
-  size_t m_last_page_size;
+
+  /**
+   Max number of pages.
+
+   Concurrency profile:
+   - written once on buffer creation
+   - read only during normal operations.
+  */
+  std::atomic<size_t> m_max_page_count{0};
+
+  /**
+   Size of the last page.
+
+   Concurrency profile:
+   - written once on buffer creation
+   - read only during normal operations.
+  */
+  std::atomic<size_t> m_last_page_size{0};
+
+  /**
+   Array of pages.
+
+   Concurrency profile:
+   - occasional write on buffer extend
+   - mostly read otherwise
+  */
   std::atomic<array_type *> m_pages[PFS_PAGE_COUNT];
-  allocator_type *m_allocator;
+
+  /**
+   Buffer allocator.
+
+   Concurrency profile:
+   - written once on buffer creation
+   - read only during normal operations.
+  */
+  std::atomic<allocator_type *> m_allocator{nullptr};
+
   native_mutex_t m_critical_section;
 };
 
@@ -1115,7 +1272,7 @@ class PFS_partitioned_buffer_scalable_container {
 
     if (partition_index >= PFS_PARTITION_COUNT) {
       *has_more = false;
-      return NULL;
+      return nullptr;
     }
 
     *has_more = true;
@@ -1353,12 +1510,12 @@ extern PFS_prepared_stmt_container global_prepared_stmt_container;
 
 class PFS_account_array : public PFS_buffer_default_array<PFS_account> {
  public:
-  PFS_single_stat *m_instr_class_waits_array;
-  PFS_stage_stat *m_instr_class_stages_array;
-  PFS_statement_stat *m_instr_class_statements_array;
-  PFS_transaction_stat *m_instr_class_transactions_array;
-  PFS_error_stat *m_instr_class_errors_array;
-  PFS_memory_shared_stat *m_instr_class_memory_array;
+  PFS_single_stat *m_instr_class_waits_array{nullptr};
+  PFS_stage_stat *m_instr_class_stages_array{nullptr};
+  PFS_statement_stat *m_instr_class_statements_array{nullptr};
+  PFS_transaction_stat *m_instr_class_transactions_array{nullptr};
+  PFS_error_stat *m_instr_class_errors_array{nullptr};
+  PFS_memory_shared_stat *m_instr_class_memory_array{nullptr};
 };
 
 class PFS_account_allocator {
@@ -1381,12 +1538,12 @@ extern PFS_account_container global_account_container;
 
 class PFS_host_array : public PFS_buffer_default_array<PFS_host> {
  public:
-  PFS_single_stat *m_instr_class_waits_array;
-  PFS_stage_stat *m_instr_class_stages_array;
-  PFS_statement_stat *m_instr_class_statements_array;
-  PFS_transaction_stat *m_instr_class_transactions_array;
-  PFS_error_stat *m_instr_class_errors_array;
-  PFS_memory_shared_stat *m_instr_class_memory_array;
+  PFS_single_stat *m_instr_class_waits_array{nullptr};
+  PFS_stage_stat *m_instr_class_stages_array{nullptr};
+  PFS_statement_stat *m_instr_class_statements_array{nullptr};
+  PFS_transaction_stat *m_instr_class_transactions_array{nullptr};
+  PFS_error_stat *m_instr_class_errors_array{nullptr};
+  PFS_memory_shared_stat *m_instr_class_memory_array{nullptr};
 };
 
 class PFS_host_allocator {
@@ -1408,24 +1565,24 @@ extern PFS_host_container global_host_container;
 
 class PFS_thread_array : public PFS_buffer_default_array<PFS_thread> {
  public:
-  PFS_single_stat *m_instr_class_waits_array;
-  PFS_stage_stat *m_instr_class_stages_array;
-  PFS_statement_stat *m_instr_class_statements_array;
-  PFS_transaction_stat *m_instr_class_transactions_array;
-  PFS_error_stat *m_instr_class_errors_array;
-  PFS_memory_safe_stat *m_instr_class_memory_array;
+  PFS_single_stat *m_instr_class_waits_array{nullptr};
+  PFS_stage_stat *m_instr_class_stages_array{nullptr};
+  PFS_statement_stat *m_instr_class_statements_array{nullptr};
+  PFS_transaction_stat *m_instr_class_transactions_array{nullptr};
+  PFS_error_stat *m_instr_class_errors_array{nullptr};
+  PFS_memory_safe_stat *m_instr_class_memory_array{nullptr};
 
-  PFS_events_waits *m_waits_history_array;
-  PFS_events_stages *m_stages_history_array;
-  PFS_events_statements *m_statements_history_array;
-  PFS_events_statements *m_statements_stack_array;
-  PFS_events_transactions *m_transactions_history_array;
-  char *m_session_connect_attrs_array;
+  PFS_events_waits *m_waits_history_array{nullptr};
+  PFS_events_stages *m_stages_history_array{nullptr};
+  PFS_events_statements *m_statements_history_array{nullptr};
+  PFS_events_statements *m_statements_stack_array{nullptr};
+  PFS_events_transactions *m_transactions_history_array{nullptr};
+  char *m_session_connect_attrs_array{nullptr};
 
-  char *m_current_stmts_text_array;
-  char *m_history_stmts_text_array;
-  unsigned char *m_current_stmts_digest_token_array;
-  unsigned char *m_history_stmts_digest_token_array;
+  char *m_current_stmts_text_array{nullptr};
+  char *m_history_stmts_text_array{nullptr};
+  unsigned char *m_current_stmts_digest_token_array{nullptr};
+  unsigned char *m_history_stmts_digest_token_array{nullptr};
 };
 
 class PFS_thread_allocator {
@@ -1447,12 +1604,12 @@ extern PFS_thread_container global_thread_container;
 
 class PFS_user_array : public PFS_buffer_default_array<PFS_user> {
  public:
-  PFS_single_stat *m_instr_class_waits_array;
-  PFS_stage_stat *m_instr_class_stages_array;
-  PFS_statement_stat *m_instr_class_statements_array;
-  PFS_transaction_stat *m_instr_class_transactions_array;
-  PFS_error_stat *m_instr_class_errors_array;
-  PFS_memory_shared_stat *m_instr_class_memory_array;
+  PFS_single_stat *m_instr_class_waits_array{nullptr};
+  PFS_stage_stat *m_instr_class_stages_array{nullptr};
+  PFS_statement_stat *m_instr_class_statements_array{nullptr};
+  PFS_transaction_stat *m_instr_class_transactions_array{nullptr};
+  PFS_error_stat *m_instr_class_errors_array{nullptr};
+  PFS_memory_shared_stat *m_instr_class_memory_array{nullptr};
 };
 
 class PFS_user_allocator {
