@@ -1,4 +1,4 @@
-/* Copyright (c) 2008, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -120,6 +120,7 @@
 #include "storage/perfschema/table_replication_applier_status_by_coordinator.h"
 #include "storage/perfschema/table_replication_applier_status_by_worker.h"
 /* For replication related perfschema tables. */
+#include "storage/perfschema/table_global_variable_attributes.h"
 #include "storage/perfschema/table_log_status.h"
 #include "storage/perfschema/table_replication_asynchronous_connection_failover.h"
 #include "storage/perfschema/table_replication_connection_configuration.h"
@@ -157,6 +158,7 @@
 #include "storage/perfschema/table_uvar_by_thread.h"
 #include "storage/perfschema/table_variables_by_thread.h"
 #include "storage/perfschema/table_variables_info.h"
+#include "storage/perfschema/table_variables_metadata.h"
 #include "thr_lock.h"
 #include "thr_mutex.h"
 
@@ -602,6 +604,8 @@ static PFS_engine_table_share *all_shares[] = {
     &table_global_variables::m_share,
     &table_session_variables::m_share,
     &table_variables_info::m_share,
+    &table_variables_metadata::m_share,
+    &table_global_variable_attributes::m_share,
     &table_persisted_variables::m_share,
     &table_user_defined_functions::m_share,
     &table_binary_log_transaction_compression_stats::m_share,
@@ -936,7 +940,7 @@ static bool allow_drop_schema_privilege() {
       in particular to drop unknown tables,
       see PFS_unknown_acl::check()
   */
-  THD *thd = current_thd;
+  const THD *thd = current_thd;
   if (thd == nullptr) {
     return false;
   }
@@ -1018,7 +1022,7 @@ static bool allow_drop_table_privilege() {
     Here, we want to prevent DROP / ALTER  while allowing TRUNCATE.
     Note that we must also allow GRANT to transfer the truncate privilege.
   */
-  THD *thd = current_thd;
+  const THD *thd = current_thd;
   if (thd == nullptr) {
     return false;
   }
@@ -1408,7 +1412,7 @@ enum ha_rkey_function PFS_key_reader::read_varchar_utf8(
     memcpy(buffer, m_remaining_key + data_offset, string_len);
     *buffer_length = (uint)string_len;
 
-    auto *pos = (uchar *)buffer;
+    const auto *pos = (uchar *)buffer;
     const uchar *end = skip_trailing_space(pos, string_len);
     *buffer_length = (uint)(end - pos);
 
@@ -1456,7 +1460,7 @@ enum ha_rkey_function PFS_key_reader::read_text_utf8(
     *buffer_length = (uint)string_len;
 
     const CHARSET_INFO *cs = &my_charset_utf8mb4_bin;
-    auto *pos = (uchar *)buffer;
+    const auto *pos = (uchar *)buffer;
     if (cs->mbmaxlen > 1) {
       size_t char_length;
       char_length =
@@ -1512,7 +1516,6 @@ void PFS_engine_index::read_key(const uchar *key, uint key_len,
   }
 
   m_fields = reader.m_parts_found;
-  m_key_fetch_count = 0;
 }
 
 /** @} */
