@@ -1,15 +1,16 @@
-/* Copyright (c) 2008, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -520,7 +521,7 @@ void PFS_thread::set_history_derived_flags() {
 
 void PFS_thread::rebase_memory_stats() {
   PFS_memory_safe_stat *stat = m_instr_class_memory_stats;
-  PFS_memory_safe_stat *stat_last = stat + memory_class_max;
+  const PFS_memory_safe_stat *stat_last = stat + memory_class_max;
   for (; stat < stat_last; stat++) {
     stat->reset();
   }
@@ -742,7 +743,7 @@ PFS_thread *create_thread(PFS_thread_class *klass, PSI_thread_seqnum seqnum,
   @return pfs pointer if found, else NULL
 */
 PFS_thread *find_thread_by_internal_id(ulonglong thread_id) {
-  PFS_thread *pfs = nullptr;
+  PFS_thread *pfs;
   uint index = 0;
 
   PFS_thread_iterator it = global_thread_container.iterate(index);
@@ -765,7 +766,7 @@ PFS_thread *find_thread_by_internal_id(ulonglong thread_id) {
   @return pfs pointer if found, else NULL
 */
 PFS_thread *find_thread_by_processlist_id(ulonglong processlist_id) {
-  PFS_thread *pfs = nullptr;
+  PFS_thread *pfs;
   uint index = 0;
 
   /* Valid processlist ID is > 0 (see Global_THD_manager) */
@@ -1022,7 +1023,6 @@ int normalize_filename(const char *filename, uint name_len,
 
   /* Resolve the absolute directory path. */
   if (my_realpath(buffer, dirbuffer, MYF(0)) != 0) {
-    buffer[0] = '\0';
     return 1;
   }
 
@@ -1073,7 +1073,7 @@ PFS_file *find_or_create_file(PFS_thread *thread, PFS_file_class *klass,
 
   PFS_file **entry;
   uint retry_count = 0;
-  const uint retry_max = 3;
+  constexpr uint retry_max = 3;
   pfs_dirty_state dirty_state;
 
 search:
@@ -1428,7 +1428,7 @@ void PFS_table::safe_aggregate_lock(PFS_table_stat *table_stat,
   assert(table_stat != nullptr);
   assert(table_share != nullptr);
 
-  PFS_table_lock_stat *from_stat = &table_stat->m_lock_stat;
+  const PFS_table_lock_stat *from_stat = &table_stat->m_lock_stat;
 
   PFS_table_share_lock *to_stat;
 
@@ -1643,7 +1643,7 @@ void reset_histogram_global() { global_statements_histogram.reset(); }
 void aggregate_all_event_names(PFS_single_stat *from_array,
                                PFS_single_stat *to_array) {
   PFS_single_stat *from;
-  PFS_single_stat *from_last;
+  const PFS_single_stat *from_last;
   PFS_single_stat *to;
 
   from = from_array;
@@ -1662,7 +1662,7 @@ void aggregate_all_event_names(PFS_single_stat *from_array,
                                PFS_single_stat *to_array_1,
                                PFS_single_stat *to_array_2) {
   PFS_single_stat *from;
-  PFS_single_stat *from_last;
+  const PFS_single_stat *from_last;
   PFS_single_stat *to_1;
   PFS_single_stat *to_2;
 
@@ -1683,7 +1683,7 @@ void aggregate_all_event_names(PFS_single_stat *from_array,
 void aggregate_all_stages(PFS_stage_stat *from_array,
                           PFS_stage_stat *to_array) {
   PFS_stage_stat *from;
-  PFS_stage_stat *from_last;
+  const PFS_stage_stat *from_last;
   PFS_stage_stat *to;
 
   from = from_array;
@@ -1702,7 +1702,7 @@ void aggregate_all_stages(PFS_stage_stat *from_array,
                           PFS_stage_stat *to_array_1,
                           PFS_stage_stat *to_array_2) {
   PFS_stage_stat *from;
-  PFS_stage_stat *from_last;
+  const PFS_stage_stat *from_last;
   PFS_stage_stat *to_1;
   PFS_stage_stat *to_2;
 
@@ -1723,7 +1723,7 @@ void aggregate_all_stages(PFS_stage_stat *from_array,
 void aggregate_all_statements(PFS_statement_stat *from_array,
                               PFS_statement_stat *to_array) {
   PFS_statement_stat *from;
-  PFS_statement_stat *from_last;
+  const PFS_statement_stat *from_last;
   PFS_statement_stat *to;
 
   from = from_array;
@@ -1742,7 +1742,7 @@ void aggregate_all_statements(PFS_statement_stat *from_array,
                               PFS_statement_stat *to_array_1,
                               PFS_statement_stat *to_array_2) {
   PFS_statement_stat *from;
-  PFS_statement_stat *from_last;
+  const PFS_statement_stat *from_last;
   PFS_statement_stat *to_1;
   PFS_statement_stat *to_2;
 
@@ -1815,7 +1815,7 @@ void aggregate_all_memory_with_reassign(bool alive,
                                         PFS_memory_shared_stat *to_array,
                                         PFS_memory_shared_stat *global_array) {
   PFS_memory_safe_stat *from;
-  PFS_memory_safe_stat *from_last;
+  const PFS_memory_safe_stat *from_last;
   PFS_memory_shared_stat *to;
 
   from = from_array;
@@ -1839,7 +1839,7 @@ void aggregate_all_memory_with_reassign(bool alive,
 void aggregate_all_memory(bool alive, PFS_memory_safe_stat *from_array,
                           PFS_memory_shared_stat *to_array) {
   PFS_memory_safe_stat *from;
-  PFS_memory_safe_stat *from_last;
+  const PFS_memory_safe_stat *from_last;
   PFS_memory_shared_stat *to;
 
   from = from_array;
@@ -1861,7 +1861,7 @@ void aggregate_all_memory(bool alive, PFS_memory_safe_stat *from_array,
 void aggregate_all_memory(bool alive, PFS_memory_shared_stat *from_array,
                           PFS_memory_shared_stat *to_array) {
   PFS_memory_shared_stat *from;
-  PFS_memory_shared_stat *from_last;
+  const PFS_memory_shared_stat *from_last;
   PFS_memory_shared_stat *to;
 
   from = from_array;
@@ -1886,7 +1886,7 @@ void aggregate_all_memory_with_reassign(bool alive,
                                         PFS_memory_shared_stat *to_array_2,
                                         PFS_memory_shared_stat *global_array) {
   PFS_memory_safe_stat *from;
-  PFS_memory_safe_stat *from_last;
+  const PFS_memory_safe_stat *from_last;
   PFS_memory_shared_stat *to_1;
   PFS_memory_shared_stat *to_2;
 
@@ -1913,7 +1913,7 @@ void aggregate_all_memory(bool alive, PFS_memory_shared_stat *from_array,
                           PFS_memory_shared_stat *to_array_1,
                           PFS_memory_shared_stat *to_array_2) {
   PFS_memory_shared_stat *from;
-  PFS_memory_shared_stat *from_last;
+  const PFS_memory_shared_stat *from_last;
   PFS_memory_shared_stat *to_1;
   PFS_memory_shared_stat *to_2;
 
@@ -1943,7 +1943,7 @@ void aggregate_thread_status(PFS_thread *thread, PFS_account *safe_account,
     return;
   }
 
-  System_status_var *status_var = get_thd_status_var(thd, &aggregated);
+  const System_status_var *status_var = get_thd_status_var(thd, &aggregated);
 
   if (unlikely(aggregated)) {
     /* THD is being closed, status has already been aggregated. */
@@ -2423,7 +2423,7 @@ void set_thread_account(PFS_thread *thread) {
 }
 
 static void fct_update_mutex_derived_flags(PFS_mutex *pfs) {
-  PFS_mutex_class *klass = sanitize_mutex_class(pfs->m_class);
+  const PFS_mutex_class *klass = sanitize_mutex_class(pfs->m_class);
   if (likely(klass != nullptr)) {
     pfs->m_enabled = klass->m_enabled && flag_global_instrumentation;
     pfs->m_timed = klass->m_timed;
@@ -2438,7 +2438,7 @@ void update_mutex_derived_flags() {
 }
 
 static void fct_update_rwlock_derived_flags(PFS_rwlock *pfs) {
-  PFS_rwlock_class *klass = sanitize_rwlock_class(pfs->m_class);
+  const PFS_rwlock_class *klass = sanitize_rwlock_class(pfs->m_class);
   if (likely(klass != nullptr)) {
     pfs->m_enabled = klass->m_enabled && flag_global_instrumentation;
     pfs->m_timed = klass->m_timed;
@@ -2453,7 +2453,7 @@ void update_rwlock_derived_flags() {
 }
 
 static void fct_update_cond_derived_flags(PFS_cond *pfs) {
-  PFS_cond_class *klass = sanitize_cond_class(pfs->m_class);
+  const PFS_cond_class *klass = sanitize_cond_class(pfs->m_class);
   if (likely(klass != nullptr)) {
     pfs->m_enabled = klass->m_enabled && flag_global_instrumentation;
     pfs->m_timed = klass->m_timed;
@@ -2468,7 +2468,7 @@ void update_cond_derived_flags() {
 }
 
 static void fct_update_file_derived_flags(PFS_file *pfs) {
-  PFS_file_class *klass = sanitize_file_class(pfs->m_class);
+  const PFS_file_class *klass = sanitize_file_class(pfs->m_class);
   if (likely(klass != nullptr)) {
     pfs->m_enabled = klass->m_enabled && flag_global_instrumentation;
     pfs->m_timed = klass->m_timed;
@@ -2483,7 +2483,7 @@ void update_file_derived_flags() {
 }
 
 static void fct_update_table_derived_flags(PFS_table *pfs) {
-  PFS_table_share *share = sanitize_table_share(pfs->m_share);
+  const PFS_table_share *share = sanitize_table_share(pfs->m_share);
   if (likely(share != nullptr)) {
     pfs->m_io_enabled = share->m_enabled && flag_global_instrumentation &&
                         global_table_io_class.m_enabled;
@@ -2504,7 +2504,7 @@ void update_table_derived_flags() {
 }
 
 static void fct_update_socket_derived_flags(PFS_socket *pfs) {
-  PFS_socket_class *klass = sanitize_socket_class(pfs->m_class);
+  const PFS_socket_class *klass = sanitize_socket_class(pfs->m_class);
   if (likely(klass != nullptr)) {
     pfs->m_enabled = klass->m_enabled && flag_global_instrumentation;
     pfs->m_timed = klass->m_timed;
@@ -2558,7 +2558,7 @@ void update_instruments_derived_flags() {
 static void fct_reset_source_file_pointers(PFS_thread *pfs_thread) {
   /* EVENTS_WAITS_CURRENT */
   PFS_events_waits *wait = pfs_thread->m_events_waits_stack;
-  PFS_events_waits *wait_last = wait + WAIT_STACK_SIZE;
+  const PFS_events_waits *wait_last = wait + WAIT_STACK_SIZE;
   for (; wait < wait_last; wait++) {
     wait->m_source_file = nullptr;
   }
@@ -2575,14 +2575,15 @@ static void fct_reset_source_file_pointers(PFS_thread *pfs_thread) {
 
   /* EVENTS_STAGES_HISTORY */
   PFS_events_stages *stage = pfs_thread->m_stages_history;
-  PFS_events_stages *stage_last = stage + events_stages_history_per_thread;
+  const PFS_events_stages *stage_last =
+      stage + events_stages_history_per_thread;
   for (; stage < stage_last; stage++) {
     stage->m_source_file = nullptr;
   }
 
   /* EVENTS_STATEMENTS_CURRENT */
   PFS_events_statements *stmt = &pfs_thread->m_statement_stack[0];
-  PFS_events_statements *stmt_last = stmt + statement_stack_max;
+  const PFS_events_statements *stmt_last = stmt + statement_stack_max;
   for (; stmt < stmt_last; stmt++) {
     stmt->m_source_file = nullptr;
   }
@@ -2599,7 +2600,7 @@ static void fct_reset_source_file_pointers(PFS_thread *pfs_thread) {
 
   /* EVENTS_TRANSACTIONS_HISTORY */
   PFS_events_transactions *trx = pfs_thread->m_transactions_history;
-  PFS_events_transactions *trx_last =
+  const PFS_events_transactions *trx_last =
       trx + events_transactions_history_per_thread;
   for (; trx < trx_last; trx++) {
     trx->m_source_file = nullptr;
@@ -2627,28 +2628,29 @@ void reset_source_file_pointers() {
 
   /* EVENTS_WAITS_HISTORY_LONG */
   PFS_events_waits *wait = events_waits_history_long_array;
-  PFS_events_waits *wait_last = wait + events_waits_history_long_size;
+  const PFS_events_waits *wait_last = wait + events_waits_history_long_size;
   for (; wait < wait_last; wait++) {
     wait->m_source_file = nullptr;
   }
 
   /* EVENTS_STAGES_HISTORY_LONG */
   PFS_events_stages *stage = events_stages_history_long_array;
-  PFS_events_stages *stage_last = stage + events_stages_history_long_size;
+  const PFS_events_stages *stage_last = stage + events_stages_history_long_size;
   for (; stage < stage_last; stage++) {
     stage->m_source_file = nullptr;
   }
 
   /* EVENTS_STATEMENTS_HISTORY_LONG */
   PFS_events_statements *stmt = events_statements_history_long_array;
-  PFS_events_statements *stmt_last = stmt + events_statements_history_long_size;
+  const PFS_events_statements *stmt_last =
+      stmt + events_statements_history_long_size;
   for (; stmt < stmt_last; stmt++) {
     stmt->m_source_file = nullptr;
   }
 
   /* EVENTS_TRANSACTIONS_HISTORY_LONG */
   PFS_events_transactions *trx = events_transactions_history_long_array;
-  PFS_events_transactions *trx_last =
+  const PFS_events_transactions *trx_last =
       trx + events_transactions_history_long_size;
   for (; trx < trx_last; trx++) {
     trx->m_source_file = nullptr;
