@@ -1,15 +1,16 @@
-/* Copyright (c) 2008, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -265,6 +266,10 @@ struct PFS_instr_class {
     return m_enforced_flags & PSI_FLAG_MEM_COLLECT;
   }
 
+  bool is_disabled() const { return m_flags & PSI_FLAG_DISABLED; }
+
+  bool is_untimed() const { return m_flags & PSI_FLAG_UNTIMED; }
+
   void set_enforced_flags(uint flags) { m_enforced_flags = flags; }
 
   void enforce_valid_flags(uint allowed_flags) {
@@ -288,10 +293,8 @@ struct PFS_instr_class {
     switch (m_type) {
       case PFS_CLASS_SOCKET:
         return true;
-        break;
       default:
         return false;
-        break;
     };
   }
 
@@ -368,22 +371,22 @@ struct PFS_ALIGNED PFS_metric_class : public PFS_instr_class {
   pfs_lock m_lock;
 
   /** Metric name with length. */
-  const char *m_metric;
-  uint m_metric_length;
+  const char *m_metric{nullptr};
+  uint m_metric_length{0};
   /** Metric group with length. */
-  const char *m_group;
-  uint m_group_length;
+  const char *m_group{nullptr};
+  uint m_group_length{0};
   /** Metric unit with length. */
-  const char *m_unit;
-  uint m_unit_length;
+  const char *m_unit{nullptr};
+  uint m_unit_length{0};
   /** Metric description with length. */
-  const char *m_description;
-  uint m_description_length;
-  MetricNumType m_num_type;
-  MetricOTELType m_metric_type;
-  PSI_metric_key m_key;
+  const char *m_description{nullptr};
+  uint m_description_length{0};
+  MetricNumType m_num_type{MetricNumType::METRIC_INTEGER};
+  MetricOTELType m_metric_type{MetricOTELType::ASYNC_COUNTER};
+  PSI_metric_key m_key{0};
   measurement_callback_t m_measurement_callback;
-  void *m_measurement_context;
+  void *m_measurement_context{nullptr};
 };
 
 /** Instrumentation metadata for a meter. */
@@ -391,18 +394,18 @@ struct PFS_ALIGNED PFS_meter_class : public PFS_instr_class {
   pfs_lock m_lock;
 
   /** Meter name with length. */
-  const char *m_meter;
-  uint m_meter_length;
+  const char *m_meter{nullptr};
+  uint m_meter_length{0};
   /** Meter export frequency in seconds. */
-  uint m_frequency;
+  uint m_frequency{0};
   /** Meter description with length. */
-  const char *m_description;
-  uint m_description_length;
-  PSI_meter_key m_key;
+  const char *m_description{nullptr};
+  uint m_description_length{0};
+  PSI_meter_key m_key{0};
 
   /** Metrics belonging to this meter. */
-  PSI_metric_key *m_metrics;
-  uint m_metrics_size;
+  PSI_metric_key *m_metrics{nullptr};
+  uint m_metrics_size{0};
 };
 
 /** Key identifying a table share. */
