@@ -1,15 +1,16 @@
-/* Copyright (c) 2022, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2022, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -67,7 +68,7 @@ static bool invalid_metric_name(const char *name, size_t max_len) {
   return false;
 }
 
-static bool invalid_metric_definition(PSI_metric_info_v1 *metric) {
+static bool invalid_metric_definition(const PSI_metric_info_v1 *metric) {
   return invalid_metric_name(metric->m_metric, MAX_METRIC_NAME_LEN) ||
          (metric->m_unit != nullptr &&
           strlen(metric->m_unit) > MAX_METRIC_UNIT_LEN) ||
@@ -75,7 +76,7 @@ static bool invalid_metric_definition(PSI_metric_info_v1 *metric) {
           strlen(metric->m_description) > MAX_METRIC_DESCRIPTION_LEN);
 }
 
-static bool invalid_meter_definition(PSI_meter_info_v1 *metric) {
+static bool invalid_meter_definition(const PSI_meter_info_v1 *metric) {
   return invalid_metric_name(metric->m_meter, MAX_METER_NAME_LEN) ||
          (metric->m_description != nullptr &&
           strlen(metric->m_description) > MAX_METER_DESCRIPTION_LEN);
@@ -406,7 +407,7 @@ void pfs_register_meters_v1(PSI_meter_info_v1 *info [[maybe_unused]],
   // notify component
   mysql_rwlock_rdlock(&LOCK_pfs_meter_notify);
   if (g_notify_callback != nullptr) {
-    for (auto meter : meters_added) {
+    for (const auto *meter : meters_added) {
       g_notify_callback(meter, MeterNotifyType::METER_ADDED);
     }
   }
@@ -452,7 +453,7 @@ void pfs_unregister_meters_v1(PSI_meter_info_v1 *info [[maybe_unused]],
   // notify component
   mysql_rwlock_rdlock(&LOCK_pfs_meter_notify);
   if (g_notify_callback != nullptr) {
-    for (auto meter : meters_removed) {
+    for (const auto *meter : meters_removed) {
       g_notify_callback(meter, MeterNotifyType::METER_REMOVED);
     }
   }
