@@ -189,14 +189,14 @@ int insert_setup_actor(const PFS_user_name *user, const PFS_host_name *host,
     pfs->m_history = history;
 
     int res;
-    pfs->m_lock.dirty_to_allocated(&dirty_state);
     res = lf_hash_insert(&setup_actor_hash, pins, &pfs);
     if (likely(res == 0)) {
       update_setup_actors_derived_flags();
+      pfs->m_lock.dirty_to_allocated(&dirty_state);
       return 0;
     }
 
-    global_setup_actor_container.deallocate(pfs);
+    global_setup_actor_container.dirty_to_free(&dirty_state, pfs);
 
     if (res > 0) {
       return HA_ERR_FOUND_DUPP_KEY;
@@ -324,7 +324,7 @@ void lookup_setup_actor(PFS_thread *thread, const PFS_user_name *user,
         lf_hash_search(&setup_actor_hash, pins, &key, sizeof(key)));
 
     if (entry && (entry != MY_LF_ERRPTR)) {
-      PFS_setup_actor *pfs = *entry;
+      const PFS_setup_actor *pfs = *entry;
       lf_hash_search_unpin(pins);
       *enabled = pfs->m_enabled;
       *history = pfs->m_history;
