@@ -1,15 +1,16 @@
-/* Copyright (c) 2015, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2015, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -145,10 +146,12 @@
 #include "storage/perfschema/pfs_instr.h"
 #include "storage/perfschema/pfs_user.h"
 
-using std::string;
-
 typedef std::vector<SHOW_VAR> Status_var_array;
 
+/* Number of system variable elements to preallocate. */
+#define SYSTEM_VARIABLE_PREALLOC 200
+typedef Prealloced_array<SHOW_VAR, SYSTEM_VARIABLE_PREALLOC> Show_var_array;
+
 /* Global array of all server and plugin-defined status variables. */
 extern Status_var_array all_status_vars;
 extern bool status_vars_inited;
@@ -211,9 +214,11 @@ class Status_variable {
         m_type(SHOW_UNDEF),
         m_scope(SHOW_SCOPE_UNDEF),
         m_charset(nullptr),
-        m_initialized(false) {}
+        m_initialized(false) {
+    m_value_str[0] = '\0';
+  }
 
-  Status_variable(const SHOW_VAR *show_var, System_status_var *status_array,
+  Status_variable(const SHOW_VAR *show_var, System_status_var *vars,
                   enum_var_type query_scope);
 
   bool is_null() const { return !m_initialized; }
@@ -229,7 +234,7 @@ class Status_variable {
 
  private:
   bool m_initialized;
-  void init(const SHOW_VAR *show_var, System_status_var *status_array,
+  void init(const SHOW_VAR *show_var, System_status_var *vars,
             enum_var_type query_scope);
 };
 
@@ -239,7 +244,7 @@ class Status_variable {
 class Find_THD_variable : public Find_THD_Impl {
  public:
   Find_THD_variable() : m_unsafe_thd(nullptr) {}
-  Find_THD_variable(THD *unsafe_thd) : m_unsafe_thd(unsafe_thd) {}
+  explicit Find_THD_variable(THD *unsafe_thd) : m_unsafe_thd(unsafe_thd) {}
 
   bool operator()(THD *thd) override;
   void set_unsafe_thd(THD *unsafe_thd) { m_unsafe_thd = unsafe_thd; }
@@ -254,9 +259,9 @@ class Find_THD_variable : public Find_THD_Impl {
 template <class Var_type>
 class PFS_variable_cache {
  public:
-  typedef Prealloced_array<Var_type, SHOW_VAR_PREALLOC> Variable_array;
+  typedef Prealloced_array<Var_type, SYSTEM_VARIABLE_PREALLOC> Variable_array;
 
-  PFS_variable_cache(bool external_init);
+  explicit PFS_variable_cache(bool external_init);
 
   virtual ~PFS_variable_cache() = 0;
 
@@ -264,13 +269,13 @@ class PFS_variable_cache {
     Build array of SHOW_VARs from the external variable source.
     Filter using session scope.
   */
-  bool initialize_session(void);
+  bool initialize_session();
 
   /**
     Build array of SHOW_VARs suitable for aggregation by user, host or account.
     Filter using session scope.
   */
-  bool initialize_client_session(void);
+  bool initialize_client_session();
 
   /**
     Build cache of GLOBAL system or status variables.
@@ -316,7 +321,7 @@ class PFS_variable_cache {
   /**
     True if variables have been materialized.
   */
-  bool is_materialized(void) { return m_materialized; }
+  bool is_materialized() { return m_materialized; }
 
   /**
     True if variables have been materialized for given THD.
@@ -368,8 +373,8 @@ class PFS_variable_cache {
     Get a validated THD from the thread manager. Execute callback function while
     inside of the thread manager locks.
   */
-  THD *get_THD(THD *thd);
-  THD *get_THD(PFS_thread *pfs_thread);
+  THD_ptr get_THD(THD *thd);
+  THD_ptr get_THD(PFS_thread *pfs_thread);
 
   /**
     Get a single variable from the cache.
@@ -390,9 +395,9 @@ class PFS_variable_cache {
   uint size() { return (uint)m_cache.size(); }
 
  private:
-  virtual bool do_initialize_global(void) { return true; }
-  virtual bool do_initialize_session(void) { return true; }
-  virtual int do_materialize_global(void) { return 1; }
+  virtual bool do_initialize_global() { return true; }
+  virtual bool do_initialize_session() { return true; }
+  virtual int do_materialize_global() { return 1; }
   virtual int do_materialize_all(THD *) { return 1; }
   virtual int do_materialize_session(THD *) { return 1; }
   virtual int do_materialize_session(PFS_thread *) { return 1; }
@@ -432,9 +437,12 @@ class PFS_variable_cache {
   /* True when cache is complete. */
   bool m_materialized;
 
-  /* Array of variables to be materialized. Last element must be null. */
+  /* Array of status variables to be materialized. Last element must be null. */
   Show_var_array m_show_var_array;
 
+  /* Array of system variable descriptors. */
+  System_variable_tracker::Array m_sys_var_tracker_array;
+
   /* Version of global hash/array. Changes when vars added/removed. */
   ulonglong m_version;
 
@@ -452,30 +460,29 @@ class PFS_variable_cache {
   Destructor.
 */
 template <class Var_type>
-PFS_variable_cache<Var_type>::~PFS_variable_cache() {}
+PFS_variable_cache<Var_type>::~PFS_variable_cache() = default;
 
 /**
   Get a validated THD from the thread manager. Execute callback function while
   while inside the thread manager lock.
 */
 template <class Var_type>
-THD *PFS_variable_cache<Var_type>::get_THD(THD *unsafe_thd) {
+THD_ptr PFS_variable_cache<Var_type>::get_THD(THD *unsafe_thd) {
   if (unsafe_thd == nullptr) {
     /*
       May happen, precisely because the pointer is unsafe
       (THD just disconnected for example).
       No need to walk Global_THD_manager for that.
     */
-    return nullptr;
+    return THD_ptr{nullptr};
   }
 
   m_thd_finder.set_unsafe_thd(unsafe_thd);
-  THD *safe_thd = Global_THD_manager::get_instance()->find_thd(&m_thd_finder);
-  return safe_thd;
+  return Global_THD_manager::get_instance()->find_thd(&m_thd_finder);
 }
 
 template <class Var_type>
-THD *PFS_variable_cache<Var_type>::get_THD(PFS_thread *pfs_thread) {
+THD_ptr PFS_variable_cache<Var_type>::get_THD(PFS_thread *pfs_thread) {
   assert(pfs_thread != nullptr);
   return get_THD(pfs_thread->m_thd);
 }
@@ -485,7 +492,7 @@ THD *PFS_variable_cache<Var_type>::get_THD(PFS_thread *pfs_thread) {
   Filter using session scope.
 */
 template <class Var_type>
-bool PFS_variable_cache<Var_type>::initialize_session(void) {
+bool PFS_variable_cache<Var_type>::initialize_session() {
   if (m_initialized) {
     return false;
   }
@@ -498,7 +505,7 @@ bool PFS_variable_cache<Var_type>::initialize_session(void) {
   Filter using session scope.
 */
 template <class Var_type>
-bool PFS_variable_cache<Var_type>::initialize_client_session(void) {
+bool PFS_variable_cache<Var_type>::initialize_client_session() {
   if (m_initialized) {
     return false;
   }
@@ -600,16 +607,16 @@ int PFS_variable_cache<Var_type>::materialize_session(PFS_thread *pfs_thread,
 */
 class PFS_system_variable_cache : public PFS_variable_cache<System_variable> {
  public:
-  PFS_system_variable_cache(bool external_init);
+  explicit PFS_system_variable_cache(bool external_init);
   bool match_scope(int scope);
-  ulonglong get_sysvar_hash_version(void) { return m_version; }
+  ulonglong get_sysvar_hash_version() { return m_version; }
   ~PFS_system_variable_cache() override { free_mem_root(); }
 
  private:
-  bool do_initialize_session(void) override;
+  bool do_initialize_session() override;
 
   /* Global */
-  int do_materialize_global(void) override;
+  int do_materialize_global() override;
   /* Session - THD */
   int do_materialize_session(THD *thd) override;
   /* Session -  PFS_thread */
@@ -626,11 +633,11 @@ class PFS_system_variable_cache : public PFS_variable_cache<System_variable> {
   /* Pointer to temporary mem_root. */
   MEM_ROOT *m_mem_sysvar_ptr;
   /* Allocate and/or assign temporary mem_root. */
-  void set_mem_root(void);
+  void set_mem_root();
   /* Mark all memory blocks as free in temporary mem_root. */
-  void clear_mem_root(void);
+  void clear_mem_root();
   /* Free mem_root memory. */
-  void free_mem_root(void);
+  void free_mem_root();
 
  protected:
   /* Build SHOW_var array. */
@@ -644,9 +651,9 @@ class PFS_system_variable_cache : public PFS_variable_cache<System_variable> {
 */
 class PFS_system_variable_info_cache : public PFS_system_variable_cache {
  public:
-  PFS_system_variable_info_cache(bool external_init)
+  explicit PFS_system_variable_info_cache(bool external_init)
       : PFS_system_variable_cache(external_init) {}
-  ~PFS_system_variable_info_cache() override {}
+  ~PFS_system_variable_info_cache() override = default;
 
  private:
   /* Global and Session - THD */
@@ -658,9 +665,9 @@ class PFS_system_variable_info_cache : public PFS_system_variable_cache {
 */
 class PFS_system_persisted_variables_cache : public PFS_system_variable_cache {
  public:
-  PFS_system_persisted_variables_cache(bool external_init)
+  explicit PFS_system_persisted_variables_cache(bool external_init)
       : PFS_system_variable_cache(external_init) {}
-  ~PFS_system_persisted_variables_cache() override {}
+  ~PFS_system_persisted_variables_cache() override = default;
 
  private:
   /* Global and Session - THD */
@@ -672,13 +679,13 @@ class PFS_system_persisted_variables_cache : public PFS_system_variable_cache {
 */
 class PFS_status_variable_cache : public PFS_variable_cache<Status_variable> {
  public:
-  PFS_status_variable_cache(bool external_init);
+  explicit PFS_status_variable_cache(bool external_init);
 
   int materialize_user(PFS_user *pfs_user);
   int materialize_host(PFS_host *pfs_host);
   int materialize_account(PFS_account *pfs_account);
 
-  ulonglong get_status_array_version(void) { return m_version; }
+  ulonglong get_status_array_version() { return m_version; }
 
  protected:
   /* Get PFS_user, account or host associated with a PFS_thread. Implemented by
@@ -689,9 +696,9 @@ class PFS_status_variable_cache : public PFS_variable_cache<Status_variable> {
   bool m_show_command;
 
  private:
-  bool do_initialize_session(void) override;
+  bool do_initialize_session() override;
 
-  int do_materialize_global(void) override;
+  int do_materialize_global() override;
   /* Global and Session - THD */
   int do_materialize_all(THD *thd) override;
   int do_materialize_session(THD *thd) override;
@@ -731,12 +738,12 @@ class PFS_status_variable_cache : public PFS_variable_cache<Status_variable> {
 
   /* For the current THD, use initial_status_vars taken from before the query
    * start. */
-  System_status_var *set_status_vars(void);
+  System_status_var *set_status_vars();
 
   /* Build the list of status variables from SHOW_VAR array. */
   void manifest(THD *thd, const SHOW_VAR *show_var_array,
-                System_status_var *status_var_array, const char *prefix,
-                bool nested_array, bool strict);
+                System_status_var *vars, const char *prefix, bool nested_array,
+                bool strict);
 };
 
 /* Callback functions to sum status variables for a given user, host or account.
@@ -747,7 +754,7 @@ void sum_account_status(PFS_client *pfs_account,
                         System_status_var *status_totals);
 
 /* Warnings issued if the global system or status variables change mid-query. */
-void system_variable_warning(void);
-void status_variable_warning(void);
+void system_variable_warning();
+void status_variable_warning();
 
 #endif
