@@ -1,15 +1,16 @@
-/* Copyright (c) 2010, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -71,8 +72,8 @@ struct PFS_connection_slice;
 */
 class PFS_connection_visitor {
  public:
-  PFS_connection_visitor() {}
-  virtual ~PFS_connection_visitor() {}
+  PFS_connection_visitor() = default;
+  virtual ~PFS_connection_visitor() = default;
   /** Visit all connections. */
   virtual void visit_global() {}
   /** Visit all connections of a host. */
@@ -160,8 +161,8 @@ class PFS_connection_iterator {
 */
 class PFS_instance_visitor {
  public:
-  PFS_instance_visitor() {}
-  virtual ~PFS_instance_visitor() {}
+  PFS_instance_visitor() = default;
+  virtual ~PFS_instance_visitor() = default;
   /** Visit a mutex class. */
   virtual void visit_mutex_class(PFS_mutex_class *) {}
   /** Visit a rwlock class. */
@@ -268,20 +269,19 @@ class PFS_instance_iterator {
 */
 class PFS_object_visitor {
  public:
-  PFS_object_visitor() {}
-  virtual ~PFS_object_visitor() {}
+  PFS_object_visitor() = default;
+  virtual ~PFS_object_visitor() = default;
   /** Visit global data. */
   virtual void visit_global() {}
   /** Visit a table share. */
   virtual void visit_table_share(PFS_table_share *) {}
   /** Visit a table share index. */
   virtual void visit_table_share_index(PFS_table_share *,
-                                       uint index MY_ATTRIBUTE((unused))) {}
+                                       uint index [[maybe_unused]]) {}
   /** Visit a table. */
   virtual void visit_table(PFS_table *) {}
   /** Visit a table index. */
-  virtual void visit_table_index(PFS_table *,
-                                 uint index MY_ATTRIBUTE((unused))) {}
+  virtual void visit_table_index(PFS_table *, uint index [[maybe_unused]]) {}
 };
 
 /**
@@ -308,7 +308,7 @@ class PFS_object_iterator {
 class PFS_connection_wait_visitor : public PFS_connection_visitor {
  public:
   /** Constructor. */
-  PFS_connection_wait_visitor(PFS_instr_class *klass);
+  explicit PFS_connection_wait_visitor(PFS_instr_class *klass);
   ~PFS_connection_wait_visitor() override;
   void visit_global() override;
   void visit_host(PFS_host *pfs) override;
@@ -351,7 +351,7 @@ class PFS_connection_all_wait_visitor : public PFS_connection_visitor {
 class PFS_connection_stage_visitor : public PFS_connection_visitor {
  public:
   /** Constructor. */
-  PFS_connection_stage_visitor(PFS_stage_class *klass);
+  explicit PFS_connection_stage_visitor(PFS_stage_class *klass);
   ~PFS_connection_stage_visitor() override;
   void visit_global() override;
   void visit_host(PFS_host *pfs) override;
@@ -372,7 +372,7 @@ class PFS_connection_stage_visitor : public PFS_connection_visitor {
 class PFS_connection_statement_visitor : public PFS_connection_visitor {
  public:
   /** Constructor. */
-  PFS_connection_statement_visitor(PFS_statement_class *klass);
+  explicit PFS_connection_statement_visitor(PFS_statement_class *klass);
   ~PFS_connection_statement_visitor() override;
   void visit_global() override;
   void visit_host(PFS_host *pfs) override;
@@ -415,7 +415,7 @@ class PFS_connection_all_statement_visitor : public PFS_connection_visitor {
 class PFS_connection_transaction_visitor : public PFS_connection_visitor {
  public:
   /** Constructor. */
-  PFS_connection_transaction_visitor(PFS_transaction_class *klass);
+  explicit PFS_connection_transaction_visitor(PFS_transaction_class *klass);
   ~PFS_connection_transaction_visitor() override;
   void visit_global() override;
   void visit_host(PFS_host *pfs) override;
@@ -478,7 +478,7 @@ class PFS_connection_stat_visitor : public PFS_connection_visitor {
 class PFS_connection_memory_visitor : public PFS_connection_visitor {
  public:
   /** Constructor. */
-  PFS_connection_memory_visitor(PFS_memory_class *klass);
+  explicit PFS_connection_memory_visitor(PFS_memory_class *klass);
   ~PFS_connection_memory_visitor() override;
   void visit_global() override;
   void visit_host(PFS_host *pfs) override;
@@ -499,7 +499,7 @@ class PFS_connection_memory_visitor : public PFS_connection_visitor {
 class PFS_connection_status_visitor : public PFS_connection_visitor {
  public:
   /** Constructor. */
-  PFS_connection_status_visitor(System_status_var *status_vars);
+  explicit PFS_connection_status_visitor(System_status_var *vars);
   ~PFS_connection_status_visitor() override;
   void visit_global() override;
   void visit_host(PFS_host *pfs) override;
