@@ -1,4 +1,4 @@
-/* Copyright (c) 2010, 2025, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
@@ -131,7 +131,7 @@ ha_rows table_ets_by_account_by_event_name::get_row_count() {
 }
 
 table_ets_by_account_by_event_name::table_ets_by_account_by_event_name()
-    : PFS_engine_table(&m_share, &m_pos), m_pos(), m_next_pos() {
+    : PFS_engine_table(&m_share, &m_pos), m_opened_index(nullptr) {
   m_normalizer = time_normalizer::get_transaction();
 }
 
@@ -162,14 +162,12 @@ int table_ets_by_account_by_event_name::rnd_next() {
 }
 
 int table_ets_by_account_by_event_name::rnd_pos(const void *pos) {
-  PFS_account *account;
-  PFS_transaction_class *transaction_class;
-
   set_position(pos);
 
-  account = global_account_container.get(m_pos.m_index_1);
+  PFS_account *account = global_account_container.get(m_pos.m_index_1);
   if (account != nullptr) {
-    transaction_class = find_transaction_class(m_pos.m_index_2);
+    PFS_transaction_class *transaction_class =
+        find_transaction_class(m_pos.m_index_2);
     if (transaction_class) {
       return make_row(account, transaction_class);
     }
@@ -180,9 +178,8 @@ int table_ets_by_account_by_event_name::rnd_pos(const void *pos) {
 
 int table_ets_by_account_by_event_name::index_init(uint idx [[maybe_unused]],
                                                    bool) {
-  PFS_index_ets_by_account_by_event_name *result = nullptr;
   assert(idx == 0);
-  result = PFS_NEW(PFS_index_ets_by_account_by_event_name);
+  auto *result = PFS_NEW(PFS_index_ets_by_account_by_event_name);
   m_opened_index = result;
   m_index = result;
   return 0;
