@@ -1,15 +1,16 @@
-/* Copyright (c) 2010, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -198,8 +199,8 @@ table_events_stages_common::table_events_stages_common(
 int table_events_stages_common::make_row(PFS_events_stages *stage) {
   ulonglong timer_end;
 
-  PFS_stage_class *unsafe = (PFS_stage_class *)stage->m_class;
-  PFS_stage_class *klass = sanitize_stage_class(unsafe);
+  auto *unsafe = (PFS_stage_class *)stage->m_class;
+  const PFS_stage_class *klass = sanitize_stage_class(unsafe);
   if (unlikely(klass == nullptr)) {
     return HA_ERR_RECORD_DELETED;
   }
@@ -219,8 +220,8 @@ int table_events_stages_common::make_row(PFS_events_stages *stage) {
   m_normalizer->to_pico(stage->m_timer_start, timer_end, &m_row.m_timer_start,
                         &m_row.m_timer_end, &m_row.m_timer_wait);
 
-  m_row.m_name = klass->m_name;
-  m_row.m_name_length = klass->m_name_length;
+  m_row.m_name = klass->m_name.str();
+  m_row.m_name_length = klass->m_name.length();
 
   make_source_column(stage->m_source_file, stage->m_source_line, m_row.m_source,
                      sizeof(m_row.m_source), m_row.m_source_length);
@@ -263,10 +264,10 @@ int table_events_stages_common::read_row_values(TABLE *table,
           }
           break;
         case 3: /* EVENT_NAME */
-          set_field_varchar_utf8(f, m_row.m_name, m_row.m_name_length);
+          set_field_varchar_utf8mb4(f, m_row.m_name, m_row.m_name_length);
           break;
         case 4: /* SOURCE */
-          set_field_varchar_utf8(f, m_row.m_source, m_row.m_source_length);
+          set_field_varchar_utf8mb4(f, m_row.m_source, m_row.m_source_length);
           break;
         case 5: /* TIMER_START */
           if (m_row.m_timer_start != 0) {
@@ -332,24 +333,25 @@ PFS_engine_table *table_events_stages_current::create(
 }
 
 table_events_stages_current::table_events_stages_current()
-    : table_events_stages_common(&m_share, &m_pos), m_pos(0), m_next_pos(0) {}
+    : table_events_stages_common(&m_share, &m_pos),
+      m_pos(0),
+      m_next_pos(0),
+      m_opened_index(nullptr) {}
 
-void table_events_stages_current::reset_position(void) {
+void table_events_stages_current::reset_position() {
   m_pos.m_index = 0;
   m_next_pos.m_index = 0;
 }
 
 int table_events_stages_current::rnd_init(bool) { return 0; }
 
-int table_events_stages_current::rnd_next(void) {
-  PFS_thread *pfs_thread;
-  PFS_events_stages *stage;
-
+int table_events_stages_current::rnd_next() {
   m_pos.set_at(&m_next_pos);
+
   PFS_thread_iterator it = global_thread_container.iterate(m_pos.m_index);
-  pfs_thread = it.scan_next(&m_pos.m_index);
+  PFS_thread *pfs_thread = it.scan_next(&m_pos.m_index);
   if (pfs_thread != nullptr) {
-    stage = &pfs_thread->m_stage_current;
+    PFS_events_stages *stage = &pfs_thread->m_stage_current;
     m_next_pos.set_after(&m_pos);
     return make_row(stage);
   }
@@ -358,22 +360,18 @@ int table_events_stages_current::rnd_next(void) {
 }
 
 int table_events_stages_current::rnd_pos(const void *pos) {
-  PFS_thread *pfs_thread;
-  PFS_events_stages *stage;
-
   set_position(pos);
 
-  pfs_thread = global_thread_container.get(m_pos.m_index);
+  PFS_thread *pfs_thread = global_thread_container.get(m_pos.m_index);
   if (pfs_thread != nullptr) {
-    stage = &pfs_thread->m_stage_current;
+    PFS_events_stages *stage = &pfs_thread->m_stage_current;
     return make_row(stage);
   }
 
   return HA_ERR_RECORD_DELETED;
 }
 
-int table_events_stages_current::index_init(uint idx MY_ATTRIBUTE((unused)),
-                                            bool) {
+int table_events_stages_current::index_init(uint idx [[maybe_unused]], bool) {
   PFS_index_events_stages *result;
   assert(idx == 0);
   result = PFS_NEW(PFS_index_events_stages);
@@ -382,7 +380,7 @@ int table_events_stages_current::index_init(uint idx MY_ATTRIBUTE((unused)),
   return 0;
 }
 
-int table_events_stages_current::index_next(void) {
+int table_events_stages_current::index_next() {
   PFS_thread *pfs_thread;
   PFS_events_stages *stage;
 
@@ -407,12 +405,12 @@ int table_events_stages_current::index_next(void) {
   return HA_ERR_END_OF_FILE;
 }
 
-int table_events_stages_current::delete_all_rows(void) {
+int table_events_stages_current::delete_all_rows() {
   reset_events_stages_current();
   return 0;
 }
 
-ha_rows table_events_stages_current::get_row_count(void) {
+ha_rows table_events_stages_current::get_row_count() {
   return global_thread_container.get_row_count();
 }
 
@@ -422,16 +420,16 @@ PFS_engine_table *table_events_stages_history::create(
 }
 
 table_events_stages_history::table_events_stages_history()
-    : table_events_stages_common(&m_share, &m_pos), m_pos(), m_next_pos() {}
+    : table_events_stages_common(&m_share, &m_pos), m_opened_index(nullptr) {}
 
-void table_events_stages_history::reset_position(void) {
+void table_events_stages_history::reset_position() {
   m_pos.reset();
   m_next_pos.reset();
 }
 
 int table_events_stages_history::rnd_init(bool) { return 0; }
 
-int table_events_stages_history::rnd_next(void) {
+int table_events_stages_history::rnd_next() {
   PFS_thread *pfs_thread;
   PFS_events_stages *stage;
   bool has_more_thread = true;
@@ -468,22 +466,19 @@ int table_events_stages_history::rnd_next(void) {
 }
 
 int table_events_stages_history::rnd_pos(const void *pos) {
-  PFS_thread *pfs_thread;
-  PFS_events_stages *stage;
-
   assert(events_stages_history_per_thread != 0);
   set_position(pos);
 
   assert(m_pos.m_index_2 < events_stages_history_per_thread);
 
-  pfs_thread = global_thread_container.get(m_pos.m_index_1);
+  const PFS_thread *pfs_thread = global_thread_container.get(m_pos.m_index_1);
   if (pfs_thread != nullptr) {
     if (!pfs_thread->m_stages_history_full &&
         (m_pos.m_index_2 >= pfs_thread->m_stages_history_index)) {
       return HA_ERR_RECORD_DELETED;
     }
 
-    stage = &pfs_thread->m_stages_history[m_pos.m_index_2];
+    PFS_events_stages *stage = &pfs_thread->m_stages_history[m_pos.m_index_2];
 
     if (stage->m_class != nullptr) {
       return make_row(stage);
@@ -493,8 +488,7 @@ int table_events_stages_history::rnd_pos(const void *pos) {
   return HA_ERR_RECORD_DELETED;
 }
 
-int table_events_stages_history::index_init(uint idx MY_ATTRIBUTE((unused)),
-                                            bool) {
+int table_events_stages_history::index_init(uint idx [[maybe_unused]], bool) {
   PFS_index_events_stages *result;
   assert(idx == 0);
   result = PFS_NEW(PFS_index_events_stages);
@@ -503,7 +497,7 @@ int table_events_stages_history::index_init(uint idx MY_ATTRIBUTE((unused)),
   return 0;
 }
 
-int table_events_stages_history::index_next(void) {
+int table_events_stages_history::index_next() {
   PFS_thread *pfs_thread;
   PFS_events_stages *stage;
   bool has_more_thread = true;
@@ -545,12 +539,12 @@ int table_events_stages_history::index_next(void) {
   return HA_ERR_END_OF_FILE;
 }
 
-int table_events_stages_history::delete_all_rows(void) {
+int table_events_stages_history::delete_all_rows() {
   reset_events_stages_history();
   return 0;
 }
 
-ha_rows table_events_stages_history::get_row_count(void) {
+ha_rows table_events_stages_history::get_row_count() {
   return events_stages_history_per_thread *
          global_thread_container.get_row_count();
 }
@@ -563,14 +557,14 @@ PFS_engine_table *table_events_stages_history_long::create(
 table_events_stages_history_long::table_events_stages_history_long()
     : table_events_stages_common(&m_share, &m_pos), m_pos(0), m_next_pos(0) {}
 
-void table_events_stages_history_long::reset_position(void) {
+void table_events_stages_history_long::reset_position() {
   m_pos.m_index = 0;
   m_next_pos.m_index = 0;
 }
 
 int table_events_stages_history_long::rnd_init(bool) { return 0; }
 
-int table_events_stages_history_long::rnd_next(void) {
+int table_events_stages_history_long::rnd_next() {
   PFS_events_stages *stage;
   uint limit;
 
@@ -626,11 +620,11 @@ int table_events_stages_history_long::rnd_pos(const void *pos) {
   return make_row(stage);
 }
 
-int table_events_stages_history_long::delete_all_rows(void) {
+int table_events_stages_history_long::delete_all_rows() {
   reset_events_stages_history_long();
   return 0;
 }
 
-ha_rows table_events_stages_history_long::get_row_count(void) {
+ha_rows table_events_stages_history_long::get_row_count() {
   return events_stages_history_long_size;
 }
