@@ -1,15 +1,16 @@
-/* Copyright (c) 2010, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -93,6 +94,10 @@ Plugin_table table_events_statements_current::m_table_def(
     "  NESTING_EVENT_TYPE ENUM('TRANSACTION', 'STATEMENT', 'STAGE', 'WAIT'),\n"
     "  NESTING_EVENT_LEVEL INTEGER,\n"
     "  STATEMENT_ID BIGINT unsigned,\n"
+    "  CPU_TIME BIGINT unsigned not null,\n"
+    "  MAX_CONTROLLED_MEMORY BIGINT unsigned not null,\n"
+    "  MAX_TOTAL_MEMORY BIGINT unsigned not null,\n"
+    "  EXECUTION_ENGINE ENUM ('PRIMARY', 'SECONDARY'),\n"
     "  PRIMARY KEY (THREAD_ID, EVENT_ID) USING HASH\n",
     /* Options */
     " ENGINE=PERFORMANCE_SCHEMA",
@@ -164,6 +169,10 @@ Plugin_table table_events_statements_history::m_table_def(
     "  NESTING_EVENT_TYPE ENUM('TRANSACTION', 'STATEMENT', 'STAGE', 'WAIT'),\n"
     "  NESTING_EVENT_LEVEL INTEGER,\n"
     "  STATEMENT_ID BIGINT unsigned,\n"
+    "  CPU_TIME BIGINT unsigned not null,\n"
+    "  MAX_CONTROLLED_MEMORY BIGINT unsigned not null,\n"
+    "  MAX_TOTAL_MEMORY BIGINT unsigned not null,\n"
+    "  EXECUTION_ENGINE ENUM ('PRIMARY', 'SECONDARY'),\n"
     "  PRIMARY KEY (THREAD_ID, EVENT_ID) USING HASH\n",
     /* Options */
     " ENGINE=PERFORMANCE_SCHEMA",
@@ -234,7 +243,11 @@ Plugin_table table_events_statements_history_long::m_table_def(
     "  NESTING_EVENT_ID BIGINT unsigned,\n"
     "  NESTING_EVENT_TYPE ENUM('TRANSACTION', 'STATEMENT', 'STAGE', 'WAIT'),\n"
     "  NESTING_EVENT_LEVEL INTEGER,\n"
-    "  STATEMENT_ID BIGINT unsigned\n",
+    "  STATEMENT_ID BIGINT unsigned,\n"
+    "  CPU_TIME BIGINT unsigned not null,\n"
+    "  MAX_CONTROLLED_MEMORY BIGINT unsigned not null,\n"
+    "  MAX_TOTAL_MEMORY BIGINT unsigned not null,\n"
+    "  EXECUTION_ENGINE ENUM ('PRIMARY', 'SECONDARY')\n",
     /* Options */
     " ENGINE=PERFORMANCE_SCHEMA",
     /* Tablespace */
@@ -292,8 +305,8 @@ int table_events_statements_common::make_row_part_1(
     PFS_events_statements *statement, sql_digest_storage *digest) {
   ulonglong timer_end;
 
-  PFS_statement_class *unsafe = (PFS_statement_class *)statement->m_class;
-  PFS_statement_class *klass = sanitize_statement_class(unsafe);
+  auto *unsafe = (PFS_statement_class *)statement->m_class;
+  const PFS_statement_class *klass = sanitize_statement_class(unsafe);
   if (unlikely(klass == nullptr)) {
     return HA_ERR_RECORD_DELETED;
   }
@@ -317,32 +330,26 @@ int table_events_statements_common::make_row_part_1(
                         &m_row.m_timer_wait);
   m_row.m_lock_time = statement->m_lock_time * MICROSEC_TO_PICOSEC;
 
-  m_row.m_name = klass->m_name;
-  m_row.m_name_length = klass->m_name_length;
+  m_row.m_name = klass->m_name.str();
+  m_row.m_name_length = klass->m_name.length();
 
-  m_row.m_current_schema_name_length = statement->m_current_schema_name_length;
-  if (m_row.m_current_schema_name_length > 0)
-    memcpy(m_row.m_current_schema_name, statement->m_current_schema_name,
-           m_row.m_current_schema_name_length);
+  m_row.m_current_schema_name = statement->m_current_schema_name;
 
   m_row.m_object_type = statement->m_sp_type;
-
-  m_row.m_schema_name_length = statement->m_schema_name_length;
-  if (m_row.m_schema_name_length > 0)
-    memcpy(m_row.m_schema_name, statement->m_schema_name,
-           m_row.m_schema_name_length);
-
-  m_row.m_object_name_length = statement->m_object_name_length;
-  if (m_row.m_object_name_length > 0)
-    memcpy(m_row.m_object_name, statement->m_object_name,
-           m_row.m_object_name_length);
+  m_row.m_schema_name = statement->m_schema_name;
+  m_row.m_object_name = statement->m_object_name;
 
   make_source_column(statement->m_source_file, statement->m_source_line,
                      m_row.m_source, sizeof(m_row.m_source),
                      m_row.m_source_length);
 
-  memcpy(m_row.m_message_text, statement->m_message_text,
-         sizeof(m_row.m_message_text));
+  m_row.m_message_text_length = statement->m_message_text_length;
+  if (m_row.m_message_text_length > 0) {
+    memcpy(m_row.m_message_text, statement->m_message_text,
+           m_row.m_message_text_length);
+  }
+  m_row.m_message_text[m_row.m_message_text_length] = '\0';
+
   memcpy(m_row.m_sqlstate, statement->m_sqlstate, SQLSTATE_LENGTH);
 
   m_row.m_sql_errno = statement->m_sql_errno;
@@ -364,6 +371,10 @@ int table_events_statements_common::make_row_part_1(
   m_row.m_sort_scan = statement->m_sort_scan;
   m_row.m_no_index_used = statement->m_no_index_used;
   m_row.m_no_good_index_used = statement->m_no_good_index_used;
+  m_row.m_cpu_time = statement->m_cpu_time * NANOSEC_TO_PICOSEC;
+  m_row.m_max_controlled_memory = statement->m_max_controlled_memory;
+  m_row.m_max_total_memory = statement->m_max_total_memory;
+  m_row.m_secondary = statement->m_secondary;
 
   /* Copy the digest storage. */
   digest->copy(&statement->m_digest_storage);
@@ -389,7 +400,7 @@ int table_events_statements_common::make_row_part_2(
   /*
     Filling up statement digest information.
   */
-  size_t safe_byte_count = digest->m_byte_count;
+  const size_t safe_byte_count = digest->m_byte_count;
   if (safe_byte_count > 0 && safe_byte_count <= pfs_max_digest_length) {
     /* Generate the DIGEST string from the digest */
     DIGEST_HASH_TO_STRING(digest->m_hash, m_row.m_digest.m_digest);
@@ -439,10 +450,10 @@ int table_events_statements_common::read_row_values(TABLE *table,
           }
           break;
         case 3: /* EVENT_NAME */
-          set_field_varchar_utf8(f, m_row.m_name, m_row.m_name_length);
+          set_field_varchar_utf8mb4(f, m_row.m_name, m_row.m_name_length);
           break;
         case 4: /* SOURCE */
-          set_field_varchar_utf8(f, m_row.m_source, m_row.m_source_length);
+          set_field_varchar_utf8mb4(f, m_row.m_source, m_row.m_source_length);
           break;
         case 5: /* TIMER_START */
           if (m_row.m_timer_start != 0) {
@@ -483,8 +494,8 @@ int table_events_statements_common::read_row_values(TABLE *table,
           break;
         case 10: /* DIGEST */
           if (m_row.m_digest.m_digest_length > 0)
-            set_field_varchar_utf8(f, m_row.m_digest.m_digest,
-                                   m_row.m_digest.m_digest_length);
+            set_field_varchar_utf8mb4(f, m_row.m_digest.m_digest,
+                                      m_row.m_digest.m_digest_length);
           else {
             f->set_null();
           }
@@ -498,35 +509,20 @@ int table_events_statements_common::read_row_values(TABLE *table,
           }
           break;
         case 12: /* CURRENT_SCHEMA */
-          if (m_row.m_current_schema_name_length)
-            set_field_varchar_utf8(f, m_row.m_current_schema_name,
-                                   m_row.m_current_schema_name_length);
-          else {
-            f->set_null();
-          }
+          set_nullable_field_schema_name(f, &m_row.m_current_schema_name);
           break;
         case 13: /* OBJECT_TYPE */
-          if (m_row.m_object_name_length > 0) {
+          if (m_row.m_object_name.length() > 0) {
             set_field_object_type(f, m_row.m_object_type);
           } else {
             f->set_null();
           }
           break;
         case 14: /* OBJECT_SCHEMA */
-          if (m_row.m_schema_name_length)
-            set_field_varchar_utf8(f, m_row.m_schema_name,
-                                   m_row.m_schema_name_length);
-          else {
-            f->set_null();
-          }
+          set_nullable_field_schema_name(f, &m_row.m_schema_name);
           break;
         case 15: /* OBJECT_NAME */
-          if (m_row.m_object_name_length)
-            set_field_varchar_utf8(f, m_row.m_object_name,
-                                   m_row.m_object_name_length);
-          else {
-            f->set_null();
-          }
+          set_nullable_field_object_name(f, &m_row.m_object_name);
           break;
         case 16: /* OBJECT_INSTANCE_BEGIN */
           f->set_null();
@@ -536,15 +532,15 @@ int table_events_statements_common::read_row_values(TABLE *table,
           break;
         case 18: /* RETURNED_SQLSTATE */
           if (m_row.m_sqlstate[0] != 0) {
-            set_field_varchar_utf8(f, m_row.m_sqlstate, SQLSTATE_LENGTH);
+            set_field_varchar_utf8mb4(f, m_row.m_sqlstate, SQLSTATE_LENGTH);
           } else {
             f->set_null();
           }
           break;
         case 19: /* MESSAGE_TEXT */
-          len = (uint)strlen(m_row.m_message_text);
+          len = m_row.m_message_text_length;
           if (len) {
-            set_field_varchar_utf8(f, m_row.m_message_text, len);
+            set_field_varchar_utf8mb4(f, m_row.m_message_text, len);
           } else {
             f->set_null();
           }
@@ -627,6 +623,18 @@ int table_events_statements_common::read_row_values(TABLE *table,
             f->set_null();
           }
           break;
+        case 42: /* CPU_TIME */
+          set_field_ulonglong(f, m_row.m_cpu_time);
+          break;
+        case 43: /* MAX_CONTROLLED_MEMORY */
+          set_field_ulonglong(f, m_row.m_max_controlled_memory);
+          break;
+        case 44: /* MAX_TOTAL_MEMORY */
+          set_field_ulonglong(f, m_row.m_max_total_memory);
+          break;
+        case 45: /* EXECUTION_ENGINE */
+          set_field_enum(f, m_row.m_secondary ? ENUM_SECONDARY : ENUM_PRIMARY);
+          break;
         default:
           assert(false);
       }
@@ -641,16 +649,17 @@ PFS_engine_table *table_events_statements_current::create(
 }
 
 table_events_statements_current::table_events_statements_current()
-    : table_events_statements_common(&m_share, &m_pos), m_pos(), m_next_pos() {}
+    : table_events_statements_common(&m_share, &m_pos),
+      m_opened_index(nullptr) {}
 
-void table_events_statements_current::reset_position(void) {
+void table_events_statements_current::reset_position() {
   m_pos.reset();
   m_next_pos.reset();
 }
 
 int table_events_statements_current::rnd_init(bool) { return 0; }
 
-int table_events_statements_current::rnd_next(void) {
+int table_events_statements_current::rnd_next() {
   PFS_thread *pfs_thread;
   PFS_events_statements *statement;
   bool has_more_thread = true;
@@ -658,7 +667,8 @@ int table_events_statements_current::rnd_next(void) {
   for (m_pos.set_at(&m_next_pos); has_more_thread; m_pos.next_thread()) {
     pfs_thread = global_thread_container.get(m_pos.m_index_1, &has_more_thread);
     if (pfs_thread != nullptr) {
-      uint safe_events_statements_count = pfs_thread->m_events_statements_count;
+      const uint safe_events_statements_count =
+          pfs_thread->m_events_statements_count;
 
       if (safe_events_statements_count == 0) {
         /* Display the last top level statement, when completed */
@@ -683,14 +693,12 @@ int table_events_statements_current::rnd_next(void) {
 }
 
 int table_events_statements_current::rnd_pos(const void *pos) {
-  PFS_thread *pfs_thread;
-  PFS_events_statements *statement;
-
   set_position(pos);
 
-  pfs_thread = global_thread_container.get(m_pos.m_index_1);
+  PFS_thread *pfs_thread = global_thread_container.get(m_pos.m_index_1);
   if (pfs_thread != nullptr) {
-    uint safe_events_statements_count = pfs_thread->m_events_statements_count;
+    const uint safe_events_statements_count =
+        pfs_thread->m_events_statements_count;
 
     if (safe_events_statements_count == 0) {
       /* Display the last top level statement, when completed */
@@ -706,7 +714,8 @@ int table_events_statements_current::rnd_pos(const void *pos) {
 
     assert(m_pos.m_index_2 < statement_stack_max);
 
-    statement = &pfs_thread->m_statement_stack[m_pos.m_index_2];
+    PFS_events_statements *statement =
+        &pfs_thread->m_statement_stack[m_pos.m_index_2];
 
     if (statement->m_class != nullptr) {
       return make_row(pfs_thread, statement);
@@ -716,7 +725,7 @@ int table_events_statements_current::rnd_pos(const void *pos) {
   return HA_ERR_RECORD_DELETED;
 }
 
-int table_events_statements_current::index_next(void) {
+int table_events_statements_current::index_next() {
   PFS_thread *pfs_thread;
   PFS_events_statements *statement;
   bool has_more_thread = true;
@@ -726,7 +735,7 @@ int table_events_statements_current::index_next(void) {
     if (pfs_thread != nullptr) {
       if (m_opened_index->match(pfs_thread)) {
         do {
-          uint safe_events_statements_count =
+          const uint safe_events_statements_count =
               pfs_thread->m_events_statements_count;
           if (safe_events_statements_count == 0) {
             /* Display the last top level statement, when completed */
@@ -783,16 +792,16 @@ int table_events_statements_current::make_row(
   return table_events_statements_common::make_row_part_2(&digest);
 }
 
-int table_events_statements_current::delete_all_rows(void) {
+int table_events_statements_current::delete_all_rows() {
   reset_events_statements_current();
   return 0;
 }
 
-ha_rows table_events_statements_current::get_row_count(void) {
+ha_rows table_events_statements_current::get_row_count() {
   return global_thread_container.get_row_count() * statement_stack_max;
 }
 
-int table_events_statements_current::index_init(uint idx MY_ATTRIBUTE((unused)),
+int table_events_statements_current::index_init(uint idx [[maybe_unused]],
                                                 bool) {
   PFS_index_events_statements *result;
   assert(idx == 0);
@@ -808,16 +817,17 @@ PFS_engine_table *table_events_statements_history::create(
 }
 
 table_events_statements_history::table_events_statements_history()
-    : table_events_statements_common(&m_share, &m_pos), m_pos(), m_next_pos() {}
+    : table_events_statements_common(&m_share, &m_pos),
+      m_opened_index(nullptr) {}
 
-void table_events_statements_history::reset_position(void) {
+void table_events_statements_history::reset_position() {
   m_pos.reset();
   m_next_pos.reset();
 }
 
 int table_events_statements_history::rnd_init(bool) { return 0; }
 
-int table_events_statements_history::rnd_next(void) {
+int table_events_statements_history::rnd_next() {
   PFS_thread *pfs_thread;
   PFS_events_statements *statement;
   bool has_more_thread = true;
@@ -854,13 +864,10 @@ int table_events_statements_history::rnd_next(void) {
 }
 
 int table_events_statements_history::rnd_pos(const void *pos) {
-  PFS_thread *pfs_thread;
-  PFS_events_statements *statement;
-
   assert(events_statements_history_per_thread != 0);
   set_position(pos);
 
-  pfs_thread = global_thread_container.get(m_pos.m_index_1);
+  PFS_thread *pfs_thread = global_thread_container.get(m_pos.m_index_1);
   if (pfs_thread != nullptr) {
     assert(m_pos.m_index_2 < events_statements_history_per_thread);
 
@@ -869,7 +876,8 @@ int table_events_statements_history::rnd_pos(const void *pos) {
       return HA_ERR_RECORD_DELETED;
     }
 
-    statement = &pfs_thread->m_statements_history[m_pos.m_index_2];
+    PFS_events_statements *statement =
+        &pfs_thread->m_statements_history[m_pos.m_index_2];
     if (statement->m_class != nullptr) {
       return make_row(pfs_thread, statement);
     }
@@ -878,7 +886,7 @@ int table_events_statements_history::rnd_pos(const void *pos) {
   return HA_ERR_RECORD_DELETED;
 }
 
-int table_events_statements_history::index_next(void) {
+int table_events_statements_history::index_next() {
   PFS_thread *pfs_thread;
   PFS_events_statements *statement;
   bool has_more_thread = true;
@@ -942,17 +950,17 @@ int table_events_statements_history::make_row(
   return table_events_statements_common::make_row_part_2(&digest);
 }
 
-int table_events_statements_history::delete_all_rows(void) {
+int table_events_statements_history::delete_all_rows() {
   reset_events_statements_history();
   return 0;
 }
 
-ha_rows table_events_statements_history::get_row_count(void) {
+ha_rows table_events_statements_history::get_row_count() {
   return events_statements_history_per_thread *
          global_thread_container.get_row_count();
 }
 
-int table_events_statements_history::index_init(uint idx MY_ATTRIBUTE((unused)),
+int table_events_statements_history::index_init(uint idx [[maybe_unused]],
                                                 bool) {
   PFS_index_events_statements *result;
   assert(idx == 0);
@@ -972,14 +980,14 @@ table_events_statements_history_long::table_events_statements_history_long()
       m_pos(0),
       m_next_pos(0) {}
 
-void table_events_statements_history_long::reset_position(void) {
+void table_events_statements_history_long::reset_position() {
   m_pos.m_index = 0;
   m_next_pos.m_index = 0;
 }
 
 int table_events_statements_history_long::rnd_init(bool) { return 0; }
 
-int table_events_statements_history_long::rnd_next(void) {
+int table_events_statements_history_long::rnd_next() {
   PFS_events_statements *statement;
   size_t limit;
 
@@ -1048,11 +1056,11 @@ int table_events_statements_history_long::make_row(
   return table_events_statements_common::make_row_part_2(&digest);
 }
 
-int table_events_statements_history_long::delete_all_rows(void) {
+int table_events_statements_history_long::delete_all_rows() {
   reset_events_statements_history_long();
   return 0;
 }
 
-ha_rows table_events_statements_history_long::get_row_count(void) {
+ha_rows table_events_statements_history_long::get_row_count() {
   return events_statements_history_long_size;
 }
