@@ -1,15 +1,16 @@
-/* Copyright (c) 2008, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -29,10 +30,10 @@
 
 #include <assert.h>
 #include "lex_string.h"
-#include "m_string.h"
 #include "my_compiler.h"
 
 #include "my_thread.h"
+#include "mysql/strings/int2str.h"
 #include "sql/field.h"
 #include "sql/plugin_table.h"
 #include "sql/table.h"
@@ -42,6 +43,7 @@
 #include "storage/perfschema/pfs_instr.h"
 #include "storage/perfschema/pfs_instr_class.h"
 #include "storage/perfschema/pfs_timer.h"
+#include "string_with_len.h"
 
 bool PFS_index_events_waits::match(PFS_thread *pfs) {
   if (m_fields >= 1) {
@@ -214,17 +216,15 @@ table_events_waits_common::table_events_waits_common(
 
 void table_events_waits_common::clear_object_columns() {
   m_row.m_object_type_length = 0;
-  m_row.m_object_schema_length = 0;
+  m_row.m_object_schema.reset();
   m_row.m_object_name_length = 0;
   m_row.m_index_name_length = 0;
 }
 
 int table_events_waits_common::make_table_object_columns(
     PFS_events_waits *wait) {
-  uint safe_index;
-  PFS_table_share *safe_table_share;
-
-  safe_table_share = sanitize_table_share(wait->m_weak_table_share);
+  PFS_table_share *safe_table_share =
+      sanitize_table_share(wait->m_weak_table_share);
   if (unlikely(safe_table_share == nullptr)) {
     return 1;
   }
@@ -239,27 +239,21 @@ int table_events_waits_common::make_table_object_columns(
 
   if (safe_table_share->get_version() == wait->m_weak_version) {
     /* OBJECT SCHEMA */
-    m_row.m_object_schema_length = safe_table_share->m_schema_name_length;
-    if (unlikely(
-            (m_row.m_object_schema_length == 0) ||
-            (m_row.m_object_schema_length > sizeof(m_row.m_object_schema)))) {
-      return 1;
-    }
-    memcpy(m_row.m_object_schema, safe_table_share->m_schema_name,
-           m_row.m_object_schema_length);
+    m_row.m_object_schema = safe_table_share->m_key.m_schema_name;
 
     /* OBJECT NAME */
-    m_row.m_object_name_length = safe_table_share->m_table_name_length;
+    m_row.m_object_name_length = safe_table_share->m_key.m_table_name.length();
     if (unlikely((m_row.m_object_name_length == 0) ||
                  (m_row.m_object_name_length > sizeof(m_row.m_object_name)))) {
       return 1;
     }
-    memcpy(m_row.m_object_name, safe_table_share->m_table_name,
+    memcpy(m_row.m_object_name, safe_table_share->m_key.m_table_name.ptr(),
            m_row.m_object_name_length);
 
     /* INDEX NAME */
-    safe_index = wait->m_index;
-    uint safe_key_count = sanitize_index_count(safe_table_share->m_key_count);
+    const uint safe_index = wait->m_index;
+    const uint safe_key_count =
+        sanitize_index_count(safe_table_share->m_key_count);
     if (safe_index < safe_key_count) {
       PFS_table_share_index *index_stat;
       index_stat = safe_table_share->find_index_stat(safe_index);
@@ -282,7 +276,7 @@ int table_events_waits_common::make_table_object_columns(
       m_row.m_index_name_length = 0;
     }
   } else {
-    m_row.m_object_schema_length = 0;
+    m_row.m_object_schema.reset();
     m_row.m_object_name_length = 0;
     m_row.m_index_name_length = 0;
   }
@@ -302,17 +296,17 @@ int table_events_waits_common::make_file_object_columns(
 
   m_row.m_object_type = "FILE";
   m_row.m_object_type_length = 4;
-  m_row.m_object_schema_length = 0;
+  m_row.m_object_schema.reset();
   m_row.m_object_instance_addr = (intptr)wait->m_object_instance_addr;
 
   if (safe_file->get_version() == wait->m_weak_version) {
     /* OBJECT NAME */
-    m_row.m_object_name_length = safe_file->m_filename_length;
+    m_row.m_object_name_length = safe_file->m_file_name.length();
     if (unlikely((m_row.m_object_name_length == 0) ||
                  (m_row.m_object_name_length > sizeof(m_row.m_object_name)))) {
       return 1;
     }
-    memcpy(m_row.m_object_name, safe_file->m_filename,
+    memcpy(m_row.m_object_name, safe_file->m_file_name.ptr(),
            m_row.m_object_name_length);
   } else {
     m_row.m_object_name_length = 0;
@@ -334,7 +328,7 @@ int table_events_waits_common::make_socket_object_columns(
 
   m_row.m_object_type = "SOCKET";
   m_row.m_object_type_length = 6;
-  m_row.m_object_schema_length = 0;
+  m_row.m_object_schema.reset();
   m_row.m_object_instance_addr = (intptr)wait->m_object_instance_addr;
 
   if (safe_socket->get_version() == wait->m_weak_version) {
@@ -343,16 +337,16 @@ int table_events_waits_common::make_socket_object_columns(
     uint port;
     char port_str[128];
     char ip_str[INET6_ADDRSTRLEN + 1];
-    uint ip_len = 0;
     port_str[0] = ':';
 
     /* Get the IP address and port number */
-    ip_len = pfs_get_socket_address(ip_str, sizeof(ip_str), &port,
-                                    &safe_socket->m_sock_addr,
-                                    safe_socket->m_addr_len);
+    const uint ip_len = pfs_get_socket_address(ip_str, sizeof(ip_str), &port,
+                                               &safe_socket->m_sock_addr,
+                                               safe_socket->m_addr_len);
 
     /* Convert port number to a string (length includes ':') */
-    size_t port_len = longlong10_to_str(port, (port_str + 1), 10) - port_str;
+    const size_t port_len =
+        longlong10_to_str(port, (port_str + 1), 10) - port_str;
 
     /* OBJECT NAME */
     m_row.m_object_name_length = ip_len + port_len;
@@ -374,6 +368,10 @@ int table_events_waits_common::make_socket_object_columns(
   return 0;
 }
 
+static void set_schema_name(PFS_schema_name *dst, const MDL_key *src) {
+  dst->set(src->db_name(), src->db_name_length());
+}
+
 int table_events_waits_common::make_metadata_lock_object_columns(
     PFS_events_waits *wait) {
   PFS_metadata_lock *safe_metadata_lock;
@@ -388,104 +386,104 @@ int table_events_waits_common::make_metadata_lock_object_columns(
     static_assert(MDL_key::NAMESPACE_END == 18,
                   "Adjust performance schema when changing enum_mdl_namespace");
 
-    MDL_key *mdl = &safe_metadata_lock->m_mdl_key;
+    const MDL_key *mdl = &safe_metadata_lock->m_mdl_key;
 
     switch (mdl->mdl_namespace()) {
       case MDL_key::GLOBAL:
         m_row.m_object_type = "GLOBAL";
         m_row.m_object_type_length = 6;
-        m_row.m_object_schema_length = 0;
+        m_row.m_object_schema.reset();
         m_row.m_object_name_length = 0;
         m_row.m_index_name_length = 0;
         break;
       case MDL_key::TABLESPACE:
         m_row.m_object_type = "TABLESPACE";
         m_row.m_object_type_length = 10;
-        m_row.m_object_schema_length = 0;
+        m_row.m_object_schema.reset();
         m_row.m_object_name_length = mdl->name_length();
         m_row.m_index_name_length = 0;
         break;
       case MDL_key::SCHEMA:
         m_row.m_object_type = "SCHEMA";
         m_row.m_object_type_length = 6;
-        m_row.m_object_schema_length = mdl->db_name_length();
+        set_schema_name(&m_row.m_object_schema, mdl);
         m_row.m_object_name_length = 0;
         m_row.m_index_name_length = 0;
         break;
       case MDL_key::TABLE:
         m_row.m_object_type = "TABLE";
         m_row.m_object_type_length = 5;
-        m_row.m_object_schema_length = mdl->db_name_length();
+        set_schema_name(&m_row.m_object_schema, mdl);
         m_row.m_object_name_length = mdl->name_length();
         m_row.m_index_name_length = 0;
         break;
       case MDL_key::FUNCTION:
         m_row.m_object_type = "FUNCTION";
         m_row.m_object_type_length = 8;
-        m_row.m_object_schema_length = mdl->db_name_length();
+        set_schema_name(&m_row.m_object_schema, mdl);
         m_row.m_object_name_length = mdl->name_length();
         m_row.m_index_name_length = 0;
         break;
       case MDL_key::PROCEDURE:
         m_row.m_object_type = "PROCEDURE";
         m_row.m_object_type_length = 9;
-        m_row.m_object_schema_length = mdl->db_name_length();
+        set_schema_name(&m_row.m_object_schema, mdl);
         m_row.m_object_name_length = mdl->name_length();
         m_row.m_index_name_length = 0;
         break;
       case MDL_key::TRIGGER:
         m_row.m_object_type = "TRIGGER";
         m_row.m_object_type_length = 7;
-        m_row.m_object_schema_length = mdl->db_name_length();
+        set_schema_name(&m_row.m_object_schema, mdl);
         m_row.m_object_name_length = mdl->name_length();
         m_row.m_index_name_length = 0;
         break;
       case MDL_key::EVENT:
         m_row.m_object_type = "EVENT";
         m_row.m_object_type_length = 5;
-        m_row.m_object_schema_length = mdl->db_name_length();
+        set_schema_name(&m_row.m_object_schema, mdl);
         m_row.m_object_name_length = mdl->name_length();
         m_row.m_index_name_length = 0;
         break;
       case MDL_key::COMMIT:
         m_row.m_object_type = "COMMIT";
         m_row.m_object_type_length = 6;
-        m_row.m_object_schema_length = 0;
+        m_row.m_object_schema.reset();
         m_row.m_object_name_length = 0;
         m_row.m_index_name_length = 0;
         break;
       case MDL_key::USER_LEVEL_LOCK:
         m_row.m_object_type = "USER LEVEL LOCK";
         m_row.m_object_type_length = 15;
-        m_row.m_object_schema_length = 0;
+        m_row.m_object_schema.reset();
         m_row.m_object_name_length = mdl->name_length();
         m_row.m_index_name_length = 0;
         break;
       case MDL_key::LOCKING_SERVICE:
         m_row.m_object_type = "LOCKING SERVICE";
         m_row.m_object_type_length = 15;
-        m_row.m_object_schema_length = mdl->db_name_length();
+        set_schema_name(&m_row.m_object_schema, mdl);
         m_row.m_object_name_length = mdl->name_length();
         m_row.m_index_name_length = 0;
         break;
       case MDL_key::SRID:
         m_row.m_object_type = "SRID";
         m_row.m_object_type_length = 4;
-        m_row.m_object_schema_length = 0;
+        m_row.m_object_schema.reset();
         m_row.m_object_name_length = mdl->name_length();
         m_row.m_index_name_length = 0;
         break;
       case MDL_key::ACL_CACHE:
         m_row.m_object_type = "ACL CACHE";
         m_row.m_object_type_length = 9;
-        m_row.m_object_schema_length = mdl->db_name_length();
+        set_schema_name(&m_row.m_object_schema, mdl);
         m_row.m_object_name_length = mdl->name_length();
         m_row.m_index_name_length = 0;
         break;
       case MDL_key::COLUMN_STATISTICS:
         m_row.m_object_type = "COLUMN STATISTICS";
         m_row.m_object_type_length = 17;
-        m_row.m_object_schema_length = mdl->db_name_length();
+        set_schema_name(&m_row.m_object_schema, mdl);
         m_row.m_object_name_length = mdl->name_length();
         // Reusing the INDEX_NAME column for COLUMN_NAME
         m_row.m_index_name_length = mdl->col_name_length();
@@ -493,46 +491,38 @@ int table_events_waits_common::make_metadata_lock_object_columns(
       case MDL_key::BACKUP_LOCK:
         m_row.m_object_type = "BACKUP_LOCK";
         m_row.m_object_type_length = sizeof("BACKUP_LOCK") - 1;
-        m_row.m_object_schema_length = 0;
+        m_row.m_object_schema.reset();
         m_row.m_object_name_length = 0;
         m_row.m_index_name_length = 0;
         break;
       case MDL_key::RESOURCE_GROUPS:
         m_row.m_object_type = "RESOURCE_GROUPS";
         m_row.m_object_type_length = 15;
-        m_row.m_object_schema_length = mdl->db_name_length();
+        set_schema_name(&m_row.m_object_schema, mdl);
         m_row.m_object_name_length = mdl->name_length();
         m_row.m_index_name_length = 0;
         break;
       case MDL_key::FOREIGN_KEY:
         m_row.m_object_type = "FOREIGN KEY";
         m_row.m_object_type_length = 11;
-        m_row.m_object_schema_length = mdl->db_name_length();
+        set_schema_name(&m_row.m_object_schema, mdl);
         m_row.m_object_name_length = mdl->name_length();
         break;
       case MDL_key::CHECK_CONSTRAINT:
         m_row.m_object_type = "CHECK CONSTRAINT";
         m_row.m_object_type_length = 16;
-        m_row.m_object_schema_length = mdl->db_name_length();
+        set_schema_name(&m_row.m_object_schema, mdl);
         m_row.m_object_name_length = mdl->name_length();
         break;
       case MDL_key::NAMESPACE_END:
       default:
         m_row.m_object_type_length = 0;
-        m_row.m_object_schema_length = 0;
+        m_row.m_object_schema.reset();
         m_row.m_object_name_length = 0;
         m_row.m_index_name_length = 0;
         break;
     }
 
-    if (m_row.m_object_schema_length > sizeof(m_row.m_object_schema)) {
-      return 1;
-    }
-    if (m_row.m_object_schema_length > 0) {
-      memcpy(m_row.m_object_schema, mdl->db_name(),
-             m_row.m_object_schema_length);
-    }
-
     if (m_row.m_object_name_length > sizeof(m_row.m_object_name)) {
       return 1;
     }
@@ -550,7 +540,7 @@ int table_events_waits_common::make_metadata_lock_object_columns(
     m_row.m_object_instance_addr = (intptr)wait->m_object_instance_addr;
   } else {
     m_row.m_object_type_length = 0;
-    m_row.m_object_schema_length = 0;
+    m_row.m_object_schema.reset();
     m_row.m_object_name_length = 0;
     m_row.m_index_name_length = 0;
     m_row.m_object_instance_addr = 0;
@@ -568,7 +558,7 @@ int table_events_waits_common::make_row(PFS_events_waits *wait) {
   PFS_instr_class *safe_class;
   ulonglong timer_end;
   /* wait normalizer for most rows. */
-  time_normalizer *normalizer = m_normalizer;
+  const time_normalizer *normalizer = m_normalizer;
 
   /*
     Design choice:
@@ -667,8 +657,8 @@ int table_events_waits_common::make_row(PFS_events_waits *wait) {
   normalizer->to_pico(wait->m_timer_start, timer_end, &m_row.m_timer_start,
                       &m_row.m_timer_end, &m_row.m_timer_wait);
 
-  m_row.m_name = safe_class->m_name;
-  m_row.m_name_length = safe_class->m_name_length;
+  m_row.m_name = safe_class->m_name.str();
+  m_row.m_name_length = safe_class->m_name.length();
 
   make_source_column(wait->m_source_file, wait->m_source_line, m_row.m_source,
                      sizeof(m_row.m_source), m_row.m_source_length);
@@ -688,7 +678,7 @@ int table_events_waits_common::make_row(PFS_events_waits *wait) {
   Different similar operations (CLOSE vs STREAMCLOSE) are displayed
   with the same name 'close'.
 */
-static const LEX_CSTRING operation_names_map[] = {
+static constexpr LEX_CSTRING operation_names_map[] = {
     /* Mutex operations */
     {STRING_WITH_LEN("lock")},
     {STRING_WITH_LEN("try_lock")},
@@ -813,10 +803,10 @@ int table_events_waits_common::read_row_values(TABLE *table, unsigned char *buf,
           }
           break;
         case 3: /* EVENT_NAME */
-          set_field_varchar_utf8(f, m_row.m_name, m_row.m_name_length);
+          set_field_varchar_utf8mb4(f, m_row.m_name, m_row.m_name_length);
           break;
         case 4: /* SOURCE */
-          set_field_varchar_utf8(f, m_row.m_source, m_row.m_source_length);
+          set_field_varchar_utf8mb4(f, m_row.m_source, m_row.m_source_length);
           break;
         case 5: /* TIMER_START */
           if (m_row.m_timer_start != 0) {
@@ -844,33 +834,28 @@ int table_events_waits_common::read_row_values(TABLE *table, unsigned char *buf,
           f->set_null();
           break;
         case 9: /* OBJECT_SCHEMA */
-          if (m_row.m_object_schema_length > 0) {
-            set_field_varchar_utf8(f, m_row.m_object_schema,
-                                   m_row.m_object_schema_length);
-          } else {
-            f->set_null();
-          }
+          set_nullable_field_schema_name(f, &m_row.m_object_schema);
           break;
         case 10: /* OBJECT_NAME */
           if (m_row.m_object_name_length > 0) {
-            set_field_varchar_utf8(f, m_row.m_object_name,
-                                   m_row.m_object_name_length);
+            set_field_varchar_utf8mb4(f, m_row.m_object_name,
+                                      m_row.m_object_name_length);
           } else {
             f->set_null();
           }
           break;
         case 11: /* INDEX_NAME */
           if (m_row.m_index_name_length > 0) {
-            set_field_varchar_utf8(f, m_row.m_index_name,
-                                   m_row.m_index_name_length);
+            set_field_varchar_utf8mb4(f, m_row.m_index_name,
+                                      m_row.m_index_name_length);
           } else {
             f->set_null();
           }
           break;
         case 12: /* OBJECT_TYPE */
           if (m_row.m_object_type_length > 0) {
-            set_field_varchar_utf8(f, m_row.m_object_type,
-                                   m_row.m_object_type_length);
+            set_field_varchar_utf8mb4(f, m_row.m_object_type,
+                                      m_row.m_object_type_length);
           } else {
             f->set_null();
           }
@@ -894,7 +879,7 @@ int table_events_waits_common::read_row_values(TABLE *table, unsigned char *buf,
           break;
         case 16: /* OPERATION */
           operation = &operation_names_map[(int)m_row.m_operation - 1];
-          set_field_varchar_utf8(f, operation->str, operation->length);
+          set_field_varchar_utf8mb4(f, operation->str, operation->length);
           break;
         case 17: /* NUMBER_OF_BYTES (also used for ROWS) */
           if ((m_row.m_operation == OPERATION_TYPE_FILEREAD) ||
@@ -929,34 +914,29 @@ PFS_engine_table *table_events_waits_current::create(PFS_engine_table_share *) {
 }
 
 table_events_waits_current::table_events_waits_current()
-    : table_events_waits_common(&m_share, &m_pos), m_pos() {}
+    : table_events_waits_common(&m_share, &m_pos), m_opened_index(nullptr) {}
 
-void table_events_waits_current::reset_position(void) {
+void table_events_waits_current::reset_position() {
   m_pos.reset();
   m_next_pos.reset();
 }
 
-PFS_events_waits *table_events_waits_current::get_wait(
-    PFS_thread *pfs_thread, uint index_2 MY_ATTRIBUTE((unused))) {
+PFS_events_waits *table_events_waits_current::get_wait(PFS_thread *pfs_thread,
+                                                       uint index_2
+                                                       [[maybe_unused]]) {
   PFS_events_waits *wait;
 
   /*
     We do not show nested events for now,
     this will be revised with TABLE I/O
   */
-  // #define ONLY_SHOW_ONE_WAIT
 
-#ifdef ONLY_SHOW_ONE_WAIT
-  if (index_2 >= 1) {
-    return NULL;
-  }
-#else
   /* m_events_waits_stack[0] is a dummy record */
-  PFS_events_waits *top_wait =
+  const PFS_events_waits *top_wait =
       &pfs_thread->m_events_waits_stack[WAIT_STACK_BOTTOM];
   wait = &pfs_thread->m_events_waits_stack[m_pos.m_index_2 + WAIT_STACK_BOTTOM];
 
-  PFS_events_waits *safe_current = pfs_thread->m_events_waits_current;
+  const PFS_events_waits *safe_current = pfs_thread->m_events_waits_current;
 
   if (safe_current == top_wait) {
     /* Display the last top level wait, when completed */
@@ -969,7 +949,6 @@ PFS_events_waits *table_events_waits_current::get_wait(
       return nullptr;
     }
   }
-#endif
 
   if (wait->m_wait_class == NO_WAIT_CLASS) {
     /*
@@ -982,7 +961,7 @@ PFS_events_waits *table_events_waits_current::get_wait(
   return wait;
 }
 
-int table_events_waits_current::rnd_next(void) {
+int table_events_waits_current::rnd_next() {
   PFS_thread *pfs_thread;
   PFS_events_waits *wait;
   bool has_more_thread = true;
@@ -1003,15 +982,12 @@ int table_events_waits_current::rnd_next(void) {
 }
 
 int table_events_waits_current::rnd_pos(const void *pos) {
-  PFS_thread *pfs_thread;
-  PFS_events_waits *wait;
-
   set_position(pos);
 
-  pfs_thread = global_thread_container.get(m_pos.m_index_1);
+  PFS_thread *pfs_thread = global_thread_container.get(m_pos.m_index_1);
   if (pfs_thread != nullptr) {
     assert(m_pos.m_index_2 < WAIT_STACK_LOGICAL_SIZE);
-    wait = get_wait(pfs_thread, m_pos.m_index_2);
+    PFS_events_waits *wait = get_wait(pfs_thread, m_pos.m_index_2);
     if (wait != nullptr) {
       return make_row(pfs_thread, wait);
     }
@@ -1020,8 +996,7 @@ int table_events_waits_current::rnd_pos(const void *pos) {
   return HA_ERR_RECORD_DELETED;
 }
 
-int table_events_waits_current::index_init(uint idx MY_ATTRIBUTE((unused)),
-                                           bool) {
+int table_events_waits_current::index_init(uint idx [[maybe_unused]], bool) {
   PFS_index_events_waits *result;
   assert(idx == 0);
   result = PFS_NEW(PFS_index_events_waits);
@@ -1030,7 +1005,7 @@ int table_events_waits_current::index_init(uint idx MY_ATTRIBUTE((unused)),
   return 0;
 }
 
-int table_events_waits_current::index_next(void) {
+int table_events_waits_current::index_next() {
   PFS_thread *pfs_thread;
   PFS_events_waits *wait;
   bool has_more_thread = true;
@@ -1077,12 +1052,12 @@ int table_events_waits_current::make_row(PFS_thread *thread,
   return 0;
 }
 
-int table_events_waits_current::delete_all_rows(void) {
+int table_events_waits_current::delete_all_rows() {
   reset_events_waits_current();
   return 0;
 }
 
-ha_rows table_events_waits_current::get_row_count(void) {
+ha_rows table_events_waits_current::get_row_count() {
   return WAIT_STACK_SIZE * global_thread_container.get_row_count();
 }
 
@@ -1091,9 +1066,9 @@ PFS_engine_table *table_events_waits_history::create(PFS_engine_table_share *) {
 }
 
 table_events_waits_history::table_events_waits_history()
-    : table_events_waits_common(&m_share, &m_pos), m_pos(), m_next_pos() {}
+    : table_events_waits_common(&m_share, &m_pos), m_opened_index(nullptr) {}
 
-void table_events_waits_history::reset_position(void) {
+void table_events_waits_history::reset_position() {
   m_pos.reset();
   m_next_pos.reset();
 }
@@ -1121,7 +1096,7 @@ PFS_events_waits *table_events_waits_history::get_wait(PFS_thread *pfs_thread,
   return wait;
 }
 
-int table_events_waits_history::rnd_next(void) {
+int table_events_waits_history::rnd_next() {
   PFS_thread *pfs_thread;
   PFS_events_waits *wait;
   bool has_more_thread = true;
@@ -1146,17 +1121,14 @@ int table_events_waits_history::rnd_next(void) {
 }
 
 int table_events_waits_history::rnd_pos(const void *pos) {
-  PFS_thread *pfs_thread;
-  PFS_events_waits *wait;
-
   assert(events_waits_history_per_thread != 0);
   set_position(pos);
 
-  pfs_thread = global_thread_container.get(m_pos.m_index_1);
+  PFS_thread *pfs_thread = global_thread_container.get(m_pos.m_index_1);
   if (pfs_thread != nullptr) {
     assert(m_pos.m_index_2 < events_waits_history_per_thread);
 
-    wait = get_wait(pfs_thread, m_pos.m_index_2);
+    PFS_events_waits *wait = get_wait(pfs_thread, m_pos.m_index_2);
     if (wait != nullptr) {
       return make_row(pfs_thread, wait);
     }
@@ -1165,8 +1137,7 @@ int table_events_waits_history::rnd_pos(const void *pos) {
   return HA_ERR_RECORD_DELETED;
 }
 
-int table_events_waits_history::index_init(uint idx MY_ATTRIBUTE((unused)),
-                                           bool) {
+int table_events_waits_history::index_init(uint idx [[maybe_unused]], bool) {
   PFS_index_events_waits *result;
   assert(idx == 0);
   result = PFS_NEW(PFS_index_events_waits);
@@ -1175,7 +1146,7 @@ int table_events_waits_history::index_init(uint idx MY_ATTRIBUTE((unused)),
   return 0;
 }
 
-int table_events_waits_history::index_next(void) {
+int table_events_waits_history::index_next() {
   PFS_thread *pfs_thread;
   PFS_events_waits *wait;
   bool has_more_thread = true;
@@ -1226,12 +1197,12 @@ int table_events_waits_history::make_row(PFS_thread *thread,
   return 0;
 }
 
-int table_events_waits_history::delete_all_rows(void) {
+int table_events_waits_history::delete_all_rows() {
   reset_events_waits_history();
   return 0;
 }
 
-ha_rows table_events_waits_history::get_row_count(void) {
+ha_rows table_events_waits_history::get_row_count() {
   return events_waits_history_per_thread *
          global_thread_container.get_row_count();
 }
@@ -1244,12 +1215,12 @@ PFS_engine_table *table_events_waits_history_long::create(
 table_events_waits_history_long::table_events_waits_history_long()
     : table_events_waits_common(&m_share, &m_pos), m_pos(0), m_next_pos(0) {}
 
-void table_events_waits_history_long::reset_position(void) {
+void table_events_waits_history_long::reset_position() {
   m_pos.m_index = 0;
   m_next_pos.m_index = 0;
 }
 
-int table_events_waits_history_long::rnd_next(void) {
+int table_events_waits_history_long::rnd_next() {
   PFS_events_waits *wait;
   uint limit;
 
@@ -1305,11 +1276,11 @@ int table_events_waits_history_long::rnd_pos(const void *pos) {
   return make_row(wait);
 }
 
-int table_events_waits_history_long::delete_all_rows(void) {
+int table_events_waits_history_long::delete_all_rows() {
   reset_events_waits_history_long();
   return 0;
 }
 
-ha_rows table_events_waits_history_long::get_row_count(void) {
+ha_rows table_events_waits_history_long::get_row_count() {
   return events_waits_history_long_size;
 }
