@@ -1,15 +1,16 @@
-/* Copyright (c) 2010, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2010, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -71,13 +72,13 @@ void set_field_utiny(Field *f, ulong value) {
 
 long get_field_tiny(Field *f) {
   assert(f->real_type() == MYSQL_TYPE_TINY);
-  auto *f2 = (Field_tiny *)f;
+  const auto *f2 = (Field_tiny *)f;
   return f2->val_int();
 }
 
 ulong get_field_utiny(Field *f) {
   assert(f->real_type() == MYSQL_TYPE_TINY);
-  auto *f2 = (Field_tiny *)f;
+  const auto *f2 = (Field_tiny *)f;
   return f2->val_int();
 }
 
@@ -96,13 +97,13 @@ void set_field_ushort(Field *f, ulong value) {
 
 long get_field_short(Field *f) {
   assert(f->real_type() == MYSQL_TYPE_SHORT);
-  auto *f2 = (Field_short *)f;
+  const auto *f2 = (Field_short *)f;
   return f2->val_int();
 }
 
 ulong get_field_ushort(Field *f) {
   assert(f->real_type() == MYSQL_TYPE_SHORT);
-  auto *f2 = (Field_short *)f;
+  const auto *f2 = (Field_short *)f;
   return f2->val_int();
 }
 
@@ -121,13 +122,13 @@ void set_field_umedium(Field *f, ulong value) {
 
 long get_field_medium(Field *f) {
   assert(f->real_type() == MYSQL_TYPE_INT24);
-  auto *f2 = (Field_medium *)f;
+  const auto *f2 = (Field_medium *)f;
   return f2->val_int();
 }
 
 ulong get_field_umedium(Field *f) {
   assert(f->real_type() == MYSQL_TYPE_INT24);
-  auto *f2 = (Field_medium *)f;
+  const auto *f2 = (Field_medium *)f;
   return f2->val_int();
 }
 
@@ -146,13 +147,13 @@ void set_field_ulong(Field *f, ulong value) {
 
 long get_field_long(Field *f) {
   assert(f->real_type() == MYSQL_TYPE_LONG);
-  auto *f2 = (Field_long *)f;
+  const auto *f2 = (Field_long *)f;
   return f2->val_int();
 }
 
 ulong get_field_ulong(Field *f) {
   assert(f->real_type() == MYSQL_TYPE_LONG);
-  auto *f2 = (Field_long *)f;
+  const auto *f2 = (Field_long *)f;
   return f2->val_int();
 }
 
@@ -171,13 +172,13 @@ void set_field_ulonglong(Field *f, ulonglong value) {
 
 longlong get_field_longlong(Field *f) {
   assert(f->real_type() == MYSQL_TYPE_LONGLONG);
-  auto *f2 = (Field_longlong *)f;
+  const auto *f2 = (Field_longlong *)f;
   return f2->val_int();
 }
 
 ulonglong get_field_ulonglong(Field *f) {
   assert(f->real_type() == MYSQL_TYPE_LONGLONG);
-  auto *f2 = (Field_longlong *)f;
+  const auto *f2 = (Field_longlong *)f;
   return f2->val_int();
 }
 
@@ -190,7 +191,7 @@ void set_field_decimal(Field *f, double value) {
 
 double get_field_decimal(Field *f) {
   assert(f->real_type() == MYSQL_TYPE_NEWDECIMAL);
-  auto *f2 = (Field_new_decimal *)f;
+  const auto *f2 = (Field_new_decimal *)f;
   return f2->val_real();
 }
 
@@ -203,7 +204,7 @@ void set_field_float(Field *f, double value) {
 
 double get_field_float(Field *f) {
   assert(f->real_type() == MYSQL_TYPE_FLOAT);
-  auto *f2 = (Field_float *)f;
+  const auto *f2 = (Field_float *)f;
   return f2->val_real();
 }
 
@@ -216,7 +217,7 @@ void set_field_double(Field *f, double value) {
 
 double get_field_double(Field *f) {
   assert(f->real_type() == MYSQL_TYPE_DOUBLE);
-  auto *f2 = (Field_double *)f;
+  const auto *f2 = (Field_double *)f;
   return f2->val_real();
 }
 
@@ -229,7 +230,7 @@ void set_field_char_utf8mb4(Field *f, const char *str, uint len) {
 
 String *get_field_char_utf8mb4(Field *f, String *val) {
   assert(f->real_type() == MYSQL_TYPE_STRING);
-  auto *f2 = (Field_string *)f;
+  const auto *f2 = (Field_string *)f;
   val = f2->val_str(nullptr, val);
   return val;
 }
@@ -237,7 +238,7 @@ String *get_field_char_utf8mb4(Field *f, String *val) {
 char *get_field_char_utf8mb4(Field *f, char *val, uint *len) {
   assert(f->real_type() == MYSQL_TYPE_STRING);
   String temp;
-  auto *f2 = (Field_string *)f;
+  const auto *f2 = (Field_string *)f;
   f2->val_str(nullptr, &temp);
   *len = temp.length();
   strncpy(val, temp.ptr(), *len);
@@ -254,14 +255,14 @@ void set_field_varchar(Field *f, const CHARSET_INFO *cs, const char *str,
 
 String *get_field_varchar_utf8mb4(Field *f, String *val) {
   assert(f->real_type() == MYSQL_TYPE_VARCHAR);
-  auto *f2 = (Field_varstring *)f;
+  const auto *f2 = (Field_varstring *)f;
   val = f2->val_str(nullptr, val);
   return val;
 }
 char *get_field_varchar_utf8mb4(Field *f, char *val, uint *len) {
   assert(f->real_type() == MYSQL_TYPE_VARCHAR);
   String temp;
-  auto *f2 = (Field_varstring *)f;
+  const auto *f2 = (Field_varstring *)f;
   f2->val_str(nullptr, &temp);
   *len = temp.length();
   strncpy(val, temp.ptr(), *len);
@@ -298,7 +299,7 @@ void set_field_text(Field *f, const char *val, size_t len,
 char *get_field_blob(Field *f, char *val, uint *len) {
   assert(f->real_type() == MYSQL_TYPE_BLOB);
   String temp;
-  auto *f2 = (Field_blob *)f;
+  const auto *f2 = (Field_blob *)f;
   f2->val_str(nullptr, &temp);
   *len = temp.length();
   strncpy(val, temp.ptr(), *len);
@@ -314,7 +315,7 @@ void set_field_enum(Field *f, ulonglong value) {
 
 ulonglong get_field_enum(Field *f) {
   assert(f->real_type() == MYSQL_TYPE_ENUM);
-  auto *f2 = (Field_enum *)f;
+  const auto *f2 = (Field_enum *)f;
   return f2->val_int();
 }
 
@@ -327,7 +328,7 @@ void set_field_set(Field *f, ulonglong value) {
 
 ulonglong get_field_set(Field *f) {
   assert(f->real_type() == MYSQL_TYPE_SET);
-  auto *f2 = (Field_set *)f;
+  const auto *f2 = (Field_set *)f;
   return f2->val_int();
 }
 
@@ -341,7 +342,7 @@ void set_field_date(Field *f, const char *value, uint len) {
 char *get_field_date(Field *f, char *val, uint *len) {
   assert(f->real_type() == MYSQL_TYPE_NEWDATE);
   String temp;
-  auto *f2 = (Field_newdate *)f;
+  const auto *f2 = (Field_newdate *)f;
   f2->val_str(&temp, nullptr);
   *len = temp.length();
   strncpy(val, temp.ptr(), *len);
@@ -358,7 +359,7 @@ void set_field_time(Field *f, const char *value, uint len) {
 char *get_field_time(Field *f, char *val, uint *len) {
   assert(f->real_type() == MYSQL_TYPE_TIME2);
   String temp;
-  auto *f2 = (Field_timef *)f;
+  const auto *f2 = (Field_timef *)f;
   f2->val_str(&temp, nullptr);
   *len = temp.length();
   strncpy(val, temp.ptr(), *len);
@@ -375,7 +376,7 @@ void set_field_datetime(Field *f, const char *value, uint len) {
 char *get_field_datetime(Field *f, char *val, uint *len) {
   assert(f->real_type() == MYSQL_TYPE_DATETIME2);
   String temp;
-  auto *f2 = (Field_datetimef *)f;
+  const auto *f2 = (Field_datetimef *)f;
   f2->val_str(&temp, nullptr);
   *len = temp.length();
   strncpy(val, temp.ptr(), *len);
@@ -392,7 +393,7 @@ void set_field_timestamp(Field *f, const char *value, uint len) {
 char *get_field_timestamp(Field *f, char *val, uint *len) {
   assert(f->real_type() == MYSQL_TYPE_TIMESTAMP2);
   String temp;
-  auto *f2 = (Field_timestampf *)f;
+  const auto *f2 = (Field_timestampf *)f;
   f2->val_str(&temp, nullptr);
   *len = temp.length();
   strncpy(val, temp.ptr(), *len);
@@ -416,7 +417,7 @@ void set_field_year(Field *f, ulong value) {
 
 ulong get_field_year(Field *f) {
   assert(f->real_type() == MYSQL_TYPE_YEAR);
-  auto *f2 = (Field_year *)f;
+  const auto *f2 = (Field_year *)f;
   return f2->val_int();
 }
 
@@ -442,6 +443,22 @@ void set_field_schema_name(Field *f, const PFS_schema_name *schema) {
   set_field_varchar_utf8mb4(f, schema->ptr(), schema->length());
 }
 
+void set_nullable_field_schema_name(Field *f,
+                                    const PFS_schema_name_view *schema) {
+  assert(f->is_nullable());
+  const size_t len = schema->length();
+  if (len == 0) {
+    f->set_null();
+  } else {
+    set_field_varchar_utf8mb4(f, schema->ptr(), len);
+  }
+}
+
+void set_field_schema_name(Field *f, const PFS_schema_name_view *schema) {
+  assert(!f->is_nullable());
+  set_field_varchar_utf8mb4(f, schema->ptr(), schema->length());
+}
+
 void set_nullable_field_object_name(Field *f, const PFS_object_name *object) {
   assert(f->is_nullable());
   const size_t len = object->length();
@@ -457,6 +474,22 @@ void set_field_object_name(Field *f, const PFS_object_name *object) {
   set_field_varchar_utf8mb4(f, object->ptr(), object->length());
 }
 
+void set_nullable_field_object_name(Field *f,
+                                    const PFS_object_name_view *object) {
+  assert(f->is_nullable());
+  const size_t len = object->length();
+  if (len == 0) {
+    f->set_null();
+  } else {
+    set_field_varchar_utf8mb4(f, object->ptr(), len);
+  }
+}
+
+void set_field_object_name(Field *f, const PFS_object_name_view *object) {
+  assert(!f->is_nullable());
+  set_field_varchar_utf8mb4(f, object->ptr(), object->length());
+}
+
 void set_nullable_field_routine_name(Field *f, const PFS_routine_name *object) {
   assert(f->is_nullable());
   const size_t len = object->length();
@@ -875,6 +908,42 @@ void PFS_object_row::set_nullable_field(uint index, Field *f) {
   }
 }
 
+void PFS_object_view_row::set_field(uint index, Field *f) {
+  switch (index) {
+    case 0: /* OBJECT_TYPE */
+      set_field_object_type(f, m_object_type);
+      break;
+    case 1: /* SCHEMA_NAME */
+      set_field_schema_name(f, &m_schema_name);
+      break;
+    case 2: /* OBJECT_NAME */
+      set_field_object_name(f, &m_object_name);
+      break;
+    default:
+      assert(false);
+  }
+}
+
+void PFS_object_view_row::set_nullable_field(uint index, Field *f) {
+  switch (index) {
+    case 0: /* OBJECT_TYPE */
+      if (m_object_type != NO_OBJECT_TYPE) {
+        set_field_object_type(f, m_object_type);
+      } else {
+        f->set_null();
+      }
+      break;
+    case 1: /* SCHEMA_NAME */
+      set_nullable_field_schema_name(f, &m_schema_name);
+      break;
+    case 2: /* OBJECT_NAME */
+      set_nullable_field_object_name(f, &m_object_name);
+      break;
+    default:
+      assert(false);
+  }
+}
+
 void PFS_column_row::set_nullable_field(uint index, Field *f) {
   switch (index) {
     case 0: /* OBJECT_TYPE */
@@ -987,6 +1056,44 @@ void PFS_index_row::set_nullable_field(uint index, Field *f) {
   }
 }
 
+void PFS_index_view_row::set_field(uint index, Field *f) {
+  switch (index) {
+    case 0: /* OBJECT_TYPE */
+    case 1: /* SCHEMA_NAME */
+    case 2: /* OBJECT_NAME */
+      m_object_row.set_field(index, f);
+      break;
+    case 3: /* INDEX_NAME */
+      if (m_index_name.length() > 0) {
+        set_field_varchar_utf8mb4(f, m_index_name.ptr(), m_index_name.length());
+      } else {
+        f->set_null();
+      }
+      break;
+    default:
+      assert(false);
+  }
+}
+
+void PFS_index_view_row::set_nullable_field(uint index, Field *f) {
+  switch (index) {
+    case 0: /* OBJECT_TYPE */
+    case 1: /* SCHEMA_NAME */
+    case 2: /* OBJECT_NAME */
+      m_object_row.set_nullable_field(index, f);
+      break;
+    case 3: /* INDEX_NAME */
+      if (m_index_name.length() > 0) {
+        set_field_varchar_utf8mb4(f, m_index_name.ptr(), m_index_name.length());
+      } else {
+        f->set_null();
+      }
+      break;
+    default:
+      assert(false);
+  }
+}
+
 void PFS_statement_stat_row::set_field(uint index, Field *f) {
   switch (index) {
     case 0: /* COUNT_STAR */
@@ -1551,20 +1658,18 @@ bool PFS_key_ulonglong::stateless_match(bool record_null,
 }
 
 bool PFS_key_ulong::do_match(bool record_null, ulong record_value) {
-  int cmp = 0;
+  int cmp;
 
   if (m_is_null) {
     cmp = (record_null ? 0 : 1);
+  } else if (record_null) {
+    cmp = -1;
+  } else if (record_value < m_key_value) {
+    cmp = -1;
+  } else if (record_value > m_key_value) {
+    cmp = +1;
   } else {
-    if (record_null) {
-      cmp = -1;
-    } else if (record_value < m_key_value) {
-      cmp = -1;
-    } else if (record_value > m_key_value) {
-      cmp = +1;
-    } else {
-      cmp = 0;
-    }
+    cmp = 0;
   }
 
   switch (m_find_flag) {
@@ -1585,20 +1690,18 @@ bool PFS_key_ulong::do_match(bool record_null, ulong record_value) {
 }
 
 bool PFS_key_ulonglong::do_match(bool record_null, ulonglong record_value) {
-  int cmp = 0;
+  int cmp;
 
   if (m_is_null) {
     cmp = (record_null ? 0 : 1);
+  } else if (record_null) {
+    cmp = -1;
+  } else if (record_value < m_key_value) {
+    cmp = -1;
+  } else if (record_value > m_key_value) {
+    cmp = +1;
   } else {
-    if (record_null) {
-      cmp = -1;
-    } else if (record_value < m_key_value) {
-      cmp = -1;
-    } else if (record_value > m_key_value) {
-      cmp = +1;
-    } else {
-      cmp = 0;
-    }
+    cmp = 0;
   }
 
   switch (m_find_flag) {
@@ -1641,16 +1744,14 @@ bool PFS_key_pstring::stateless_match(bool record_null,
                                m_key_value_length) == 0);
   }
 
-  int cmp = 0;
+  int cmp;
 
   if (is_null) {
     cmp = record_null ? 0 : 1;
+  } else if (record_null) {
+    cmp = -1;
   } else {
-    if (record_null) {
-      cmp = -1;
-    } else {
-      cmp = native_strncasecmp(record_string, m_key_value, m_key_value_length);
-    }
+    cmp = native_strncasecmp(record_string, m_key_value, m_key_value_length);
   }
 
   switch (find_flag) {
@@ -1699,7 +1800,7 @@ bool PFS_key_thread_id::match(const PFS_thread *pfs) {
 }
 
 bool PFS_key_thread_id::match_owner(const PFS_table *pfs) {
-  PFS_thread *thread = sanitize_thread(pfs->m_thread_owner);
+  const PFS_thread *thread = sanitize_thread(pfs->m_thread_owner);
 
   if (thread == nullptr) {
     return do_match(true, 0);
@@ -1710,7 +1811,7 @@ bool PFS_key_thread_id::match_owner(const PFS_table *pfs) {
 }
 
 bool PFS_key_thread_id::match_owner(const PFS_socket *pfs) {
-  PFS_thread *thread = sanitize_thread(pfs->m_thread_owner);
+  const PFS_thread *thread = sanitize_thread(pfs->m_thread_owner);
 
   if (thread == nullptr) {
     return do_match(true, 0);
@@ -1720,7 +1821,7 @@ bool PFS_key_thread_id::match_owner(const PFS_socket *pfs) {
 }
 
 bool PFS_key_thread_id::match_owner(const PFS_mutex *pfs) {
-  PFS_thread *thread = sanitize_thread(pfs->m_owner);
+  const PFS_thread *thread = sanitize_thread(pfs->m_owner);
 
   if (thread == nullptr) {
     return do_match(true, 0);
@@ -1740,7 +1841,7 @@ bool PFS_key_thread_id::match_owner(const PFS_metadata_lock *pfs) {
 }
 
 bool PFS_key_thread_id::match_writer(const PFS_rwlock *pfs) {
-  PFS_thread *thread = sanitize_thread(pfs->m_writer);
+  const PFS_thread *thread = sanitize_thread(pfs->m_writer);
 
   if (thread == nullptr) {
     return do_match(true, 0);
@@ -1810,11 +1911,10 @@ bool PFS_key_socket_id::match(const PFS_socket *pfs) {
 bool PFS_key_port::match(const PFS_socket *pfs) {
   bool record_null = (pfs->m_addr_len == 0);
   uint port = 0;
-  char ip[INET6_ADDRSTRLEN + 1];
-  uint ip_len = 0;
   if (!record_null) {
-    ip_len = pfs_get_socket_address(ip, sizeof(ip), &port, &pfs->m_sock_addr,
-                                    pfs->m_addr_len);
+    char ip[INET6_ADDRSTRLEN + 1];
+    const uint ip_len = pfs_get_socket_address(
+        ip, sizeof(ip), &port, &pfs->m_sock_addr, pfs->m_addr_len);
     record_null = (ip_len == 0);
   }
   return do_match(record_null, (int32)port);
@@ -1836,7 +1936,7 @@ bool PFS_key_error_number::match_error_index(uint error_index) {
 }
 
 bool PFS_key_thread_name::match(const PFS_thread *pfs) {
-  PFS_thread_class *klass = sanitize_thread_class(pfs->m_class);
+  const PFS_thread_class *klass = sanitize_thread_class(pfs->m_class);
   if (klass == nullptr) {
     return false;
   }
@@ -1853,7 +1953,7 @@ bool PFS_key_event_name::match(const PFS_instr_class *pfs) {
 }
 
 bool PFS_key_event_name::match(const PFS_mutex *pfs) {
-  PFS_mutex_class *safe_class = sanitize_mutex_class(pfs->m_class);
+  const PFS_mutex_class *safe_class = sanitize_mutex_class(pfs->m_class);
   if (unlikely(safe_class == nullptr)) {
     return false;
   }
@@ -1862,7 +1962,7 @@ bool PFS_key_event_name::match(const PFS_mutex *pfs) {
 }
 
 bool PFS_key_event_name::match(const PFS_rwlock *pfs) {
-  PFS_rwlock_class *safe_class = sanitize_rwlock_class(pfs->m_class);
+  const PFS_rwlock_class *safe_class = sanitize_rwlock_class(pfs->m_class);
   if (unlikely(safe_class == nullptr)) {
     return false;
   }
@@ -1870,7 +1970,7 @@ bool PFS_key_event_name::match(const PFS_rwlock *pfs) {
 }
 
 bool PFS_key_event_name::match(const PFS_cond *pfs) {
-  PFS_cond_class *safe_class = sanitize_cond_class(pfs->m_class);
+  const PFS_cond_class *safe_class = sanitize_cond_class(pfs->m_class);
   if (unlikely(safe_class == nullptr)) {
     return false;
   }
@@ -1878,7 +1978,7 @@ bool PFS_key_event_name::match(const PFS_cond *pfs) {
 }
 
 bool PFS_key_event_name::match(const PFS_file *pfs) {
-  PFS_file_class *safe_class = sanitize_file_class(pfs->m_class);
+  const PFS_file_class *safe_class = sanitize_file_class(pfs->m_class);
   if (unlikely(safe_class == nullptr)) {
     return false;
   }
@@ -1886,7 +1986,7 @@ bool PFS_key_event_name::match(const PFS_file *pfs) {
 }
 
 bool PFS_key_event_name::match(const PFS_socket *pfs) {
-  PFS_socket_class *safe_class = sanitize_socket_class(pfs->m_class);
+  const PFS_socket_class *safe_class = sanitize_socket_class(pfs->m_class);
   if (unlikely(safe_class == nullptr)) {
     return false;
   }
@@ -1971,7 +2071,7 @@ bool PFS_key_event_name::match_view(uint view) {
 }
 
 bool PFS_key_meter_name::match(PFS_meter_class *pfs) {
-  PFS_meter_class *safe_class = sanitize_meter_class(pfs);
+  const PFS_meter_class *safe_class = sanitize_meter_class(pfs);
   if (unlikely(safe_class == nullptr)) {
     return false;
   }
@@ -1979,7 +2079,7 @@ bool PFS_key_meter_name::match(PFS_meter_class *pfs) {
 }
 
 bool PFS_key_metric_name::match(PFS_metric_class *pfs) {
-  PFS_metric_class *safe_class = sanitize_metric_class(pfs);
+  const PFS_metric_class *safe_class = sanitize_metric_class(pfs);
   if (unlikely(safe_class == nullptr)) {
     return false;
   }
@@ -2182,20 +2282,18 @@ bool PFS_key_object_type::match(const PFS_program *pfs) {
 
 bool PFS_key_object_type::do_match(bool record_null,
                                    enum_object_type record_value) {
-  int cmp = 0;
+  int cmp;
 
   if (m_is_null) {
     cmp = (record_null ? 0 : 1);
+  } else if (record_null) {
+    cmp = -1;
+  } else if (record_value < m_object_type) {
+    cmp = -1;
+  } else if (record_value > m_object_type) {
+    cmp = +1;
   } else {
-    if (record_null) {
-      cmp = -1;
-    } else if (record_value < m_object_type) {
-      cmp = -1;
-    } else if (record_value > m_object_type) {
-      cmp = +1;
-    } else {
-      cmp = 0;
-    }
+    cmp = 0;
   }
 
   switch (m_find_flag) {
@@ -2250,20 +2348,18 @@ bool PFS_key_object_type_enum::match(const PFS_program *pfs) {
 
 bool PFS_key_object_type_enum::do_match(bool record_null,
                                         enum_object_type record_value) {
-  int cmp = 0;
+  int cmp;
 
   if (m_is_null) {
     cmp = (record_null ? 0 : 1);
+  } else if (record_null) {
+    cmp = -1;
+  } else if (record_value < m_object_type) {
+    cmp = -1;
+  } else if (record_value > m_object_type) {
+    cmp = +1;
   } else {
-    if (record_null) {
-      cmp = -1;
-    } else if (record_value < m_object_type) {
-      cmp = -1;
-    } else if (record_value > m_object_type) {
-      cmp = +1;
-    } else {
-      cmp = 0;
-    }
+    cmp = 0;
   }
 
   switch (m_find_flag) {
