@@ -1,25 +1,26 @@
-/* Copyright (c) 2008, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2024, Oracle and/or its affiliates.
 
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License, version 2.0,
-  as published by the Free Software Foundation.
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License, version 2.0,
+as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
-  but not limited to OpenSSL) that is licensed under separate terms,
-  as designated in a particular file or component or in included license
-  documentation.  The authors of MySQL hereby grant you an additional
-  permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+This program is designed to work with certain software (including
+but not limited to OpenSSL) that is licensed under separate terms,
+as designated in a particular file or component or in included license
+documentation.  The authors of MySQL hereby grant you an additional
+permission to link the program and your derivative works with the
+separately licensed software that they have either included with
+the program or referenced in the documentation.
 
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License, version 2.0, for more details.
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License, version 2.0, for more details.
 
-  You should have received a copy of the GNU General Public License
-  along with this program; if not, write to the Free Software
-  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
-  */
+You should have received a copy of the GNU General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA
+*/
 
 #ifndef PFS_TABLE_HELPER_H
 #define PFS_TABLE_HELPER_H
@@ -568,7 +569,7 @@ struct PFS_digest_row {
   /** Column DIGEST. */
   char m_digest[DIGEST_HASH_TO_STRING_LENGTH + 1];
   /** Length in bytes of @c m_digest. */
-  uint m_digest_length;
+  uint m_digest_length{0};
   /** Column DIGEST_TEXT. */
   String m_digest_text;
 
@@ -601,7 +602,7 @@ struct PFS_event_name_row {
 /** Row fragment for columns OBJECT_TYPE, SCHEMA_NAME, OBJECT_NAME. */
 struct PFS_object_row {
   /** Column OBJECT_TYPE. */
-  enum_object_type m_object_type;
+  enum_object_type m_object_type{NO_OBJECT_TYPE};
   /** Column SCHEMA_NAME. */
   PFS_schema_name m_schema_name;
   /** Column OBJECT_NAME. */
@@ -618,7 +619,7 @@ struct PFS_object_row {
 /** Row fragment for columns OBJECT_TYPE, SCHEMA_NAME, OBJECT_NAME. */
 struct PFS_object_view_row {
   /** Column OBJECT_TYPE. */
-  enum_object_type m_object_type;
+  enum_object_type m_object_type{NO_OBJECT_TYPE};
   /** Column SCHEMA_NAME. */
   PFS_schema_name_view m_schema_name;
   /** Column OBJECT_NAME. */
@@ -633,19 +634,19 @@ struct PFS_object_view_row {
  */
 struct PFS_column_row {
   /** Column OBJECT_TYPE. */
-  enum_object_type m_object_type;
+  enum_object_type m_object_type{NO_OBJECT_TYPE};
   /** Column SCHEMA_NAME. */
   char m_schema_name[NAME_LEN];
   /** Length in bytes of @c m_schema_name. */
-  size_t m_schema_name_length;
+  size_t m_schema_name_length{0};
   /** Column OBJECT_NAME. */
   char m_object_name[NAME_LEN];
   /** Length in bytes of @c m_object_name. */
-  size_t m_object_name_length;
+  size_t m_object_name_length{0};
   /** Column OBJECT_NAME. */
   char m_column_name[NAME_LEN];
   /** Length in bytes of @c m_column_name. */
-  size_t m_column_name_length;
+  size_t m_column_name_length{0};
 
   /** Build a row from a memory buffer. */
   int make_row(const MDL_key *mdl);
@@ -661,7 +662,7 @@ struct PFS_index_row {
   /** Column INDEX_NAME. */
   char m_index_name[NAME_LEN];
   /** Length in bytes of @c m_index_name. */
-  size_t m_index_name_length;
+  size_t m_index_name_length{0};
 
   /** Build a row from a memory buffer. */
   int make_index_name(PFS_table_share_index *pfs_index, uint table_index);
@@ -1142,8 +1143,8 @@ struct PFS_variable_value_row {
   int make_row(const CHARSET_INFO *cs, const char *str, size_t length);
 
   char m_str[1024];
-  uint m_length;
-  const CHARSET_INFO *m_charset;
+  uint m_length{0};
+  const CHARSET_INFO *m_charset{nullptr};
 };
 
 struct PFS_user_variable_value_row {
@@ -1421,7 +1422,9 @@ template <int SIZE>
 class PFS_key_string : public PFS_key_pstring {
  public:
   explicit PFS_key_string(const char *name)
-      : PFS_key_pstring(name), m_key_value_length(0) {}
+      : PFS_key_pstring(name), m_key_value_length(0) {
+    m_key_value[0] = '\0';
+  }
 
   ~PFS_key_string() override = default;
 
