@@ -1,15 +1,16 @@
-/* Copyright (c) 2008, 2023, Oracle and/or its affiliates.
+/* Copyright (c) 2008, 2024, Oracle and/or its affiliates.
 
   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License, version 2.0,
   as published by the Free Software Foundation.
 
-  This program is also distributed with certain software (including
+  This program is designed to work with certain software (including
   but not limited to OpenSSL) that is licensed under separate terms,
   as designated in a particular file or component or in included license
   documentation.  The authors of MySQL hereby grant you an additional
   permission to link the program and your derivative works with the
-  separately licensed software that they have included with MySQL.
+  separately licensed software that they have either included with
+  the program or referenced in the documentation.
 
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -159,7 +160,8 @@ table_setup_consumers::table_setup_consumers()
     : PFS_engine_table(&m_share, &m_pos),
       m_row(nullptr),
       m_pos(0),
-      m_next_pos(0) {}
+      m_next_pos(0),
+      m_opened_index(nullptr) {}
 
 void table_setup_consumers::reset_position() {
   m_pos.m_index = 0;
@@ -191,9 +193,8 @@ int table_setup_consumers::rnd_pos(const void *pos) {
 }
 
 int table_setup_consumers::index_init(uint idx [[maybe_unused]], bool) {
-  PFS_index_setup_consumers *result = nullptr;
   assert(idx == 0);
-  result = PFS_NEW(PFS_index_setup_consumers);
+  auto *result = PFS_NEW(PFS_index_setup_consumers);
   m_opened_index = result;
   m_index = result;
   return 0;
