@@ -1,15 +1,16 @@
-/* Copyright (c) 2002, 2021, Oracle and/or its affiliates.
+/* Copyright (c) 2002, 2024, Oracle and/or its affiliates.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2.0,
    as published by the Free Software Foundation.
 
-   This program is also distributed with certain software (including
+   This program is designed to work with certain software (including
    but not limited to OpenSSL) that is licensed under separate terms,
    as designated in a particular file or component or in included license
    documentation.  The authors of MySQL hereby grant you an additional
    permission to link the program and your derivative works with the
-   separately licensed software that they have included with MySQL.
+   separately licensed software that they have either included with
+   the program or referenced in the documentation.
 
    Without limiting anything contained in the foregoing, this file,
    which is part of C Driver for MySQL (Connector/C), is also subject to the
@@ -25,24 +26,21 @@
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301  USA */
 
-#include <assert.h>
-#include <string.h>
-#include <sys/types.h>
-
 #include <algorithm>
+#include <cassert>
+#include <cstdint>
+#include <cstring>
 
-#include "m_ctype.h"
-#include "m_string.h"
+#include "m_string.h"  // skip_trailing_space
 #include "my_compiler.h"
-
-#include "my_inttypes.h"
-#include "my_macros.h"
+#include "mysql/strings/m_ctype.h"
+#include "strings/m_ctype_internals.h"
 #include "strings/str_uca_type.h"
 #include "template_utils.h"
 
 size_t my_caseup_str_mb(const CHARSET_INFO *cs, char *str) {
-  uint32 l;
-  const uchar *map = cs->to_upper;
+  uint32_t l = 0;
+  const uint8_t *map = cs->to_upper;
   char *str_orig = str;
 
   while (*str) {
@@ -50,7 +48,7 @@ size_t my_caseup_str_mb(const CHARSET_INFO *cs, char *str) {
     if ((l = my_ismbchar(cs, str, str + cs->mbmaxlen)))
       str += l;
     else {
-      *str = (char)map[(uchar)*str];
+      *str = (char)map[(uint8_t)*str];
       str++;
     }
   }
@@ -58,8 +56,8 @@ size_t my_caseup_str_mb(const CHARSET_INFO *cs, char *str) {
 }
 
 size_t my_casedn_str_mb(const CHARSET_INFO *cs, char *str) {
-  uint32 l;
-  const uchar *map = cs->to_lower;
+  uint32_t l = 0;
+  const uint8_t *map = cs->to_lower;
   char *str_orig = str;
 
   while (*str) {
@@ -67,7 +65,7 @@ size_t my_casedn_str_mb(const CHARSET_INFO *cs, char *str) {
     if ((l = my_ismbchar(cs, str, str + cs->mbmaxlen)))
       str += l;
     else {
-      *str = (char)map[(uchar)*str];
+      *str = (char)map[(uint8_t)*str];
       str++;
     }
   }
@@ -75,7 +73,7 @@ size_t my_casedn_str_mb(const CHARSET_INFO *cs, char *str) {
 }
 
 static inline const MY_UNICASE_CHARACTER *get_case_info_for_ch(
-    const CHARSET_INFO *cs, uint page, uint offs) {
+    const CHARSET_INFO *cs, unsigned page, unsigned offs) {
   const MY_UNICASE_CHARACTER *p;
   return cs->caseinfo ? ((p = cs->caseinfo->page[page]) ? &p[offs] : nullptr)
                       : nullptr;
@@ -85,11 +83,11 @@ static inline const MY_UNICASE_CHARACTER *get_case_info_for_ch(
   For character sets which don't change octet length in case conversion.
 */
 size_t my_caseup_mb(const CHARSET_INFO *cs, char *src, size_t srclen,
-                    char *dst MY_ATTRIBUTE((unused)),
-                    size_t dstlen MY_ATTRIBUTE((unused))) {
-  uint32 l;
+                    char *dst [[maybe_unused]],
+                    size_t dstlen [[maybe_unused]]) {
+  uint32_t l = 0;
   char *srcend = src + srclen;
-  const uchar *map = cs->to_upper;
+  const uint8_t *map = cs->to_upper;
 
   assert(cs->caseup_multiply == 1);
   assert(src == dst && srclen == dstlen);
@@ -98,13 +96,13 @@ size_t my_caseup_mb(const CHARSET_INFO *cs, char *src, size_t srclen,
   while (src < srcend) {
     if ((l = my_ismbchar(cs, src, srcend))) {
       const MY_UNICASE_CHARACTER *ch;
-      if ((ch = get_case_info_for_ch(cs, (uchar)src[0], (uchar)src[1]))) {
+      if ((ch = get_case_info_for_ch(cs, (uint8_t)src[0], (uint8_t)src[1]))) {
         *src++ = ch->toupper >> 8;
         *src++ = ch->toupper & 0xFF;
       } else
         src += l;
     } else {
-      *src = (char)map[(uchar)*src];
+      *src = (char)map[(uint8_t)*src];
       src++;
     }
   }
@@ -112,11 +110,11 @@ size_t my_caseup_mb(const CHARSET_INFO *cs, char *src, size_t srclen,
 }
 
 size_t my_casedn_mb(const CHARSET_INFO *cs, char *src, size_t srclen,
-                    char *dst MY_ATTRIBUTE((unused)),
-                    size_t dstlen MY_ATTRIBUTE((unused))) {
-  uint32 l;
+                    char *dst [[maybe_unused]],
+                    size_t dstlen [[maybe_unused]]) {
+  uint32_t l = 0;
   char *srcend = src + srclen;
-  const uchar *map = cs->to_lower;
+  const uint8_t *map = cs->to_lower;
 
   assert(cs->casedn_multiply == 1);
   assert(src == dst && srclen == dstlen);
@@ -125,13 +123,13 @@ size_t my_casedn_mb(const CHARSET_INFO *cs, char *src, size_t srclen,
   while (src < srcend) {
     if ((l = my_ismbchar(cs, src, srcend))) {
       const MY_UNICASE_CHARACTER *ch;
-      if ((ch = get_case_info_for_ch(cs, (uchar)src[0], (uchar)src[1]))) {
+      if ((ch = get_case_info_for_ch(cs, (uint8_t)src[0], (uint8_t)src[1]))) {
         *src++ = ch->tolower >> 8;
         *src++ = ch->tolower & 0xFF;
       } else
         src += l;
     } else {
-      *src = (char)map[(uchar)*src];
+      *src = (char)map[(uint8_t)*src];
       src++;
     }
   }
@@ -149,8 +147,8 @@ size_t my_casedn_mb(const CHARSET_INFO *cs, char *src, size_t srclen,
 */
 static size_t my_casefold_mb_varlen(const CHARSET_INFO *cs, char *src,
                                     size_t srclen, char *dst,
-                                    size_t dstlen MY_ATTRIBUTE((unused)),
-                                    const uchar *map, size_t is_upper) {
+                                    size_t dstlen [[maybe_unused]],
+                                    const uint8_t *map, size_t is_upper) {
   char *srcend = src + srclen, *dst0 = dst;
 
   assert(cs->mbmaxlen == 2);
@@ -159,7 +157,7 @@ static size_t my_casefold_mb_varlen(const CHARSET_INFO *cs, char *src,
     size_t mblen = my_ismbchar(cs, src, srcend);
     if (mblen) {
       const MY_UNICASE_CHARACTER *ch;
-      if ((ch = get_case_info_for_ch(cs, (uchar)src[0], (uchar)src[1]))) {
+      if ((ch = get_case_info_for_ch(cs, (uint8_t)src[0], (uint8_t)src[1]))) {
         int code = is_upper ? ch->toupper : ch->tolower;
         src += 2;
         if (code > 0xFF) *dst++ = code >> 8;
@@ -169,7 +167,7 @@ static size_t my_casefold_mb_varlen(const CHARSET_INFO *cs, char *src,
         *dst++ = *src++;
       }
     } else {
-      *dst++ = (char)map[(uchar)*src++];
+      *dst++ = (char)map[(uint8_t)*src++];
     }
   }
   return (size_t)(dst - dst0);
@@ -194,8 +192,8 @@ size_t my_caseup_mb_varlen(const CHARSET_INFO *cs, char *src, size_t srclen,
  */
 
 int my_strcasecmp_mb(const CHARSET_INFO *cs, const char *s, const char *t) {
-  uint32 l;
-  const uchar *map = cs->to_upper;
+  uint32_t l = 0;
+  const uint8_t *map = cs->to_upper;
 
   while (*s && *t) {
     /* Pointing after the '\0' is safe here. */
@@ -203,7 +201,7 @@ int my_strcasecmp_mb(const CHARSET_INFO *cs, const char *s, const char *t) {
       while (l--)
         if (*s++ != *t++) return 1;
     } else if (my_mbcharlen(cs, *t) != 1 ||
-               map[(uchar)*s++] != map[(uchar)*t++])
+               map[(uint8_t)*s++] != map[(uint8_t)*t++])
       return 1;
   }
   /* At least one of '*s' and '*t' is zero here. */
@@ -221,17 +219,17 @@ int my_strcasecmp_mb(const CHARSET_INFO *cs, const char *s, const char *t) {
 #define INC_PTR(cs, A, B) \
   A += (my_ismbchar(cs, A, B) ? my_ismbchar(cs, A, B) : 1)
 
-#define likeconv(s, A) (uchar)(s)->sort_order[(uchar)(A)]
+#define likeconv(s, A) (uint8_t)(s)->sort_order[(uint8_t)(A)]
 
 static int my_wildcmp_mb_impl(const CHARSET_INFO *cs, const char *str,
                               const char *str_end, const char *wildstr_arg,
                               const char *wildend_arg, int escape, int w_one,
                               int w_many, int recurse_level) {
   int result = -1; /* Not found, using wildcards */
-  const uchar *wildstr = pointer_cast<const uchar *>(wildstr_arg);
-  const uchar *wildend = pointer_cast<const uchar *>(wildend_arg);
+  const uint8_t *wildstr = pointer_cast<const uint8_t *>(wildstr_arg);
+  const uint8_t *wildend = pointer_cast<const uint8_t *>(wildend_arg);
 
-  if (my_string_stack_guard && my_string_stack_guard(recurse_level)) return 1;
+  if (my_string_stack_guard && my_string_stack_guard(recurse_level)) return -1;
   while (wildstr != wildend) {
     while (*wildstr != w_many && *wildstr != w_one) {
       int l;
@@ -256,9 +254,7 @@ static int my_wildcmp_mb_impl(const CHARSET_INFO *cs, const char *str,
       if (wildstr == wildend) break;
     }
     if (*wildstr == w_many) { /* Found w_many */
-      uchar cmp;
-      const uchar *mb = wildstr;
-      int mb_len = 0;
+      uint8_t cmp = 0;
 
       wildstr++;
       /* Remove any '%' and '_' from the wild search string */
@@ -277,14 +273,14 @@ static int my_wildcmp_mb_impl(const CHARSET_INFO *cs, const char *str,
       if ((cmp = *wildstr) == escape && wildstr + 1 != wildend)
         cmp = *++wildstr;
 
-      mb = wildstr;
-      mb_len = my_ismbchar(cs, wildstr, wildend);
-      INC_PTR(cs, wildstr, wildend); /* This is compared trough cmp */
+      const uint8_t *mb = wildstr;
+      unsigned mb_len = my_ismbchar(cs, wildstr, wildend);
+      INC_PTR(cs, wildstr, wildend); /* This is compared through cmp */
       cmp = likeconv(cs, cmp);
       do {
         for (;;) {
           if (str >= str_end) return -1;
-          if (mb_len) {
+          if (mb_len > 0) {
             if (str + mb_len <= str_end && memcmp(str, mb, mb_len) == 0) {
               str += mb_len;
               break;
@@ -320,19 +316,19 @@ size_t my_numchars_mb(const CHARSET_INFO *cs, const char *pos,
                       const char *end) {
   size_t count = 0;
   while (pos < end) {
-    uint mb_len;
+    unsigned mb_len = 0;
     pos += (mb_len = my_ismbchar(cs, pos, end)) ? mb_len : 1;
     count++;
   }
   return count;
 }
 
-size_t my_charpos_mb(const CHARSET_INFO *cs, const char *pos, const char *end,
-                     size_t length) {
+size_t my_charpos_mb3(const CHARSET_INFO *cs, const char *pos, const char *end,
+                      size_t length) {
   const char *start = pos;
 
   while (length && pos < end) {
-    uint mb_len;
+    unsigned mb_len = 0;
     pos += (mb_len = my_ismbchar(cs, pos, end)) ? mb_len : 1;
     length--;
   }
@@ -347,8 +343,8 @@ size_t my_well_formed_len_mb(const CHARSET_INFO *cs, const char *b,
     my_wc_t wc;
     int mb_len;
 
-    if ((mb_len = cs->cset->mb_wc(cs, &wc, pointer_cast<const uchar *>(b),
-                                  pointer_cast<const uchar *>(e))) <= 0) {
+    if ((mb_len = cs->cset->mb_wc(cs, &wc, pointer_cast<const uint8_t *>(b),
+                                  pointer_cast<const uint8_t *>(e))) <= 0) {
       *error = b < e ? 1 : 0;
       break;
     }
@@ -358,56 +354,46 @@ size_t my_well_formed_len_mb(const CHARSET_INFO *cs, const char *b,
   return (size_t)(b - b_start);
 }
 
-uint my_instr_mb(const CHARSET_INFO *cs, const char *b, size_t b_length,
-                 const char *s, size_t s_length, my_match_t *match,
-                 uint nmatch) {
+bool my_instr_mb(const CHARSET_INFO *cs, const char *b, size_t b_length,
+                 const char *s, size_t s_length, my_match_t *match) {
   const char *end, *b0;
   int res = 0;
 
-  if (s_length <= b_length) {
-    if (!s_length) {
-      if (nmatch) {
-        match->beg = 0;
-        match->end = 0;
-        match->mb_len = 0;
-      }
-      return 1; /* Empty string is always found */
-    }
-
-    b0 = b;
-    end = b + b_length - s_length + 1;
-
-    while (b < end) {
-      int mb_len;
-
-      if (!cs->coll->strnncoll(cs, pointer_cast<const uchar *>(b), s_length,
-                               pointer_cast<const uchar *>(s), s_length,
-                               false)) {
-        if (nmatch) {
-          match[0].beg = 0;
-          match[0].end = (uint)(b - b0);
-          match[0].mb_len = res;
-          if (nmatch > 1) {
-            match[1].beg = match[0].end;
-            match[1].end = match[0].end + (uint)s_length;
-            match[1].mb_len = 0; /* Not computed */
-          }
-        }
-        return 2;
-      }
-      mb_len = (mb_len = my_ismbchar(cs, b, end)) ? mb_len : 1;
-      b += mb_len;
-      b_length -= mb_len;
-      res++;
+  if (!s_length) {
+    if (match != nullptr) {
+      match->end = 0;
+      match->mb_len = 0;
     }
+    return true; /* Empty string is always found */
   }
-  return 0;
+
+  b0 = b;
+  end = b + b_length;
+
+  while (b < end) {
+    int mb_len;
+
+    if (!cs->coll->strnncoll(cs, pointer_cast<const uint8_t *>(b), b_length,
+                             pointer_cast<const uint8_t *>(s), s_length,
+                             true /* t_is_prefix */)) {
+      if (match != nullptr) {
+        match->end = static_cast<unsigned>(b - b0);
+        match->mb_len = res;
+      }
+      return true;
+    }
+    mb_len = (mb_len = my_ismbchar(cs, b, end)) ? mb_len : 1;
+    b += mb_len;
+    b_length -= mb_len;
+    res++;
+  }
+  return false;
 }
 
 /* BINARY collations handlers for MB charsets */
 
-int my_strnncoll_mb_bin(const CHARSET_INFO *cs MY_ATTRIBUTE((unused)),
-                        const uchar *s, size_t slen, const uchar *t,
+int my_strnncoll_mb_bin(const CHARSET_INFO *cs [[maybe_unused]],
+                        const uint8_t *s, size_t slen, const uint8_t *t,
                         size_t tlen, bool t_is_prefix) {
   size_t len = std::min(slen, tlen);
   int cmp = len == 0 ? 0 : memcmp(s, t, len);
@@ -419,7 +405,7 @@ int my_strnncoll_mb_bin(const CHARSET_INFO *cs MY_ATTRIBUTE((unused)),
 
   SYNOPSIS
     my_strnncollsp_mb_bin()
-    cs			Chararacter set
+    cs			Character set
     s			String to compare
     slen		Length of 's'
     t			String to compare
@@ -436,14 +422,13 @@ int my_strnncoll_mb_bin(const CHARSET_INFO *cs MY_ATTRIBUTE((unused)),
     0 if strings are equal
 */
 
-int my_strnncollsp_mb_bin(const CHARSET_INFO *cs MY_ATTRIBUTE((unused)),
-                          const uchar *a, size_t a_length, const uchar *b,
+int my_strnncollsp_mb_bin(const CHARSET_INFO *cs [[maybe_unused]],
+                          const uint8_t *a, size_t a_length, const uint8_t *b,
                           size_t b_length) {
-  const uchar *end;
-  size_t length;
-  int res;
+  size_t length = 0;
+  int res = 0;
 
-  end = a + (length = std::min(a_length, b_length));
+  const uint8_t *end = a + (length = std::min(a_length, b_length));
   while (a < end) {
     if (*a++ != *b++) return ((int)a[-1] - (int)b[-1]);
   }
@@ -481,13 +466,13 @@ int my_strnncollsp_mb_bin(const CHARSET_INFO *cs MY_ATTRIBUTE((unused)),
     switch (cs->cset->ismbchar(cs, (const char *)src, (const char *)se)) {   \
       case 4:                                                                \
         *dst++ = *src++;                                                     \
-        /* fall through */                                                   \
+        [[fallthrough]];                                                     \
       case 3:                                                                \
         *dst++ = *src++;                                                     \
-        /* fall through */                                                   \
+        [[fallthrough]];                                                     \
       case 2:                                                                \
         *dst++ = *src++;                                                     \
-        /* fall through */                                                   \
+        [[fallthrough]];                                                     \
       case 0:                                                                \
         *dst++ = *src++; /* byte in range 0x80..0xFF which is not MB head */ \
     }                                                                        \
@@ -498,13 +483,13 @@ int my_strnncollsp_mb_bin(const CHARSET_INFO *cs MY_ATTRIBUTE((unused)),
   characters having multibyte weights *equal* to their codes:
   cp932, euckr, gb2312, sjis, eucjpms, ujis.
 */
-size_t my_strnxfrm_mb(const CHARSET_INFO *cs, uchar *dst, size_t dstlen,
-                      uint nweights, const uchar *src, size_t srclen,
-                      uint flags) {
-  uchar *d0 = dst;
-  uchar *de = dst + dstlen;
-  const uchar *se = src + srclen;
-  const uchar *sort_order = cs->sort_order;
+size_t my_strnxfrm_mb(const CHARSET_INFO *cs, uint8_t *dst, size_t dstlen,
+                      unsigned nweights, const uint8_t *src, size_t srclen,
+                      unsigned flags) {
+  uint8_t *d0 = dst;
+  uint8_t *de = dst + dstlen;
+  const uint8_t *se = src + srclen;
+  const uint8_t *sort_order = cs->sort_order;
 
   assert(cs->mbmaxlen <= 4);
 
@@ -535,7 +520,7 @@ size_t my_strnxfrm_mb(const CHARSET_INFO *cs, uchar *dst, size_t dstlen,
   }
 
   /*
-    A thourough loop, checking all possible limits:
+    A thorough loop, checking all possible limits:
     "se", "nweights" and "de".
   */
   for (; src < se && nweights && dst < de; nweights--) {
@@ -557,15 +542,15 @@ pad:
   return my_strxfrm_pad(cs, d0, dst, de, nweights, flags);
 }
 
-int my_strcasecmp_mb_bin(const CHARSET_INFO *cs MY_ATTRIBUTE((unused)),
-                         const char *s, const char *t) {
+int my_strcasecmp_mb_bin(const CHARSET_INFO *cs [[maybe_unused]], const char *s,
+                         const char *t) {
   return strcmp(s, t);
 }
 
-void my_hash_sort_mb_bin(const CHARSET_INFO *cs MY_ATTRIBUTE((unused)),
-                         const uchar *key, size_t len, uint64 *nr1,
-                         uint64 *nr2) {
-  const uchar *pos = key;
+void my_hash_sort_mb_bin(const CHARSET_INFO *cs [[maybe_unused]],
+                         const uint8_t *key, size_t len, uint64_t *nr1,
+                         uint64_t *nr2) {
+  const uint8_t *pos = key;
 
   /*
      Remove trailing spaces. We have to do this to be able to compare
@@ -575,7 +560,8 @@ void my_hash_sort_mb_bin(const CHARSET_INFO *cs MY_ATTRIBUTE((unused)),
 
   for (; pos < key; pos++) {
     nr1[0] ^=
-        (uint64)((((uint)nr1[0] & 63) + nr2[0]) * ((uint)*pos)) + (nr1[0] << 8);
+        (uint64_t)((((unsigned)nr1[0] & 63) + nr2[0]) * ((unsigned)*pos)) +
+        (nr1[0] << 8);
     nr2[0] += 3;
   }
 }
@@ -600,7 +586,7 @@ void my_hash_sort_mb_bin(const CHARSET_INFO *cs MY_ATTRIBUTE((unused)),
 */
 static void pad_max_char(const CHARSET_INFO *cs, char *str, char *end) {
   char buf[10];
-  char buflen;
+  int buflen = 0;
 
   if (!(cs->state & MY_CS_UNICODE)) {
     if (cs->max_sort_char <= 255) {
@@ -620,14 +606,15 @@ static void pad_max_char(const CHARSET_INFO *cs, char *str, char *end) {
       buflen = 4;
     }
   } else {
-    buflen = cs->cset->wc_mb(cs, cs->max_sort_char, (uchar *)buf,
-                             (uchar *)buf + sizeof(buf));
+    buflen =
+        cs->cset->wc_mb(cs, cs->max_sort_char, pointer_cast<uint8_t *>(buf),
+                        pointer_cast<uint8_t *>(buf) + sizeof(buf));
   }
 
   assert(buflen > 0);
   do {
     if ((str + buflen) <= end) {
-      /* Enough space for the characer */
+      /* Enough space for the character */
       memcpy(str, buf, buflen);
       str += buflen;
     } else {
@@ -661,7 +648,7 @@ bool my_like_range_mb(const CHARSET_INFO *cs, const char *ptr,
                       size_t ptr_length, char escape, char w_one, char w_many,
                       size_t res_length, char *min_str, char *max_str,
                       size_t *min_length, size_t *max_length) {
-  uint mb_len;
+  unsigned mb_len = 0;
   const char *end = ptr + ptr_length;
   char *min_org = min_str;
   char *min_end = min_str + res_length;
@@ -733,10 +720,10 @@ bool my_like_range_mb(const CHARSET_INFO *cs, const char *ptr,
         'ab\min\min\min\min' and 'ab\max\max\max\max'.
 
       */
-      const char *contraction_flags = nullptr;
+      const MY_UCA_INFO::flags_type *contraction_flags = nullptr;
       if (cs->uca) contraction_flags = cs->uca->contraction_flags;
       if (contraction_flags && ptr + 1 < end &&
-          my_uca_can_be_contraction_head(contraction_flags, (uchar)*ptr)) {
+          my_uca_can_be_contraction_head(contraction_flags, (uint8_t)*ptr)) {
         /* Ptr[0] is a contraction head. */
 
         if (ptr[1] == w_one || ptr[1] == w_many) {
@@ -756,9 +743,10 @@ bool my_like_range_mb(const CHARSET_INFO *cs, const char *ptr,
           is not a contraction, then we put only ptr[0],
           and continue with ptr[1] on the next loop.
         */
-        if (my_uca_can_be_contraction_tail(contraction_flags, (uchar)ptr[1]) &&
+        if (my_uca_can_be_contraction_tail(contraction_flags,
+                                           (uint8_t)ptr[1]) &&
             my_uca_contraction2_weight(cs->uca->contraction_nodes,
-                                       (uchar)ptr[0], ptr[1])) {
+                                       (uint8_t)ptr[0], ptr[1])) {
           /* Contraction found */
           if (maxcharlen == 1 || min_str + 1 >= min_end) {
             /* Both contraction parts don't fit, quit */
@@ -821,8 +809,8 @@ bool my_like_range_generic(const CHARSET_INFO *cs, const char *ptr,
   for (; charlen > 0; charlen--) {
     my_wc_t wc, wc2;
     int res;
-    if ((res = cs->cset->mb_wc(cs, &wc, pointer_cast<const uchar *>(ptr),
-                               pointer_cast<const uchar *>(end))) <= 0) {
+    if ((res = cs->cset->mb_wc(cs, &wc, pointer_cast<const uint8_t *>(ptr),
+                               pointer_cast<const uint8_t *>(end))) <= 0) {
       if (res == MY_CS_ILSEQ) /* Bad sequence */
         return true;          /* min_length and max_length are not important */
       break;                  /* End of the string */
@@ -830,37 +818,39 @@ bool my_like_range_generic(const CHARSET_INFO *cs, const char *ptr,
     ptr += res;
 
     if (wc == (my_wc_t)escape) {
-      if ((res = cs->cset->mb_wc(cs, &wc, pointer_cast<const uchar *>(ptr),
-                                 pointer_cast<const uchar *>(end))) <= 0) {
+      if ((res = cs->cset->mb_wc(cs, &wc, pointer_cast<const uint8_t *>(ptr),
+                                 pointer_cast<const uint8_t *>(end))) <= 0) {
         if (res == MY_CS_ILSEQ)
           return true; /* min_length and max_length are not important */
-        /*
-           End of the string: Escape is the last character.
-           Put escape as a normal character.
-           We'll will leave the loop on the next iteration.
-        */
+                       /*
+                          End of the string: Escape is the last character.
+                          Put escape as a normal character.
+                          We'll will leave the loop on the next iteration.
+                       */
       } else
         ptr += res;
 
       /* Put escape character to min_str and max_str  */
-      if ((res = cs->cset->wc_mb(cs, wc, (uchar *)min_str, (uchar *)min_end)) <=
-          0)
+      if ((res = cs->cset->wc_mb(cs, wc, pointer_cast<uint8_t *>(min_str),
+                                 pointer_cast<uint8_t *>(min_end))) <= 0)
         goto pad_set_lengths; /* No space */
       min_str += res;
 
-      if ((res = cs->cset->wc_mb(cs, wc, (uchar *)max_str, (uchar *)max_end)) <=
-          0)
+      if ((res = cs->cset->wc_mb(cs, wc, pointer_cast<uint8_t *>(max_str),
+                                 pointer_cast<uint8_t *>(max_end))) <= 0)
         goto pad_set_lengths; /* No space */
       max_str += res;
       continue;
     } else if (wc == (my_wc_t)w_one) {
-      if ((res = cs->cset->wc_mb(cs, cs->min_sort_char, (uchar *)min_str,
-                                 (uchar *)min_end)) <= 0)
+      if ((res = cs->cset->wc_mb(cs, cs->min_sort_char,
+                                 pointer_cast<uint8_t *>(min_str),
+                                 pointer_cast<uint8_t *>(min_end))) <= 0)
         goto pad_set_lengths;
       min_str += res;
 
-      if ((res = cs->cset->wc_mb(cs, cs->max_sort_char, (uchar *)max_str,
-                                 (uchar *)max_end)) <= 0)
+      if ((res = cs->cset->wc_mb(cs, cs->max_sort_char,
+                                 pointer_cast<uint8_t *>(max_str),
+                                 pointer_cast<uint8_t *>(max_end))) <= 0)
         goto pad_set_lengths;
       max_str += res;
       continue;
@@ -876,13 +866,13 @@ bool my_like_range_generic(const CHARSET_INFO *cs, const char *ptr,
       goto pad_min_max;
     }
 
-    const char *contraction_flags = nullptr;
+    const MY_UCA_INFO::flags_type *contraction_flags = nullptr;
     if (cs->uca) contraction_flags = cs->uca->contraction_flags;
     if (contraction_flags &&
         my_uca_can_be_contraction_head(contraction_flags, wc) &&
-        (res = cs->cset->mb_wc(cs, &wc2, pointer_cast<const uchar *>(ptr),
-                               pointer_cast<const uchar *>(end))) > 0) {
-      const uint16 *weight;
+        (res = cs->cset->mb_wc(cs, &wc2, pointer_cast<const uint8_t *>(ptr),
+                               pointer_cast<const uint8_t *>(end))) > 0) {
+      const uint16_t *weight = nullptr;
       if ((wc2 == (my_wc_t)w_one || wc2 == (my_wc_t)w_many)) {
         /* Contraction head followed by a wildcard */
         *min_length = *max_length = res_length;
@@ -904,13 +894,13 @@ bool my_like_range_generic(const CHARSET_INFO *cs, const char *ptr,
         charlen--;
 
         /* Put contraction head */
-        if ((res = cs->cset->wc_mb(cs, wc, (uchar *)min_str,
-                                   (uchar *)min_end)) <= 0)
+        if ((res = cs->cset->wc_mb(cs, wc, pointer_cast<uint8_t *>(min_str),
+                                   pointer_cast<uint8_t *>(min_end))) <= 0)
           goto pad_set_lengths;
         min_str += res;
 
-        if ((res = cs->cset->wc_mb(cs, wc, (uchar *)max_str,
-                                   (uchar *)max_end)) <= 0)
+        if ((res = cs->cset->wc_mb(cs, wc, pointer_cast<uint8_t *>(max_str),
+                                   pointer_cast<uint8_t *>(max_end))) <= 0)
           goto pad_set_lengths;
         max_str += res;
         wc = wc2; /* Prepare to put contraction tail */
@@ -918,12 +908,12 @@ bool my_like_range_generic(const CHARSET_INFO *cs, const char *ptr,
     }
 
     /* Normal character, or contraction tail */
-    if ((res = cs->cset->wc_mb(cs, wc, (uchar *)min_str, (uchar *)min_end)) <=
-        0)
+    if ((res = cs->cset->wc_mb(cs, wc, pointer_cast<uint8_t *>(min_str),
+                               pointer_cast<uint8_t *>(min_end))) <= 0)
       goto pad_set_lengths;
     min_str += res;
-    if ((res = cs->cset->wc_mb(cs, wc, (uchar *)max_str, (uchar *)max_end)) <=
-        0)
+    if ((res = cs->cset->wc_mb(cs, wc, pointer_cast<uint8_t *>(max_str),
+                               pointer_cast<uint8_t *>(max_end))) <= 0)
       goto pad_set_lengths;
     max_str += res;
   }
@@ -959,10 +949,10 @@ static int my_wildcmp_mb_bin_impl(const CHARSET_INFO *cs, const char *str,
                                   const char *wildend_arg, int escape,
                                   int w_one, int w_many, int recurse_level) {
   int result = -1; /* Not found, using wildcards */
-  const uchar *wildstr = pointer_cast<const uchar *>(wildstr_arg);
-  const uchar *wildend = pointer_cast<const uchar *>(wildend_arg);
+  const uint8_t *wildstr = pointer_cast<const uint8_t *>(wildstr_arg);
+  const uint8_t *wildend = pointer_cast<const uint8_t *>(wildend_arg);
 
-  if (my_string_stack_guard && my_string_stack_guard(recurse_level)) return 1;
+  if (my_string_stack_guard && my_string_stack_guard(recurse_level)) return -1;
   while (wildstr != wildend) {
     while (*wildstr != w_many && *wildstr != w_one) {
       int l;
@@ -971,7 +961,7 @@ static int my_wildcmp_mb_bin_impl(const CHARSET_INFO *cs, const char *str,
         if (str + l > str_end || memcmp(str, wildstr, l) != 0) return 1;
         str += l;
         wildstr += l;
-      } else if (str == str_end || *wildstr++ != static_cast<uchar>(*str++))
+      } else if (str == str_end || *wildstr++ != static_cast<uint8_t>(*str++))
         return (1); /* No match */
       if (wildstr == wildend)
         return (str != str_end); /* Match if both are at end */
@@ -987,7 +977,6 @@ static int my_wildcmp_mb_bin_impl(const CHARSET_INFO *cs, const char *str,
     }
     if (*wildstr == w_many) { /* Found w_many */
       int cmp;
-      const uchar *mb = wildstr;
       int mb_len = 0;
 
       wildstr++;
@@ -1007,9 +996,9 @@ static int my_wildcmp_mb_bin_impl(const CHARSET_INFO *cs, const char *str,
       if ((cmp = *wildstr) == escape && wildstr + 1 != wildend)
         cmp = *++wildstr;
 
-      mb = wildstr;
+      const uint8_t *mb = wildstr;
       mb_len = my_ismbchar(cs, wildstr, wildend);
-      INC_PTR(cs, wildstr, wildend); /* This is compared trough cmp */
+      INC_PTR(cs, wildstr, wildend); /* This is compared through cmp */
       do {
         for (;;) {
           if (str >= str_end) return -1;
@@ -1019,7 +1008,7 @@ static int my_wildcmp_mb_bin_impl(const CHARSET_INFO *cs, const char *str,
               break;
             }
           } else if (!my_ismbchar(cs, str, str_end) &&
-                     static_cast<uchar>(*str) == cmp) {
+                     static_cast<uint8_t>(*str) == cmp) {
             str++;
             break;
           }
@@ -1294,9 +1283,9 @@ size_t my_numcells_mb(const CHARSET_INFO *cs, const char *b, const char *e) {
 
   while (b < e) {
     int mb_len;
-    uint pg;
-    if ((mb_len = cs->cset->mb_wc(cs, &wc, pointer_cast<const uchar *>(b),
-                                  pointer_cast<const uchar *>(e))) <= 0 ||
+    unsigned pg = 0;
+    if ((mb_len = cs->cset->mb_wc(cs, &wc, pointer_cast<const uint8_t *>(b),
+                                  pointer_cast<const uint8_t *>(e))) <= 0 ||
         wc > 0xFFFF) {
       /*
         Let's think a wrong sequence takes 1 dysplay cell.
@@ -1320,8 +1309,8 @@ size_t my_numcells_mb(const CHARSET_INFO *cs, const char *b, const char *e) {
   return clen;
 }
 
-int my_mb_ctype_mb(const CHARSET_INFO *cs, int *ctype, const uchar *s,
-                   const uchar *e) {
+int my_mb_ctype_mb(const CHARSET_INFO *cs, int *ctype, const uint8_t *s,
+                   const uint8_t *e) {
   my_wc_t wc;
   int res = cs->cset->mb_wc(cs, &wc, s, e);
   if (res <= 0 || wc > 0xFFFF)
