@@ -32,6 +32,7 @@
 
 #include <gtest/gtest.h>
 
+#include "buf0flu.h"
 #include "clone0api.h" /* clone_init(), clone_free() */
 #include "fil0fil.h"
 #include "log0buf.h"
@@ -60,9 +61,7 @@ static std::map<std::string, std::vector<std::string>> log_sync_points = {
       "log_wfs_after_reserving_before_buf_size_2",
       "log_wait_for_space_in_buf_middle"}},
 
-    {"log_buffer_reserve",
-     {"log_buffer_reserve_before_buf_limit_sn",
-      "log_buffer_reserve_before_confirmation"}},
+    {"log_buffer_reserve", {"log_buffer_reserve_before_buf_limit_sn"}},
 
     {"log_buffer_write",
      {"log_buffer_write_before_memcpy",
@@ -80,9 +79,6 @@ static std::map<std::string, std::vector<std::string>> log_sync_points = {
       "log_writer_after_archiver_check", "log_writer_before_lsn_update",
       "log_writer_before_buf_limit_update", "log_writer_write_end"}},
 
-    {"log_closer",
-     {"log_advance_dpa_before_reclaim", "log_advance_dpa_before_update"}},
-
     {"log_checkpointer",
      {"log_get_available_for_chkp_lsn_before_buf_pool",
       "log_get_available_for_chkp_lsn_before_dpa",
@@ -133,7 +129,7 @@ static bool log_test_general_init() {
   srv_log_write_events = 64;
   srv_log_flush_events = 64;
   srv_log_recent_written_size = 4 * 4096;
-  srv_log_recent_closed_size = 4 * 4096;
+  srv_buf_flush_list_added_size = 4 * 4096;
   srv_log_writer_threads = true;
   srv_log_wait_for_write_spin_delay = 0;
   srv_log_wait_for_flush_timeout = 100000;
@@ -203,6 +199,7 @@ static bool log_test_init() {
 
   fil_open_system_tablespace_files();
 
+  buf_flush_list_added = Buf_flush_list_added_lsns::create();
   err = log_start(log, flushed_lsn, flushed_lsn);
   ut_a(err == DB_SUCCESS);
 
@@ -265,7 +262,7 @@ static void run_threads(TFunctor f, size_t n_threads) {
         [&f, &ready, &started](size_t thread_no) {
           ++ready;
           while (!started) {
-            std::this_thread::yield();
+            std::this_thread::sleep_for(std::chrono::seconds(0));
           }
           f(thread_no);
         },
@@ -273,7 +270,7 @@ static void run_threads(TFunctor f, size_t n_threads) {
   }
 
   while (ready < n_threads) {
-    std::this_thread::yield();
+    std::this_thread::sleep_for(std::chrono::seconds(0));
   }
   started = true;
 
@@ -309,9 +306,9 @@ static lsn_t write_single_mlog_test(Log_test::Key key) {
     log_buffer_set_first_record_group(log, end_lsn);
   }
 
-  log_buffer_write_completed(log, handle.start_lsn, end_lsn);
+  log_buffer_write_completed(log, handle.start_lsn, end_lsn, true);
 
-  log_wait_for_space_in_log_recent_closed(log, handle.start_lsn);
+  buf_flush_list_added->wait_to_add(handle.start_lsn);
 
   Log_test::Page page;
   page.key = key;
@@ -320,8 +317,8 @@ static lsn_t write_single_mlog_test(Log_test::Key key) {
   page.newest_modification = handle.end_lsn;
 
   log_test->add_dirty_page(page);
-
-  log_buffer_close(log, handle);
+  buf_flush_list_added->report_added(page.oldest_modification,
+                                     page.newest_modification);
 
   return end_lsn;
 }
@@ -335,6 +332,7 @@ static lsn_t write_multi_mlog_tests(Log_test::Key key, size_t n) {
   std::queue<uchar *> record_end;
   Log_test::Value value_left = 0;
   log_t &log = *log_sys;
+  bool is_last_block{false};
 
   /* We need to prepare all log records first, because we need to know
   how much space we have to reserve in redo log. */
@@ -403,12 +401,15 @@ static lsn_t write_multi_mlog_tests(Log_test::Key key, size_t n) {
     ptr = end;
     ut_a(ptr <= buf + MLOG_TEST_MAX_GROUP_LEN);
 
-    if (left_to_write == 0 && group_start_lsn / OS_FILE_LOG_BLOCK_SIZE !=
-                                  end_lsn / OS_FILE_LOG_BLOCK_SIZE) {
-      log_buffer_set_first_record_group(log, end_lsn);
+    if (left_to_write == 0) {
+      is_last_block = true;
+      if (group_start_lsn / OS_FILE_LOG_BLOCK_SIZE !=
+          end_lsn / OS_FILE_LOG_BLOCK_SIZE) {
+        log_buffer_set_first_record_group(log, end_lsn);
+      }
     }
 
-    log_buffer_write_completed(log, start_lsn, end_lsn);
+    log_buffer_write_completed(log, start_lsn, end_lsn, is_last_block);
 
     start_lsn = end_lsn;
   }
@@ -416,7 +417,7 @@ static lsn_t write_multi_mlog_tests(Log_test::Key key, size_t n) {
   ut_a(group_end_lsn == end_lsn);
   ut_a(left_to_write == 0);
 
-  log_wait_for_space_in_log_recent_closed(log, handle.start_lsn);
+  buf_flush_list_added->wait_to_add(handle.start_lsn);
 
   Log_test::Page page;
   page.key = key;
@@ -426,8 +427,8 @@ static lsn_t write_multi_mlog_tests(Log_test::Key key, size_t n) {
 
   log_test->add_dirty_page(page);
 
-  log_buffer_close(log, handle);
-
+  buf_flush_list_added->report_added(page.oldest_modification,
+                                     page.newest_modification);
   return group_end_lsn;
 }
 
@@ -560,6 +561,8 @@ static void log_test_close() {
 
   log_test_general_close();
 
+  buf_flush_list_added.reset(nullptr);
+
   ASSERT_FALSE(found_missing);
 }
 
@@ -745,6 +748,4 @@ TEST(log0log, log_flush_notifier) { test_single("log_flush_notifier"); }
 
 TEST(log0log, log_users) { test_single("log_users"); }
 
-TEST(log0log, log_closer) { test_single("log_closer"); }
-
 TEST(log0log, log_checkpointer) { test_single("log_checkpointer"); }
